// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/container.proto
// Protobuf C++ Version: 7.36.1

#ifndef pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_2epb_2eh
#define pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "pkg/sentry/seccheck/points/common.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto;
}  // extern "C"
namespace gvisor {
namespace container {
class Start;
struct StartGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern StartGlobalsTypeInternal Start_globals_;
extern const ::google::protobuf::internal::ClassDataFull Start_class_data_;
#else
extern const StartGlobalsTypeInternal Start_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace container
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace gvisor {
namespace container {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Start final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.container.Start) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Start() : Start(nullptr) {}
  ~Start() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Start* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Start));
  }
#endif

  template <typename = void>
  explicit constexpr Start(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Start(const Start& from) : Start(nullptr, from) {}
  inline Start(Start&& from) noexcept : Start(nullptr, ::std::move(from)) {}
  inline Start& operator=(const Start& from) {
    CopyFrom(from);
    return *this;
  }
  inline Start& operator=(Start&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Start& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Start>(&Start_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Start& a, Start& b) { a.Swap(&b); }
  inline void Swap(Start* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Start* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Start* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Start>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Start& from);
  using Super_::MergeFrom;
  void MergeFrom(const Start& from) { Start::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Start* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.container.Start"; }

  explicit Start(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Start(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Start& from);
  Start(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Start&& from) noexcept
      : Start(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kArgsFieldNumber = 4,
    kEnvFieldNumber = 5,
    kIdFieldNumber = 2,
    kCwdFieldNumber = 3,
    kContextDataFieldNumber = 1,
    kTerminalFieldNumber = 6,
  };
  // repeated string args = 4;
  [[nodiscard]] int args_size() const;
  private:
  int _internal_args_size() const;

  public:
  void clear_args() ;
  [[nodiscard]] const ::std::string& args(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_args(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_args(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_args();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_args(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      args() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_args();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_args() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_args();

  public:
  // repeated string env = 5;
  [[nodiscard]] int env_size() const;
  private:
  int _internal_env_size() const;

  public:
  void clear_env() ;
  [[nodiscard]] const ::std::string& env(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_env(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_env(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_env();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_env(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      env() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_env();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_env() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_env();

  public:
  // string id = 2;
  void clear_id() ;
  [[nodiscard]] const ::std::string& id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_id();
  void set_allocated_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_id();

  public:
  // string cwd = 3;
  void clear_cwd() ;
  [[nodiscard]] const ::std::string& cwd() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cwd(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_cwd();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_cwd();
  void set_allocated_cwd(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_cwd() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_cwd(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_cwd();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // bool terminal = 6;
  void clear_terminal() ;
  [[nodiscard]] bool terminal() const;
  void set_terminal(bool value);

  private:
  bool _internal_terminal() const;
  void _internal_set_terminal(bool value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.container.Start)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          1, 43,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Start& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> args_;
    ::google::protobuf::RepeatedPtrField<::std::string> env_;
    ::google::protobuf::internal::ArenaStringPtr id_;
    ::google::protobuf::internal::ArenaStringPtr cwd_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    bool terminal_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Start

// .gvisor.common.ContextData context_data = 1;
inline bool Start::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Start::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Start::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.context_data)
  return _internal_context_data();
}
inline void Start::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.container.Start.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Start::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Start::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.container.Start.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Start::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Start::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.container.Start.context_data)
  return _msg;
}
inline void Start::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.container.Start.context_data)
}

// string id = 2;
inline void Start::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& Start::id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.id)
  return _internal_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Start::set_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.container.Start.id)
}
inline ::std::string* PROTOBUF_NONNULL Start::mutable_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_id();
  // @@protoc_insertion_point(field_mutable:gvisor.container.Start.id)
  return _s;
}
inline const ::std::string& Start::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_.Get();
}
inline void Start::_internal_set_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Start::_internal_mutable_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Start::release_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.container.Start.id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.id_.Set("", GetArena());
  }
  return released;
}
inline void Start::set_allocated_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.id_.IsDefault()) {
    _impl_.id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.container.Start.id)
}

// string cwd = 3;
inline void Start::clear_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cwd_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& Start::cwd() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.cwd)
  return _internal_cwd();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Start::set_cwd(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.cwd_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.container.Start.cwd)
}
inline ::std::string* PROTOBUF_NONNULL Start::mutable_cwd()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_cwd();
  // @@protoc_insertion_point(field_mutable:gvisor.container.Start.cwd)
  return _s;
}
inline const ::std::string& Start::_internal_cwd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cwd_.Get();
}
inline void Start::_internal_set_cwd(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cwd_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Start::_internal_mutable_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.cwd_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Start::release_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.container.Start.cwd)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.cwd_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.cwd_.Set("", GetArena());
  }
  return released;
}
inline void Start::set_allocated_cwd(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.cwd_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.cwd_.IsDefault()) {
    _impl_.cwd_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.container.Start.cwd)
}

// repeated string args = 4;
inline int Start::_internal_args_size() const {
  return _internal_args().size();
}
inline int Start::args_size() const {
  return _internal_args_size();
}
inline void Start::clear_args() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.args_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL Start::add_args()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_args()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.container.Start.args)
  return _s;
}
inline const ::std::string& Start::args(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.args)
  return _internal_args().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Start::mutable_args(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.container.Start.args)
  return _internal_mutable_args()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Start::set_args(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_args()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.container.Start.args)
}
template <typename Arg_, typename... Args_>
inline void Start::add_args(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_args(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:gvisor.container.Start.args)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Start::args()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.container.Start.args)
  return _internal_args();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Start::mutable_args() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.container.Start.args)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_args();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Start::_internal_args() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.args_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Start::_internal_mutable_args() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.args_;
}

// repeated string env = 5;
inline int Start::_internal_env_size() const {
  return _internal_env().size();
}
inline int Start::env_size() const {
  return _internal_env_size();
}
inline void Start::clear_env() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.env_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL Start::add_env()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_env()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.container.Start.env)
  return _s;
}
inline const ::std::string& Start::env(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.env)
  return _internal_env().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Start::mutable_env(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.container.Start.env)
  return _internal_mutable_env()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Start::set_env(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_env()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.container.Start.env)
}
template <typename Arg_, typename... Args_>
inline void Start::add_env(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_env(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:gvisor.container.Start.env)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Start::env()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.container.Start.env)
  return _internal_env();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Start::mutable_env() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.container.Start.env)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_env();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Start::_internal_env() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.env_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Start::_internal_mutable_env() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.env_;
}

// bool terminal = 6;
inline void Start::clear_terminal() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.terminal_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline bool Start::terminal() const {
  // @@protoc_insertion_point(field_get:gvisor.container.Start.terminal)
  return _internal_terminal();
}
inline void Start::set_terminal(bool value) {
  _internal_set_terminal(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.container.Start.terminal)
}
inline bool Start::_internal_terminal() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.terminal_;
}
inline void Start::_internal_set_terminal(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.terminal_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace container
}  // namespace gvisor


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_2epb_2eh
