// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/common.proto
// Protobuf C++ Version: 7.36.1

#ifndef pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto_2epb_2eh
#define pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/generated_enum_reflection.h"
#include "google/protobuf/unknown_field_set.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto;
}  // extern "C"
namespace gvisor {
namespace common {
enum MessageType : int;
extern const uint32_t MessageType_internal_data_[];
class ContextData;
struct ContextDataGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ContextDataGlobalsTypeInternal ContextData_globals_;
extern const ::google::protobuf::internal::ClassDataFull ContextData_class_data_;
#else
extern const ContextDataGlobalsTypeInternal ContextData_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Credentials;
struct CredentialsGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern CredentialsGlobalsTypeInternal Credentials_globals_;
extern const ::google::protobuf::internal::ClassDataFull Credentials_class_data_;
#else
extern const CredentialsGlobalsTypeInternal Credentials_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Handshake;
struct HandshakeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern HandshakeGlobalsTypeInternal Handshake_globals_;
extern const ::google::protobuf::internal::ClassDataFull Handshake_class_data_;
#else
extern const HandshakeGlobalsTypeInternal Handshake_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace common
}  // namespace gvisor
namespace google {
namespace protobuf {
template <>
internal::EnumTraitsT<::gvisor::common::MessageType_internal_data_>
    internal::EnumTraitsImpl::value<::gvisor::common::MessageType>;
}  // namespace protobuf
}  // namespace google

namespace gvisor {
namespace common {
enum MessageType : int {
  MESSAGE_UNKNOWN = 0,
  MESSAGE_CONTAINER_START = 1,
  MESSAGE_SENTRY_CLONE = 2,
  MESSAGE_SENTRY_EXEC = 3,
  MESSAGE_SENTRY_EXIT_NOTIFY_PARENT = 4,
  MESSAGE_SENTRY_TASK_EXIT = 5,
  MESSAGE_SYSCALL_RAW = 6,
  MESSAGE_SYSCALL_OPEN = 7,
  MESSAGE_SYSCALL_CLOSE = 8,
  MESSAGE_SYSCALL_READ = 9,
  MESSAGE_SYSCALL_CONNECT = 10,
  MESSAGE_SYSCALL_EXECVE = 11,
  MESSAGE_SYSCALL_SOCKET = 12,
  MESSAGE_SYSCALL_CHDIR = 13,
  MESSAGE_SYSCALL_SETID = 14,
  MESSAGE_SYSCALL_SETRESID = 15,
  MESSAGE_SYSCALL_PRLIMIT64 = 16,
  MESSAGE_SYSCALL_PIPE = 17,
  MESSAGE_SYSCALL_FCNTL = 18,
  MESSAGE_SYSCALL_DUP = 19,
  MESSAGE_SYSCALL_SIGNALFD = 20,
  MESSAGE_SYSCALL_CHROOT = 21,
  MESSAGE_SYSCALL_EVENTFD = 22,
  MESSAGE_SYSCALL_CLONE = 23,
  MESSAGE_SYSCALL_BIND = 24,
  MESSAGE_SYSCALL_ACCEPT = 25,
  MESSAGE_SYSCALL_TIMERFD_CREATE = 26,
  MESSAGE_SYSCALL_TIMERFD_SETTIME = 27,
  MESSAGE_SYSCALL_TIMERFD_GETTIME = 28,
  MESSAGE_SYSCALL_FORK = 29,
  MESSAGE_SYSCALL_INOTIFY_INIT = 30,
  MESSAGE_SYSCALL_INOTIFY_ADD_WATCH = 31,
  MESSAGE_SYSCALL_INOTIFY_RM_WATCH = 32,
  MESSAGE_SYSCALL_SOCKETPAIR = 33,
  MESSAGE_SYSCALL_WRITE = 34,
  MessageType_INT_MIN_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::min(),
  MessageType_INT_MAX_SENTINEL_DO_NOT_USE_ =
      ::std::numeric_limits<::int32_t>::max(),
};

extern const uint32_t MessageType_internal_data_[];
inline constexpr MessageType MessageType_MIN =
    static_cast<MessageType>(0);
inline constexpr MessageType MessageType_MAX =
    static_cast<MessageType>(34);
[[nodiscard]] inline bool MessageType_IsValid(int value) {
  return 0 <= value && value <= 34;
}
inline constexpr int MessageType_ARRAYSIZE = 34 + 1;
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
MessageType_descriptor();
[[nodiscard]] inline auto ProtobufInternalGetEnumDescriptor(MessageType) {
  return MessageType_descriptor();
}
template <typename T>
[[nodiscard]] const ::std::string& MessageType_Name(T value) {
  static_assert(::std::is_same<T, MessageType>::value ||
                    ::std::is_integral<T>::value,
                "Incorrect type passed to MessageType_Name().");
  return MessageType_Name(static_cast<MessageType>(value));
}
template <>
[[nodiscard]] inline const ::std::string& MessageType_Name(MessageType value) {
  return ::google::protobuf::internal::NameOfDenseEnum<MessageType_descriptor, 0, 34>(
      static_cast<int>(value));
}
[[nodiscard]] inline bool MessageType_Parse(
    ::absl::string_view name, MessageType* PROTOBUF_NONNULL value) {
  return ::google::protobuf::internal::ParseNamedEnum<MessageType>(MessageType_descriptor(), name,
                                           value);
}
using ::google::protobuf::internal::generated_enum::AbslParseFlag;
using ::google::protobuf::internal::generated_enum::AbslUnparseFlag;

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Handshake final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.common.Handshake) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Handshake() : Handshake(nullptr) {}
  ~Handshake() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Handshake* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Handshake));
  }
#endif

  template <typename = void>
  explicit constexpr Handshake(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Handshake(const Handshake& from) : Handshake(nullptr, from) {}
  inline Handshake(Handshake&& from) noexcept : Handshake(nullptr, ::std::move(from)) {}
  inline Handshake& operator=(const Handshake& from) {
    CopyFrom(from);
    return *this;
  }
  inline Handshake& operator=(Handshake&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Handshake& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Handshake>(&Handshake_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Handshake& a, Handshake& b) { a.Swap(&b); }
  inline void Swap(Handshake* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Handshake* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Handshake* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Handshake>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Handshake& from);
  using Super_::MergeFrom;
  void MergeFrom(const Handshake& from) { Handshake::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Handshake* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.common.Handshake"; }

  explicit Handshake(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Handshake(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Handshake& from);
  Handshake(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Handshake&& from) noexcept
      : Handshake(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kVersionFieldNumber = 1,
  };
  // uint32 version = 1;
  void clear_version() ;
  [[nodiscard]] ::uint32_t version() const;
  void set_version(::uint32_t value);

  private:
  ::uint32_t _internal_version() const;
  void _internal_set_version(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.common.Handshake)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<0, 1,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Handshake& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint32_t version_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Credentials final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.common.Credentials) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Credentials() : Credentials(nullptr) {}
  ~Credentials() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Credentials* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Credentials));
  }
#endif

  template <typename = void>
  explicit constexpr Credentials(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Credentials(const Credentials& from) : Credentials(nullptr, from) {}
  inline Credentials(Credentials&& from) noexcept : Credentials(nullptr, ::std::move(from)) {}
  inline Credentials& operator=(const Credentials& from) {
    CopyFrom(from);
    return *this;
  }
  inline Credentials& operator=(Credentials&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Credentials& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Credentials>(&Credentials_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Credentials& a, Credentials& b) { a.Swap(&b); }
  inline void Swap(Credentials* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Credentials* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Credentials* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Credentials>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Credentials& from);
  using Super_::MergeFrom;
  void MergeFrom(const Credentials& from) { Credentials::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Credentials* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.common.Credentials"; }

  explicit Credentials(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Credentials(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Credentials& from);
  Credentials(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Credentials&& from) noexcept
      : Credentials(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kRealUidFieldNumber = 1,
    kEffectiveUidFieldNumber = 2,
    kSavedUidFieldNumber = 3,
    kRealGidFieldNumber = 4,
    kEffectiveGidFieldNumber = 5,
    kSavedGidFieldNumber = 6,
  };
  // uint32 real_uid = 1;
  void clear_real_uid() ;
  [[nodiscard]] ::uint32_t real_uid() const;
  void set_real_uid(::uint32_t value);

  private:
  ::uint32_t _internal_real_uid() const;
  void _internal_set_real_uid(::uint32_t value);

  public:
  // uint32 effective_uid = 2;
  void clear_effective_uid() ;
  [[nodiscard]] ::uint32_t effective_uid() const;
  void set_effective_uid(::uint32_t value);

  private:
  ::uint32_t _internal_effective_uid() const;
  void _internal_set_effective_uid(::uint32_t value);

  public:
  // uint32 saved_uid = 3;
  void clear_saved_uid() ;
  [[nodiscard]] ::uint32_t saved_uid() const;
  void set_saved_uid(::uint32_t value);

  private:
  ::uint32_t _internal_saved_uid() const;
  void _internal_set_saved_uid(::uint32_t value);

  public:
  // uint32 real_gid = 4;
  void clear_real_gid() ;
  [[nodiscard]] ::uint32_t real_gid() const;
  void set_real_gid(::uint32_t value);

  private:
  ::uint32_t _internal_real_gid() const;
  void _internal_set_real_gid(::uint32_t value);

  public:
  // uint32 effective_gid = 5;
  void clear_effective_gid() ;
  [[nodiscard]] ::uint32_t effective_gid() const;
  void set_effective_gid(::uint32_t value);

  private:
  ::uint32_t _internal_effective_gid() const;
  void _internal_set_effective_gid(::uint32_t value);

  public:
  // uint32 saved_gid = 6;
  void clear_saved_gid() ;
  [[nodiscard]] ::uint32_t saved_gid() const;
  void set_saved_gid(::uint32_t value);

  private:
  ::uint32_t _internal_saved_gid() const;
  void _internal_set_saved_gid(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.common.Credentials)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Credentials& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint32_t real_uid_;
    ::uint32_t effective_uid_;
    ::uint32_t saved_uid_;
    ::uint32_t real_gid_;
    ::uint32_t effective_gid_;
    ::uint32_t saved_gid_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ContextData final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.common.ContextData) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline ContextData() : ContextData(nullptr) {}
  ~ContextData() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ContextData* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ContextData));
  }
#endif

  template <typename = void>
  explicit constexpr ContextData(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ContextData(const ContextData& from) : ContextData(nullptr, from) {}
  inline ContextData(ContextData&& from) noexcept : ContextData(nullptr, ::std::move(from)) {}
  inline ContextData& operator=(const ContextData& from) {
    CopyFrom(from);
    return *this;
  }
  inline ContextData& operator=(ContextData&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ContextData& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ContextData>(&ContextData_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ContextData& a, ContextData& b) { a.Swap(&b); }
  inline void Swap(ContextData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ContextData* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ContextData* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<ContextData>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const ContextData& from);
  using Super_::MergeFrom;
  void MergeFrom(const ContextData& from) { ContextData::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ContextData* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.common.ContextData"; }

  explicit ContextData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ContextData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ContextData& from);
  ContextData(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ContextData&& from) noexcept
      : ContextData(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContainerIdFieldNumber = 6,
    kCwdFieldNumber = 8,
    kProcessNameFieldNumber = 9,
    kCredentialsFieldNumber = 7,
    kTimeNsFieldNumber = 1,
    kThreadStartTimeNsFieldNumber = 3,
    kThreadIdFieldNumber = 2,
    kThreadGroupIdFieldNumber = 4,
    kThreadGroupStartTimeNsFieldNumber = 5,
  };
  // string container_id = 6;
  void clear_container_id() ;
  [[nodiscard]] const ::std::string& container_id() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_container_id(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_container_id();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_container_id();
  void set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_container_id() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_container_id(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_container_id();

  public:
  // string cwd = 8;
  void clear_cwd() ;
  [[nodiscard]] const ::std::string& cwd() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_cwd(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_cwd();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_cwd();
  void set_allocated_cwd(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_cwd() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_cwd(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_cwd();

  public:
  // string process_name = 9;
  void clear_process_name() ;
  [[nodiscard]] const ::std::string& process_name() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_process_name(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_process_name();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_process_name();
  void set_allocated_process_name(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_process_name() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_process_name(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_process_name();

  public:
  // .gvisor.common.Credentials credentials = 7;
  [[nodiscard]] bool has_credentials() const;
  void clear_credentials() ;
  [[nodiscard]] const ::gvisor::common::Credentials& credentials() const;
  [[nodiscard]] ::gvisor::common::Credentials* PROTOBUF_NULLABLE release_credentials();
  ::gvisor::common::Credentials* PROTOBUF_NONNULL mutable_credentials();
  void set_allocated_credentials(::gvisor::common::Credentials* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_credentials(::gvisor::common::Credentials* PROTOBUF_NULLABLE value);
  ::gvisor::common::Credentials* PROTOBUF_NULLABLE unsafe_arena_release_credentials();

  private:
  const ::gvisor::common::Credentials& _internal_credentials() const;
  ::gvisor::common::Credentials* PROTOBUF_NONNULL _internal_mutable_credentials();

  public:
  // int64 time_ns = 1;
  void clear_time_ns() ;
  [[nodiscard]] ::int64_t time_ns() const;
  void set_time_ns(::int64_t value);

  private:
  ::int64_t _internal_time_ns() const;
  void _internal_set_time_ns(::int64_t value);

  public:
  // int64 thread_start_time_ns = 3;
  void clear_thread_start_time_ns() ;
  [[nodiscard]] ::int64_t thread_start_time_ns() const;
  void set_thread_start_time_ns(::int64_t value);

  private:
  ::int64_t _internal_thread_start_time_ns() const;
  void _internal_set_thread_start_time_ns(::int64_t value);

  public:
  // int32 thread_id = 2;
  void clear_thread_id() ;
  [[nodiscard]] ::int32_t thread_id() const;
  void set_thread_id(::int32_t value);

  private:
  ::int32_t _internal_thread_id() const;
  void _internal_set_thread_id(::int32_t value);

  public:
  // int32 thread_group_id = 4;
  void clear_thread_group_id() ;
  [[nodiscard]] ::int32_t thread_group_id() const;
  void set_thread_group_id(::int32_t value);

  private:
  ::int32_t _internal_thread_group_id() const;
  void _internal_set_thread_group_id(::int32_t value);

  public:
  // int64 thread_group_start_time_ns = 5;
  void clear_thread_group_start_time_ns() ;
  [[nodiscard]] ::int64_t thread_group_start_time_ns() const;
  void set_thread_group_start_time_ns(::int64_t value);

  private:
  ::int64_t _internal_thread_group_start_time_ns() const;
  void _internal_set_thread_group_start_time_ns(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.common.ContextData)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          1, 69,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ContextData& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr container_id_;
    ::google::protobuf::internal::ArenaStringPtr cwd_;
    ::google::protobuf::internal::ArenaStringPtr process_name_;
    ::gvisor::common::Credentials* PROTOBUF_NULLABLE credentials_;
    ::int64_t time_ns_;
    ::int64_t thread_start_time_ns_;
    ::int32_t thread_id_;
    ::int32_t thread_group_id_;
    ::int64_t thread_group_start_time_ns_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Handshake

// uint32 version = 1;
inline void Handshake::clear_version() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint32_t Handshake::version() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Handshake.version)
  return _internal_version();
}
inline void Handshake::set_version(::uint32_t value) {
  _internal_set_version(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:gvisor.common.Handshake.version)
}
inline ::uint32_t Handshake::_internal_version() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.version_;
}
inline void Handshake::_internal_set_version(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.version_ = value;
}

// -------------------------------------------------------------------

// Credentials

// uint32 real_uid = 1;
inline void Credentials::clear_real_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.real_uid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint32_t Credentials::real_uid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.real_uid)
  return _internal_real_uid();
}
inline void Credentials::set_real_uid(::uint32_t value) {
  _internal_set_real_uid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.real_uid)
}
inline ::uint32_t Credentials::_internal_real_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.real_uid_;
}
inline void Credentials::_internal_set_real_uid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.real_uid_ = value;
}

// uint32 effective_uid = 2;
inline void Credentials::clear_effective_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.effective_uid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint32_t Credentials::effective_uid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.effective_uid)
  return _internal_effective_uid();
}
inline void Credentials::set_effective_uid(::uint32_t value) {
  _internal_set_effective_uid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.effective_uid)
}
inline ::uint32_t Credentials::_internal_effective_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.effective_uid_;
}
inline void Credentials::_internal_set_effective_uid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.effective_uid_ = value;
}

// uint32 saved_uid = 3;
inline void Credentials::clear_saved_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.saved_uid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint32_t Credentials::saved_uid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.saved_uid)
  return _internal_saved_uid();
}
inline void Credentials::set_saved_uid(::uint32_t value) {
  _internal_set_saved_uid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.saved_uid)
}
inline ::uint32_t Credentials::_internal_saved_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.saved_uid_;
}
inline void Credentials::_internal_set_saved_uid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.saved_uid_ = value;
}

// uint32 real_gid = 4;
inline void Credentials::clear_real_gid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.real_gid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint32_t Credentials::real_gid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.real_gid)
  return _internal_real_gid();
}
inline void Credentials::set_real_gid(::uint32_t value) {
  _internal_set_real_gid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.real_gid)
}
inline ::uint32_t Credentials::_internal_real_gid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.real_gid_;
}
inline void Credentials::_internal_set_real_gid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.real_gid_ = value;
}

// uint32 effective_gid = 5;
inline void Credentials::clear_effective_gid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.effective_gid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint32_t Credentials::effective_gid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.effective_gid)
  return _internal_effective_gid();
}
inline void Credentials::set_effective_gid(::uint32_t value) {
  _internal_set_effective_gid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.effective_gid)
}
inline ::uint32_t Credentials::_internal_effective_gid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.effective_gid_;
}
inline void Credentials::_internal_set_effective_gid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.effective_gid_ = value;
}

// uint32 saved_gid = 6;
inline void Credentials::clear_saved_gid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.saved_gid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint32_t Credentials::saved_gid() const {
  // @@protoc_insertion_point(field_get:gvisor.common.Credentials.saved_gid)
  return _internal_saved_gid();
}
inline void Credentials::set_saved_gid(::uint32_t value) {
  _internal_set_saved_gid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.common.Credentials.saved_gid)
}
inline ::uint32_t Credentials::_internal_saved_gid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.saved_gid_;
}
inline void Credentials::_internal_set_saved_gid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.saved_gid_ = value;
}

// -------------------------------------------------------------------

// ContextData

// int64 time_ns = 1;
inline void ContextData::clear_time_ns() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_ns_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int64_t ContextData::time_ns() const {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.time_ns)
  return _internal_time_ns();
}
inline void ContextData::set_time_ns(::int64_t value) {
  _internal_set_time_ns(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.time_ns)
}
inline ::int64_t ContextData::_internal_time_ns() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.time_ns_;
}
inline void ContextData::_internal_set_time_ns(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.time_ns_ = value;
}

// int32 thread_id = 2;
inline void ContextData::clear_thread_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int32_t ContextData::thread_id() const {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.thread_id)
  return _internal_thread_id();
}
inline void ContextData::set_thread_id(::int32_t value) {
  _internal_set_thread_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.thread_id)
}
inline ::int32_t ContextData::_internal_thread_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.thread_id_;
}
inline void ContextData::_internal_set_thread_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_id_ = value;
}

// int64 thread_start_time_ns = 3;
inline void ContextData::clear_thread_start_time_ns() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_start_time_ns_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int64_t ContextData::thread_start_time_ns() const {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.thread_start_time_ns)
  return _internal_thread_start_time_ns();
}
inline void ContextData::set_thread_start_time_ns(::int64_t value) {
  _internal_set_thread_start_time_ns(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.thread_start_time_ns)
}
inline ::int64_t ContextData::_internal_thread_start_time_ns() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.thread_start_time_ns_;
}
inline void ContextData::_internal_set_thread_start_time_ns(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_start_time_ns_ = value;
}

// int32 thread_group_id = 4;
inline void ContextData::clear_thread_group_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_group_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int32_t ContextData::thread_group_id() const {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.thread_group_id)
  return _internal_thread_group_id();
}
inline void ContextData::set_thread_group_id(::int32_t value) {
  _internal_set_thread_group_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.thread_group_id)
}
inline ::int32_t ContextData::_internal_thread_group_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.thread_group_id_;
}
inline void ContextData::_internal_set_thread_group_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_group_id_ = value;
}

// int64 thread_group_start_time_ns = 5;
inline void ContextData::clear_thread_group_start_time_ns() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_group_start_time_ns_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::int64_t ContextData::thread_group_start_time_ns() const {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.thread_group_start_time_ns)
  return _internal_thread_group_start_time_ns();
}
inline void ContextData::set_thread_group_start_time_ns(::int64_t value) {
  _internal_set_thread_group_start_time_ns(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.thread_group_start_time_ns)
}
inline ::int64_t ContextData::_internal_thread_group_start_time_ns() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.thread_group_start_time_ns_;
}
inline void ContextData::_internal_set_thread_group_start_time_ns(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.thread_group_start_time_ns_ = value;
}

// string container_id = 6;
inline void ContextData::clear_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& ContextData::container_id() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.container_id)
  return _internal_container_id();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContextData::set_container_id(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.container_id_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.container_id)
}
inline ::std::string* PROTOBUF_NONNULL ContextData::mutable_container_id()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_container_id();
  // @@protoc_insertion_point(field_mutable:gvisor.common.ContextData.container_id)
  return _s;
}
inline const ::std::string& ContextData::_internal_container_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.container_id_.Get();
}
inline void ContextData::_internal_set_container_id(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.container_id_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContextData::_internal_mutable_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.container_id_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContextData::release_container_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.common.ContextData.container_id)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.container_id_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.container_id_.Set("", GetArena());
  }
  return released;
}
inline void ContextData::set_allocated_container_id(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.container_id_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.container_id_.IsDefault()) {
    _impl_.container_id_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.common.ContextData.container_id)
}

// .gvisor.common.Credentials credentials = 7;
inline bool ContextData::has_credentials() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.credentials_ != nullptr);
  return value;
}
inline void ContextData::clear_credentials() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.credentials_ != nullptr) _impl_.credentials_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::common::Credentials& ContextData::_internal_credentials() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::Credentials* p = _impl_.credentials_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::Credentials>(&::gvisor::common::Credentials_globals_);
}
inline const ::gvisor::common::Credentials& ContextData::credentials() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.credentials)
  return _internal_credentials();
}
inline void ContextData::unsafe_arena_set_allocated_credentials(
    ::gvisor::common::Credentials* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.credentials_);
  }
  _impl_.credentials_ = reinterpret_cast<::gvisor::common::Credentials*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.common.ContextData.credentials)
}
inline ::gvisor::common::Credentials* PROTOBUF_NULLABLE ContextData::release_credentials() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::common::Credentials* released = _impl_.credentials_;
  _impl_.credentials_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::Credentials* PROTOBUF_NULLABLE ContextData::unsafe_arena_release_credentials() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.common.ContextData.credentials)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::common::Credentials* temp = _impl_.credentials_;
  _impl_.credentials_ = nullptr;
  return temp;
}
inline ::gvisor::common::Credentials* PROTOBUF_NONNULL ContextData::_internal_mutable_credentials() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.credentials_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::Credentials>(GetArena());
    _impl_.credentials_ = reinterpret_cast<::gvisor::common::Credentials*>(p);
  }
  return _impl_.credentials_;
}
inline ::gvisor::common::Credentials* PROTOBUF_NONNULL ContextData::mutable_credentials()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::common::Credentials* _msg = _internal_mutable_credentials();
  // @@protoc_insertion_point(field_mutable:gvisor.common.ContextData.credentials)
  return _msg;
}
inline void ContextData::set_allocated_credentials(::gvisor::common::Credentials* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.credentials_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.credentials_ = reinterpret_cast<::gvisor::common::Credentials*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.common.ContextData.credentials)
}

// string cwd = 8;
inline void ContextData::clear_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cwd_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& ContextData::cwd() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.cwd)
  return _internal_cwd();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContextData::set_cwd(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.cwd_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.cwd)
}
inline ::std::string* PROTOBUF_NONNULL ContextData::mutable_cwd()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_cwd();
  // @@protoc_insertion_point(field_mutable:gvisor.common.ContextData.cwd)
  return _s;
}
inline const ::std::string& ContextData::_internal_cwd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cwd_.Get();
}
inline void ContextData::_internal_set_cwd(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cwd_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContextData::_internal_mutable_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.cwd_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContextData::release_cwd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.common.ContextData.cwd)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.cwd_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.cwd_.Set("", GetArena());
  }
  return released;
}
inline void ContextData::set_allocated_cwd(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.cwd_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.cwd_.IsDefault()) {
    _impl_.cwd_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.common.ContextData.cwd)
}

// string process_name = 9;
inline void ContextData::clear_process_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.process_name_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& ContextData::process_name() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.common.ContextData.process_name)
  return _internal_process_name();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ContextData::set_process_name(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.process_name_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.common.ContextData.process_name)
}
inline ::std::string* PROTOBUF_NONNULL ContextData::mutable_process_name()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_process_name();
  // @@protoc_insertion_point(field_mutable:gvisor.common.ContextData.process_name)
  return _s;
}
inline const ::std::string& ContextData::_internal_process_name() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.process_name_.Get();
}
inline void ContextData::_internal_set_process_name(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.process_name_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ContextData::_internal_mutable_process_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.process_name_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ContextData::release_process_name() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.common.ContextData.process_name)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.process_name_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.process_name_.Set("", GetArena());
  }
  return released;
}
inline void ContextData::set_allocated_process_name(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.process_name_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.process_name_.IsDefault()) {
    _impl_.process_name_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.common.ContextData.process_name)
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace common
}  // namespace gvisor


namespace google {
namespace protobuf {

template <>
struct is_proto_enum<::gvisor::common::MessageType> : std::true_type {};
template <>
inline const EnumDescriptor* PROTOBUF_NONNULL GetEnumDescriptor<::gvisor::common::MessageType>() {
  return ::gvisor::common::MessageType_descriptor();
}

}  // namespace protobuf
}  // namespace google

// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto_2epb_2eh
