// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/sentry.proto
// Protobuf C++ Version: 7.36.1

#include "pkg/sentry/seccheck/points/sentry.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::gvisor::sentry::CloneInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::sentry::ExecveInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::sentry::ExitNotifyParentInfo
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::sentry::TaskExit
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace gvisor {
namespace sentry {
class TaskExit::_Internal {
 public:
  using HasBits = decltype(::std::declval<TaskExit>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TaskExit, _impl_._has_bits_);
};

constexpr TaskExit::ParseTableT_ TaskExit::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TaskExit, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::sentry::TaskExit>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 exit_status = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TaskExit, _impl_.exit_status_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.exit_status_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.context_data_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 exit_status = 2;
      {PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.exit_status_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr TaskExit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_status_{0} {}

template <typename>
constexpr TaskExit::TaskExit(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TaskExit::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TaskExit(arena);
}
constexpr auto TaskExit::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(TaskExit), alignof(TaskExit));
}
constexpr auto TaskExit::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TaskExit::MergeImpl,
          Super_::GetNewImpl<TaskExit>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TaskExit::SharedDtor,
          Super_::GetClearImpl<TaskExit>(), &TaskExit::ByteSizeLong,
              &TaskExit::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TaskExit, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TaskExitGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TaskExitGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TaskExit_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TaskExit::InternalGenerateClassData_(
            _default, &TaskExit_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TaskExit>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TaskExitGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TaskExit _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TaskExit>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TaskExitGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TaskExitGlobalsTypeInternal TaskExit_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TaskExit_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TaskExit_globals_.GetClassData();
#else
  return TaskExit_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExitNotifyParentInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExitNotifyParentInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_._has_bits_);
};

constexpr ExitNotifyParentInfo::ParseTableT_ ExitNotifyParentInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::sentry::ExitNotifyParentInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 exit_status = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ExitNotifyParentInfo, _impl_.exit_status_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.exit_status_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.context_data_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 exit_status = 2;
      {PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.exit_status_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ExitNotifyParentInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_status_{0} {}

template <typename>
constexpr ExitNotifyParentInfo::ExitNotifyParentInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExitNotifyParentInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExitNotifyParentInfo(arena);
}
constexpr auto ExitNotifyParentInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ExitNotifyParentInfo), alignof(ExitNotifyParentInfo));
}
constexpr auto ExitNotifyParentInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExitNotifyParentInfo::MergeImpl,
          Super_::GetNewImpl<ExitNotifyParentInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExitNotifyParentInfo::SharedDtor,
          Super_::GetClearImpl<ExitNotifyParentInfo>(), &ExitNotifyParentInfo::ByteSizeLong,
              &ExitNotifyParentInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExitNotifyParentInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExitNotifyParentInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExitNotifyParentInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExitNotifyParentInfo::InternalGenerateClassData_(
            _default, &ExitNotifyParentInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExitNotifyParentInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExitNotifyParentInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExitNotifyParentInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExitNotifyParentInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExitNotifyParentInfoGlobalsTypeInternal ExitNotifyParentInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExitNotifyParentInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExitNotifyParentInfo_globals_.GetClassData();
#else
  return ExitNotifyParentInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ExecveInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<ExecveInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_._has_bits_);
};

constexpr ExecveInfo::ParseTableT_ ExecveInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::sentry::ExecveInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // bytes binary_sha256 = 8;
      {::_pbi::TcParser::FastBS1,
       {66, 3, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_sha256_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 4, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.context_data_)}},
      // string binary_path = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_path_)}},
      // repeated string argv = 3;
      {::_pbi::TcParser::FastUR1,
       {26, 0, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.argv_)}},
      // repeated string env = 4;
      {::_pbi::TcParser::FastUR1,
       {34, 1, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.env_)}},
      // uint32 binary_mode = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ExecveInfo, _impl_.binary_mode_), 5>(),
       {40, 5, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_mode_)}},
      // uint32 binary_uid = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ExecveInfo, _impl_.binary_uid_), 6>(),
       {48, 6, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_uid_)}},
      // uint32 binary_gid = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ExecveInfo, _impl_.binary_gid_), 7>(),
       {56, 7, 0,
        PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_gid_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.context_data_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string binary_path = 2;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_path_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string argv = 3;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.argv_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string env = 4;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.env_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // uint32 binary_mode = 5;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_mode_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 binary_uid = 6;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_uid_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 binary_gid = 7;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_gid_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // bytes binary_sha256 = 8;
      {PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_sha256_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
    }},
    {{
      "\30\0\13\4\3\0\0\0\0\0\0\0\0\0\0\0"
      "gvisor.sentry.ExecveInfo"
      "binary_path"
      "argv"
      "env"
    }},
  };
}


inline constexpr ExecveInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : argv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::sentry::ExecveInfo,
            PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.argv_)>()
         }
        ,
        env_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::sentry::ExecveInfo,
            PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.env_)>()
         }
        ,
        binary_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        binary_sha256_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        binary_mode_{0u},
        binary_uid_{0u},
        binary_gid_{0u} {}

template <typename>
constexpr ExecveInfo::ExecveInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ExecveInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ExecveInfo(arena);
}
constexpr auto ExecveInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ExecveInfo), alignof(ExecveInfo));
}
constexpr auto ExecveInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ExecveInfo::MergeImpl,
          Super_::GetNewImpl<ExecveInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ExecveInfo::SharedDtor,
          Super_::GetClearImpl<ExecveInfo>(), &ExecveInfo::ByteSizeLong,
              &ExecveInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExecveInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExecveInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ExecveInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ExecveInfo::InternalGenerateClassData_(
            _default, &ExecveInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ExecveInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExecveInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ExecveInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ExecveInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExecveInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExecveInfoGlobalsTypeInternal ExecveInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ExecveInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ExecveInfo_globals_.GetClassData();
#else
  return ExecveInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class CloneInfo::_Internal {
 public:
  using HasBits = decltype(::std::declval<CloneInfo>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_._has_bits_);
};

constexpr CloneInfo::ParseTableT_ CloneInfo::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967234,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::sentry::CloneInfo>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.context_data_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // int32 created_thread_id = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(CloneInfo, _impl_.created_thread_id_), 1>(),
       {24, 1, 0,
        PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_id_)}},
      // int32 created_thread_group_id = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(CloneInfo, _impl_.created_thread_group_id_), 2>(),
       {32, 2, 0,
        PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_group_id_)}},
      // int64 created_thread_start_time_ns = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(CloneInfo, _impl_.created_thread_start_time_ns_), 3>(),
       {40, 3, 0,
        PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_start_time_ns_)}},
      // uint64 flags = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(CloneInfo, _impl_.flags_), 4>(),
       {48, 4, 0,
        PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // int32 created_thread_id = 3;
      {PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_id_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 created_thread_group_id = 4;
      {PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_group_id_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 created_thread_start_time_ns = 5;
      {PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.created_thread_start_time_ns_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // uint64 flags = 6;
      {PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.flags_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr CloneInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        created_thread_id_{0},
        created_thread_group_id_{0},
        created_thread_start_time_ns_{::int64_t{0}},
        flags_{::uint64_t{0u}} {}

template <typename>
constexpr CloneInfo::CloneInfo(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL CloneInfo::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) CloneInfo(arena);
}
constexpr auto CloneInfo::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(CloneInfo), alignof(CloneInfo));
}
constexpr auto CloneInfo::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &CloneInfo::MergeImpl,
          Super_::GetNewImpl<CloneInfo>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &CloneInfo::SharedDtor,
          Super_::GetClearImpl<CloneInfo>(), &CloneInfo::ByteSizeLong,
              &CloneInfo::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloneInfoGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloneInfoGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 CloneInfo_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(CloneInfo::InternalGenerateClassData_(
            _default, &CloneInfo_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<CloneInfo>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloneInfoGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) CloneInfo _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<CloneInfo>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloneInfoGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloneInfoGlobalsTypeInternal CloneInfo_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* CloneInfo_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return CloneInfo_globals_.GetClassData();
#else
  return CloneInfo_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace sentry
}  // namespace gvisor
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto = nullptr;
const ::uint32_t
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_.created_thread_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_.created_thread_group_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_.created_thread_start_time_ns_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::CloneInfo, _impl_.flags_),
        0,
        1,
        2,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_._has_bits_),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.binary_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.argv_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.env_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.binary_mode_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.binary_uid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.binary_gid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.binary_sha256_),
        4,
        2,
        0,
        1,
        5,
        6,
        7,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExitNotifyParentInfo, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExitNotifyParentInfo, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExitNotifyParentInfo, _impl_.exit_status_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::TaskExit, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::TaskExit, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::sentry::TaskExit, _impl_.exit_status_),
        0,
        1,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::gvisor::sentry::CloneInfo)},
        {13, sizeof(::gvisor::sentry::ExecveInfo)},
        {32, sizeof(::gvisor::sentry::ExitNotifyParentInfo)},
        {39, sizeof(::gvisor::sentry::TaskExit)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::gvisor::sentry::CloneInfo_globals_,
        &::gvisor::sentry::ExecveInfo_globals_,
        &::gvisor::sentry::ExitNotifyParentInfo_globals_,
        &::gvisor::sentry::TaskExit_globals_,
};
const char descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\'pkg/sentry/seccheck/points/sentry.prot"
    "o\022\rgvisor.sentry\032\'pkg/sentry/seccheck/po"
    "ints/common.proto\"\256\001\n\tCloneInfo\0220\n\014conte"
    "xt_data\030\001 \001(\0132\032.gvisor.common.ContextDat"
    "a\022\031\n\021created_thread_id\030\003 \001(\005\022\037\n\027created_"
    "thread_group_id\030\004 \001(\005\022$\n\034created_thread_"
    "start_time_ns\030\005 \001(\003\022\r\n\005flags\030\006 \001(\004\"\302\001\n\nE"
    "xecveInfo\0220\n\014context_data\030\001 \001(\0132\032.gvisor"
    ".common.ContextData\022\023\n\013binary_path\030\002 \001(\t"
    "\022\014\n\004argv\030\003 \003(\t\022\013\n\003env\030\004 \003(\t\022\023\n\013binary_mo"
    "de\030\005 \001(\r\022\022\n\nbinary_uid\030\006 \001(\r\022\022\n\nbinary_g"
    "id\030\007 \001(\r\022\025\n\rbinary_sha256\030\010 \001(\014\"]\n\024ExitN"
    "otifyParentInfo\0220\n\014context_data\030\001 \001(\0132\032."
    "gvisor.common.ContextData\022\023\n\013exit_status"
    "\030\002 \001(\005\"Q\n\010TaskExit\0220\n\014context_data\030\001 \001(\013"
    "2\032.gvisor.common.ContextData\022\023\n\013exit_sta"
    "tus\030\002 \001(\005b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_deps[1] = {
        &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
};
static ::absl::once_flag descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto = {
    false,
    false,
    657,
    descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
    "pkg/sentry/seccheck/points/sentry.proto",
    &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_once,
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_deps,
    1,
    4,
    schemas,
    file_message_globals,
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto::offsets,
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto,
};
namespace gvisor {
namespace sentry {
// ===================================================================

void CloneInfo::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
CloneInfo::CloneInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, CloneInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.sentry.CloneInfo)
}
PROTOBUF_NDEBUG_INLINE CloneInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::sentry::CloneInfo& from_msg)
      : _has_bits_{from._has_bits_} {}

CloneInfo::CloneInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const CloneInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, CloneInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  CloneInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, created_thread_id_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, created_thread_id_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, created_thread_id_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.sentry.CloneInfo)
}
PROTOBUF_NDEBUG_INLINE CloneInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void CloneInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
CloneInfo::~CloneInfo() {
  // @@protoc_insertion_point(destructor:gvisor.sentry.CloneInfo)
  SharedDtor(*this);
}
inline void CloneInfo::SharedDtor(MessageLite& self) {
  CloneInfo& this_ = static_cast<CloneInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull CloneInfo_class_data_ =
        CloneInfo::InternalGenerateClassData_(CloneInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloneInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloneInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(CloneInfo_class_data_.tc_table);
  return CloneInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
CloneInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&CloneInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&CloneInfo_globals_));
  return CloneInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const CloneInfo::ParseTableT_
    CloneInfo::_table_ =
        CloneInfo::InternalGenerateParseTable_(CloneInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void CloneInfo::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.sentry.CloneInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
    this_._impl_.context_data_->Clear();
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000001eU)) {
    ::memset(&this_._impl_.created_thread_id_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.created_thread_id_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL CloneInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const CloneInfo& this_ = static_cast<const CloneInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL CloneInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const CloneInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.sentry.CloneInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // int32 created_thread_id = 3;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_created_thread_id() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<3>(
              stream, this_._internal_created_thread_id(), target);
    }
  }

  // int32 created_thread_group_id = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_created_thread_group_id() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_created_thread_group_id(), target);
    }
  }

  // int64 created_thread_start_time_ns = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_created_thread_start_time_ns() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<5>(
              stream, this_._internal_created_thread_start_time_ns(), target);
    }
  }

  // uint64 flags = 6;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.sentry.CloneInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t CloneInfo::ByteSizeLong(const MessageLite& base) {
  const CloneInfo& this_ = static_cast<const CloneInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t CloneInfo::ByteSizeLong() const {
  const CloneInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.sentry.CloneInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // int32 created_thread_id = 3;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_created_thread_id() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_created_thread_id());
      }
    }
    // int32 created_thread_group_id = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_created_thread_group_id() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_created_thread_group_id());
      }
    }
    // int64 created_thread_start_time_ns = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_created_thread_start_time_ns() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_created_thread_start_time_ns());
      }
    }
    // uint64 flags = 6;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void CloneInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<CloneInfo*>(&to_msg);
  auto& from = static_cast<const CloneInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.sentry.CloneInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_created_thread_id() != 0) {
        _this->_impl_.created_thread_id_ = from._impl_.created_thread_id_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_created_thread_group_id() != 0) {
        _this->_impl_.created_thread_group_id_ = from._impl_.created_thread_group_id_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_created_thread_start_time_ns() != 0) {
        _this->_impl_.created_thread_start_time_ns_ = from._impl_.created_thread_start_time_ns_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void CloneInfo::CopyFrom(const CloneInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.sentry.CloneInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void CloneInfo::InternalSwap(CloneInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.flags_)
      + sizeof(CloneInfo::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(CloneInfo, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata CloneInfo::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void ExecveInfo::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
ExecveInfo::ExecveInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExecveInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.sentry.ExecveInfo)
}
PROTOBUF_NDEBUG_INLINE ExecveInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::sentry::ExecveInfo& from_msg)
      : _has_bits_{from._has_bits_},
        argv_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::sentry::ExecveInfo,
              PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.argv_)>()
          , arena, from.argv_
        }
        ,
        env_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::sentry::ExecveInfo,
              PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.env_)>()
          , arena, from.env_
        }
        ,
        binary_path_(arena, from.binary_path_),
        binary_sha256_(arena, from.binary_sha256_) {}

ExecveInfo::ExecveInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExecveInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExecveInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExecveInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, binary_mode_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, binary_mode_),
           offsetof(Impl_, binary_gid_) -
               offsetof(Impl_, binary_mode_) +
               sizeof(Impl_::binary_gid_));

  // @@protoc_insertion_point(copy_constructor:gvisor.sentry.ExecveInfo)
}
PROTOBUF_NDEBUG_INLINE ExecveInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : argv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::sentry::ExecveInfo,
            PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.argv_)>()
         }
        ,
        env_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::sentry::ExecveInfo,
            PROTOBUF_FIELD_OFFSET(::gvisor::sentry::ExecveInfo, _impl_.env_)>()
         }
        ,
        binary_path_(arena),
        binary_sha256_(arena) {}

inline void ExecveInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, binary_gid_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::binary_gid_));
}
ExecveInfo::~ExecveInfo() {
  // @@protoc_insertion_point(destructor:gvisor.sentry.ExecveInfo)
  SharedDtor(*this);
}
inline void ExecveInfo::SharedDtor(MessageLite& self) {
  ExecveInfo& this_ = static_cast<ExecveInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.binary_path_.Destroy();
  this_._impl_.binary_sha256_.Destroy();
  delete this_._impl_.context_data_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExecveInfo_class_data_ =
        ExecveInfo::InternalGenerateClassData_(ExecveInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExecveInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExecveInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExecveInfo_class_data_.tc_table);
  return ExecveInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExecveInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExecveInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExecveInfo_globals_));
  return ExecveInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExecveInfo::ParseTableT_
    ExecveInfo::_table_ =
        ExecveInfo::InternalGenerateParseTable_(ExecveInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExecveInfo::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.sentry.ExecveInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.argv_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.env_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      this_._impl_.binary_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      this_._impl_.binary_sha256_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000e0U)) {
    ::memset(&this_._impl_.binary_mode_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.binary_gid_) -
                 reinterpret_cast<char*>(&this_._impl_.binary_mode_)) +
                 sizeof(_impl_.binary_gid_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExecveInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExecveInfo& this_ = static_cast<const ExecveInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExecveInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExecveInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.sentry.ExecveInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // string binary_path = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_binary_path().empty()) {
      const ::std::string& _s = this_._internal_binary_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.sentry.ExecveInfo.binary_path");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // repeated string argv = 3;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_argv_size(); i < n; ++i) {
      const auto& s = this_._internal_argv().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.sentry.ExecveInfo.argv");
      target = stream->WriteString(3, s, target);
    }
  }

  // repeated string env = 4;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_env_size(); i < n; ++i) {
      const auto& s = this_._internal_env().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.sentry.ExecveInfo.env");
      target = stream->WriteString(4, s, target);
    }
  }

  // uint32 binary_mode = 5;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_binary_mode() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          5, this_._internal_binary_mode(), target);
    }
  }

  // uint32 binary_uid = 6;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_binary_uid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          6, this_._internal_binary_uid(), target);
    }
  }

  // uint32 binary_gid = 7;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_binary_gid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          7, this_._internal_binary_gid(), target);
    }
  }

  // bytes binary_sha256 = 8;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_binary_sha256().empty()) {
      const ::std::string& _s = this_._internal_binary_sha256();
      target = stream->WriteBytesMaybeAliased(8, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.sentry.ExecveInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExecveInfo::ByteSizeLong(const MessageLite& base) {
  const ExecveInfo& this_ = static_cast<const ExecveInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExecveInfo::ByteSizeLong() const {
  const ExecveInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.sentry.ExecveInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated string argv = 3;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_argv().size());
      for (int i = 0, n = this_._internal_argv().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_argv().Get(i));
      }
    }
    // repeated string env = 4;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_env().size());
      for (int i = 0, n = this_._internal_env().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_env().Get(i));
      }
    }
    // string binary_path = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_binary_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_binary_path());
      }
    }
    // bytes binary_sha256 = 8;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_binary_sha256().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_binary_sha256());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // uint32 binary_mode = 5;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_binary_mode() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_binary_mode());
      }
    }
    // uint32 binary_uid = 6;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_binary_uid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_binary_uid());
      }
    }
    // uint32 binary_gid = 7;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_binary_gid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_binary_gid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExecveInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExecveInfo*>(&to_msg);
  auto& from = static_cast<const ExecveInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.sentry.ExecveInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_argv()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_argv());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_env()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_env());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_binary_path().empty()) {
        _this->_internal_set_binary_path(from._internal_binary_path());
      } else {
        if (_this->_impl_.binary_path_.IsDefault()) {
          _this->_internal_set_binary_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_binary_sha256().empty()) {
        _this->_internal_set_binary_sha256(from._internal_binary_sha256());
      } else {
        if (_this->_impl_.binary_sha256_.IsDefault()) {
          _this->_internal_set_binary_sha256("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_binary_mode() != 0) {
        _this->_impl_.binary_mode_ = from._impl_.binary_mode_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_binary_uid() != 0) {
        _this->_impl_.binary_uid_ = from._impl_.binary_uid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_binary_gid() != 0) {
        _this->_impl_.binary_gid_ = from._impl_.binary_gid_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExecveInfo::CopyFrom(const ExecveInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.sentry.ExecveInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExecveInfo::InternalSwap(ExecveInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.argv_.InternalSwap(&other->_impl_.argv_);
  _impl_.env_.InternalSwap(&other->_impl_.env_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.binary_path_, &other->_impl_.binary_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.binary_sha256_, &other->_impl_.binary_sha256_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.binary_gid_)
      + sizeof(ExecveInfo::_impl_.binary_gid_)
      - PROTOBUF_FIELD_OFFSET(ExecveInfo, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata ExecveInfo::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void ExitNotifyParentInfo::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
ExitNotifyParentInfo::ExitNotifyParentInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExitNotifyParentInfo_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.sentry.ExitNotifyParentInfo)
}
PROTOBUF_NDEBUG_INLINE ExitNotifyParentInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::sentry::ExitNotifyParentInfo& from_msg)
      : _has_bits_{from._has_bits_} {}

ExitNotifyParentInfo::ExitNotifyParentInfo(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ExitNotifyParentInfo& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ExitNotifyParentInfo_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ExitNotifyParentInfo* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_status_ = from._impl_.exit_status_;

  // @@protoc_insertion_point(copy_constructor:gvisor.sentry.ExitNotifyParentInfo)
}
PROTOBUF_NDEBUG_INLINE ExitNotifyParentInfo::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void ExitNotifyParentInfo::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, exit_status_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::exit_status_));
}
ExitNotifyParentInfo::~ExitNotifyParentInfo() {
  // @@protoc_insertion_point(destructor:gvisor.sentry.ExitNotifyParentInfo)
  SharedDtor(*this);
}
inline void ExitNotifyParentInfo::SharedDtor(MessageLite& self) {
  ExitNotifyParentInfo& this_ = static_cast<ExitNotifyParentInfo&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ExitNotifyParentInfo_class_data_ =
        ExitNotifyParentInfo::InternalGenerateClassData_(ExitNotifyParentInfo_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExitNotifyParentInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExitNotifyParentInfo_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ExitNotifyParentInfo_class_data_.tc_table);
  return ExitNotifyParentInfo_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ExitNotifyParentInfo::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ExitNotifyParentInfo_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ExitNotifyParentInfo_globals_));
  return ExitNotifyParentInfo_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ExitNotifyParentInfo::ParseTableT_
    ExitNotifyParentInfo::_table_ =
        ExitNotifyParentInfo::InternalGenerateParseTable_(ExitNotifyParentInfo_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ExitNotifyParentInfo::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.sentry.ExitNotifyParentInfo)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
    this_._impl_.context_data_->Clear();
  }
  this_._impl_.exit_status_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ExitNotifyParentInfo::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ExitNotifyParentInfo& this_ = static_cast<const ExitNotifyParentInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ExitNotifyParentInfo::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ExitNotifyParentInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.sentry.ExitNotifyParentInfo)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // int32 exit_status = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_exit_status() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_exit_status(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.sentry.ExitNotifyParentInfo)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ExitNotifyParentInfo::ByteSizeLong(const MessageLite& base) {
  const ExitNotifyParentInfo& this_ = static_cast<const ExitNotifyParentInfo&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ExitNotifyParentInfo::ByteSizeLong() const {
  const ExitNotifyParentInfo& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.sentry.ExitNotifyParentInfo)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // int32 exit_status = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_exit_status() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_exit_status());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ExitNotifyParentInfo::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ExitNotifyParentInfo*>(&to_msg);
  auto& from = static_cast<const ExitNotifyParentInfo&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.sentry.ExitNotifyParentInfo)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_exit_status() != 0) {
        _this->_impl_.exit_status_ = from._impl_.exit_status_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ExitNotifyParentInfo::CopyFrom(const ExitNotifyParentInfo& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.sentry.ExitNotifyParentInfo)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ExitNotifyParentInfo::InternalSwap(ExitNotifyParentInfo* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.exit_status_)
      + sizeof(ExitNotifyParentInfo::_impl_.exit_status_)
      - PROTOBUF_FIELD_OFFSET(ExitNotifyParentInfo, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata ExitNotifyParentInfo::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void TaskExit::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
TaskExit::TaskExit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TaskExit_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.sentry.TaskExit)
}
PROTOBUF_NDEBUG_INLINE TaskExit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::sentry::TaskExit& from_msg)
      : _has_bits_{from._has_bits_} {}

TaskExit::TaskExit(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TaskExit& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TaskExit_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TaskExit* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_status_ = from._impl_.exit_status_;

  // @@protoc_insertion_point(copy_constructor:gvisor.sentry.TaskExit)
}
PROTOBUF_NDEBUG_INLINE TaskExit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void TaskExit::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, exit_status_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::exit_status_));
}
TaskExit::~TaskExit() {
  // @@protoc_insertion_point(destructor:gvisor.sentry.TaskExit)
  SharedDtor(*this);
}
inline void TaskExit::SharedDtor(MessageLite& self) {
  TaskExit& this_ = static_cast<TaskExit&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TaskExit_class_data_ =
        TaskExit::InternalGenerateClassData_(TaskExit_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TaskExit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TaskExit_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TaskExit_class_data_.tc_table);
  return TaskExit_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TaskExit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TaskExit_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TaskExit_globals_));
  return TaskExit_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TaskExit::ParseTableT_
    TaskExit::_table_ =
        TaskExit::InternalGenerateParseTable_(TaskExit_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TaskExit::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.sentry.TaskExit)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
    this_._impl_.context_data_->Clear();
  }
  this_._impl_.exit_status_ = 0;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TaskExit::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TaskExit& this_ = static_cast<const TaskExit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TaskExit::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TaskExit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.sentry.TaskExit)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // int32 exit_status = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_exit_status() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_exit_status(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.sentry.TaskExit)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TaskExit::ByteSizeLong(const MessageLite& base) {
  const TaskExit& this_ = static_cast<const TaskExit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TaskExit::ByteSizeLong() const {
  const TaskExit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.sentry.TaskExit)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // int32 exit_status = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_exit_status() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_exit_status());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TaskExit::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TaskExit*>(&to_msg);
  auto& from = static_cast<const TaskExit&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.sentry.TaskExit)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_exit_status() != 0) {
        _this->_impl_.exit_status_ = from._impl_.exit_status_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TaskExit::CopyFrom(const TaskExit& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.sentry.TaskExit)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TaskExit::InternalSwap(TaskExit* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.exit_status_)
      + sizeof(TaskExit::_impl_.exit_status_)
      - PROTOBUF_FIELD_OFFSET(TaskExit, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata TaskExit::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace sentry
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
