// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/sentry.proto
// Protobuf C++ Version: 7.36.1

#ifndef pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_2epb_2eh
#define pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "pkg/sentry/seccheck/points/common.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto;
}  // extern "C"
namespace gvisor {
namespace sentry {
class CloneInfo;
struct CloneInfoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern CloneInfoGlobalsTypeInternal CloneInfo_globals_;
extern const ::google::protobuf::internal::ClassDataFull CloneInfo_class_data_;
#else
extern const CloneInfoGlobalsTypeInternal CloneInfo_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExecveInfo;
struct ExecveInfoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExecveInfoGlobalsTypeInternal ExecveInfo_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExecveInfo_class_data_;
#else
extern const ExecveInfoGlobalsTypeInternal ExecveInfo_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ExitNotifyParentInfo;
struct ExitNotifyParentInfoGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExitNotifyParentInfoGlobalsTypeInternal ExitNotifyParentInfo_globals_;
extern const ::google::protobuf::internal::ClassDataFull ExitNotifyParentInfo_class_data_;
#else
extern const ExitNotifyParentInfoGlobalsTypeInternal ExitNotifyParentInfo_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class TaskExit;
struct TaskExitGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TaskExitGlobalsTypeInternal TaskExit_globals_;
extern const ::google::protobuf::internal::ClassDataFull TaskExit_class_data_;
#else
extern const TaskExitGlobalsTypeInternal TaskExit_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace sentry
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace gvisor {
namespace sentry {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TaskExit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.sentry.TaskExit) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline TaskExit() : TaskExit(nullptr) {}
  ~TaskExit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TaskExit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TaskExit));
  }
#endif

  template <typename = void>
  explicit constexpr TaskExit(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TaskExit(const TaskExit& from) : TaskExit(nullptr, from) {}
  inline TaskExit(TaskExit&& from) noexcept : TaskExit(nullptr, ::std::move(from)) {}
  inline TaskExit& operator=(const TaskExit& from) {
    CopyFrom(from);
    return *this;
  }
  inline TaskExit& operator=(TaskExit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TaskExit& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TaskExit>(&TaskExit_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(TaskExit& a, TaskExit& b) { a.Swap(&b); }
  inline void Swap(TaskExit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TaskExit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TaskExit* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<TaskExit>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const TaskExit& from);
  using Super_::MergeFrom;
  void MergeFrom(const TaskExit& from) { TaskExit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TaskExit* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.sentry.TaskExit"; }

  explicit TaskExit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TaskExit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TaskExit& from);
  TaskExit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TaskExit&& from) noexcept
      : TaskExit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitStatusFieldNumber = 2,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // int32 exit_status = 2;
  void clear_exit_status() ;
  [[nodiscard]] ::int32_t exit_status() const;
  void set_exit_status(::int32_t value);

  private:
  ::int32_t _internal_exit_status() const;
  void _internal_set_exit_status(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.sentry.TaskExit)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TaskExit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::int32_t exit_status_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExitNotifyParentInfo final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.sentry.ExitNotifyParentInfo) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline ExitNotifyParentInfo() : ExitNotifyParentInfo(nullptr) {}
  ~ExitNotifyParentInfo() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExitNotifyParentInfo* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExitNotifyParentInfo));
  }
#endif

  template <typename = void>
  explicit constexpr ExitNotifyParentInfo(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExitNotifyParentInfo(const ExitNotifyParentInfo& from) : ExitNotifyParentInfo(nullptr, from) {}
  inline ExitNotifyParentInfo(ExitNotifyParentInfo&& from) noexcept : ExitNotifyParentInfo(nullptr, ::std::move(from)) {}
  inline ExitNotifyParentInfo& operator=(const ExitNotifyParentInfo& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExitNotifyParentInfo& operator=(ExitNotifyParentInfo&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExitNotifyParentInfo& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExitNotifyParentInfo>(&ExitNotifyParentInfo_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(ExitNotifyParentInfo& a, ExitNotifyParentInfo& b) { a.Swap(&b); }
  inline void Swap(ExitNotifyParentInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExitNotifyParentInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExitNotifyParentInfo* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<ExitNotifyParentInfo>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const ExitNotifyParentInfo& from);
  using Super_::MergeFrom;
  void MergeFrom(const ExitNotifyParentInfo& from) { ExitNotifyParentInfo::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExitNotifyParentInfo* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.sentry.ExitNotifyParentInfo"; }

  explicit ExitNotifyParentInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExitNotifyParentInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExitNotifyParentInfo& from);
  ExitNotifyParentInfo(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExitNotifyParentInfo&& from) noexcept
      : ExitNotifyParentInfo(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitStatusFieldNumber = 2,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // int32 exit_status = 2;
  void clear_exit_status() ;
  [[nodiscard]] ::int32_t exit_status() const;
  void set_exit_status(::int32_t value);

  private:
  ::int32_t _internal_exit_status() const;
  void _internal_set_exit_status(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.sentry.ExitNotifyParentInfo)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExitNotifyParentInfo& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::int32_t exit_status_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ExecveInfo final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.sentry.ExecveInfo) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline ExecveInfo() : ExecveInfo(nullptr) {}
  ~ExecveInfo() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ExecveInfo* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ExecveInfo));
  }
#endif

  template <typename = void>
  explicit constexpr ExecveInfo(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ExecveInfo(const ExecveInfo& from) : ExecveInfo(nullptr, from) {}
  inline ExecveInfo(ExecveInfo&& from) noexcept : ExecveInfo(nullptr, ::std::move(from)) {}
  inline ExecveInfo& operator=(const ExecveInfo& from) {
    CopyFrom(from);
    return *this;
  }
  inline ExecveInfo& operator=(ExecveInfo&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ExecveInfo& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ExecveInfo>(&ExecveInfo_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(ExecveInfo& a, ExecveInfo& b) { a.Swap(&b); }
  inline void Swap(ExecveInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ExecveInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ExecveInfo* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<ExecveInfo>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const ExecveInfo& from);
  using Super_::MergeFrom;
  void MergeFrom(const ExecveInfo& from) { ExecveInfo::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ExecveInfo* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.sentry.ExecveInfo"; }

  explicit ExecveInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ExecveInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ExecveInfo& from);
  ExecveInfo(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ExecveInfo&& from) noexcept
      : ExecveInfo(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kArgvFieldNumber = 3,
    kEnvFieldNumber = 4,
    kBinaryPathFieldNumber = 2,
    kBinarySha256FieldNumber = 8,
    kContextDataFieldNumber = 1,
    kBinaryModeFieldNumber = 5,
    kBinaryUidFieldNumber = 6,
    kBinaryGidFieldNumber = 7,
  };
  // repeated string argv = 3;
  [[nodiscard]] int argv_size() const;
  private:
  int _internal_argv_size() const;

  public:
  void clear_argv() ;
  [[nodiscard]] const ::std::string& argv(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_argv(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_argv(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_argv();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_argv(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      argv() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_argv();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_argv() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_argv();

  public:
  // repeated string env = 4;
  [[nodiscard]] int env_size() const;
  private:
  int _internal_env_size() const;

  public:
  void clear_env() ;
  [[nodiscard]] const ::std::string& env(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_env(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_env(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_env();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_env(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      env() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_env();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_env() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_env();

  public:
  // string binary_path = 2;
  void clear_binary_path() ;
  [[nodiscard]] const ::std::string& binary_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_binary_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_binary_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_binary_path();
  void set_allocated_binary_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_binary_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_binary_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_binary_path();

  public:
  // bytes binary_sha256 = 8;
  void clear_binary_sha256() ;
  [[nodiscard]] const ::std::string& binary_sha256() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_binary_sha256(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_binary_sha256();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_binary_sha256();
  void set_allocated_binary_sha256(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_binary_sha256() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_binary_sha256(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_binary_sha256();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // uint32 binary_mode = 5;
  void clear_binary_mode() ;
  [[nodiscard]] ::uint32_t binary_mode() const;
  void set_binary_mode(::uint32_t value);

  private:
  ::uint32_t _internal_binary_mode() const;
  void _internal_set_binary_mode(::uint32_t value);

  public:
  // uint32 binary_uid = 6;
  void clear_binary_uid() ;
  [[nodiscard]] ::uint32_t binary_uid() const;
  void set_binary_uid(::uint32_t value);

  private:
  ::uint32_t _internal_binary_uid() const;
  void _internal_set_binary_uid(::uint32_t value);

  public:
  // uint32 binary_gid = 7;
  void clear_binary_gid() ;
  [[nodiscard]] ::uint32_t binary_gid() const;
  void set_binary_gid(::uint32_t value);

  private:
  ::uint32_t _internal_binary_gid() const;
  void _internal_set_binary_gid(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.sentry.ExecveInfo)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 8,
                          1, 59,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ExecveInfo& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> argv_;
    ::google::protobuf::RepeatedPtrField<::std::string> env_;
    ::google::protobuf::internal::ArenaStringPtr binary_path_;
    ::google::protobuf::internal::ArenaStringPtr binary_sha256_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::uint32_t binary_mode_;
    ::uint32_t binary_uid_;
    ::uint32_t binary_gid_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED CloneInfo final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.sentry.CloneInfo) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline CloneInfo() : CloneInfo(nullptr) {}
  ~CloneInfo() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(CloneInfo* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(CloneInfo));
  }
#endif

  template <typename = void>
  explicit constexpr CloneInfo(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline CloneInfo(const CloneInfo& from) : CloneInfo(nullptr, from) {}
  inline CloneInfo(CloneInfo&& from) noexcept : CloneInfo(nullptr, ::std::move(from)) {}
  inline CloneInfo& operator=(const CloneInfo& from) {
    CopyFrom(from);
    return *this;
  }
  inline CloneInfo& operator=(CloneInfo&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const CloneInfo& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<CloneInfo>(&CloneInfo_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(CloneInfo& a, CloneInfo& b) { a.Swap(&b); }
  inline void Swap(CloneInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(CloneInfo* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] CloneInfo* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<CloneInfo>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const CloneInfo& from);
  using Super_::MergeFrom;
  void MergeFrom(const CloneInfo& from) { CloneInfo::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(CloneInfo* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.sentry.CloneInfo"; }

  explicit CloneInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  CloneInfo(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const CloneInfo& from);
  CloneInfo(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, CloneInfo&& from) noexcept
      : CloneInfo(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kCreatedThreadIdFieldNumber = 3,
    kCreatedThreadGroupIdFieldNumber = 4,
    kCreatedThreadStartTimeNsFieldNumber = 5,
    kFlagsFieldNumber = 6,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // int32 created_thread_id = 3;
  void clear_created_thread_id() ;
  [[nodiscard]] ::int32_t created_thread_id() const;
  void set_created_thread_id(::int32_t value);

  private:
  ::int32_t _internal_created_thread_id() const;
  void _internal_set_created_thread_id(::int32_t value);

  public:
  // int32 created_thread_group_id = 4;
  void clear_created_thread_group_id() ;
  [[nodiscard]] ::int32_t created_thread_group_id() const;
  void set_created_thread_group_id(::int32_t value);

  private:
  ::int32_t _internal_created_thread_group_id() const;
  void _internal_set_created_thread_group_id(::int32_t value);

  public:
  // int64 created_thread_start_time_ns = 5;
  void clear_created_thread_start_time_ns() ;
  [[nodiscard]] ::int64_t created_thread_start_time_ns() const;
  void set_created_thread_start_time_ns(::int64_t value);

  private:
  ::int64_t _internal_created_thread_start_time_ns() const;
  void _internal_set_created_thread_start_time_ns(::int64_t value);

  public:
  // uint64 flags = 6;
  void clear_flags() ;
  [[nodiscard]] ::uint64_t flags() const;
  void set_flags(::uint64_t value);

  private:
  ::uint64_t _internal_flags() const;
  void _internal_set_flags(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.sentry.CloneInfo)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          1, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const CloneInfo& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::int32_t created_thread_id_;
    ::int32_t created_thread_group_id_;
    ::int64_t created_thread_start_time_ns_;
    ::uint64_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// CloneInfo

// .gvisor.common.ContextData context_data = 1;
inline bool CloneInfo::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& CloneInfo::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& CloneInfo::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.CloneInfo.context_data)
  return _internal_context_data();
}
inline void CloneInfo::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.sentry.CloneInfo.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE CloneInfo::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE CloneInfo::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.CloneInfo.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL CloneInfo::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL CloneInfo::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.CloneInfo.context_data)
  return _msg;
}
inline void CloneInfo::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.CloneInfo.context_data)
}

// int32 created_thread_id = 3;
inline void CloneInfo::clear_created_thread_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t CloneInfo::created_thread_id() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.CloneInfo.created_thread_id)
  return _internal_created_thread_id();
}
inline void CloneInfo::set_created_thread_id(::int32_t value) {
  _internal_set_created_thread_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.CloneInfo.created_thread_id)
}
inline ::int32_t CloneInfo::_internal_created_thread_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_thread_id_;
}
inline void CloneInfo::_internal_set_created_thread_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_id_ = value;
}

// int32 created_thread_group_id = 4;
inline void CloneInfo::clear_created_thread_group_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_group_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::int32_t CloneInfo::created_thread_group_id() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.CloneInfo.created_thread_group_id)
  return _internal_created_thread_group_id();
}
inline void CloneInfo::set_created_thread_group_id(::int32_t value) {
  _internal_set_created_thread_group_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.CloneInfo.created_thread_group_id)
}
inline ::int32_t CloneInfo::_internal_created_thread_group_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_thread_group_id_;
}
inline void CloneInfo::_internal_set_created_thread_group_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_group_id_ = value;
}

// int64 created_thread_start_time_ns = 5;
inline void CloneInfo::clear_created_thread_start_time_ns() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_start_time_ns_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int64_t CloneInfo::created_thread_start_time_ns() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.CloneInfo.created_thread_start_time_ns)
  return _internal_created_thread_start_time_ns();
}
inline void CloneInfo::set_created_thread_start_time_ns(::int64_t value) {
  _internal_set_created_thread_start_time_ns(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.CloneInfo.created_thread_start_time_ns)
}
inline ::int64_t CloneInfo::_internal_created_thread_start_time_ns() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.created_thread_start_time_ns_;
}
inline void CloneInfo::_internal_set_created_thread_start_time_ns(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.created_thread_start_time_ns_ = value;
}

// uint64 flags = 6;
inline void CloneInfo::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t CloneInfo::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.CloneInfo.flags)
  return _internal_flags();
}
inline void CloneInfo::set_flags(::uint64_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.CloneInfo.flags)
}
inline ::uint64_t CloneInfo::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void CloneInfo::_internal_set_flags(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// ExecveInfo

// .gvisor.common.ContextData context_data = 1;
inline bool ExecveInfo::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& ExecveInfo::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& ExecveInfo::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.context_data)
  return _internal_context_data();
}
inline void ExecveInfo::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.sentry.ExecveInfo.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE ExecveInfo::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE ExecveInfo::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.ExecveInfo.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL ExecveInfo::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL ExecveInfo::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExecveInfo.context_data)
  return _msg;
}
inline void ExecveInfo::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.ExecveInfo.context_data)
}

// string binary_path = 2;
inline void ExecveInfo::clear_binary_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& ExecveInfo::binary_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.binary_path)
  return _internal_binary_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecveInfo::set_binary_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.binary_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.binary_path)
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::mutable_binary_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_binary_path();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExecveInfo.binary_path)
  return _s;
}
inline const ::std::string& ExecveInfo::_internal_binary_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_path_.Get();
}
inline void ExecveInfo::_internal_set_binary_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::_internal_mutable_binary_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.binary_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecveInfo::release_binary_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.ExecveInfo.binary_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.binary_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.binary_path_.Set("", GetArena());
  }
  return released;
}
inline void ExecveInfo::set_allocated_binary_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.binary_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.binary_path_.IsDefault()) {
    _impl_.binary_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.ExecveInfo.binary_path)
}

// repeated string argv = 3;
inline int ExecveInfo::_internal_argv_size() const {
  return _internal_argv().size();
}
inline int ExecveInfo::argv_size() const {
  return _internal_argv_size();
}
inline void ExecveInfo::clear_argv() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.argv_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::add_argv()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_argv()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.sentry.ExecveInfo.argv)
  return _s;
}
inline const ::std::string& ExecveInfo::argv(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.argv)
  return _internal_argv().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::mutable_argv(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExecveInfo.argv)
  return _internal_mutable_argv()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ExecveInfo::set_argv(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_argv()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.argv)
}
template <typename Arg_, typename... Args_>
inline void ExecveInfo::add_argv(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_argv(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:gvisor.sentry.ExecveInfo.argv)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ExecveInfo::argv()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.sentry.ExecveInfo.argv)
  return _internal_argv();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecveInfo::mutable_argv() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.sentry.ExecveInfo.argv)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_argv();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ExecveInfo::_internal_argv() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.argv_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecveInfo::_internal_mutable_argv() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.argv_;
}

// repeated string env = 4;
inline int ExecveInfo::_internal_env_size() const {
  return _internal_env().size();
}
inline int ExecveInfo::env_size() const {
  return _internal_env_size();
}
inline void ExecveInfo::clear_env() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.env_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::add_env()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_env()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.sentry.ExecveInfo.env)
  return _s;
}
inline const ::std::string& ExecveInfo::env(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.env)
  return _internal_env().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::mutable_env(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExecveInfo.env)
  return _internal_mutable_env()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void ExecveInfo::set_env(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_env()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.env)
}
template <typename Arg_, typename... Args_>
inline void ExecveInfo::add_env(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_env(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:gvisor.sentry.ExecveInfo.env)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& ExecveInfo::env()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.sentry.ExecveInfo.env)
  return _internal_env();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecveInfo::mutable_env() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.sentry.ExecveInfo.env)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_env();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
ExecveInfo::_internal_env() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.env_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
ExecveInfo::_internal_mutable_env() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.env_;
}

// uint32 binary_mode = 5;
inline void ExecveInfo::clear_binary_mode() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_mode_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint32_t ExecveInfo::binary_mode() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.binary_mode)
  return _internal_binary_mode();
}
inline void ExecveInfo::set_binary_mode(::uint32_t value) {
  _internal_set_binary_mode(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.binary_mode)
}
inline ::uint32_t ExecveInfo::_internal_binary_mode() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_mode_;
}
inline void ExecveInfo::_internal_set_binary_mode(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_mode_ = value;
}

// uint32 binary_uid = 6;
inline void ExecveInfo::clear_binary_uid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_uid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint32_t ExecveInfo::binary_uid() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.binary_uid)
  return _internal_binary_uid();
}
inline void ExecveInfo::set_binary_uid(::uint32_t value) {
  _internal_set_binary_uid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.binary_uid)
}
inline ::uint32_t ExecveInfo::_internal_binary_uid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_uid_;
}
inline void ExecveInfo::_internal_set_binary_uid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_uid_ = value;
}

// uint32 binary_gid = 7;
inline void ExecveInfo::clear_binary_gid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_gid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::uint32_t ExecveInfo::binary_gid() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.binary_gid)
  return _internal_binary_gid();
}
inline void ExecveInfo::set_binary_gid(::uint32_t value) {
  _internal_set_binary_gid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.binary_gid)
}
inline ::uint32_t ExecveInfo::_internal_binary_gid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_gid_;
}
inline void ExecveInfo::_internal_set_binary_gid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_gid_ = value;
}

// bytes binary_sha256 = 8;
inline void ExecveInfo::clear_binary_sha256() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_sha256_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& ExecveInfo::binary_sha256() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExecveInfo.binary_sha256)
  return _internal_binary_sha256();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void ExecveInfo::set_binary_sha256(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.binary_sha256_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExecveInfo.binary_sha256)
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::mutable_binary_sha256()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_binary_sha256();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExecveInfo.binary_sha256)
  return _s;
}
inline const ::std::string& ExecveInfo::_internal_binary_sha256() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.binary_sha256_.Get();
}
inline void ExecveInfo::_internal_set_binary_sha256(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.binary_sha256_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL ExecveInfo::_internal_mutable_binary_sha256() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.binary_sha256_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE ExecveInfo::release_binary_sha256() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.ExecveInfo.binary_sha256)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.binary_sha256_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.binary_sha256_.Set("", GetArena());
  }
  return released;
}
inline void ExecveInfo::set_allocated_binary_sha256(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.binary_sha256_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.binary_sha256_.IsDefault()) {
    _impl_.binary_sha256_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.ExecveInfo.binary_sha256)
}

// -------------------------------------------------------------------

// ExitNotifyParentInfo

// .gvisor.common.ContextData context_data = 1;
inline bool ExitNotifyParentInfo::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& ExitNotifyParentInfo::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& ExitNotifyParentInfo::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExitNotifyParentInfo.context_data)
  return _internal_context_data();
}
inline void ExitNotifyParentInfo::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.sentry.ExitNotifyParentInfo.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE ExitNotifyParentInfo::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE ExitNotifyParentInfo::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.ExitNotifyParentInfo.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL ExitNotifyParentInfo::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL ExitNotifyParentInfo::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.ExitNotifyParentInfo.context_data)
  return _msg;
}
inline void ExitNotifyParentInfo::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.ExitNotifyParentInfo.context_data)
}

// int32 exit_status = 2;
inline void ExitNotifyParentInfo::clear_exit_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_status_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t ExitNotifyParentInfo::exit_status() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.ExitNotifyParentInfo.exit_status)
  return _internal_exit_status();
}
inline void ExitNotifyParentInfo::set_exit_status(::int32_t value) {
  _internal_set_exit_status(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.ExitNotifyParentInfo.exit_status)
}
inline ::int32_t ExitNotifyParentInfo::_internal_exit_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exit_status_;
}
inline void ExitNotifyParentInfo::_internal_set_exit_status(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_status_ = value;
}

// -------------------------------------------------------------------

// TaskExit

// .gvisor.common.ContextData context_data = 1;
inline bool TaskExit::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& TaskExit::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& TaskExit::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.sentry.TaskExit.context_data)
  return _internal_context_data();
}
inline void TaskExit::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.sentry.TaskExit.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TaskExit::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TaskExit::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.sentry.TaskExit.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TaskExit::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TaskExit::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.sentry.TaskExit.context_data)
  return _msg;
}
inline void TaskExit::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.sentry.TaskExit.context_data)
}

// int32 exit_status = 2;
inline void TaskExit::clear_exit_status() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_status_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int32_t TaskExit::exit_status() const {
  // @@protoc_insertion_point(field_get:gvisor.sentry.TaskExit.exit_status)
  return _internal_exit_status();
}
inline void TaskExit::set_exit_status(::int32_t value) {
  _internal_set_exit_status(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.sentry.TaskExit.exit_status)
}
inline ::int32_t TaskExit::_internal_exit_status() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.exit_status_;
}
inline void TaskExit::_internal_set_exit_status(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.exit_status_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace sentry
}  // namespace gvisor


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // pkg_2fsentry_2fseccheck_2fpoints_2fsentry_2eproto_2epb_2eh
