// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/common.proto
// Protobuf C++ Version: 7.36.1

#include "pkg/sentry/seccheck/points/common.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::gvisor::common::Handshake
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::common::Credentials
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::common::ContextData
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace gvisor {
namespace common {
class Handshake::_Internal {
 public:
  using HasBits = decltype(::std::declval<Handshake>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Handshake, _impl_._has_bits_);
};

constexpr Handshake::ParseTableT_ Handshake::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Handshake, _impl_._has_bits_),
      0, // no _extensions_
      1, 0,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967294,  // skipmap
      offsetof(ParseTableT_, field_entries),
      1,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::common::Handshake>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint32 version = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Handshake, _impl_.version_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Handshake, _impl_.version_)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint32 version = 1;
      {PROTOBUF_FIELD_OFFSET(Handshake, _impl_.version_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Handshake::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : version_{0u} {}

template <typename>
constexpr Handshake::Handshake(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Handshake::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Handshake(arena);
}
constexpr auto Handshake::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Handshake), alignof(Handshake));
}
constexpr auto Handshake::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Handshake::MergeImpl,
          Super_::GetNewImpl<Handshake>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Handshake::SharedDtor,
          Super_::GetClearImpl<Handshake>(), &Handshake::ByteSizeLong,
              &Handshake::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Handshake, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct HandshakeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr HandshakeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Handshake_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Handshake::InternalGenerateClassData_(
            _default, &Handshake_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Handshake>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~HandshakeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Handshake _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Handshake>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(HandshakeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST HandshakeGlobalsTypeInternal Handshake_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Handshake_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Handshake_globals_.GetClassData();
#else
  return Handshake_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Credentials::_Internal {
 public:
  using HasBits = decltype(::std::declval<Credentials>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Credentials, _impl_._has_bits_);
};

constexpr Credentials::ParseTableT_ Credentials::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Credentials, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::common::Credentials>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // uint32 real_uid = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.real_uid_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.real_uid_)}},
      // uint32 effective_uid = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.effective_uid_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.effective_uid_)}},
      // uint32 saved_uid = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.saved_uid_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.saved_uid_)}},
      // uint32 real_gid = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.real_gid_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.real_gid_)}},
      // uint32 effective_gid = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.effective_gid_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.effective_gid_)}},
      // uint32 saved_gid = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Credentials, _impl_.saved_gid_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Credentials, _impl_.saved_gid_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // uint32 real_uid = 1;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.real_uid_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 effective_uid = 2;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.effective_uid_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 saved_uid = 3;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.saved_uid_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 real_gid = 4;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.real_gid_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 effective_gid = 5;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.effective_gid_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 saved_gid = 6;
      {PROTOBUF_FIELD_OFFSET(Credentials, _impl_.saved_gid_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Credentials::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : real_uid_{0u},
        effective_uid_{0u},
        saved_uid_{0u},
        real_gid_{0u},
        effective_gid_{0u},
        saved_gid_{0u} {}

template <typename>
constexpr Credentials::Credentials(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Credentials::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Credentials(arena);
}
constexpr auto Credentials::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Credentials), alignof(Credentials));
}
constexpr auto Credentials::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Credentials::MergeImpl,
          Super_::GetNewImpl<Credentials>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Credentials::SharedDtor,
          Super_::GetClearImpl<Credentials>(), &Credentials::ByteSizeLong,
              &Credentials::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Credentials, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CredentialsGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CredentialsGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Credentials_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Credentials::InternalGenerateClassData_(
            _default, &Credentials_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Credentials>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CredentialsGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Credentials _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Credentials>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CredentialsGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CredentialsGlobalsTypeInternal Credentials_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Credentials_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Credentials_globals_.GetClassData();
#else
  return Credentials_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ContextData::_Internal {
 public:
  using HasBits = decltype(::std::declval<ContextData>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ContextData, _impl_._has_bits_);
};

constexpr ContextData::ParseTableT_ ContextData::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ContextData, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::common::ContextData>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // int64 time_ns = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ContextData, _impl_.time_ns_), 4>(),
       {8, 4, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.time_ns_)}},
      // int32 thread_id = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ContextData, _impl_.thread_id_), 6>(),
       {16, 6, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_id_)}},
      // int64 thread_start_time_ns = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ContextData, _impl_.thread_start_time_ns_), 5>(),
       {24, 5, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_start_time_ns_)}},
      // int32 thread_group_id = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(ContextData, _impl_.thread_group_id_), 7>(),
       {32, 7, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_group_id_)}},
      // int64 thread_group_start_time_ns = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(ContextData, _impl_.thread_group_start_time_ns_), 8>(),
       {40, 8, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_group_start_time_ns_)}},
      // string container_id = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 0, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.container_id_)}},
      // .gvisor.common.Credentials credentials = 7;
      {::_pbi::TcParser::FastMtS1,
       {58, 3, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.credentials_)}},
      // string cwd = 8;
      {::_pbi::TcParser::FastUS1,
       {66, 1, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.cwd_)}},
      // string process_name = 9;
      {::_pbi::TcParser::FastUS1,
       {74, 2, 0,
        PROTOBUF_FIELD_OFFSET(ContextData, _impl_.process_name_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 time_ns = 1;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.time_ns_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int32 thread_id = 2;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_id_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 thread_start_time_ns = 3;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_start_time_ns_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int32 thread_group_id = 4;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_group_id_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 thread_group_start_time_ns = 5;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_group_start_time_ns_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string container_id = 6;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.container_id_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .gvisor.common.Credentials credentials = 7;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.credentials_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string cwd = 8;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.cwd_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string process_name = 9;
      {PROTOBUF_FIELD_OFFSET(ContextData, _impl_.process_name_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::Credentials>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::Credentials_globals_},
        #endif
    }},
    {{
      "\31\0\0\0\0\0\14\0\3\14\0\0\0\0\0\0"
      "gvisor.common.ContextData"
      "container_id"
      "cwd"
      "process_name"
    }},
  };
}


inline constexpr ContextData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : container_id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        cwd_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        process_name_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        credentials_{nullptr},
        time_ns_{::int64_t{0}},
        thread_start_time_ns_{::int64_t{0}},
        thread_id_{0},
        thread_group_id_{0},
        thread_group_start_time_ns_{::int64_t{0}} {}

template <typename>
constexpr ContextData::ContextData(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ContextData::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ContextData(arena);
}
constexpr auto ContextData::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(ContextData), alignof(ContextData));
}
constexpr auto ContextData::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ContextData::MergeImpl,
          Super_::GetNewImpl<ContextData>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ContextData::SharedDtor,
          Super_::GetClearImpl<ContextData>(), &ContextData::ByteSizeLong,
              &ContextData::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ContextData, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ContextDataGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ContextDataGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ContextData_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ContextData::InternalGenerateClassData_(
            _default, &ContextData_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ContextData>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ContextDataGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ContextData _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ContextData>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ContextDataGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ContextDataGlobalsTypeInternal ContextData_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ContextData_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ContextData_globals_.GetClassData();
#else
  return ContextData_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace common
}  // namespace gvisor
static const ::_pb::EnumDescriptor* PROTOBUF_NONNULL
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto[1];
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto = nullptr;
const ::uint32_t
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Handshake, _impl_._has_bits_),
        4, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Handshake, _impl_.version_),
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.real_uid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.effective_uid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.saved_uid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.real_gid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.effective_gid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::Credentials, _impl_.saved_gid_),
        0,
        1,
        2,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.time_ns_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.thread_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.thread_start_time_ns_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.thread_group_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.thread_group_start_time_ns_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.container_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.credentials_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.cwd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::common::ContextData, _impl_.process_name_),
        4,
        6,
        5,
        7,
        8,
        0,
        3,
        1,
        2,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::gvisor::common::Handshake)},
        {5, sizeof(::gvisor::common::Credentials)},
        {20, sizeof(::gvisor::common::ContextData)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::gvisor::common::Handshake_globals_,
        &::gvisor::common::Credentials_globals_,
        &::gvisor::common::ContextData_globals_,
};
const char descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n\'pkg/sentry/seccheck/points/common.prot"
    "o\022\rgvisor.common\"\034\n\tHandshake\022\017\n\007version"
    "\030\001 \001(\r\"\205\001\n\013Credentials\022\020\n\010real_uid\030\001 \001(\r"
    "\022\025\n\reffective_uid\030\002 \001(\r\022\021\n\tsaved_uid\030\003 \001"
    "(\r\022\020\n\010real_gid\030\004 \001(\r\022\025\n\reffective_gid\030\005 "
    "\001(\r\022\021\n\tsaved_gid\030\006 \001(\r\"\366\001\n\013ContextData\022\017"
    "\n\007time_ns\030\001 \001(\003\022\021\n\tthread_id\030\002 \001(\005\022\034\n\024th"
    "read_start_time_ns\030\003 \001(\003\022\027\n\017thread_group"
    "_id\030\004 \001(\005\022\"\n\032thread_group_start_time_ns\030"
    "\005 \001(\003\022\024\n\014container_id\030\006 \001(\t\022/\n\013credentia"
    "ls\030\007 \001(\0132\032.gvisor.common.Credentials\022\013\n\003"
    "cwd\030\010 \001(\t\022\024\n\014process_name\030\t \001(\t*\217\010\n\013Mess"
    "ageType\022\023\n\017MESSAGE_UNKNOWN\020\000\022\033\n\027MESSAGE_"
    "CONTAINER_START\020\001\022\030\n\024MESSAGE_SENTRY_CLON"
    "E\020\002\022\027\n\023MESSAGE_SENTRY_EXEC\020\003\022%\n!MESSAGE_"
    "SENTRY_EXIT_NOTIFY_PARENT\020\004\022\034\n\030MESSAGE_S"
    "ENTRY_TASK_EXIT\020\005\022\027\n\023MESSAGE_SYSCALL_RAW"
    "\020\006\022\030\n\024MESSAGE_SYSCALL_OPEN\020\007\022\031\n\025MESSAGE_"
    "SYSCALL_CLOSE\020\010\022\030\n\024MESSAGE_SYSCALL_READ\020"
    "\t\022\033\n\027MESSAGE_SYSCALL_CONNECT\020\n\022\032\n\026MESSAG"
    "E_SYSCALL_EXECVE\020\013\022\032\n\026MESSAGE_SYSCALL_SO"
    "CKET\020\014\022\031\n\025MESSAGE_SYSCALL_CHDIR\020\r\022\031\n\025MES"
    "SAGE_SYSCALL_SETID\020\016\022\034\n\030MESSAGE_SYSCALL_"
    "SETRESID\020\017\022\035\n\031MESSAGE_SYSCALL_PRLIMIT64\020"
    "\020\022\030\n\024MESSAGE_SYSCALL_PIPE\020\021\022\031\n\025MESSAGE_S"
    "YSCALL_FCNTL\020\022\022\027\n\023MESSAGE_SYSCALL_DUP\020\023\022"
    "\034\n\030MESSAGE_SYSCALL_SIGNALFD\020\024\022\032\n\026MESSAGE"
    "_SYSCALL_CHROOT\020\025\022\033\n\027MESSAGE_SYSCALL_EVE"
    "NTFD\020\026\022\031\n\025MESSAGE_SYSCALL_CLONE\020\027\022\030\n\024MES"
    "SAGE_SYSCALL_BIND\020\030\022\032\n\026MESSAGE_SYSCALL_A"
    "CCEPT\020\031\022\"\n\036MESSAGE_SYSCALL_TIMERFD_CREAT"
    "E\020\032\022#\n\037MESSAGE_SYSCALL_TIMERFD_SETTIME\020\033"
    "\022#\n\037MESSAGE_SYSCALL_TIMERFD_GETTIME\020\034\022\030\n"
    "\024MESSAGE_SYSCALL_FORK\020\035\022 \n\034MESSAGE_SYSCA"
    "LL_INOTIFY_INIT\020\036\022%\n!MESSAGE_SYSCALL_INO"
    "TIFY_ADD_WATCH\020\037\022$\n MESSAGE_SYSCALL_INOT"
    "IFY_RM_WATCH\020 \022\036\n\032MESSAGE_SYSCALL_SOCKET"
    "PAIR\020!\022\031\n\025MESSAGE_SYSCALL_WRITE\020\"b\006proto"
    "3"
};
static ::absl::once_flag descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto = {
    false,
    false,
    1521,
    descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
    "pkg/sentry/seccheck/points/common.proto",
    &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto_once,
    nullptr,
    0,
    3,
    schemas,
    file_message_globals,
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto::offsets,
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
};
namespace gvisor {
namespace common {
[[nodiscard]] const ::google::protobuf::EnumDescriptor* PROTOBUF_NONNULL
MessageType_descriptor() {
  ::google::protobuf::internal::AssignDescriptors(&descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto);
  return file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto[0];
}
PROTOBUF_CONSTINIT const uint32_t MessageType_internal_data_[] = {
    2293760u, 0u, };
// ===================================================================

Handshake::Handshake(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Handshake_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.common.Handshake)
}
Handshake::Handshake(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Handshake& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Handshake_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Handshake::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Handshake::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  _impl_.version_ = {};
}
Handshake::~Handshake() {
  // @@protoc_insertion_point(destructor:gvisor.common.Handshake)
  SharedDtor(*this);
}
inline void Handshake::SharedDtor(MessageLite& self) {
  Handshake& this_ = static_cast<Handshake&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Handshake_class_data_ =
        Handshake::InternalGenerateClassData_(Handshake_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Handshake::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Handshake_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Handshake_class_data_.tc_table);
  return Handshake_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Handshake::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Handshake_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Handshake_globals_));
  return Handshake_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Handshake::ParseTableT_
    Handshake::_table_ =
        Handshake::InternalGenerateParseTable_(Handshake_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Handshake::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.common.Handshake)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  this_._impl_.version_ = 0u;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Handshake::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Handshake& this_ = static_cast<const Handshake&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Handshake::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Handshake& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.common.Handshake)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint32 version = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_version() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          1, this_._internal_version(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.common.Handshake)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Handshake::ByteSizeLong(const MessageLite& base) {
  const Handshake& this_ = static_cast<const Handshake&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Handshake::ByteSizeLong() const {
  const Handshake& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.common.Handshake)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

   {
    // uint32 version = 1;
    cached_has_bits = this_._impl_._has_bits_[0];
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_version() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_version());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Handshake::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Handshake*>(&to_msg);
  auto& from = static_cast<const Handshake&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.common.Handshake)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (from._internal_version() != 0) {
      _this->_impl_.version_ = from._impl_.version_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Handshake::CopyFrom(const Handshake& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.common.Handshake)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Handshake::InternalSwap(Handshake* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  swap(_impl_.version_, other->_impl_.version_);
}

::google::protobuf::Metadata Handshake::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Credentials::Credentials(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Credentials_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.common.Credentials)
}
Credentials::Credentials(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Credentials& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Credentials_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Credentials::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Credentials::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, real_uid_),
           0,
           offsetof(Impl_, saved_gid_) -
               offsetof(Impl_, real_uid_) +
               sizeof(Impl_::saved_gid_));
}
Credentials::~Credentials() {
  // @@protoc_insertion_point(destructor:gvisor.common.Credentials)
  SharedDtor(*this);
}
inline void Credentials::SharedDtor(MessageLite& self) {
  Credentials& this_ = static_cast<Credentials&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Credentials_class_data_ =
        Credentials::InternalGenerateClassData_(Credentials_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Credentials::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Credentials_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Credentials_class_data_.tc_table);
  return Credentials_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Credentials::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Credentials_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Credentials_globals_));
  return Credentials_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Credentials::ParseTableT_
    Credentials::_table_ =
        Credentials::InternalGenerateParseTable_(Credentials_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Credentials::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.common.Credentials)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    ::memset(&this_._impl_.real_uid_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.saved_gid_) -
                 reinterpret_cast<char*>(&this_._impl_.real_uid_)) +
                 sizeof(_impl_.saved_gid_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Credentials::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Credentials& this_ = static_cast<const Credentials&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Credentials::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Credentials& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.common.Credentials)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint32 real_uid = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_real_uid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          1, this_._internal_real_uid(), target);
    }
  }

  // uint32 effective_uid = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_effective_uid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          2, this_._internal_effective_uid(), target);
    }
  }

  // uint32 saved_uid = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_saved_uid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          3, this_._internal_saved_uid(), target);
    }
  }

  // uint32 real_gid = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_real_gid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          4, this_._internal_real_gid(), target);
    }
  }

  // uint32 effective_gid = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_effective_gid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          5, this_._internal_effective_gid(), target);
    }
  }

  // uint32 saved_gid = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_saved_gid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          6, this_._internal_saved_gid(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.common.Credentials)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Credentials::ByteSizeLong(const MessageLite& base) {
  const Credentials& this_ = static_cast<const Credentials&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Credentials::ByteSizeLong() const {
  const Credentials& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.common.Credentials)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // uint32 real_uid = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_real_uid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_real_uid());
      }
    }
    // uint32 effective_uid = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_effective_uid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_effective_uid());
      }
    }
    // uint32 saved_uid = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_saved_uid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_saved_uid());
      }
    }
    // uint32 real_gid = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_real_gid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_real_gid());
      }
    }
    // uint32 effective_gid = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_effective_gid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_effective_gid());
      }
    }
    // uint32 saved_gid = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_saved_gid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_saved_gid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Credentials::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Credentials*>(&to_msg);
  auto& from = static_cast<const Credentials&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.common.Credentials)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_real_uid() != 0) {
        _this->_impl_.real_uid_ = from._impl_.real_uid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_effective_uid() != 0) {
        _this->_impl_.effective_uid_ = from._impl_.effective_uid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_saved_uid() != 0) {
        _this->_impl_.saved_uid_ = from._impl_.saved_uid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_real_gid() != 0) {
        _this->_impl_.real_gid_ = from._impl_.real_gid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_effective_gid() != 0) {
        _this->_impl_.effective_gid_ = from._impl_.effective_gid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_saved_gid() != 0) {
        _this->_impl_.saved_gid_ = from._impl_.saved_gid_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Credentials::CopyFrom(const Credentials& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.common.Credentials)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Credentials::InternalSwap(Credentials* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Credentials, _impl_.saved_gid_)
      + sizeof(Credentials::_impl_.saved_gid_)
      - PROTOBUF_FIELD_OFFSET(Credentials, _impl_.real_uid_)>(
          reinterpret_cast<char*>(&_impl_.real_uid_),
          reinterpret_cast<char*>(&other->_impl_.real_uid_));
}

::google::protobuf::Metadata Credentials::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ContextData::ContextData(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ContextData_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.common.ContextData)
}
PROTOBUF_NDEBUG_INLINE ContextData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::common::ContextData& from_msg)
      : _has_bits_{from._has_bits_},
        container_id_(arena, from.container_id_),
        cwd_(arena, from.cwd_),
        process_name_(arena, from.process_name_) {}

ContextData::ContextData(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ContextData& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ContextData_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ContextData* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.credentials_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.credentials_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, time_ns_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, time_ns_),
           offsetof(Impl_, thread_group_start_time_ns_) -
               offsetof(Impl_, time_ns_) +
               sizeof(Impl_::thread_group_start_time_ns_));

  // @@protoc_insertion_point(copy_constructor:gvisor.common.ContextData)
}
PROTOBUF_NDEBUG_INLINE ContextData::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : container_id_(arena),
        cwd_(arena),
        process_name_(arena) {}

inline void ContextData::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, credentials_),
           0,
           offsetof(Impl_, thread_group_start_time_ns_) -
               offsetof(Impl_, credentials_) +
               sizeof(Impl_::thread_group_start_time_ns_));
}
ContextData::~ContextData() {
  // @@protoc_insertion_point(destructor:gvisor.common.ContextData)
  SharedDtor(*this);
}
inline void ContextData::SharedDtor(MessageLite& self) {
  ContextData& this_ = static_cast<ContextData&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.container_id_.Destroy();
  this_._impl_.cwd_.Destroy();
  this_._impl_.process_name_.Destroy();
  delete this_._impl_.credentials_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ContextData_class_data_ =
        ContextData::InternalGenerateClassData_(ContextData_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ContextData::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ContextData_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ContextData_class_data_.tc_table);
  return ContextData_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ContextData::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ContextData_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ContextData_globals_));
  return ContextData_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ContextData::ParseTableT_
    ContextData::_table_ =
        ContextData::InternalGenerateParseTable_(ContextData_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ContextData::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.common.ContextData)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.container_id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.cwd_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      this_._impl_.process_name_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.credentials_ != nullptr);
      this_._impl_.credentials_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f0U)) {
    ::memset(&this_._impl_.time_ns_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.thread_group_id_) -
                 reinterpret_cast<char*>(&this_._impl_.time_ns_)) +
                 sizeof(_impl_.thread_group_id_));
  }
  this_._impl_.thread_group_start_time_ns_ = ::int64_t{0};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ContextData::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ContextData& this_ = static_cast<const ContextData&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ContextData::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ContextData& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.common.ContextData)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 time_ns = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_time_ns() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_time_ns(), target);
    }
  }

  // int32 thread_id = 2;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_thread_id() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<2>(
              stream, this_._internal_thread_id(), target);
    }
  }

  // int64 thread_start_time_ns = 3;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_thread_start_time_ns() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<3>(
              stream, this_._internal_thread_start_time_ns(), target);
    }
  }

  // int32 thread_group_id = 4;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_thread_group_id() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_thread_group_id(), target);
    }
  }

  // int64 thread_group_start_time_ns = 5;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_thread_group_start_time_ns() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<5>(
              stream, this_._internal_thread_group_start_time_ns(), target);
    }
  }

  // string container_id = 6;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_container_id().empty()) {
      const ::std::string& _s = this_._internal_container_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.common.ContextData.container_id");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // .gvisor.common.Credentials credentials = 7;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        7, *this_._impl_.credentials_, this_._impl_.credentials_->GetCachedSize(), target,
        stream);
  }

  // string cwd = 8;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_cwd().empty()) {
      const ::std::string& _s = this_._internal_cwd();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.common.ContextData.cwd");
      target = stream->WriteStringMaybeAliased(8, _s, target);
    }
  }

  // string process_name = 9;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_process_name().empty()) {
      const ::std::string& _s = this_._internal_process_name();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.common.ContextData.process_name");
      target = stream->WriteStringMaybeAliased(9, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.common.ContextData)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ContextData::ByteSizeLong(const MessageLite& base) {
  const ContextData& this_ = static_cast<const ContextData&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ContextData::ByteSizeLong() const {
  const ContextData& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.common.ContextData)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string container_id = 6;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_container_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_container_id());
      }
    }
    // string cwd = 8;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_cwd().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_cwd());
      }
    }
    // string process_name = 9;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_process_name().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_process_name());
      }
    }
    // .gvisor.common.Credentials credentials = 7;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.credentials_);
    }
    // int64 time_ns = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_time_ns() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_time_ns());
      }
    }
    // int64 thread_start_time_ns = 3;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_thread_start_time_ns() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_thread_start_time_ns());
      }
    }
    // int32 thread_id = 2;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_thread_id() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_thread_id());
      }
    }
    // int32 thread_group_id = 4;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_thread_group_id() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_thread_group_id());
      }
    }
  }
   {
    // int64 thread_group_start_time_ns = 5;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_thread_group_start_time_ns() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_thread_group_start_time_ns());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ContextData::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ContextData*>(&to_msg);
  auto& from = static_cast<const ContextData&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.common.ContextData)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_container_id().empty()) {
        _this->_internal_set_container_id(from._internal_container_id());
      } else {
        if (_this->_impl_.container_id_.IsDefault()) {
          _this->_internal_set_container_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_cwd().empty()) {
        _this->_internal_set_cwd(from._internal_cwd());
      } else {
        if (_this->_impl_.cwd_.IsDefault()) {
          _this->_internal_set_cwd("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_process_name().empty()) {
        _this->_internal_set_process_name(from._internal_process_name());
      } else {
        if (_this->_impl_.process_name_.IsDefault()) {
          _this->_internal_set_process_name("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.credentials_ != nullptr);
      if (_this->_impl_.credentials_ == nullptr) {
        _this->_impl_.credentials_ = Super_::CopyConstruct(arena, *from._impl_.credentials_);
      } else {
        _this->_impl_.credentials_->MergeFrom(*from._impl_.credentials_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_time_ns() != 0) {
        _this->_impl_.time_ns_ = from._impl_.time_ns_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_thread_start_time_ns() != 0) {
        _this->_impl_.thread_start_time_ns_ = from._impl_.thread_start_time_ns_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_thread_id() != 0) {
        _this->_impl_.thread_id_ = from._impl_.thread_id_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_thread_group_id() != 0) {
        _this->_impl_.thread_group_id_ = from._impl_.thread_group_id_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_thread_group_start_time_ns() != 0) {
      _this->_impl_.thread_group_start_time_ns_ = from._impl_.thread_group_start_time_ns_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ContextData::CopyFrom(const ContextData& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.common.ContextData)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ContextData::InternalSwap(ContextData* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.container_id_, &other->_impl_.container_id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.cwd_, &other->_impl_.cwd_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.process_name_, &other->_impl_.process_name_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ContextData, _impl_.thread_group_start_time_ns_)
      + sizeof(ContextData::_impl_.thread_group_start_time_ns_)
      - PROTOBUF_FIELD_OFFSET(ContextData, _impl_.credentials_)>(
          reinterpret_cast<char*>(&_impl_.credentials_),
          reinterpret_cast<char*>(&other->_impl_.credentials_));
}

::google::protobuf::Metadata ContextData::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace common
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
