// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/container.proto
// Protobuf C++ Version: 7.36.1

#include "pkg/sentry/seccheck/points/container.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::gvisor::container::Start
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace gvisor {
namespace container {
class Start::_Internal {
 public:
  using HasBits = decltype(::std::declval<Start>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Start, _impl_._has_bits_);
};

constexpr Start::ParseTableT_ Start::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Start, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      1,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::container::Start>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 4, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.context_data_)}},
      // string id = 2;
      {::_pbi::TcParser::FastUS1,
       {18, 2, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.id_)}},
      // string cwd = 3;
      {::_pbi::TcParser::FastUS1,
       {26, 3, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.cwd_)}},
      // repeated string args = 4;
      {::_pbi::TcParser::FastUR1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.args_)}},
      // repeated string env = 5;
      {::_pbi::TcParser::FastUR1,
       {42, 1, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.env_)}},
      // bool terminal = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Start, _impl_.terminal_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Start, _impl_.terminal_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.context_data_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // string id = 2;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.id_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string cwd = 3;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.cwd_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string args = 4;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.args_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string env = 5;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.env_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // bool terminal = 6;
      {PROTOBUF_FIELD_OFFSET(Start, _impl_.terminal_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
    }},
    {{
      "\26\0\2\3\4\3\0\0"
      "gvisor.container.Start"
      "id"
      "cwd"
      "args"
      "env"
    }},
  };
}


inline constexpr Start::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : args_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::container::Start,
            PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.args_)>()
         }
        ,
        env_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::container::Start,
            PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.env_)>()
         }
        ,
        id_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        cwd_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        terminal_{false} {}

template <typename>
constexpr Start::Start(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Start::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Start(arena);
}
constexpr auto Start::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Start), alignof(Start));
}
constexpr auto Start::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Start::MergeImpl,
          Super_::GetNewImpl<Start>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Start::SharedDtor,
          Super_::GetClearImpl<Start>(), &Start::ByteSizeLong,
              &Start::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Start, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StartGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StartGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Start_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Start::InternalGenerateClassData_(
            _default, &Start_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Start>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StartGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Start _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Start>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StartGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StartGlobalsTypeInternal Start_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Start_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Start_globals_.GetClassData();
#else
  return Start_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace container
}  // namespace gvisor
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto = nullptr;
const ::uint32_t
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.cwd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.args_),
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.env_),
        PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.terminal_),
        4,
        2,
        3,
        0,
        1,
        5,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::gvisor::container::Start)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::gvisor::container::Start_globals_,
};
const char descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n*pkg/sentry/seccheck/points/container.p"
    "roto\022\020gvisor.container\032\'pkg/sentry/secch"
    "eck/points/common.proto\"\177\n\005Start\0220\n\014cont"
    "ext_data\030\001 \001(\0132\032.gvisor.common.ContextDa"
    "ta\022\n\n\002id\030\002 \001(\t\022\013\n\003cwd\030\003 \001(\t\022\014\n\004args\030\004 \003("
    "\t\022\013\n\003env\030\005 \003(\t\022\020\n\010terminal\030\006 \001(\010b\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_deps[1] = {
        &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
};
static ::absl::once_flag descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto = {
    false,
    false,
    240,
    descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto,
    "pkg/sentry/seccheck/points/container.proto",
    &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_once,
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto_deps,
    1,
    1,
    schemas,
    file_message_globals,
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto::offsets,
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto,
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto,
};
namespace gvisor {
namespace container {
// ===================================================================

void Start::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
Start::Start(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Start_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.container.Start)
}
PROTOBUF_NDEBUG_INLINE Start::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::container::Start& from_msg)
      : _has_bits_{from._has_bits_},
        args_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::container::Start,
              PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.args_)>()
          , arena, from.args_
        }
        ,
        env_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::container::Start,
              PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.env_)>()
          , arena, from.env_
        }
        ,
        id_(arena, from.id_),
        cwd_(arena, from.cwd_) {}

Start::Start(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Start& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Start_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Start* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.terminal_ = from._impl_.terminal_;

  // @@protoc_insertion_point(copy_constructor:gvisor.container.Start)
}
PROTOBUF_NDEBUG_INLINE Start::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : args_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::container::Start,
            PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.args_)>()
         }
        ,
        env_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::container::Start,
            PROTOBUF_FIELD_OFFSET(::gvisor::container::Start, _impl_.env_)>()
         }
        ,
        id_(arena),
        cwd_(arena) {}

inline void Start::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, terminal_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::terminal_));
}
Start::~Start() {
  // @@protoc_insertion_point(destructor:gvisor.container.Start)
  SharedDtor(*this);
}
inline void Start::SharedDtor(MessageLite& self) {
  Start& this_ = static_cast<Start&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.id_.Destroy();
  this_._impl_.cwd_.Destroy();
  delete this_._impl_.context_data_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Start_class_data_ =
        Start::InternalGenerateClassData_(Start_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Start::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Start_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Start_class_data_.tc_table);
  return Start_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Start::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Start_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Start_globals_));
  return Start_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Start::ParseTableT_
    Start::_table_ =
        Start::InternalGenerateParseTable_(Start_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Start::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.container.Start)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.args_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.env_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      this_._impl_.id_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      this_._impl_.cwd_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
  }
  this_._impl_.terminal_ = false;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Start::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Start& this_ = static_cast<const Start&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Start::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Start& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.container.Start)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // string id = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_id().empty()) {
      const ::std::string& _s = this_._internal_id();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.container.Start.id");
      target = stream->WriteStringMaybeAliased(2, _s, target);
    }
  }

  // string cwd = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_cwd().empty()) {
      const ::std::string& _s = this_._internal_cwd();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.container.Start.cwd");
      target = stream->WriteStringMaybeAliased(3, _s, target);
    }
  }

  // repeated string args = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_args_size(); i < n; ++i) {
      const auto& s = this_._internal_args().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.container.Start.args");
      target = stream->WriteString(4, s, target);
    }
  }

  // repeated string env = 5;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_env_size(); i < n; ++i) {
      const auto& s = this_._internal_env().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.container.Start.env");
      target = stream->WriteString(5, s, target);
    }
  }

  // bool terminal = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_terminal() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          6, this_._internal_terminal(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.container.Start)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Start::ByteSizeLong(const MessageLite& base) {
  const Start& this_ = static_cast<const Start&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Start::ByteSizeLong() const {
  const Start& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.container.Start)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // repeated string args = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_args().size());
      for (int i = 0, n = this_._internal_args().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_args().Get(i));
      }
    }
    // repeated string env = 5;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_env().size());
      for (int i = 0, n = this_._internal_env().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_env().Get(i));
      }
    }
    // string id = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_id().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_id());
      }
    }
    // string cwd = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_cwd().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_cwd());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // bool terminal = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_terminal() != 0) {
        total_size += 2;
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Start::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Start*>(&to_msg);
  auto& from = static_cast<const Start&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.container.Start)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_args()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_args());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_env()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_env());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_id().empty()) {
        _this->_internal_set_id(from._internal_id());
      } else {
        if (_this->_impl_.id_.IsDefault()) {
          _this->_internal_set_id("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_cwd().empty()) {
        _this->_internal_set_cwd(from._internal_cwd());
      } else {
        if (_this->_impl_.cwd_.IsDefault()) {
          _this->_internal_set_cwd("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_terminal() != 0) {
        _this->_impl_.terminal_ = from._impl_.terminal_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Start::CopyFrom(const Start& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.container.Start)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Start::InternalSwap(Start* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.args_.InternalSwap(&other->_impl_.args_);
  _impl_.env_.InternalSwap(&other->_impl_.env_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.id_, &other->_impl_.id_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.cwd_, &other->_impl_.cwd_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Start, _impl_.terminal_)
      + sizeof(Start::_impl_.terminal_)
      - PROTOBUF_FIELD_OFFSET(Start, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Start::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace container
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcontainer_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
