// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/syscall.proto
// Protobuf C++ Version: 7.36.1

#ifndef pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_2epb_2eh
#define pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_2epb_2eh

#include <limits>
#include <string>
#include <type_traits>
#include <utility>

// clang-format off
#include "google/protobuf/runtime_version.h"
#if PROTOBUF_VERSION != 7036001
#error "Protobuf C++ gencode is built with an incompatible version of"
#error "Protobuf C++ headers/runtime. See"
#error "https://protobuf.dev/support/cross-version-runtime-guarantee/#cpp"
#endif
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/arena.h"
#include "google/protobuf/arenastring.h"
#include "google/protobuf/generated_message_tctable_decl.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/metadata_lite.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/message.h"
#include "google/protobuf/message_lite.h"
#include "google/protobuf/repeated_field.h"  // IWYU pragma: export
#include "google/protobuf/extension_set.h"  // IWYU pragma: export
#include "google/protobuf/unknown_field_set.h"
#include "pkg/sentry/seccheck/points/common.pb.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"

#define PROTOBUF_INTERNAL_EXPORT_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto

// Internal implementation detail -- do not use these members.
struct TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto {
  static const ::uint32_t offsets[];
};
extern "C" {
extern const ::google::protobuf::internal::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
}  // extern "C"
namespace gvisor {
namespace syscall {
class Accept;
struct AcceptGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern AcceptGlobalsTypeInternal Accept_globals_;
extern const ::google::protobuf::internal::ClassDataFull Accept_class_data_;
#else
extern const AcceptGlobalsTypeInternal Accept_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Bind;
struct BindGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern BindGlobalsTypeInternal Bind_globals_;
extern const ::google::protobuf::internal::ClassDataFull Bind_class_data_;
#else
extern const BindGlobalsTypeInternal Bind_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Chdir;
struct ChdirGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ChdirGlobalsTypeInternal Chdir_globals_;
extern const ::google::protobuf::internal::ClassDataFull Chdir_class_data_;
#else
extern const ChdirGlobalsTypeInternal Chdir_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Chroot;
struct ChrootGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ChrootGlobalsTypeInternal Chroot_globals_;
extern const ::google::protobuf::internal::ClassDataFull Chroot_class_data_;
#else
extern const ChrootGlobalsTypeInternal Chroot_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Clone;
struct CloneGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern CloneGlobalsTypeInternal Clone_globals_;
extern const ::google::protobuf::internal::ClassDataFull Clone_class_data_;
#else
extern const CloneGlobalsTypeInternal Clone_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Close;
struct CloseGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern CloseGlobalsTypeInternal Close_globals_;
extern const ::google::protobuf::internal::ClassDataFull Close_class_data_;
#else
extern const CloseGlobalsTypeInternal Close_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Connect;
struct ConnectGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ConnectGlobalsTypeInternal Connect_globals_;
extern const ::google::protobuf::internal::ClassDataFull Connect_class_data_;
#else
extern const ConnectGlobalsTypeInternal Connect_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Dup;
struct DupGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern DupGlobalsTypeInternal Dup_globals_;
extern const ::google::protobuf::internal::ClassDataFull Dup_class_data_;
#else
extern const DupGlobalsTypeInternal Dup_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Eventfd;
struct EventfdGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern EventfdGlobalsTypeInternal Eventfd_globals_;
extern const ::google::protobuf::internal::ClassDataFull Eventfd_class_data_;
#else
extern const EventfdGlobalsTypeInternal Eventfd_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Execve;
struct ExecveGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExecveGlobalsTypeInternal Execve_globals_;
extern const ::google::protobuf::internal::ClassDataFull Execve_class_data_;
#else
extern const ExecveGlobalsTypeInternal Execve_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Exit;
struct ExitGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ExitGlobalsTypeInternal Exit_globals_;
extern const ::google::protobuf::internal::ClassDataFull Exit_class_data_;
#else
extern const ExitGlobalsTypeInternal Exit_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Fcntl;
struct FcntlGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern FcntlGlobalsTypeInternal Fcntl_globals_;
extern const ::google::protobuf::internal::ClassDataFull Fcntl_class_data_;
#else
extern const FcntlGlobalsTypeInternal Fcntl_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Fork;
struct ForkGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ForkGlobalsTypeInternal Fork_globals_;
extern const ::google::protobuf::internal::ClassDataFull Fork_class_data_;
#else
extern const ForkGlobalsTypeInternal Fork_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class InotifyAddWatch;
struct InotifyAddWatchGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern InotifyAddWatchGlobalsTypeInternal InotifyAddWatch_globals_;
extern const ::google::protobuf::internal::ClassDataFull InotifyAddWatch_class_data_;
#else
extern const InotifyAddWatchGlobalsTypeInternal InotifyAddWatch_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class InotifyInit;
struct InotifyInitGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern InotifyInitGlobalsTypeInternal InotifyInit_globals_;
extern const ::google::protobuf::internal::ClassDataFull InotifyInit_class_data_;
#else
extern const InotifyInitGlobalsTypeInternal InotifyInit_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class InotifyRmWatch;
struct InotifyRmWatchGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern InotifyRmWatchGlobalsTypeInternal InotifyRmWatch_globals_;
extern const ::google::protobuf::internal::ClassDataFull InotifyRmWatch_class_data_;
#else
extern const InotifyRmWatchGlobalsTypeInternal InotifyRmWatch_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class ItimerSpec;
struct ItimerSpecGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ItimerSpecGlobalsTypeInternal ItimerSpec_globals_;
extern const ::google::protobuf::internal::ClassDataFull ItimerSpec_class_data_;
#else
extern const ItimerSpecGlobalsTypeInternal ItimerSpec_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Open;
struct OpenGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern OpenGlobalsTypeInternal Open_globals_;
extern const ::google::protobuf::internal::ClassDataFull Open_class_data_;
#else
extern const OpenGlobalsTypeInternal Open_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Pipe;
struct PipeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern PipeGlobalsTypeInternal Pipe_globals_;
extern const ::google::protobuf::internal::ClassDataFull Pipe_class_data_;
#else
extern const PipeGlobalsTypeInternal Pipe_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Prlimit;
struct PrlimitGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern PrlimitGlobalsTypeInternal Prlimit_globals_;
extern const ::google::protobuf::internal::ClassDataFull Prlimit_class_data_;
#else
extern const PrlimitGlobalsTypeInternal Prlimit_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Read;
struct ReadGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern ReadGlobalsTypeInternal Read_globals_;
extern const ::google::protobuf::internal::ClassDataFull Read_class_data_;
#else
extern const ReadGlobalsTypeInternal Read_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Setid;
struct SetidGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SetidGlobalsTypeInternal Setid_globals_;
extern const ::google::protobuf::internal::ClassDataFull Setid_class_data_;
#else
extern const SetidGlobalsTypeInternal Setid_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Setresid;
struct SetresidGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SetresidGlobalsTypeInternal Setresid_globals_;
extern const ::google::protobuf::internal::ClassDataFull Setresid_class_data_;
#else
extern const SetresidGlobalsTypeInternal Setresid_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Signalfd;
struct SignalfdGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SignalfdGlobalsTypeInternal Signalfd_globals_;
extern const ::google::protobuf::internal::ClassDataFull Signalfd_class_data_;
#else
extern const SignalfdGlobalsTypeInternal Signalfd_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Socket;
struct SocketGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SocketGlobalsTypeInternal Socket_globals_;
extern const ::google::protobuf::internal::ClassDataFull Socket_class_data_;
#else
extern const SocketGlobalsTypeInternal Socket_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class SocketPair;
struct SocketPairGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SocketPairGlobalsTypeInternal SocketPair_globals_;
extern const ::google::protobuf::internal::ClassDataFull SocketPair_class_data_;
#else
extern const SocketPairGlobalsTypeInternal SocketPair_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class StructRlimit;
struct StructRlimitGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern StructRlimitGlobalsTypeInternal StructRlimit_globals_;
extern const ::google::protobuf::internal::ClassDataFull StructRlimit_class_data_;
#else
extern const StructRlimitGlobalsTypeInternal StructRlimit_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Syscall;
struct SyscallGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern SyscallGlobalsTypeInternal Syscall_globals_;
extern const ::google::protobuf::internal::ClassDataFull Syscall_class_data_;
#else
extern const SyscallGlobalsTypeInternal Syscall_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class TimerfdCreate;
struct TimerfdCreateGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TimerfdCreateGlobalsTypeInternal TimerfdCreate_globals_;
extern const ::google::protobuf::internal::ClassDataFull TimerfdCreate_class_data_;
#else
extern const TimerfdCreateGlobalsTypeInternal TimerfdCreate_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class TimerfdGetTime;
struct TimerfdGetTimeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TimerfdGetTimeGlobalsTypeInternal TimerfdGetTime_globals_;
extern const ::google::protobuf::internal::ClassDataFull TimerfdGetTime_class_data_;
#else
extern const TimerfdGetTimeGlobalsTypeInternal TimerfdGetTime_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class TimerfdSetTime;
struct TimerfdSetTimeGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TimerfdSetTimeGlobalsTypeInternal TimerfdSetTime_globals_;
extern const ::google::protobuf::internal::ClassDataFull TimerfdSetTime_class_data_;
#else
extern const TimerfdSetTimeGlobalsTypeInternal TimerfdSetTime_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Timespec;
struct TimespecGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern TimespecGlobalsTypeInternal Timespec_globals_;
extern const ::google::protobuf::internal::ClassDataFull Timespec_class_data_;
#else
extern const TimespecGlobalsTypeInternal Timespec_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
class Write;
struct WriteGlobalsTypeInternal;
#ifndef PROTOBUF_MESSAGE_GLOBALS
extern WriteGlobalsTypeInternal Write_globals_;
extern const ::google::protobuf::internal::ClassDataFull Write_class_data_;
#else
extern const WriteGlobalsTypeInternal Write_globals_;
#endif  // PROTOBUF_MESSAGE_GLOBALS
}  // namespace syscall
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google

namespace gvisor {
namespace syscall {

// ===================================================================


// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Timespec final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Timespec) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Timespec() : Timespec(nullptr) {}
  ~Timespec() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Timespec* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Timespec));
  }
#endif

  template <typename = void>
  explicit constexpr Timespec(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Timespec(const Timespec& from) : Timespec(nullptr, from) {}
  inline Timespec(Timespec&& from) noexcept : Timespec(nullptr, ::std::move(from)) {}
  inline Timespec& operator=(const Timespec& from) {
    CopyFrom(from);
    return *this;
  }
  inline Timespec& operator=(Timespec&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Timespec& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Timespec>(&Timespec_globals_);
  }
  static constexpr int kIndexInFileMessages = 24;
  friend void swap(Timespec& a, Timespec& b) { a.Swap(&b); }
  inline void Swap(Timespec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Timespec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Timespec* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Timespec>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Timespec& from);
  using Super_::MergeFrom;
  void MergeFrom(const Timespec& from) { Timespec::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Timespec* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Timespec"; }

  explicit Timespec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Timespec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Timespec& from);
  Timespec(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Timespec&& from) noexcept
      : Timespec(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kSecFieldNumber = 1,
    kNsecFieldNumber = 2,
  };
  // int64 sec = 1;
  void clear_sec() ;
  [[nodiscard]] ::int64_t sec() const;
  void set_sec(::int64_t value);

  private:
  ::int64_t _internal_sec() const;
  void _internal_set_sec(::int64_t value);

  public:
  // int64 nsec = 2;
  void clear_nsec() ;
  [[nodiscard]] ::int64_t nsec() const;
  void set_nsec(::int64_t value);

  private:
  ::int64_t _internal_nsec() const;
  void _internal_set_nsec(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Timespec)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Timespec& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t sec_;
    ::int64_t nsec_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED StructRlimit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.StructRlimit) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline StructRlimit() : StructRlimit(nullptr) {}
  ~StructRlimit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(StructRlimit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(StructRlimit));
  }
#endif

  template <typename = void>
  explicit constexpr StructRlimit(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline StructRlimit(const StructRlimit& from) : StructRlimit(nullptr, from) {}
  inline StructRlimit(StructRlimit&& from) noexcept : StructRlimit(nullptr, ::std::move(from)) {}
  inline StructRlimit& operator=(const StructRlimit& from) {
    CopyFrom(from);
    return *this;
  }
  inline StructRlimit& operator=(StructRlimit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const StructRlimit& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<StructRlimit>(&StructRlimit_globals_);
  }
  static constexpr int kIndexInFileMessages = 12;
  friend void swap(StructRlimit& a, StructRlimit& b) { a.Swap(&b); }
  inline void Swap(StructRlimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(StructRlimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] StructRlimit* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<StructRlimit>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const StructRlimit& from);
  using Super_::MergeFrom;
  void MergeFrom(const StructRlimit& from) { StructRlimit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(StructRlimit* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.StructRlimit"; }

  explicit StructRlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  StructRlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StructRlimit& from);
  StructRlimit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, StructRlimit&& from) noexcept
      : StructRlimit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kCurFieldNumber = 1,
    kMaxFieldNumber = 2,
  };
  // uint64 cur = 1;
  void clear_cur() ;
  [[nodiscard]] ::uint64_t cur() const;
  void set_cur(::uint64_t value);

  private:
  ::uint64_t _internal_cur() const;
  void _internal_set_cur(::uint64_t value);

  public:
  // uint64 max = 2;
  void clear_max() ;
  [[nodiscard]] ::uint64_t max() const;
  void set_max(::uint64_t value);

  private:
  ::uint64_t _internal_max() const;
  void _internal_set_max(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.StructRlimit)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const StructRlimit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::uint64_t cur_;
    ::uint64_t max_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Exit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Exit) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Exit() : Exit(nullptr) {}
  ~Exit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Exit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Exit));
  }
#endif

  template <typename = void>
  explicit constexpr Exit(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Exit(const Exit& from) : Exit(nullptr, from) {}
  inline Exit(Exit&& from) noexcept : Exit(nullptr, ::std::move(from)) {}
  inline Exit& operator=(const Exit& from) {
    CopyFrom(from);
    return *this;
  }
  inline Exit& operator=(Exit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Exit& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Exit>(&Exit_globals_);
  }
  static constexpr int kIndexInFileMessages = 0;
  friend void swap(Exit& a, Exit& b) { a.Swap(&b); }
  inline void Swap(Exit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Exit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Exit* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Exit>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Exit& from);
  using Super_::MergeFrom;
  void MergeFrom(const Exit& from) { Exit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Exit* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Exit"; }

  explicit Exit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Exit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Exit& from);
  Exit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Exit&& from) noexcept
      : Exit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kResultFieldNumber = 1,
    kErrornoFieldNumber = 2,
  };
  // int64 result = 1;
  void clear_result() ;
  [[nodiscard]] ::int64_t result() const;
  void set_result(::int64_t value);

  private:
  ::int64_t _internal_result() const;
  void _internal_set_result(::int64_t value);

  public:
  // int64 errorno = 2;
  void clear_errorno() ;
  [[nodiscard]] ::int64_t errorno() const;
  void set_errorno(::int64_t value);

  private:
  ::int64_t _internal_errorno() const;
  void _internal_set_errorno(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Exit)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          0, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Exit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::int64_t result_;
    ::int64_t errorno_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED ItimerSpec final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.ItimerSpec) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline ItimerSpec() : ItimerSpec(nullptr) {}
  ~ItimerSpec() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(ItimerSpec* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(ItimerSpec));
  }
#endif

  template <typename = void>
  explicit constexpr ItimerSpec(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline ItimerSpec(const ItimerSpec& from) : ItimerSpec(nullptr, from) {}
  inline ItimerSpec(ItimerSpec&& from) noexcept : ItimerSpec(nullptr, ::std::move(from)) {}
  inline ItimerSpec& operator=(const ItimerSpec& from) {
    CopyFrom(from);
    return *this;
  }
  inline ItimerSpec& operator=(ItimerSpec&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const ItimerSpec& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<ItimerSpec>(&ItimerSpec_globals_);
  }
  static constexpr int kIndexInFileMessages = 25;
  friend void swap(ItimerSpec& a, ItimerSpec& b) { a.Swap(&b); }
  inline void Swap(ItimerSpec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(ItimerSpec* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] ItimerSpec* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<ItimerSpec>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const ItimerSpec& from);
  using Super_::MergeFrom;
  void MergeFrom(const ItimerSpec& from) { ItimerSpec::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(ItimerSpec* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.ItimerSpec"; }

  explicit ItimerSpec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  ItimerSpec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const ItimerSpec& from);
  ItimerSpec(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, ItimerSpec&& from) noexcept
      : ItimerSpec(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kIntervalFieldNumber = 1,
    kValueFieldNumber = 2,
  };
  // .gvisor.syscall.Timespec interval = 1;
  [[nodiscard]] bool has_interval() const;
  void clear_interval() ;
  [[nodiscard]] const ::gvisor::syscall::Timespec& interval() const;
  [[nodiscard]] ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE release_interval();
  ::gvisor::syscall::Timespec* PROTOBUF_NONNULL mutable_interval();
  void set_allocated_interval(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_interval(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE unsafe_arena_release_interval();

  private:
  const ::gvisor::syscall::Timespec& _internal_interval() const;
  ::gvisor::syscall::Timespec* PROTOBUF_NONNULL _internal_mutable_interval();

  public:
  // .gvisor.syscall.Timespec value = 2;
  [[nodiscard]] bool has_value() const;
  void clear_value() ;
  [[nodiscard]] const ::gvisor::syscall::Timespec& value() const;
  [[nodiscard]] ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE release_value();
  ::gvisor::syscall::Timespec* PROTOBUF_NONNULL mutable_value();
  void set_allocated_value(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_value(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE unsafe_arena_release_value();

  private:
  const ::gvisor::syscall::Timespec& _internal_value() const;
  ::gvisor::syscall::Timespec* PROTOBUF_NONNULL _internal_mutable_value();

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.ItimerSpec)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<1, 2,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const ItimerSpec& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE interval_;
    ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Write final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Write) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Write() : Write(nullptr) {}
  ~Write() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Write* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Write));
  }
#endif

  template <typename = void>
  explicit constexpr Write(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Write(const Write& from) : Write(nullptr, from) {}
  inline Write(Write&& from) noexcept : Write(nullptr, ::std::move(from)) {}
  inline Write& operator=(const Write& from) {
    CopyFrom(from);
    return *this;
  }
  inline Write& operator=(Write&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Write& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Write>(&Write_globals_);
  }
  static constexpr int kIndexInFileMessages = 5;
  friend void swap(Write& a, Write& b) { a.Swap(&b); }
  inline void Swap(Write* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Write* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Write* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Write>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Write& from);
  using Super_::MergeFrom;
  void MergeFrom(const Write& from) { Write::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Write* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Write"; }

  explicit Write(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Write(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Write& from);
  Write(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Write&& from) noexcept
      : Write(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kCountFieldNumber = 6,
    kOffsetFieldNumber = 8,
    kHasOffsetFieldNumber = 7,
    kFlagsFieldNumber = 9,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // uint64 count = 6;
  void clear_count() ;
  [[nodiscard]] ::uint64_t count() const;
  void set_count(::uint64_t value);

  private:
  ::uint64_t _internal_count() const;
  void _internal_set_count(::uint64_t value);

  public:
  // int64 offset = 8;
  void clear_offset() ;
  [[nodiscard]] ::int64_t offset() const;
  void set_offset(::int64_t value);

  private:
  ::int64_t _internal_offset() const;
  void _internal_set_offset(::int64_t value);

  public:
  // bool has_offset = 7;
  void clear_has_offset() ;
  [[nodiscard]] bool has_offset() const;
  void set_has_offset(bool value);

  private:
  bool _internal_has_offset() const;
  void _internal_set_has_offset(bool value);

  public:
  // uint32 flags = 9;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Write)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          2, 44,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Write& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    ::uint64_t count_;
    ::int64_t offset_;
    bool has_offset_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TimerfdSetTime final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.TimerfdSetTime) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline TimerfdSetTime() : TimerfdSetTime(nullptr) {}
  ~TimerfdSetTime() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TimerfdSetTime* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TimerfdSetTime));
  }
#endif

  template <typename = void>
  explicit constexpr TimerfdSetTime(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TimerfdSetTime(const TimerfdSetTime& from) : TimerfdSetTime(nullptr, from) {}
  inline TimerfdSetTime(TimerfdSetTime&& from) noexcept : TimerfdSetTime(nullptr, ::std::move(from)) {}
  inline TimerfdSetTime& operator=(const TimerfdSetTime& from) {
    CopyFrom(from);
    return *this;
  }
  inline TimerfdSetTime& operator=(TimerfdSetTime&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TimerfdSetTime& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TimerfdSetTime>(&TimerfdSetTime_globals_);
  }
  static constexpr int kIndexInFileMessages = 26;
  friend void swap(TimerfdSetTime& a, TimerfdSetTime& b) { a.Swap(&b); }
  inline void Swap(TimerfdSetTime* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TimerfdSetTime* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TimerfdSetTime* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<TimerfdSetTime>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const TimerfdSetTime& from);
  using Super_::MergeFrom;
  void MergeFrom(const TimerfdSetTime& from) { TimerfdSetTime::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TimerfdSetTime* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.TimerfdSetTime"; }

  explicit TimerfdSetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TimerfdSetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TimerfdSetTime& from);
  TimerfdSetTime(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TimerfdSetTime&& from) noexcept
      : TimerfdSetTime(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kNewValueFieldNumber = 7,
    kOldValueFieldNumber = 8,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kFlagsFieldNumber = 6,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // .gvisor.syscall.ItimerSpec new_value = 7;
  [[nodiscard]] bool has_new_value() const;
  void clear_new_value() ;
  [[nodiscard]] const ::gvisor::syscall::ItimerSpec& new_value() const;
  [[nodiscard]] ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE release_new_value();
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL mutable_new_value();
  void set_allocated_new_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_new_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE unsafe_arena_release_new_value();

  private:
  const ::gvisor::syscall::ItimerSpec& _internal_new_value() const;
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL _internal_mutable_new_value();

  public:
  // .gvisor.syscall.ItimerSpec old_value = 8;
  [[nodiscard]] bool has_old_value() const;
  void clear_old_value() ;
  [[nodiscard]] const ::gvisor::syscall::ItimerSpec& old_value() const;
  [[nodiscard]] ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE release_old_value();
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL mutable_old_value();
  void set_allocated_old_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_old_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE unsafe_arena_release_old_value();

  private:
  const ::gvisor::syscall::ItimerSpec& _internal_old_value() const;
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL _internal_mutable_old_value();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // int32 flags = 6;
  void clear_flags() ;
  [[nodiscard]] ::int32_t flags() const;
  void set_flags(::int32_t value);

  private:
  ::int32_t _internal_flags() const;
  void _internal_set_flags(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.TimerfdSetTime)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 8,
                          4, 53,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TimerfdSetTime& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE new_value_;
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE old_value_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::int32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TimerfdGetTime final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.TimerfdGetTime) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline TimerfdGetTime() : TimerfdGetTime(nullptr) {}
  ~TimerfdGetTime() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TimerfdGetTime* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TimerfdGetTime));
  }
#endif

  template <typename = void>
  explicit constexpr TimerfdGetTime(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TimerfdGetTime(const TimerfdGetTime& from) : TimerfdGetTime(nullptr, from) {}
  inline TimerfdGetTime(TimerfdGetTime&& from) noexcept : TimerfdGetTime(nullptr, ::std::move(from)) {}
  inline TimerfdGetTime& operator=(const TimerfdGetTime& from) {
    CopyFrom(from);
    return *this;
  }
  inline TimerfdGetTime& operator=(TimerfdGetTime&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TimerfdGetTime& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TimerfdGetTime>(&TimerfdGetTime_globals_);
  }
  static constexpr int kIndexInFileMessages = 27;
  friend void swap(TimerfdGetTime& a, TimerfdGetTime& b) { a.Swap(&b); }
  inline void Swap(TimerfdGetTime* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TimerfdGetTime* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TimerfdGetTime* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<TimerfdGetTime>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const TimerfdGetTime& from);
  using Super_::MergeFrom;
  void MergeFrom(const TimerfdGetTime& from) { TimerfdGetTime::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TimerfdGetTime* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.TimerfdGetTime"; }

  explicit TimerfdGetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TimerfdGetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TimerfdGetTime& from);
  TimerfdGetTime(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TimerfdGetTime&& from) noexcept
      : TimerfdGetTime(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kCurValueFieldNumber = 6,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // .gvisor.syscall.ItimerSpec cur_value = 6;
  [[nodiscard]] bool has_cur_value() const;
  void clear_cur_value() ;
  [[nodiscard]] const ::gvisor::syscall::ItimerSpec& cur_value() const;
  [[nodiscard]] ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE release_cur_value();
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL mutable_cur_value();
  void set_allocated_cur_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_cur_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE unsafe_arena_release_cur_value();

  private:
  const ::gvisor::syscall::ItimerSpec& _internal_cur_value() const;
  ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL _internal_mutable_cur_value();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.TimerfdGetTime)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          3, 45,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TimerfdGetTime& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE cur_value_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED TimerfdCreate final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.TimerfdCreate) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline TimerfdCreate() : TimerfdCreate(nullptr) {}
  ~TimerfdCreate() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(TimerfdCreate* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(TimerfdCreate));
  }
#endif

  template <typename = void>
  explicit constexpr TimerfdCreate(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline TimerfdCreate(const TimerfdCreate& from) : TimerfdCreate(nullptr, from) {}
  inline TimerfdCreate(TimerfdCreate&& from) noexcept : TimerfdCreate(nullptr, ::std::move(from)) {}
  inline TimerfdCreate& operator=(const TimerfdCreate& from) {
    CopyFrom(from);
    return *this;
  }
  inline TimerfdCreate& operator=(TimerfdCreate&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const TimerfdCreate& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<TimerfdCreate>(&TimerfdCreate_globals_);
  }
  static constexpr int kIndexInFileMessages = 23;
  friend void swap(TimerfdCreate& a, TimerfdCreate& b) { a.Swap(&b); }
  inline void Swap(TimerfdCreate* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(TimerfdCreate* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] TimerfdCreate* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<TimerfdCreate>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const TimerfdCreate& from);
  using Super_::MergeFrom;
  void MergeFrom(const TimerfdCreate& from) { TimerfdCreate::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(TimerfdCreate* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.TimerfdCreate"; }

  explicit TimerfdCreate(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  TimerfdCreate(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const TimerfdCreate& from);
  TimerfdCreate(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, TimerfdCreate&& from) noexcept
      : TimerfdCreate(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kClockIdFieldNumber = 4,
    kFlagsFieldNumber = 5,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 clock_id = 4;
  void clear_clock_id() ;
  [[nodiscard]] ::int32_t clock_id() const;
  void set_clock_id(::int32_t value);

  private:
  ::int32_t _internal_clock_id() const;
  void _internal_set_clock_id(::int32_t value);

  public:
  // int32 flags = 5;
  void clear_flags() ;
  [[nodiscard]] ::int32_t flags() const;
  void set_flags(::int32_t value);

  private:
  ::int32_t _internal_flags() const;
  void _internal_set_flags(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.TimerfdCreate)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const TimerfdCreate& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t clock_id_;
    ::int32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Syscall final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Syscall) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Syscall() : Syscall(nullptr) {}
  ~Syscall() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Syscall* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Syscall));
  }
#endif

  template <typename = void>
  explicit constexpr Syscall(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Syscall(const Syscall& from) : Syscall(nullptr, from) {}
  inline Syscall(Syscall&& from) noexcept : Syscall(nullptr, ::std::move(from)) {}
  inline Syscall& operator=(const Syscall& from) {
    CopyFrom(from);
    return *this;
  }
  inline Syscall& operator=(Syscall&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Syscall& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Syscall>(&Syscall_globals_);
  }
  static constexpr int kIndexInFileMessages = 1;
  friend void swap(Syscall& a, Syscall& b) { a.Swap(&b); }
  inline void Swap(Syscall* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Syscall* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Syscall* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Syscall>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Syscall& from);
  using Super_::MergeFrom;
  void MergeFrom(const Syscall& from) { Syscall::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Syscall* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Syscall"; }

  explicit Syscall(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Syscall(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Syscall& from);
  Syscall(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Syscall&& from) noexcept
      : Syscall(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 4,
    kArg1FieldNumber = 5,
    kArg2FieldNumber = 6,
    kArg3FieldNumber = 7,
    kArg4FieldNumber = 8,
    kArg5FieldNumber = 9,
    kArg6FieldNumber = 10,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 4;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // uint64 arg1 = 5;
  void clear_arg1() ;
  [[nodiscard]] ::uint64_t arg1() const;
  void set_arg1(::uint64_t value);

  private:
  ::uint64_t _internal_arg1() const;
  void _internal_set_arg1(::uint64_t value);

  public:
  // uint64 arg2 = 6;
  void clear_arg2() ;
  [[nodiscard]] ::uint64_t arg2() const;
  void set_arg2(::uint64_t value);

  private:
  ::uint64_t _internal_arg2() const;
  void _internal_set_arg2(::uint64_t value);

  public:
  // uint64 arg3 = 7;
  void clear_arg3() ;
  [[nodiscard]] ::uint64_t arg3() const;
  void set_arg3(::uint64_t value);

  private:
  ::uint64_t _internal_arg3() const;
  void _internal_set_arg3(::uint64_t value);

  public:
  // uint64 arg4 = 8;
  void clear_arg4() ;
  [[nodiscard]] ::uint64_t arg4() const;
  void set_arg4(::uint64_t value);

  private:
  ::uint64_t _internal_arg4() const;
  void _internal_set_arg4(::uint64_t value);

  public:
  // uint64 arg5 = 9;
  void clear_arg5() ;
  [[nodiscard]] ::uint64_t arg5() const;
  void set_arg5(::uint64_t value);

  private:
  ::uint64_t _internal_arg5() const;
  void _internal_set_arg5(::uint64_t value);

  public:
  // uint64 arg6 = 10;
  void clear_arg6() ;
  [[nodiscard]] ::uint64_t arg6() const;
  void set_arg6(::uint64_t value);

  private:
  ::uint64_t _internal_arg6() const;
  void _internal_set_arg6(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Syscall)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Syscall& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::uint64_t arg1_;
    ::uint64_t arg2_;
    ::uint64_t arg3_;
    ::uint64_t arg4_;
    ::uint64_t arg5_;
    ::uint64_t arg6_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED SocketPair final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.SocketPair) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline SocketPair() : SocketPair(nullptr) {}
  ~SocketPair() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(SocketPair* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(SocketPair));
  }
#endif

  template <typename = void>
  explicit constexpr SocketPair(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline SocketPair(const SocketPair& from) : SocketPair(nullptr, from) {}
  inline SocketPair(SocketPair&& from) noexcept : SocketPair(nullptr, ::std::move(from)) {}
  inline SocketPair& operator=(const SocketPair& from) {
    CopyFrom(from);
    return *this;
  }
  inline SocketPair& operator=(SocketPair&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const SocketPair& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<SocketPair>(&SocketPair_globals_);
  }
  static constexpr int kIndexInFileMessages = 32;
  friend void swap(SocketPair& a, SocketPair& b) { a.Swap(&b); }
  inline void Swap(SocketPair* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(SocketPair* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] SocketPair* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<SocketPair>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const SocketPair& from);
  using Super_::MergeFrom;
  void MergeFrom(const SocketPair& from) { SocketPair::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(SocketPair* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.SocketPair"; }

  explicit SocketPair(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  SocketPair(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const SocketPair& from);
  SocketPair(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, SocketPair&& from) noexcept
      : SocketPair(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kDomainFieldNumber = 4,
    kTypeFieldNumber = 5,
    kProtocolFieldNumber = 6,
    kSocket1FieldNumber = 7,
    kSocket2FieldNumber = 8,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 domain = 4;
  void clear_domain() ;
  [[nodiscard]] ::int32_t domain() const;
  void set_domain(::int32_t value);

  private:
  ::int32_t _internal_domain() const;
  void _internal_set_domain(::int32_t value);

  public:
  // int32 type = 5;
  void clear_type() ;
  [[nodiscard]] ::int32_t type() const;
  void set_type(::int32_t value);

  private:
  ::int32_t _internal_type() const;
  void _internal_set_type(::int32_t value);

  public:
  // int32 protocol = 6;
  void clear_protocol() ;
  [[nodiscard]] ::int32_t protocol() const;
  void set_protocol(::int32_t value);

  private:
  ::int32_t _internal_protocol() const;
  void _internal_set_protocol(::int32_t value);

  public:
  // int32 socket1 = 7;
  void clear_socket1() ;
  [[nodiscard]] ::int32_t socket1() const;
  void set_socket1(::int32_t value);

  private:
  ::int32_t _internal_socket1() const;
  void _internal_set_socket1(::int32_t value);

  public:
  // int32 socket2 = 8;
  void clear_socket2() ;
  [[nodiscard]] ::int32_t socket2() const;
  void set_socket2(::int32_t value);

  private:
  ::int32_t _internal_socket2() const;
  void _internal_set_socket2(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.SocketPair)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 8,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const SocketPair& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t domain_;
    ::int32_t type_;
    ::int32_t protocol_;
    ::int32_t socket1_;
    ::int32_t socket2_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Socket final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Socket) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Socket() : Socket(nullptr) {}
  ~Socket() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Socket* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Socket));
  }
#endif

  template <typename = void>
  explicit constexpr Socket(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Socket(const Socket& from) : Socket(nullptr, from) {}
  inline Socket(Socket&& from) noexcept : Socket(nullptr, ::std::move(from)) {}
  inline Socket& operator=(const Socket& from) {
    CopyFrom(from);
    return *this;
  }
  inline Socket& operator=(Socket&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Socket& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Socket>(&Socket_globals_);
  }
  static constexpr int kIndexInFileMessages = 8;
  friend void swap(Socket& a, Socket& b) { a.Swap(&b); }
  inline void Swap(Socket* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Socket* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Socket* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Socket>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Socket& from);
  using Super_::MergeFrom;
  void MergeFrom(const Socket& from) { Socket::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Socket* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Socket"; }

  explicit Socket(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Socket(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Socket& from);
  Socket(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Socket&& from) noexcept
      : Socket(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kDomainFieldNumber = 4,
    kTypeFieldNumber = 5,
    kProtocolFieldNumber = 6,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 domain = 4;
  void clear_domain() ;
  [[nodiscard]] ::int32_t domain() const;
  void set_domain(::int32_t value);

  private:
  ::int32_t _internal_domain() const;
  void _internal_set_domain(::int32_t value);

  public:
  // int32 type = 5;
  void clear_type() ;
  [[nodiscard]] ::int32_t type() const;
  void set_type(::int32_t value);

  private:
  ::int32_t _internal_type() const;
  void _internal_set_type(::int32_t value);

  public:
  // int32 protocol = 6;
  void clear_protocol() ;
  [[nodiscard]] ::int32_t protocol() const;
  void set_protocol(::int32_t value);

  private:
  ::int32_t _internal_protocol() const;
  void _internal_set_protocol(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Socket)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Socket& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t domain_;
    ::int32_t type_;
    ::int32_t protocol_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Signalfd final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Signalfd) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Signalfd() : Signalfd(nullptr) {}
  ~Signalfd() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Signalfd* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Signalfd));
  }
#endif

  template <typename = void>
  explicit constexpr Signalfd(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Signalfd(const Signalfd& from) : Signalfd(nullptr, from) {}
  inline Signalfd(Signalfd&& from) noexcept : Signalfd(nullptr, ::std::move(from)) {}
  inline Signalfd& operator=(const Signalfd& from) {
    CopyFrom(from);
    return *this;
  }
  inline Signalfd& operator=(Signalfd&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Signalfd& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Signalfd>(&Signalfd_globals_);
  }
  static constexpr int kIndexInFileMessages = 17;
  friend void swap(Signalfd& a, Signalfd& b) { a.Swap(&b); }
  inline void Swap(Signalfd* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Signalfd* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Signalfd* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Signalfd>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Signalfd& from);
  using Super_::MergeFrom;
  void MergeFrom(const Signalfd& from) { Signalfd::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Signalfd* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Signalfd"; }

  explicit Signalfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Signalfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Signalfd& from);
  Signalfd(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Signalfd&& from) noexcept
      : Signalfd(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kFlagsFieldNumber = 7,
    kSigsetFieldNumber = 6,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // int32 flags = 7;
  void clear_flags() ;
  [[nodiscard]] ::int32_t flags() const;
  void set_flags(::int32_t value);

  private:
  ::int32_t _internal_flags() const;
  void _internal_set_flags(::int32_t value);

  public:
  // uint64 sigset = 6;
  void clear_sigset() ;
  [[nodiscard]] ::uint64_t sigset() const;
  void set_sigset(::uint64_t value);

  private:
  ::uint64_t _internal_sigset() const;
  void _internal_set_sigset(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Signalfd)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 39,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Signalfd& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::int32_t flags_;
    ::uint64_t sigset_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Setresid final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Setresid) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Setresid() : Setresid(nullptr) {}
  ~Setresid() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Setresid* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Setresid));
  }
#endif

  template <typename = void>
  explicit constexpr Setresid(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Setresid(const Setresid& from) : Setresid(nullptr, from) {}
  inline Setresid(Setresid&& from) noexcept : Setresid(nullptr, ::std::move(from)) {}
  inline Setresid& operator=(const Setresid& from) {
    CopyFrom(from);
    return *this;
  }
  inline Setresid& operator=(Setresid&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Setresid& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Setresid>(&Setresid_globals_);
  }
  static constexpr int kIndexInFileMessages = 10;
  friend void swap(Setresid& a, Setresid& b) { a.Swap(&b); }
  inline void Swap(Setresid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Setresid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Setresid* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Setresid>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Setresid& from);
  using Super_::MergeFrom;
  void MergeFrom(const Setresid& from) { Setresid::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Setresid* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Setresid"; }

  explicit Setresid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Setresid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Setresid& from);
  Setresid(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Setresid&& from) noexcept
      : Setresid(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kRidFieldNumber = 4,
    kEidFieldNumber = 5,
    kSidFieldNumber = 6,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // uint32 rid = 4;
  void clear_rid() ;
  [[nodiscard]] ::uint32_t rid() const;
  void set_rid(::uint32_t value);

  private:
  ::uint32_t _internal_rid() const;
  void _internal_set_rid(::uint32_t value);

  public:
  // uint32 eid = 5;
  void clear_eid() ;
  [[nodiscard]] ::uint32_t eid() const;
  void set_eid(::uint32_t value);

  private:
  ::uint32_t _internal_eid() const;
  void _internal_set_eid(::uint32_t value);

  public:
  // uint32 sid = 6;
  void clear_sid() ;
  [[nodiscard]] ::uint32_t sid() const;
  void set_sid(::uint32_t value);

  private:
  ::uint32_t _internal_sid() const;
  void _internal_set_sid(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Setresid)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Setresid& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::uint32_t rid_;
    ::uint32_t eid_;
    ::uint32_t sid_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Setid final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Setid) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Setid() : Setid(nullptr) {}
  ~Setid() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Setid* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Setid));
  }
#endif

  template <typename = void>
  explicit constexpr Setid(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Setid(const Setid& from) : Setid(nullptr, from) {}
  inline Setid(Setid&& from) noexcept : Setid(nullptr, ::std::move(from)) {}
  inline Setid& operator=(const Setid& from) {
    CopyFrom(from);
    return *this;
  }
  inline Setid& operator=(Setid&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Setid& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Setid>(&Setid_globals_);
  }
  static constexpr int kIndexInFileMessages = 11;
  friend void swap(Setid& a, Setid& b) { a.Swap(&b); }
  inline void Swap(Setid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Setid* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Setid* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Setid>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Setid& from);
  using Super_::MergeFrom;
  void MergeFrom(const Setid& from) { Setid::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Setid* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Setid"; }

  explicit Setid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Setid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Setid& from);
  Setid(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Setid&& from) noexcept
      : Setid(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kIdFieldNumber = 4,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // uint32 id = 4;
  void clear_id() ;
  [[nodiscard]] ::uint32_t id() const;
  void set_id(::uint32_t value);

  private:
  ::uint32_t _internal_id() const;
  void _internal_set_id(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Setid)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Setid& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::uint32_t id_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Read final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Read) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Read() : Read(nullptr) {}
  ~Read() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Read* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Read));
  }
#endif

  template <typename = void>
  explicit constexpr Read(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Read(const Read& from) : Read(nullptr, from) {}
  inline Read(Read&& from) noexcept : Read(nullptr, ::std::move(from)) {}
  inline Read& operator=(const Read& from) {
    CopyFrom(from);
    return *this;
  }
  inline Read& operator=(Read&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Read& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Read>(&Read_globals_);
  }
  static constexpr int kIndexInFileMessages = 4;
  friend void swap(Read& a, Read& b) { a.Swap(&b); }
  inline void Swap(Read* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Read* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Read* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Read>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Read& from);
  using Super_::MergeFrom;
  void MergeFrom(const Read& from) { Read::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Read* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Read"; }

  explicit Read(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Read(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Read& from);
  Read(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Read&& from) noexcept
      : Read(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kCountFieldNumber = 6,
    kOffsetFieldNumber = 8,
    kHasOffsetFieldNumber = 7,
    kFlagsFieldNumber = 9,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // uint64 count = 6;
  void clear_count() ;
  [[nodiscard]] ::uint64_t count() const;
  void set_count(::uint64_t value);

  private:
  ::uint64_t _internal_count() const;
  void _internal_set_count(::uint64_t value);

  public:
  // int64 offset = 8;
  void clear_offset() ;
  [[nodiscard]] ::int64_t offset() const;
  void set_offset(::int64_t value);

  private:
  ::int64_t _internal_offset() const;
  void _internal_set_offset(::int64_t value);

  public:
  // bool has_offset = 7;
  void clear_has_offset() ;
  [[nodiscard]] bool has_offset() const;
  void set_has_offset(bool value);

  private:
  bool _internal_has_offset() const;
  void _internal_set_has_offset(bool value);

  public:
  // uint32 flags = 9;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Read)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          2, 43,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Read& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    ::uint64_t count_;
    ::int64_t offset_;
    bool has_offset_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Prlimit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Prlimit) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Prlimit() : Prlimit(nullptr) {}
  ~Prlimit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Prlimit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Prlimit));
  }
#endif

  template <typename = void>
  explicit constexpr Prlimit(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Prlimit(const Prlimit& from) : Prlimit(nullptr, from) {}
  inline Prlimit(Prlimit&& from) noexcept : Prlimit(nullptr, ::std::move(from)) {}
  inline Prlimit& operator=(const Prlimit& from) {
    CopyFrom(from);
    return *this;
  }
  inline Prlimit& operator=(Prlimit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Prlimit& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Prlimit>(&Prlimit_globals_);
  }
  static constexpr int kIndexInFileMessages = 13;
  friend void swap(Prlimit& a, Prlimit& b) { a.Swap(&b); }
  inline void Swap(Prlimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Prlimit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Prlimit* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Prlimit>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Prlimit& from);
  using Super_::MergeFrom;
  void MergeFrom(const Prlimit& from) { Prlimit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Prlimit* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Prlimit"; }

  explicit Prlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Prlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Prlimit& from);
  Prlimit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Prlimit&& from) noexcept
      : Prlimit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kNewLimitFieldNumber = 6,
    kOldLimitFieldNumber = 7,
    kSysnoFieldNumber = 3,
    kResourceFieldNumber = 5,
    kPidFieldNumber = 4,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // .gvisor.syscall.StructRlimit new_limit = 6;
  [[nodiscard]] bool has_new_limit() const;
  void clear_new_limit() ;
  [[nodiscard]] const ::gvisor::syscall::StructRlimit& new_limit() const;
  [[nodiscard]] ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE release_new_limit();
  ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL mutable_new_limit();
  void set_allocated_new_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_new_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE unsafe_arena_release_new_limit();

  private:
  const ::gvisor::syscall::StructRlimit& _internal_new_limit() const;
  ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL _internal_mutable_new_limit();

  public:
  // .gvisor.syscall.StructRlimit old_limit = 7;
  [[nodiscard]] bool has_old_limit() const;
  void clear_old_limit() ;
  [[nodiscard]] const ::gvisor::syscall::StructRlimit& old_limit() const;
  [[nodiscard]] ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE release_old_limit();
  ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL mutable_old_limit();
  void set_allocated_old_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_old_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE unsafe_arena_release_old_limit();

  private:
  const ::gvisor::syscall::StructRlimit& _internal_old_limit() const;
  ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL _internal_mutable_old_limit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 resource = 5;
  void clear_resource() ;
  [[nodiscard]] ::int64_t resource() const;
  void set_resource(::int64_t value);

  private:
  ::int64_t _internal_resource() const;
  void _internal_set_resource(::int64_t value);

  public:
  // int32 pid = 4;
  void clear_pid() ;
  [[nodiscard]] ::int32_t pid() const;
  void set_pid(::int32_t value);

  private:
  ::int32_t _internal_pid() const;
  void _internal_set_pid(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Prlimit)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          4, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Prlimit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE new_limit_;
    ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE old_limit_;
    ::uint64_t sysno_;
    ::int64_t resource_;
    ::int32_t pid_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Pipe final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Pipe) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Pipe() : Pipe(nullptr) {}
  ~Pipe() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Pipe* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Pipe));
  }
#endif

  template <typename = void>
  explicit constexpr Pipe(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Pipe(const Pipe& from) : Pipe(nullptr, from) {}
  inline Pipe(Pipe&& from) noexcept : Pipe(nullptr, ::std::move(from)) {}
  inline Pipe& operator=(const Pipe& from) {
    CopyFrom(from);
    return *this;
  }
  inline Pipe& operator=(Pipe&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Pipe& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Pipe>(&Pipe_globals_);
  }
  static constexpr int kIndexInFileMessages = 14;
  friend void swap(Pipe& a, Pipe& b) { a.Swap(&b); }
  inline void Swap(Pipe* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Pipe* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Pipe* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Pipe>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Pipe& from);
  using Super_::MergeFrom;
  void MergeFrom(const Pipe& from) { Pipe::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Pipe* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Pipe"; }

  explicit Pipe(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Pipe(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Pipe& from);
  Pipe(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Pipe&& from) noexcept
      : Pipe(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFlagsFieldNumber = 4,
    kReaderFieldNumber = 5,
    kWriterFieldNumber = 6,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // uint32 flags = 4;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // int32 reader = 5;
  void clear_reader() ;
  [[nodiscard]] ::int32_t reader() const;
  void set_reader(::int32_t value);

  private:
  ::int32_t _internal_reader() const;
  void _internal_set_reader(::int32_t value);

  public:
  // int32 writer = 6;
  void clear_writer() ;
  [[nodiscard]] ::int32_t writer() const;
  void set_writer(::int32_t value);

  private:
  ::int32_t _internal_writer() const;
  void _internal_set_writer(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Pipe)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Pipe& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::uint32_t flags_;
    ::int32_t reader_;
    ::int32_t writer_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Open final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Open) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Open() : Open(nullptr) {}
  ~Open() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Open* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Open));
  }
#endif

  template <typename = void>
  explicit constexpr Open(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Open(const Open& from) : Open(nullptr, from) {}
  inline Open(Open&& from) noexcept : Open(nullptr, ::std::move(from)) {}
  inline Open& operator=(const Open& from) {
    CopyFrom(from);
    return *this;
  }
  inline Open& operator=(Open&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Open& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Open>(&Open_globals_);
  }
  static constexpr int kIndexInFileMessages = 2;
  friend void swap(Open& a, Open& b) { a.Swap(&b); }
  inline void Swap(Open* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Open* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Open* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Open>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Open& from);
  using Super_::MergeFrom;
  void MergeFrom(const Open& from) { Open::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Open* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Open"; }

  explicit Open(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Open(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Open& from);
  Open(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Open&& from) noexcept
      : Open(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kPathnameFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kFlagsFieldNumber = 7,
    kModeFieldNumber = 8,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // string pathname = 6;
  void clear_pathname() ;
  [[nodiscard]] const ::std::string& pathname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pathname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pathname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pathname();
  void set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pathname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pathname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pathname();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // uint32 flags = 7;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // uint32 mode = 8;
  void clear_mode() ;
  [[nodiscard]] ::uint32_t mode() const;
  void set_mode(::uint32_t value);

  private:
  ::uint32_t _internal_mode() const;
  void _internal_set_mode(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Open)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 8,
                          2, 51,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Open& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr pathname_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    ::uint32_t flags_;
    ::uint32_t mode_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED InotifyRmWatch final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.InotifyRmWatch) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline InotifyRmWatch() : InotifyRmWatch(nullptr) {}
  ~InotifyRmWatch() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(InotifyRmWatch* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(InotifyRmWatch));
  }
#endif

  template <typename = void>
  explicit constexpr InotifyRmWatch(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline InotifyRmWatch(const InotifyRmWatch& from) : InotifyRmWatch(nullptr, from) {}
  inline InotifyRmWatch(InotifyRmWatch&& from) noexcept : InotifyRmWatch(nullptr, ::std::move(from)) {}
  inline InotifyRmWatch& operator=(const InotifyRmWatch& from) {
    CopyFrom(from);
    return *this;
  }
  inline InotifyRmWatch& operator=(InotifyRmWatch&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const InotifyRmWatch& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<InotifyRmWatch>(&InotifyRmWatch_globals_);
  }
  static constexpr int kIndexInFileMessages = 31;
  friend void swap(InotifyRmWatch& a, InotifyRmWatch& b) { a.Swap(&b); }
  inline void Swap(InotifyRmWatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(InotifyRmWatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] InotifyRmWatch* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<InotifyRmWatch>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const InotifyRmWatch& from);
  using Super_::MergeFrom;
  void MergeFrom(const InotifyRmWatch& from) { InotifyRmWatch::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(InotifyRmWatch* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.InotifyRmWatch"; }

  explicit InotifyRmWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  InotifyRmWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const InotifyRmWatch& from);
  InotifyRmWatch(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, InotifyRmWatch&& from) noexcept
      : InotifyRmWatch(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kWdFieldNumber = 6,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // int32 wd = 6;
  void clear_wd() ;
  [[nodiscard]] ::int32_t wd() const;
  void set_wd(::int32_t value);

  private:
  ::int32_t _internal_wd() const;
  void _internal_set_wd(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.InotifyRmWatch)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 45,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const InotifyRmWatch& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::int32_t wd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED InotifyInit final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.InotifyInit) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline InotifyInit() : InotifyInit(nullptr) {}
  ~InotifyInit() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(InotifyInit* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(InotifyInit));
  }
#endif

  template <typename = void>
  explicit constexpr InotifyInit(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline InotifyInit(const InotifyInit& from) : InotifyInit(nullptr, from) {}
  inline InotifyInit(InotifyInit&& from) noexcept : InotifyInit(nullptr, ::std::move(from)) {}
  inline InotifyInit& operator=(const InotifyInit& from) {
    CopyFrom(from);
    return *this;
  }
  inline InotifyInit& operator=(InotifyInit&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const InotifyInit& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<InotifyInit>(&InotifyInit_globals_);
  }
  static constexpr int kIndexInFileMessages = 29;
  friend void swap(InotifyInit& a, InotifyInit& b) { a.Swap(&b); }
  inline void Swap(InotifyInit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(InotifyInit* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] InotifyInit* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<InotifyInit>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const InotifyInit& from);
  using Super_::MergeFrom;
  void MergeFrom(const InotifyInit& from) { InotifyInit::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(InotifyInit* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.InotifyInit"; }

  explicit InotifyInit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  InotifyInit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const InotifyInit& from);
  InotifyInit(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, InotifyInit&& from) noexcept
      : InotifyInit(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFlagsFieldNumber = 4,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 flags = 4;
  void clear_flags() ;
  [[nodiscard]] ::int32_t flags() const;
  void set_flags(::int32_t value);

  private:
  ::int32_t _internal_flags() const;
  void _internal_set_flags(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.InotifyInit)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const InotifyInit& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED InotifyAddWatch final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.InotifyAddWatch) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline InotifyAddWatch() : InotifyAddWatch(nullptr) {}
  ~InotifyAddWatch() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(InotifyAddWatch* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(InotifyAddWatch));
  }
#endif

  template <typename = void>
  explicit constexpr InotifyAddWatch(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline InotifyAddWatch(const InotifyAddWatch& from) : InotifyAddWatch(nullptr, from) {}
  inline InotifyAddWatch(InotifyAddWatch&& from) noexcept : InotifyAddWatch(nullptr, ::std::move(from)) {}
  inline InotifyAddWatch& operator=(const InotifyAddWatch& from) {
    CopyFrom(from);
    return *this;
  }
  inline InotifyAddWatch& operator=(InotifyAddWatch&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const InotifyAddWatch& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<InotifyAddWatch>(&InotifyAddWatch_globals_);
  }
  static constexpr int kIndexInFileMessages = 30;
  friend void swap(InotifyAddWatch& a, InotifyAddWatch& b) { a.Swap(&b); }
  inline void Swap(InotifyAddWatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(InotifyAddWatch* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] InotifyAddWatch* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<InotifyAddWatch>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const InotifyAddWatch& from);
  using Super_::MergeFrom;
  void MergeFrom(const InotifyAddWatch& from) { InotifyAddWatch::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(InotifyAddWatch* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.InotifyAddWatch"; }

  explicit InotifyAddWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  InotifyAddWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const InotifyAddWatch& from);
  InotifyAddWatch(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, InotifyAddWatch&& from) noexcept
      : InotifyAddWatch(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kPathnameFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kMaskFieldNumber = 7,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // string pathname = 6;
  void clear_pathname() ;
  [[nodiscard]] const ::std::string& pathname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pathname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pathname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pathname();
  void set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pathname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pathname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pathname();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // uint32 mask = 7;
  void clear_mask() ;
  [[nodiscard]] ::uint32_t mask() const;
  void set_mask(::uint32_t value);

  private:
  ::uint32_t _internal_mask() const;
  void _internal_set_mask(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.InotifyAddWatch)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 54,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const InotifyAddWatch& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr pathname_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::uint32_t mask_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Fork final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Fork) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Fork() : Fork(nullptr) {}
  ~Fork() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Fork* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Fork));
  }
#endif

  template <typename = void>
  explicit constexpr Fork(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Fork(const Fork& from) : Fork(nullptr, from) {}
  inline Fork(Fork&& from) noexcept : Fork(nullptr, ::std::move(from)) {}
  inline Fork& operator=(const Fork& from) {
    CopyFrom(from);
    return *this;
  }
  inline Fork& operator=(Fork&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Fork& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Fork>(&Fork_globals_);
  }
  static constexpr int kIndexInFileMessages = 28;
  friend void swap(Fork& a, Fork& b) { a.Swap(&b); }
  inline void Swap(Fork* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Fork* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Fork* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Fork>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Fork& from);
  using Super_::MergeFrom;
  void MergeFrom(const Fork& from) { Fork::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Fork* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Fork"; }

  explicit Fork(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Fork(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Fork& from);
  Fork(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Fork&& from) noexcept
      : Fork(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Fork)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 3,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Fork& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Fcntl final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Fcntl) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Fcntl() : Fcntl(nullptr) {}
  ~Fcntl() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Fcntl* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Fcntl));
  }
#endif

  template <typename = void>
  explicit constexpr Fcntl(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Fcntl(const Fcntl& from) : Fcntl(nullptr, from) {}
  inline Fcntl(Fcntl&& from) noexcept : Fcntl(nullptr, ::std::move(from)) {}
  inline Fcntl& operator=(const Fcntl& from) {
    CopyFrom(from);
    return *this;
  }
  inline Fcntl& operator=(Fcntl&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Fcntl& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Fcntl>(&Fcntl_globals_);
  }
  static constexpr int kIndexInFileMessages = 15;
  friend void swap(Fcntl& a, Fcntl& b) { a.Swap(&b); }
  inline void Swap(Fcntl* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Fcntl* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Fcntl* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Fcntl>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Fcntl& from);
  using Super_::MergeFrom;
  void MergeFrom(const Fcntl& from) { Fcntl::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Fcntl* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Fcntl"; }

  explicit Fcntl(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Fcntl(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Fcntl& from);
  Fcntl(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Fcntl&& from) noexcept
      : Fcntl(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kCmdFieldNumber = 6,
    kArgsFieldNumber = 7,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // int32 cmd = 6;
  void clear_cmd() ;
  [[nodiscard]] ::int32_t cmd() const;
  void set_cmd(::int32_t value);

  private:
  ::int32_t _internal_cmd() const;
  void _internal_set_cmd(::int32_t value);

  public:
  // int64 args = 7;
  void clear_args() ;
  [[nodiscard]] ::int64_t args() const;
  void set_args(::int64_t value);

  private:
  ::int64_t _internal_args() const;
  void _internal_set_args(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Fcntl)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 36,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Fcntl& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::int32_t cmd_;
    ::int64_t args_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Execve final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Execve) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Execve() : Execve(nullptr) {}
  ~Execve() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Execve* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Execve));
  }
#endif

  template <typename = void>
  explicit constexpr Execve(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Execve(const Execve& from) : Execve(nullptr, from) {}
  inline Execve(Execve&& from) noexcept : Execve(nullptr, ::std::move(from)) {}
  inline Execve& operator=(const Execve& from) {
    CopyFrom(from);
    return *this;
  }
  inline Execve& operator=(Execve&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Execve& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Execve>(&Execve_globals_);
  }
  static constexpr int kIndexInFileMessages = 7;
  friend void swap(Execve& a, Execve& b) { a.Swap(&b); }
  inline void Swap(Execve* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Execve* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Execve* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Execve>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Execve& from);
  using Super_::MergeFrom;
  void MergeFrom(const Execve& from) { Execve::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Execve* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Execve"; }

  explicit Execve(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Execve(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Execve& from);
  Execve(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Execve&& from) noexcept
      : Execve(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kArgvFieldNumber = 7,
    kEnvvFieldNumber = 8,
    kFdPathFieldNumber = 5,
    kPathnameFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kFlagsFieldNumber = 9,
  };
  // repeated string argv = 7;
  [[nodiscard]] int argv_size() const;
  private:
  int _internal_argv_size() const;

  public:
  void clear_argv() ;
  [[nodiscard]] const ::std::string& argv(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_argv(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_argv(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_argv();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_argv(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      argv() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_argv();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_argv() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_argv();

  public:
  // repeated string envv = 8;
  [[nodiscard]] int envv_size() const;
  private:
  int _internal_envv_size() const;

  public:
  void clear_envv() ;
  [[nodiscard]] const ::std::string& envv(int index) const;
  ::std::string* PROTOBUF_NONNULL mutable_envv(int index);
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_envv(int index, Arg_&& value, Args_... args);
  ::std::string* PROTOBUF_NONNULL add_envv();
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void add_envv(Arg_&& value, Args_... args);
  [[nodiscard]] const
      ::google::protobuf::RepeatedPtrField<::std::string>&
      envv() const;
  [[nodiscard]] ::google::protobuf::RepeatedPtrField<::std::string>*
      PROTOBUF_NONNULL
      mutable_envv();

  private:
  const ::google::protobuf::RepeatedPtrField<::std::string>& _internal_envv() const;
  ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL _internal_mutable_envv();

  public:
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // string pathname = 6;
  void clear_pathname() ;
  [[nodiscard]] const ::std::string& pathname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pathname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pathname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pathname();
  void set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pathname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pathname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pathname();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // uint32 flags = 9;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Execve)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<4, 9,
                          2, 61,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Execve& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::RepeatedPtrField<::std::string> argv_;
    ::google::protobuf::RepeatedPtrField<::std::string> envv_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr pathname_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Eventfd final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Eventfd) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Eventfd() : Eventfd(nullptr) {}
  ~Eventfd() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Eventfd* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Eventfd));
  }
#endif

  template <typename = void>
  explicit constexpr Eventfd(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Eventfd(const Eventfd& from) : Eventfd(nullptr, from) {}
  inline Eventfd(Eventfd&& from) noexcept : Eventfd(nullptr, ::std::move(from)) {}
  inline Eventfd& operator=(const Eventfd& from) {
    CopyFrom(from);
    return *this;
  }
  inline Eventfd& operator=(Eventfd&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Eventfd& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Eventfd>(&Eventfd_globals_);
  }
  static constexpr int kIndexInFileMessages = 19;
  friend void swap(Eventfd& a, Eventfd& b) { a.Swap(&b); }
  inline void Swap(Eventfd* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Eventfd* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Eventfd* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Eventfd>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Eventfd& from);
  using Super_::MergeFrom;
  void MergeFrom(const Eventfd& from) { Eventfd::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Eventfd* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Eventfd"; }

  explicit Eventfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Eventfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Eventfd& from);
  Eventfd(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Eventfd&& from) noexcept
      : Eventfd(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kValFieldNumber = 4,
    kFlagsFieldNumber = 5,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 val = 4;
  void clear_val() ;
  [[nodiscard]] ::int32_t val() const;
  void set_val(::int32_t value);

  private:
  ::int32_t _internal_val() const;
  void _internal_set_val(::int32_t value);

  public:
  // uint32 flags = 5;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Eventfd)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Eventfd& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t val_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Dup final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Dup) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Dup() : Dup(nullptr) {}
  ~Dup() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Dup* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Dup));
  }
#endif

  template <typename = void>
  explicit constexpr Dup(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Dup(const Dup& from) : Dup(nullptr, from) {}
  inline Dup(Dup&& from) noexcept : Dup(nullptr, ::std::move(from)) {}
  inline Dup& operator=(const Dup& from) {
    CopyFrom(from);
    return *this;
  }
  inline Dup& operator=(Dup&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Dup& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Dup>(&Dup_globals_);
  }
  static constexpr int kIndexInFileMessages = 16;
  friend void swap(Dup& a, Dup& b) { a.Swap(&b); }
  inline void Swap(Dup* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Dup* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Dup* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Dup>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Dup& from);
  using Super_::MergeFrom;
  void MergeFrom(const Dup& from) { Dup::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Dup* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Dup"; }

  explicit Dup(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Dup(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Dup& from);
  Dup(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Dup&& from) noexcept
      : Dup(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kOldFdFieldNumber = 4,
    kNewFdFieldNumber = 5,
    kFlagsFieldNumber = 7,
  };
  // string fd_path = 6;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 old_fd = 4;
  void clear_old_fd() ;
  [[nodiscard]] ::int32_t old_fd() const;
  void set_old_fd(::int32_t value);

  private:
  ::int32_t _internal_old_fd() const;
  void _internal_set_old_fd(::int32_t value);

  public:
  // int32 new_fd = 5;
  void clear_new_fd() ;
  [[nodiscard]] ::int32_t new_fd() const;
  void set_new_fd(::int32_t value);

  private:
  ::int32_t _internal_new_fd() const;
  void _internal_set_new_fd(::int32_t value);

  public:
  // uint32 flags = 7;
  void clear_flags() ;
  [[nodiscard]] ::uint32_t flags() const;
  void set_flags(::uint32_t value);

  private:
  ::uint32_t _internal_flags() const;
  void _internal_set_flags(::uint32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Dup)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 34,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Dup& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t old_fd_;
    ::int32_t new_fd_;
    ::uint32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Connect final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Connect) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Connect() : Connect(nullptr) {}
  ~Connect() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Connect* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Connect));
  }
#endif

  template <typename = void>
  explicit constexpr Connect(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Connect(const Connect& from) : Connect(nullptr, from) {}
  inline Connect(Connect&& from) noexcept : Connect(nullptr, ::std::move(from)) {}
  inline Connect& operator=(const Connect& from) {
    CopyFrom(from);
    return *this;
  }
  inline Connect& operator=(Connect&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Connect& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Connect>(&Connect_globals_);
  }
  static constexpr int kIndexInFileMessages = 6;
  friend void swap(Connect& a, Connect& b) { a.Swap(&b); }
  inline void Swap(Connect* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Connect* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Connect* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Connect>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Connect& from);
  using Super_::MergeFrom;
  void MergeFrom(const Connect& from) { Connect::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Connect* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Connect"; }

  explicit Connect(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Connect(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Connect& from);
  Connect(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Connect&& from) noexcept
      : Connect(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kAddressFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // bytes address = 6;
  void clear_address() ;
  [[nodiscard]] const ::std::string& address() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_address(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_address();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_address();
  void set_allocated_address(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_address() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_address(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_address();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Connect)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 38,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Connect& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr address_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Close final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Close) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Close() : Close(nullptr) {}
  ~Close() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Close* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Close));
  }
#endif

  template <typename = void>
  explicit constexpr Close(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Close(const Close& from) : Close(nullptr, from) {}
  inline Close(Close&& from) noexcept : Close(nullptr, ::std::move(from)) {}
  inline Close& operator=(const Close& from) {
    CopyFrom(from);
    return *this;
  }
  inline Close& operator=(Close&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Close& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Close>(&Close_globals_);
  }
  static constexpr int kIndexInFileMessages = 3;
  friend void swap(Close& a, Close& b) { a.Swap(&b); }
  inline void Swap(Close* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Close* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Close* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Close>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Close& from);
  using Super_::MergeFrom;
  void MergeFrom(const Close& from) { Close::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Close* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Close"; }

  explicit Close(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Close(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Close& from);
  Close(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Close&& from) noexcept
      : Close(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Close)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 5,
                          2, 36,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Close& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Clone final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Clone) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Clone() : Clone(nullptr) {}
  ~Clone() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Clone* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Clone));
  }
#endif

  template <typename = void>
  explicit constexpr Clone(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Clone(const Clone& from) : Clone(nullptr, from) {}
  inline Clone(Clone&& from) noexcept : Clone(nullptr, ::std::move(from)) {}
  inline Clone& operator=(const Clone& from) {
    CopyFrom(from);
    return *this;
  }
  inline Clone& operator=(Clone&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Clone& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Clone>(&Clone_globals_);
  }
  static constexpr int kIndexInFileMessages = 20;
  friend void swap(Clone& a, Clone& b) { a.Swap(&b); }
  inline void Swap(Clone* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Clone* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Clone* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Clone>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Clone& from);
  using Super_::MergeFrom;
  void MergeFrom(const Clone& from) { Clone::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Clone* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Clone"; }

  explicit Clone(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Clone(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Clone& from);
  Clone(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Clone&& from) noexcept
      : Clone(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFlagsFieldNumber = 4,
    kStackFieldNumber = 5,
    kNewTidFieldNumber = 6,
    kTlsFieldNumber = 7,
  };
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // uint64 flags = 4;
  void clear_flags() ;
  [[nodiscard]] ::uint64_t flags() const;
  void set_flags(::uint64_t value);

  private:
  ::uint64_t _internal_flags() const;
  void _internal_set_flags(::uint64_t value);

  public:
  // uint64 stack = 5;
  void clear_stack() ;
  [[nodiscard]] ::uint64_t stack() const;
  void set_stack(::uint64_t value);

  private:
  ::uint64_t _internal_stack() const;
  void _internal_set_stack(::uint64_t value);

  public:
  // uint64 new_tid = 6;
  void clear_new_tid() ;
  [[nodiscard]] ::uint64_t new_tid() const;
  void set_new_tid(::uint64_t value);

  private:
  ::uint64_t _internal_new_tid() const;
  void _internal_set_new_tid(::uint64_t value);

  public:
  // uint64 tls = 7;
  void clear_tls() ;
  [[nodiscard]] ::uint64_t tls() const;
  void set_tls(::uint64_t value);

  private:
  ::uint64_t _internal_tls() const;
  void _internal_set_tls(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Clone)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 0,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Clone& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::uint64_t flags_;
    ::uint64_t stack_;
    ::uint64_t new_tid_;
    ::uint64_t tls_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Chroot final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Chroot) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Chroot() : Chroot(nullptr) {}
  ~Chroot() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Chroot* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Chroot));
  }
#endif

  template <typename = void>
  explicit constexpr Chroot(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Chroot(const Chroot& from) : Chroot(nullptr, from) {}
  inline Chroot(Chroot&& from) noexcept : Chroot(nullptr, ::std::move(from)) {}
  inline Chroot& operator=(const Chroot& from) {
    CopyFrom(from);
    return *this;
  }
  inline Chroot& operator=(Chroot&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Chroot& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Chroot>(&Chroot_globals_);
  }
  static constexpr int kIndexInFileMessages = 18;
  friend void swap(Chroot& a, Chroot& b) { a.Swap(&b); }
  inline void Swap(Chroot* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Chroot* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Chroot* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Chroot>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Chroot& from);
  using Super_::MergeFrom;
  void MergeFrom(const Chroot& from) { Chroot::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Chroot* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Chroot"; }

  explicit Chroot(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Chroot(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Chroot& from);
  Chroot(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Chroot&& from) noexcept
      : Chroot(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kPathnameFieldNumber = 4,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
  };
  // string pathname = 4;
  void clear_pathname() ;
  [[nodiscard]] const ::std::string& pathname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pathname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pathname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pathname();
  void set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pathname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pathname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pathname();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Chroot)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<2, 4,
                          2, 38,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Chroot& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr pathname_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Chdir final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Chdir) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Chdir() : Chdir(nullptr) {}
  ~Chdir() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Chdir* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Chdir));
  }
#endif

  template <typename = void>
  explicit constexpr Chdir(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Chdir(const Chdir& from) : Chdir(nullptr, from) {}
  inline Chdir(Chdir&& from) noexcept : Chdir(nullptr, ::std::move(from)) {}
  inline Chdir& operator=(const Chdir& from) {
    CopyFrom(from);
    return *this;
  }
  inline Chdir& operator=(Chdir&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Chdir& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Chdir>(&Chdir_globals_);
  }
  static constexpr int kIndexInFileMessages = 9;
  friend void swap(Chdir& a, Chdir& b) { a.Swap(&b); }
  inline void Swap(Chdir* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Chdir* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Chdir* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Chdir>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Chdir& from);
  using Super_::MergeFrom;
  void MergeFrom(const Chdir& from) { Chdir::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Chdir* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Chdir"; }

  explicit Chdir(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Chdir(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Chdir& from);
  Chdir(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Chdir&& from) noexcept
      : Chdir(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kPathnameFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // string pathname = 6;
  void clear_pathname() ;
  [[nodiscard]] const ::std::string& pathname() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_pathname(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_pathname();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_pathname();
  void set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_pathname() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_pathname(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_pathname();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int64 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int64_t fd() const;
  void set_fd(::int64_t value);

  private:
  ::int64_t _internal_fd() const;
  void _internal_set_fd(::int64_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Chdir)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 44,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Chdir& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr pathname_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int64_t fd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Bind final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Bind) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Bind() : Bind(nullptr) {}
  ~Bind() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Bind* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Bind));
  }
#endif

  template <typename = void>
  explicit constexpr Bind(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Bind(const Bind& from) : Bind(nullptr, from) {}
  inline Bind(Bind&& from) noexcept : Bind(nullptr, ::std::move(from)) {}
  inline Bind& operator=(const Bind& from) {
    CopyFrom(from);
    return *this;
  }
  inline Bind& operator=(Bind&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Bind& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Bind>(&Bind_globals_);
  }
  static constexpr int kIndexInFileMessages = 21;
  friend void swap(Bind& a, Bind& b) { a.Swap(&b); }
  inline void Swap(Bind* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Bind* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Bind* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Bind>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Bind& from);
  using Super_::MergeFrom;
  void MergeFrom(const Bind& from) { Bind::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Bind* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Bind"; }

  explicit Bind(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Bind(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Bind& from);
  Bind(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Bind&& from) noexcept
      : Bind(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kAddressFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // bytes address = 6;
  void clear_address() ;
  [[nodiscard]] const ::std::string& address() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_address(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_address();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_address();
  void set_allocated_address(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_address() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_address(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_address();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Bind)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 6,
                          2, 35,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Bind& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr address_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};
// -------------------------------------------------------------------

class  PROTOBUF_FUTURE_ADD_EARLY_WARN_UNUSED Accept final : public ::google::protobuf::Message
/* @@protoc_insertion_point(class_definition:gvisor.syscall.Accept) */ {
  using Super_ = ::google::protobuf::Message;

 public:
  inline Accept() : Accept(nullptr) {}
  ~Accept() PROTOBUF_FINAL;

#if defined(PROTOBUF_CUSTOM_VTABLE)
  void operator delete(Accept* PROTOBUF_NONNULL msg, ::std::destroying_delete_t) {
    SharedDtor(*msg);
    ::google::protobuf::internal::SizedDelete(msg, sizeof(Accept));
  }
#endif

  template <typename = void>
  explicit constexpr Accept(::google::protobuf::internal::ConstantInitialized,
                           const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
                               class_data);

  inline Accept(const Accept& from) : Accept(nullptr, from) {}
  inline Accept(Accept&& from) noexcept : Accept(nullptr, ::std::move(from)) {}
  inline Accept& operator=(const Accept& from) {
    CopyFrom(from);
    return *this;
  }
  inline Accept& operator=(Accept&& from) noexcept {
    if (this == &from) return *this;
    if (::google::protobuf::internal::CanMoveWithInternalSwap(GetArena(), from.GetArena())) {
      InternalSwap(&from);
    } else {
      CopyFrom(from);
    }
    return *this;
  }

  [[nodiscard]] inline const ::google::protobuf::UnknownFieldSet& unknown_fields() const
      ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance);
  }
  [[nodiscard]] inline ::google::protobuf::UnknownFieldSet* PROTOBUF_NONNULL
  mutable_unknown_fields() ABSL_ATTRIBUTE_LIFETIME_BOUND {
    return _internal_metadata_.mutable_unknown_fields<::google::protobuf::UnknownFieldSet>();
  }

  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL descriptor() {
    return GetDescriptor();
  }
  [[nodiscard]] static const ::google::protobuf::Descriptor* PROTOBUF_NONNULL
  GetDescriptor() {
    return default_instance().GetMetadata().descriptor;
  }
  [[nodiscard]] static const ::google::protobuf::Reflection* PROTOBUF_NONNULL GetReflection() {
    return default_instance().GetMetadata().reflection;
  }
  [[nodiscard]] static const Accept& default_instance() {
    return *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<Accept>(&Accept_globals_);
  }
  static constexpr int kIndexInFileMessages = 22;
  friend void swap(Accept& a, Accept& b) { a.Swap(&b); }
  inline void Swap(Accept* PROTOBUF_NONNULL other) {
    if (other == this) return;
    if (::google::protobuf::internal::CanUseInternalSwap(GetArena(), other->GetArena())) {
      InternalSwap(other);
    } else {
      ::google::protobuf::internal::GenericSwap(this, other);
    }
  }
  void UnsafeArenaSwap(Accept* PROTOBUF_NONNULL other) {
    if (other == this) return;
    ABSL_DCHECK(GetArena() == other->GetArena());
    InternalSwap(other);
  }

  // implements Message ----------------------------------------------

  [[nodiscard]] Accept* PROTOBUF_NONNULL
  New(::google::protobuf::Arena* PROTOBUF_NULLABLE arena = nullptr) const {
    return Super_::DefaultConstruct<Accept>(arena);
  }
  using Super_::CopyFrom;
  void CopyFrom(const Accept& from);
  using Super_::MergeFrom;
  void MergeFrom(const Accept& from) { Accept::MergeImpl(*this, from); }

  private:
  static void MergeImpl(::google::protobuf::MessageLite& to_msg,
                        const ::google::protobuf::MessageLite& from_msg);

  public:
  [[nodiscard]] bool IsInitialized() const {
    return true;
  }
  ABSL_ATTRIBUTE_REINITIALIZES void Clear() PROTOBUF_FINAL;
  #if defined(PROTOBUF_CUSTOM_VTABLE)
  private:
  [[nodiscard]] static ::size_t ByteSizeLong(const ::google::protobuf::MessageLite& msg);
  [[nodiscard]] static ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      const ::google::protobuf::MessageLite& msg, ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream);

  public:
  [[nodiscard]] ::size_t ByteSizeLong() const { return ByteSizeLong(*this); }
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
    return _InternalSerialize(*this, target, stream);
  }
  #else   // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] ::size_t ByteSizeLong() const final;
  [[nodiscard]] ::uint8_t* PROTOBUF_NONNULL _InternalSerialize(
      ::uint8_t* PROTOBUF_NONNULL target,
      ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const final;
  #endif  // PROTOBUF_CUSTOM_VTABLE
  [[nodiscard]] int GetCachedSize() const {
    return _impl_._cached_size_.Get();
  }

  private:
  void SharedCtor(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static void SharedDtor(MessageLite& self);
  void InternalSwap(Accept* PROTOBUF_NONNULL other);
 private:
  static ::absl::string_view FullMessageName() { return "gvisor.syscall.Accept"; }

  explicit Accept(::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  Accept(::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Accept& from);
  Accept(
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, Accept&& from) noexcept
      : Accept(arena) {
    *this = ::std::move(from);
  }
  const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL GetClassData() const PROTOBUF_FINAL;
  static void* PROTOBUF_NONNULL PlacementNew_(
      const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
      ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
  static constexpr auto InternalNewImpl_();

 public:
  static constexpr auto InternalGenerateClassData_(
      const MessageLite& prototype,
      const ::google::protobuf::internal::TcParseTableBase* PROTOBUF_NULLABLE tc_table = nullptr);

  [[nodiscard]] ::google::protobuf::Metadata GetMetadata() const;
  // nested types ----------------------------------------------------

  // accessors -------------------------------------------------------
  enum : int {
    kFdPathFieldNumber = 5,
    kAddressFieldNumber = 6,
    kContextDataFieldNumber = 1,
    kExitFieldNumber = 2,
    kSysnoFieldNumber = 3,
    kFdFieldNumber = 4,
    kFlagsFieldNumber = 7,
  };
  // string fd_path = 5;
  void clear_fd_path() ;
  [[nodiscard]] const ::std::string& fd_path() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_fd_path(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_fd_path();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_fd_path();
  void set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_fd_path() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_fd_path(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_fd_path();

  public:
  // bytes address = 6;
  void clear_address() ;
  [[nodiscard]] const ::std::string& address() const;
  template <typename Arg_ = const ::std::string&, typename... Args_>
  void set_address(Arg_&& arg, Args_... args);
  ::std::string* PROTOBUF_NONNULL mutable_address();
  [[nodiscard]] ::std::string* PROTOBUF_NULLABLE release_address();
  void set_allocated_address(::std::string* PROTOBUF_NULLABLE value);

  private:
  const ::std::string& _internal_address() const;
  PROTOBUF_ALWAYS_INLINE void _internal_set_address(const ::std::string& value);
  ::std::string* PROTOBUF_NONNULL _internal_mutable_address();

  public:
  // .gvisor.common.ContextData context_data = 1;
  [[nodiscard]] bool has_context_data() const;
  void clear_context_data() ;
  [[nodiscard]] const ::gvisor::common::ContextData& context_data() const;
  [[nodiscard]] ::gvisor::common::ContextData* PROTOBUF_NULLABLE release_context_data();
  ::gvisor::common::ContextData* PROTOBUF_NONNULL mutable_context_data();
  void set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value);
  ::gvisor::common::ContextData* PROTOBUF_NULLABLE unsafe_arena_release_context_data();

  private:
  const ::gvisor::common::ContextData& _internal_context_data() const;
  ::gvisor::common::ContextData* PROTOBUF_NONNULL _internal_mutable_context_data();

  public:
  // .gvisor.syscall.Exit exit = 2;
  [[nodiscard]] bool has_exit() const;
  void clear_exit() ;
  [[nodiscard]] const ::gvisor::syscall::Exit& exit() const;
  [[nodiscard]] ::gvisor::syscall::Exit* PROTOBUF_NULLABLE release_exit();
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL mutable_exit();
  void set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  void unsafe_arena_set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value);
  ::gvisor::syscall::Exit* PROTOBUF_NULLABLE unsafe_arena_release_exit();

  private:
  const ::gvisor::syscall::Exit& _internal_exit() const;
  ::gvisor::syscall::Exit* PROTOBUF_NONNULL _internal_mutable_exit();

  public:
  // uint64 sysno = 3;
  void clear_sysno() ;
  [[nodiscard]] ::uint64_t sysno() const;
  void set_sysno(::uint64_t value);

  private:
  ::uint64_t _internal_sysno() const;
  void _internal_set_sysno(::uint64_t value);

  public:
  // int32 fd = 4;
  void clear_fd() ;
  [[nodiscard]] ::int32_t fd() const;
  void set_fd(::int32_t value);

  private:
  ::int32_t _internal_fd() const;
  void _internal_set_fd(::int32_t value);

  public:
  // int32 flags = 7;
  void clear_flags() ;
  [[nodiscard]] ::int32_t flags() const;
  void set_flags(::int32_t value);

  private:
  ::int32_t _internal_flags() const;
  void _internal_set_flags(::int32_t value);

  public:
  // @@protoc_insertion_point(class_scope:gvisor.syscall.Accept)
 private:
  class _Internal;
  using ParseTableT_ =
      ::google::protobuf::internal::TcParseTable<3, 7,
                          2, 37,
                          2>;
  static constexpr ParseTableT_ InternalGenerateParseTable_(
      const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL class_data);
  friend class ::google::protobuf::internal::TcParser;
  #ifndef PROTOBUF_MESSAGE_GLOBALS
  static const ParseTableT_ _table_;
  #endif

  friend class ::google::protobuf::MessageLite;
  friend class ::google::protobuf::Arena;
  friend ::google::protobuf::internal::PrivateAccess;
  template <typename T>
  friend class ::google::protobuf::Arena::InternalHelper;
  using InternalArenaConstructable_ = void;
  using DestructorSkippable_ = void;
  struct Impl_ {
    inline explicit constexpr Impl_(::google::protobuf::internal::InternalVisibility visibility,
                                    ::google::protobuf::internal::ConstantInitialized) noexcept;
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena);
    inline explicit Impl_(
        ::google::protobuf::internal::InternalVisibility visibility,
        ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
        const Accept& from_msg);
    ::google::protobuf::internal::HasBits<1> _has_bits_;
    ::google::protobuf::internal::CachedSize _cached_size_;
    ::google::protobuf::internal::ArenaStringPtr fd_path_;
    ::google::protobuf::internal::ArenaStringPtr address_;
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE context_data_;
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE exit_;
    ::uint64_t sysno_;
    ::int32_t fd_;
    ::int32_t flags_;
    PROTOBUF_TSAN_DECLARE_MEMBER
  };
  union { Impl_ _impl_; };
  friend struct ::TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto;
};

// ===================================================================




// ===================================================================


#ifdef __GNUC__
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wstrict-aliasing"
#endif  // __GNUC__
// -------------------------------------------------------------------

// Exit

// int64 result = 1;
inline void Exit::clear_result() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.result_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int64_t Exit::result() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Exit.result)
  return _internal_result();
}
inline void Exit::set_result(::int64_t value) {
  _internal_set_result(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Exit.result)
}
inline ::int64_t Exit::_internal_result() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.result_;
}
inline void Exit::_internal_set_result(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.result_ = value;
}

// int64 errorno = 2;
inline void Exit::clear_errorno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.errorno_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t Exit::errorno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Exit.errorno)
  return _internal_errorno();
}
inline void Exit::set_errorno(::int64_t value) {
  _internal_set_errorno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Exit.errorno)
}
inline ::int64_t Exit::_internal_errorno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.errorno_;
}
inline void Exit::_internal_set_errorno(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.errorno_ = value;
}

// -------------------------------------------------------------------

// Syscall

// .gvisor.common.ContextData context_data = 1;
inline bool Syscall::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Syscall::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Syscall::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.context_data)
  return _internal_context_data();
}
inline void Syscall::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Syscall.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Syscall::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Syscall::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Syscall.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Syscall::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Syscall::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Syscall.context_data)
  return _msg;
}
inline void Syscall::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Syscall.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Syscall::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Syscall::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Syscall::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Syscall::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.exit)
  return _internal_exit();
}
inline void Syscall::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Syscall.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Syscall::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Syscall::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Syscall.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Syscall::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Syscall::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Syscall.exit)
  return _msg;
}
inline void Syscall::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Syscall.exit)
}

// uint64 sysno = 4;
inline void Syscall::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Syscall::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.sysno)
  return _internal_sysno();
}
inline void Syscall::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.sysno)
}
inline ::uint64_t Syscall::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Syscall::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// uint64 arg1 = 5;
inline void Syscall::clear_arg1() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg1_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Syscall::arg1() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg1)
  return _internal_arg1();
}
inline void Syscall::set_arg1(::uint64_t value) {
  _internal_set_arg1(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg1)
}
inline ::uint64_t Syscall::_internal_arg1() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg1_;
}
inline void Syscall::_internal_set_arg1(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg1_ = value;
}

// uint64 arg2 = 6;
inline void Syscall::clear_arg2() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg2_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Syscall::arg2() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg2)
  return _internal_arg2();
}
inline void Syscall::set_arg2(::uint64_t value) {
  _internal_set_arg2(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg2)
}
inline ::uint64_t Syscall::_internal_arg2() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg2_;
}
inline void Syscall::_internal_set_arg2(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg2_ = value;
}

// uint64 arg3 = 7;
inline void Syscall::clear_arg3() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg3_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t Syscall::arg3() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg3)
  return _internal_arg3();
}
inline void Syscall::set_arg3(::uint64_t value) {
  _internal_set_arg3(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg3)
}
inline ::uint64_t Syscall::_internal_arg3() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg3_;
}
inline void Syscall::_internal_set_arg3(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg3_ = value;
}

// uint64 arg4 = 8;
inline void Syscall::clear_arg4() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg4_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t Syscall::arg4() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg4)
  return _internal_arg4();
}
inline void Syscall::set_arg4(::uint64_t value) {
  _internal_set_arg4(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg4)
}
inline ::uint64_t Syscall::_internal_arg4() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg4_;
}
inline void Syscall::_internal_set_arg4(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg4_ = value;
}

// uint64 arg5 = 9;
inline void Syscall::clear_arg5() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg5_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::uint64_t Syscall::arg5() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg5)
  return _internal_arg5();
}
inline void Syscall::set_arg5(::uint64_t value) {
  _internal_set_arg5(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg5)
}
inline ::uint64_t Syscall::_internal_arg5() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg5_;
}
inline void Syscall::_internal_set_arg5(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg5_ = value;
}

// uint64 arg6 = 10;
inline void Syscall::clear_arg6() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg6_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint64_t Syscall::arg6() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Syscall.arg6)
  return _internal_arg6();
}
inline void Syscall::set_arg6(::uint64_t value) {
  _internal_set_arg6(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Syscall.arg6)
}
inline ::uint64_t Syscall::_internal_arg6() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.arg6_;
}
inline void Syscall::_internal_set_arg6(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.arg6_ = value;
}

// -------------------------------------------------------------------

// Open

// .gvisor.common.ContextData context_data = 1;
inline bool Open::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Open::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Open::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.context_data)
  return _internal_context_data();
}
inline void Open::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Open.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Open::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Open::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Open.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Open::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Open::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Open.context_data)
  return _msg;
}
inline void Open::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Open.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Open::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Open::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& Open::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Open::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.exit)
  return _internal_exit();
}
inline void Open::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Open.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Open::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Open::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Open.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Open::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Open::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Open.exit)
  return _msg;
}
inline void Open::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Open.exit)
}

// uint64 sysno = 3;
inline void Open::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Open::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.sysno)
  return _internal_sysno();
}
inline void Open::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.sysno)
}
inline ::uint64_t Open::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Open::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Open::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int64_t Open::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.fd)
  return _internal_fd();
}
inline void Open::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.fd)
}
inline ::int64_t Open::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Open::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Open::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Open::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Open::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Open::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Open.fd_path)
  return _s;
}
inline const ::std::string& Open::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Open::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Open::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Open::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Open.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Open::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Open.fd_path)
}

// string pathname = 6;
inline void Open::clear_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Open::pathname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.pathname)
  return _internal_pathname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Open::set_pathname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pathname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.pathname)
}
inline ::std::string* PROTOBUF_NONNULL Open::mutable_pathname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pathname();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Open.pathname)
  return _s;
}
inline const ::std::string& Open::_internal_pathname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pathname_.Get();
}
inline void Open::_internal_set_pathname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Open::_internal_mutable_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pathname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Open::release_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Open.pathname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pathname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pathname_.Set("", GetArena());
  }
  return released;
}
inline void Open::set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pathname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pathname_.IsDefault()) {
    _impl_.pathname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Open.pathname)
}

// uint32 flags = 7;
inline void Open::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint32_t Open::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.flags)
  return _internal_flags();
}
inline void Open::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.flags)
}
inline ::uint32_t Open::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Open::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// uint32 mode = 8;
inline void Open::clear_mode() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mode_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::uint32_t Open::mode() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Open.mode)
  return _internal_mode();
}
inline void Open::set_mode(::uint32_t value) {
  _internal_set_mode(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Open.mode)
}
inline ::uint32_t Open::_internal_mode() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mode_;
}
inline void Open::_internal_set_mode(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mode_ = value;
}

// -------------------------------------------------------------------

// Close

// .gvisor.common.ContextData context_data = 1;
inline bool Close::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Close::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Close::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Close.context_data)
  return _internal_context_data();
}
inline void Close::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Close.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Close::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Close::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Close.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Close::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Close::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Close.context_data)
  return _msg;
}
inline void Close::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Close.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Close::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Close::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Close::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Close::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Close.exit)
  return _internal_exit();
}
inline void Close::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Close.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Close::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Close::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Close.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Close::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Close::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Close.exit)
  return _msg;
}
inline void Close::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Close.exit)
}

// uint64 sysno = 3;
inline void Close::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Close::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Close.sysno)
  return _internal_sysno();
}
inline void Close::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Close.sysno)
}
inline ::uint64_t Close::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Close::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Close::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int64_t Close::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Close.fd)
  return _internal_fd();
}
inline void Close::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Close.fd)
}
inline ::int64_t Close::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Close::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Close::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Close::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Close.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Close::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Close.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Close::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Close.fd_path)
  return _s;
}
inline const ::std::string& Close::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Close::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Close::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Close::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Close.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Close::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Close.fd_path)
}

// -------------------------------------------------------------------

// Read

// .gvisor.common.ContextData context_data = 1;
inline bool Read::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Read::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Read::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.context_data)
  return _internal_context_data();
}
inline void Read::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Read.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Read::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Read::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Read.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Read::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Read::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Read.context_data)
  return _msg;
}
inline void Read::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Read.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Read::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Read::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Read::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Read::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.exit)
  return _internal_exit();
}
inline void Read::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Read.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Read::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Read::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Read.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Read::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Read::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Read.exit)
  return _msg;
}
inline void Read::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Read.exit)
}

// uint64 sysno = 3;
inline void Read::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Read::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.sysno)
  return _internal_sysno();
}
inline void Read::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.sysno)
}
inline ::uint64_t Read::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Read::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Read::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int64_t Read::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.fd)
  return _internal_fd();
}
inline void Read::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.fd)
}
inline ::int64_t Read::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Read::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Read::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Read::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Read::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Read::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Read.fd_path)
  return _s;
}
inline const ::std::string& Read::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Read::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Read::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Read::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Read.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Read::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Read.fd_path)
}

// uint64 count = 6;
inline void Read::clear_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t Read::count() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.count)
  return _internal_count();
}
inline void Read::set_count(::uint64_t value) {
  _internal_set_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.count)
}
inline ::uint64_t Read::_internal_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.count_;
}
inline void Read::_internal_set_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = value;
}

// bool has_offset = 7;
inline void Read::clear_has_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.has_offset_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline bool Read::has_offset() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.has_offset)
  return _internal_has_offset();
}
inline void Read::set_has_offset(bool value) {
  _internal_set_has_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.has_offset)
}
inline bool Read::_internal_has_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.has_offset_;
}
inline void Read::_internal_set_has_offset(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.has_offset_ = value;
}

// int64 offset = 8;
inline void Read::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int64_t Read::offset() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.offset)
  return _internal_offset();
}
inline void Read::set_offset(::int64_t value) {
  _internal_set_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.offset)
}
inline ::int64_t Read::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline void Read::_internal_set_offset(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = value;
}

// uint32 flags = 9;
inline void Read::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint32_t Read::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Read.flags)
  return _internal_flags();
}
inline void Read::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Read.flags)
}
inline ::uint32_t Read::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Read::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Write

// .gvisor.common.ContextData context_data = 1;
inline bool Write::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Write::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Write::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.context_data)
  return _internal_context_data();
}
inline void Write::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Write.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Write::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Write::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Write.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Write::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Write::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Write.context_data)
  return _msg;
}
inline void Write::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Write.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Write::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Write::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Write::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Write::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.exit)
  return _internal_exit();
}
inline void Write::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Write.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Write::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Write::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Write.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Write::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Write::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Write.exit)
  return _msg;
}
inline void Write::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Write.exit)
}

// uint64 sysno = 3;
inline void Write::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Write::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.sysno)
  return _internal_sysno();
}
inline void Write::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.sysno)
}
inline ::uint64_t Write::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Write::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Write::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int64_t Write::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.fd)
  return _internal_fd();
}
inline void Write::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.fd)
}
inline ::int64_t Write::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Write::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Write::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Write::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Write::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Write::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Write.fd_path)
  return _s;
}
inline const ::std::string& Write::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Write::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Write::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Write::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Write.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Write::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Write.fd_path)
}

// uint64 count = 6;
inline void Write::clear_count() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t Write::count() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.count)
  return _internal_count();
}
inline void Write::set_count(::uint64_t value) {
  _internal_set_count(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.count)
}
inline ::uint64_t Write::_internal_count() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.count_;
}
inline void Write::_internal_set_count(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.count_ = value;
}

// bool has_offset = 7;
inline void Write::clear_has_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.has_offset_ = false;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline bool Write::has_offset() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.has_offset)
  return _internal_has_offset();
}
inline void Write::set_has_offset(bool value) {
  _internal_set_has_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.has_offset)
}
inline bool Write::_internal_has_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.has_offset_;
}
inline void Write::_internal_set_has_offset(bool value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.has_offset_ = value;
}

// int64 offset = 8;
inline void Write::clear_offset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int64_t Write::offset() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.offset)
  return _internal_offset();
}
inline void Write::set_offset(::int64_t value) {
  _internal_set_offset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.offset)
}
inline ::int64_t Write::_internal_offset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.offset_;
}
inline void Write::_internal_set_offset(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.offset_ = value;
}

// uint32 flags = 9;
inline void Write::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint32_t Write::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Write.flags)
  return _internal_flags();
}
inline void Write::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Write.flags)
}
inline ::uint32_t Write::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Write::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Connect

// .gvisor.common.ContextData context_data = 1;
inline bool Connect::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Connect::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Connect::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.context_data)
  return _internal_context_data();
}
inline void Connect::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Connect.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Connect::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Connect::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Connect.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Connect::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Connect::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Connect.context_data)
  return _msg;
}
inline void Connect::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Connect.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Connect::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Connect::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& Connect::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Connect::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.exit)
  return _internal_exit();
}
inline void Connect::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Connect.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Connect::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Connect::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Connect.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Connect::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Connect::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Connect.exit)
  return _msg;
}
inline void Connect::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Connect.exit)
}

// uint64 sysno = 3;
inline void Connect::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Connect::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.sysno)
  return _internal_sysno();
}
inline void Connect::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Connect.sysno)
}
inline ::uint64_t Connect::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Connect::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Connect::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int64_t Connect::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.fd)
  return _internal_fd();
}
inline void Connect::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Connect.fd)
}
inline ::int64_t Connect::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Connect::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Connect::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Connect::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Connect::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Connect.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Connect::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Connect.fd_path)
  return _s;
}
inline const ::std::string& Connect::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Connect::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Connect::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Connect::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Connect.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Connect::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Connect.fd_path)
}

// bytes address = 6;
inline void Connect::clear_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Connect::address() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Connect.address)
  return _internal_address();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Connect::set_address(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.address_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Connect.address)
}
inline ::std::string* PROTOBUF_NONNULL Connect::mutable_address()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_address();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Connect.address)
  return _s;
}
inline const ::std::string& Connect::_internal_address() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.address_.Get();
}
inline void Connect::_internal_set_address(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Connect::_internal_mutable_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.address_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Connect::release_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Connect.address)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.address_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.address_.Set("", GetArena());
  }
  return released;
}
inline void Connect::set_allocated_address(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.address_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.address_.IsDefault()) {
    _impl_.address_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Connect.address)
}

// -------------------------------------------------------------------

// Execve

// .gvisor.common.ContextData context_data = 1;
inline bool Execve::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Execve::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Execve::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.context_data)
  return _internal_context_data();
}
inline void Execve::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Execve.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Execve::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Execve::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Execve.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Execve::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Execve::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.context_data)
  return _msg;
}
inline void Execve::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Execve.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Execve::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000020U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Execve::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline const ::gvisor::syscall::Exit& Execve::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Execve::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.exit)
  return _internal_exit();
}
inline void Execve::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Execve.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Execve::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Execve::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Execve.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Execve::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Execve::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.exit)
  return _msg;
}
inline void Execve::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Execve.exit)
}

// uint64 sysno = 3;
inline void Execve::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t Execve::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.sysno)
  return _internal_sysno();
}
inline void Execve::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.sysno)
}
inline ::uint64_t Execve::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Execve::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Execve::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int64_t Execve::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.fd)
  return _internal_fd();
}
inline void Execve::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.fd)
}
inline ::int64_t Execve::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Execve::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Execve::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::std::string& Execve::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Execve::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Execve::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.fd_path)
  return _s;
}
inline const ::std::string& Execve::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Execve::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Execve::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Execve::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Execve.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000004U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Execve::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Execve.fd_path)
}

// string pathname = 6;
inline void Execve::clear_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::std::string& Execve::pathname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.pathname)
  return _internal_pathname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Execve::set_pathname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  _impl_.pathname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.pathname)
}
inline ::std::string* PROTOBUF_NONNULL Execve::mutable_pathname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::std::string* _s = _internal_mutable_pathname();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.pathname)
  return _s;
}
inline const ::std::string& Execve::_internal_pathname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pathname_.Get();
}
inline void Execve::_internal_set_pathname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Execve::_internal_mutable_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pathname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Execve::release_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Execve.pathname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000008U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  auto* released = _impl_.pathname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pathname_.Set("", GetArena());
  }
  return released;
}
inline void Execve::set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  _impl_.pathname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pathname_.IsDefault()) {
    _impl_.pathname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Execve.pathname)
}

// repeated string argv = 7;
inline int Execve::_internal_argv_size() const {
  return _internal_argv().size();
}
inline int Execve::argv_size() const {
  return _internal_argv_size();
}
inline void Execve::clear_argv() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.argv_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::std::string* PROTOBUF_NONNULL Execve::add_argv()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_argv()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.syscall.Execve.argv)
  return _s;
}
inline const ::std::string& Execve::argv(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.argv)
  return _internal_argv().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Execve::mutable_argv(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.argv)
  return _internal_mutable_argv()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Execve::set_argv(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_argv()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.argv)
}
template <typename Arg_, typename... Args_>
inline void Execve::add_argv(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_argv(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_add:gvisor.syscall.Execve.argv)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Execve::argv()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.syscall.Execve.argv)
  return _internal_argv();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Execve::mutable_argv() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.syscall.Execve.argv)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_argv();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Execve::_internal_argv() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.argv_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Execve::_internal_mutable_argv() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.argv_;
}

// repeated string envv = 8;
inline int Execve::_internal_envv_size() const {
  return _internal_envv().size();
}
inline int Execve::envv_size() const {
  return _internal_envv_size();
}
inline void Execve::clear_envv() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.envv_.Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::std::string* PROTOBUF_NONNULL Execve::add_envv()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::std::string* _s =
      _internal_mutable_envv()->InternalAddWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), GetArena());
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add_mutable:gvisor.syscall.Execve.envv)
  return _s;
}
inline const ::std::string& Execve::envv(int index) const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.envv)
  return _internal_envv().Get(index);
}
inline ::std::string* PROTOBUF_NONNULL Execve::mutable_envv(int index)
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Execve.envv)
  return _internal_mutable_envv()->Mutable(index);
}
template <typename Arg_, typename... Args_>
inline void Execve::set_envv(int index, Arg_&& value, Args_... args) {
  ::google::protobuf::internal::AssignToString(
      *_internal_mutable_envv()->Mutable(index),
      ::std::forward<Arg_>(value), args... );
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.envv)
}
template <typename Arg_, typename... Args_>
inline void Execve::add_envv(Arg_&& value, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::google::protobuf::internal::AddToRepeatedPtrField(
      ::google::protobuf::MessageLite::internal_visibility(), GetArena(),
      *_internal_mutable_envv(), ::std::forward<Arg_>(value),
      args... );
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_add:gvisor.syscall.Execve.envv)
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>& Execve::envv()
    const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_list:gvisor.syscall.Execve.envv)
  return _internal_envv();
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Execve::mutable_envv() ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_mutable_list:gvisor.syscall.Execve.envv)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _internal_mutable_envv();
}
inline const ::google::protobuf::RepeatedPtrField<::std::string>&
Execve::_internal_envv() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.envv_;
}
inline ::google::protobuf::RepeatedPtrField<::std::string>* PROTOBUF_NONNULL
Execve::_internal_mutable_envv() {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return &_impl_.envv_;
}

// uint32 flags = 9;
inline void Execve::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000100U);
}
inline ::uint32_t Execve::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Execve.flags)
  return _internal_flags();
}
inline void Execve::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000100U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Execve.flags)
}
inline ::uint32_t Execve::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Execve::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Socket

// .gvisor.common.ContextData context_data = 1;
inline bool Socket::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Socket::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Socket::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.context_data)
  return _internal_context_data();
}
inline void Socket::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Socket.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Socket::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Socket::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Socket.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Socket::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Socket::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Socket.context_data)
  return _msg;
}
inline void Socket::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Socket.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Socket::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Socket::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Socket::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Socket::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.exit)
  return _internal_exit();
}
inline void Socket::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Socket.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Socket::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Socket::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Socket.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Socket::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Socket::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Socket.exit)
  return _msg;
}
inline void Socket::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Socket.exit)
}

// uint64 sysno = 3;
inline void Socket::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Socket::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.sysno)
  return _internal_sysno();
}
inline void Socket::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Socket.sysno)
}
inline ::uint64_t Socket::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Socket::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 domain = 4;
inline void Socket::clear_domain() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.domain_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t Socket::domain() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.domain)
  return _internal_domain();
}
inline void Socket::set_domain(::int32_t value) {
  _internal_set_domain(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Socket.domain)
}
inline ::int32_t Socket::_internal_domain() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.domain_;
}
inline void Socket::_internal_set_domain(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.domain_ = value;
}

// int32 type = 5;
inline void Socket::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Socket::type() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.type)
  return _internal_type();
}
inline void Socket::set_type(::int32_t value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Socket.type)
}
inline ::int32_t Socket::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_;
}
inline void Socket::_internal_set_type(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// int32 protocol = 6;
inline void Socket::clear_protocol() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Socket::protocol() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Socket.protocol)
  return _internal_protocol();
}
inline void Socket::set_protocol(::int32_t value) {
  _internal_set_protocol(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Socket.protocol)
}
inline ::int32_t Socket::_internal_protocol() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.protocol_;
}
inline void Socket::_internal_set_protocol(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = value;
}

// -------------------------------------------------------------------

// Chdir

// .gvisor.common.ContextData context_data = 1;
inline bool Chdir::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Chdir::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Chdir::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.context_data)
  return _internal_context_data();
}
inline void Chdir::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Chdir.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Chdir::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Chdir::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chdir.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Chdir::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Chdir::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chdir.context_data)
  return _msg;
}
inline void Chdir::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chdir.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Chdir::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Chdir::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& Chdir::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Chdir::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.exit)
  return _internal_exit();
}
inline void Chdir::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Chdir.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Chdir::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Chdir::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chdir.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Chdir::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Chdir::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chdir.exit)
  return _msg;
}
inline void Chdir::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chdir.exit)
}

// uint64 sysno = 3;
inline void Chdir::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Chdir::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.sysno)
  return _internal_sysno();
}
inline void Chdir::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chdir.sysno)
}
inline ::uint64_t Chdir::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Chdir::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int64 fd = 4;
inline void Chdir::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int64_t Chdir::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.fd)
  return _internal_fd();
}
inline void Chdir::set_fd(::int64_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chdir.fd)
}
inline ::int64_t Chdir::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Chdir::_internal_set_fd(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Chdir::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Chdir::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Chdir::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chdir.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Chdir::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chdir.fd_path)
  return _s;
}
inline const ::std::string& Chdir::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Chdir::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Chdir::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Chdir::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chdir.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Chdir::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chdir.fd_path)
}

// string pathname = 6;
inline void Chdir::clear_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Chdir::pathname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chdir.pathname)
  return _internal_pathname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Chdir::set_pathname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pathname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chdir.pathname)
}
inline ::std::string* PROTOBUF_NONNULL Chdir::mutable_pathname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pathname();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chdir.pathname)
  return _s;
}
inline const ::std::string& Chdir::_internal_pathname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pathname_.Get();
}
inline void Chdir::_internal_set_pathname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Chdir::_internal_mutable_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pathname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Chdir::release_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chdir.pathname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pathname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pathname_.Set("", GetArena());
  }
  return released;
}
inline void Chdir::set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pathname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pathname_.IsDefault()) {
    _impl_.pathname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chdir.pathname)
}

// -------------------------------------------------------------------

// Setresid

// .gvisor.common.ContextData context_data = 1;
inline bool Setresid::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Setresid::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Setresid::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.context_data)
  return _internal_context_data();
}
inline void Setresid::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Setresid.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Setresid::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Setresid::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Setresid.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Setresid::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Setresid::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Setresid.context_data)
  return _msg;
}
inline void Setresid::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Setresid.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Setresid::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Setresid::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Setresid::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Setresid::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.exit)
  return _internal_exit();
}
inline void Setresid::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Setresid.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Setresid::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Setresid::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Setresid.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Setresid::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Setresid::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Setresid.exit)
  return _msg;
}
inline void Setresid::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Setresid.exit)
}

// uint64 sysno = 3;
inline void Setresid::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Setresid::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.sysno)
  return _internal_sysno();
}
inline void Setresid::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setresid.sysno)
}
inline ::uint64_t Setresid::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Setresid::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// uint32 rid = 4;
inline void Setresid::clear_rid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint32_t Setresid::rid() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.rid)
  return _internal_rid();
}
inline void Setresid::set_rid(::uint32_t value) {
  _internal_set_rid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setresid.rid)
}
inline ::uint32_t Setresid::_internal_rid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.rid_;
}
inline void Setresid::_internal_set_rid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.rid_ = value;
}

// uint32 eid = 5;
inline void Setresid::clear_eid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.eid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint32_t Setresid::eid() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.eid)
  return _internal_eid();
}
inline void Setresid::set_eid(::uint32_t value) {
  _internal_set_eid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setresid.eid)
}
inline ::uint32_t Setresid::_internal_eid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.eid_;
}
inline void Setresid::_internal_set_eid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.eid_ = value;
}

// uint32 sid = 6;
inline void Setresid::clear_sid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sid_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint32_t Setresid::sid() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setresid.sid)
  return _internal_sid();
}
inline void Setresid::set_sid(::uint32_t value) {
  _internal_set_sid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setresid.sid)
}
inline ::uint32_t Setresid::_internal_sid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sid_;
}
inline void Setresid::_internal_set_sid(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sid_ = value;
}

// -------------------------------------------------------------------

// Setid

// .gvisor.common.ContextData context_data = 1;
inline bool Setid::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Setid::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Setid::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setid.context_data)
  return _internal_context_data();
}
inline void Setid::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Setid.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Setid::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Setid::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Setid.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Setid::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Setid::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Setid.context_data)
  return _msg;
}
inline void Setid::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Setid.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Setid::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Setid::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Setid::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Setid::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setid.exit)
  return _internal_exit();
}
inline void Setid::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Setid.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Setid::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Setid::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Setid.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Setid::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Setid::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Setid.exit)
  return _msg;
}
inline void Setid::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Setid.exit)
}

// uint64 sysno = 3;
inline void Setid::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Setid::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setid.sysno)
  return _internal_sysno();
}
inline void Setid::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setid.sysno)
}
inline ::uint64_t Setid::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Setid::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// uint32 id = 4;
inline void Setid::clear_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint32_t Setid::id() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Setid.id)
  return _internal_id();
}
inline void Setid::set_id(::uint32_t value) {
  _internal_set_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Setid.id)
}
inline ::uint32_t Setid::_internal_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.id_;
}
inline void Setid::_internal_set_id(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.id_ = value;
}

// -------------------------------------------------------------------

// StructRlimit

// uint64 cur = 1;
inline void StructRlimit::clear_cur() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cur_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::uint64_t StructRlimit::cur() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.StructRlimit.cur)
  return _internal_cur();
}
inline void StructRlimit::set_cur(::uint64_t value) {
  _internal_set_cur(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.StructRlimit.cur)
}
inline ::uint64_t StructRlimit::_internal_cur() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cur_;
}
inline void StructRlimit::_internal_set_cur(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cur_ = value;
}

// uint64 max = 2;
inline void StructRlimit::clear_max() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::uint64_t StructRlimit::max() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.StructRlimit.max)
  return _internal_max();
}
inline void StructRlimit::set_max(::uint64_t value) {
  _internal_set_max(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.StructRlimit.max)
}
inline ::uint64_t StructRlimit::_internal_max() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.max_;
}
inline void StructRlimit::_internal_set_max(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.max_ = value;
}

// -------------------------------------------------------------------

// Prlimit

// .gvisor.common.ContextData context_data = 1;
inline bool Prlimit::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Prlimit::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Prlimit::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.context_data)
  return _internal_context_data();
}
inline void Prlimit::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Prlimit.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Prlimit::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Prlimit::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Prlimit.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Prlimit::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Prlimit::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Prlimit.context_data)
  return _msg;
}
inline void Prlimit::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Prlimit.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Prlimit::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Prlimit::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Prlimit::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Prlimit::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.exit)
  return _internal_exit();
}
inline void Prlimit::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Prlimit.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Prlimit::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Prlimit::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Prlimit.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Prlimit::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Prlimit::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Prlimit.exit)
  return _msg;
}
inline void Prlimit::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Prlimit.exit)
}

// uint64 sysno = 3;
inline void Prlimit::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Prlimit::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.sysno)
  return _internal_sysno();
}
inline void Prlimit::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Prlimit.sysno)
}
inline ::uint64_t Prlimit::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Prlimit::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 pid = 4;
inline void Prlimit::clear_pid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pid_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int32_t Prlimit::pid() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.pid)
  return _internal_pid();
}
inline void Prlimit::set_pid(::int32_t value) {
  _internal_set_pid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Prlimit.pid)
}
inline ::int32_t Prlimit::_internal_pid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pid_;
}
inline void Prlimit::_internal_set_pid(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pid_ = value;
}

// int64 resource = 5;
inline void Prlimit::clear_resource() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.resource_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int64_t Prlimit::resource() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.resource)
  return _internal_resource();
}
inline void Prlimit::set_resource(::int64_t value) {
  _internal_set_resource(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Prlimit.resource)
}
inline ::int64_t Prlimit::_internal_resource() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.resource_;
}
inline void Prlimit::_internal_set_resource(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.resource_ = value;
}

// .gvisor.syscall.StructRlimit new_limit = 6;
inline bool Prlimit::has_new_limit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.new_limit_ != nullptr);
  return value;
}
inline void Prlimit::clear_new_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.new_limit_ != nullptr) _impl_.new_limit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::StructRlimit& Prlimit::_internal_new_limit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::StructRlimit* p = _impl_.new_limit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::StructRlimit>(&::gvisor::syscall::StructRlimit_globals_);
}
inline const ::gvisor::syscall::StructRlimit& Prlimit::new_limit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.new_limit)
  return _internal_new_limit();
}
inline void Prlimit::unsafe_arena_set_allocated_new_limit(
    ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.new_limit_);
  }
  _impl_.new_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Prlimit.new_limit)
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE Prlimit::release_new_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::StructRlimit* released = _impl_.new_limit_;
  _impl_.new_limit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE Prlimit::unsafe_arena_release_new_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Prlimit.new_limit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::StructRlimit* temp = _impl_.new_limit_;
  _impl_.new_limit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL Prlimit::_internal_mutable_new_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.new_limit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::StructRlimit>(GetArena());
    _impl_.new_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(p);
  }
  return _impl_.new_limit_;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL Prlimit::mutable_new_limit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::StructRlimit* _msg = _internal_mutable_new_limit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Prlimit.new_limit)
  return _msg;
}
inline void Prlimit::set_allocated_new_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.new_limit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.new_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Prlimit.new_limit)
}

// .gvisor.syscall.StructRlimit old_limit = 7;
inline bool Prlimit::has_old_limit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.old_limit_ != nullptr);
  return value;
}
inline void Prlimit::clear_old_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.old_limit_ != nullptr) _impl_.old_limit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::StructRlimit& Prlimit::_internal_old_limit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::StructRlimit* p = _impl_.old_limit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::StructRlimit>(&::gvisor::syscall::StructRlimit_globals_);
}
inline const ::gvisor::syscall::StructRlimit& Prlimit::old_limit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Prlimit.old_limit)
  return _internal_old_limit();
}
inline void Prlimit::unsafe_arena_set_allocated_old_limit(
    ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.old_limit_);
  }
  _impl_.old_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Prlimit.old_limit)
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE Prlimit::release_old_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::StructRlimit* released = _impl_.old_limit_;
  _impl_.old_limit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE Prlimit::unsafe_arena_release_old_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Prlimit.old_limit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::StructRlimit* temp = _impl_.old_limit_;
  _impl_.old_limit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL Prlimit::_internal_mutable_old_limit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.old_limit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::StructRlimit>(GetArena());
    _impl_.old_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(p);
  }
  return _impl_.old_limit_;
}
inline ::gvisor::syscall::StructRlimit* PROTOBUF_NONNULL Prlimit::mutable_old_limit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::StructRlimit* _msg = _internal_mutable_old_limit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Prlimit.old_limit)
  return _msg;
}
inline void Prlimit::set_allocated_old_limit(::gvisor::syscall::StructRlimit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.old_limit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.old_limit_ = reinterpret_cast<::gvisor::syscall::StructRlimit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Prlimit.old_limit)
}

// -------------------------------------------------------------------

// Pipe

// .gvisor.common.ContextData context_data = 1;
inline bool Pipe::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Pipe::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Pipe::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.context_data)
  return _internal_context_data();
}
inline void Pipe::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Pipe.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Pipe::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Pipe::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Pipe.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Pipe::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Pipe::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Pipe.context_data)
  return _msg;
}
inline void Pipe::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Pipe.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Pipe::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Pipe::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Pipe::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Pipe::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.exit)
  return _internal_exit();
}
inline void Pipe::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Pipe.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Pipe::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Pipe::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Pipe.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Pipe::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Pipe::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Pipe.exit)
  return _msg;
}
inline void Pipe::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Pipe.exit)
}

// uint64 sysno = 3;
inline void Pipe::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Pipe::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.sysno)
  return _internal_sysno();
}
inline void Pipe::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Pipe.sysno)
}
inline ::uint64_t Pipe::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Pipe::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// uint32 flags = 4;
inline void Pipe::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint32_t Pipe::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.flags)
  return _internal_flags();
}
inline void Pipe::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Pipe.flags)
}
inline ::uint32_t Pipe::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Pipe::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// int32 reader = 5;
inline void Pipe::clear_reader() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reader_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Pipe::reader() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.reader)
  return _internal_reader();
}
inline void Pipe::set_reader(::int32_t value) {
  _internal_set_reader(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Pipe.reader)
}
inline ::int32_t Pipe::_internal_reader() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.reader_;
}
inline void Pipe::_internal_set_reader(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.reader_ = value;
}

// int32 writer = 6;
inline void Pipe::clear_writer() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.writer_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Pipe::writer() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Pipe.writer)
  return _internal_writer();
}
inline void Pipe::set_writer(::int32_t value) {
  _internal_set_writer(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Pipe.writer)
}
inline ::int32_t Pipe::_internal_writer() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.writer_;
}
inline void Pipe::_internal_set_writer(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.writer_ = value;
}

// -------------------------------------------------------------------

// Fcntl

// .gvisor.common.ContextData context_data = 1;
inline bool Fcntl::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Fcntl::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Fcntl::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.context_data)
  return _internal_context_data();
}
inline void Fcntl::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Fcntl.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Fcntl::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Fcntl::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Fcntl.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Fcntl::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Fcntl::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Fcntl.context_data)
  return _msg;
}
inline void Fcntl::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Fcntl.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Fcntl::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Fcntl::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Fcntl::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Fcntl::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.exit)
  return _internal_exit();
}
inline void Fcntl::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Fcntl.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Fcntl::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Fcntl::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Fcntl.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Fcntl::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Fcntl::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Fcntl.exit)
  return _msg;
}
inline void Fcntl::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Fcntl.exit)
}

// uint64 sysno = 3;
inline void Fcntl::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Fcntl::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.sysno)
  return _internal_sysno();
}
inline void Fcntl::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fcntl.sysno)
}
inline ::uint64_t Fcntl::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Fcntl::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void Fcntl::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Fcntl::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.fd)
  return _internal_fd();
}
inline void Fcntl::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fcntl.fd)
}
inline ::int32_t Fcntl::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Fcntl::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Fcntl::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Fcntl::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Fcntl::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fcntl.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Fcntl::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Fcntl.fd_path)
  return _s;
}
inline const ::std::string& Fcntl::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Fcntl::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Fcntl::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Fcntl::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Fcntl.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Fcntl::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Fcntl.fd_path)
}

// int32 cmd = 6;
inline void Fcntl::clear_cmd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cmd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Fcntl::cmd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.cmd)
  return _internal_cmd();
}
inline void Fcntl::set_cmd(::int32_t value) {
  _internal_set_cmd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fcntl.cmd)
}
inline ::int32_t Fcntl::_internal_cmd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.cmd_;
}
inline void Fcntl::_internal_set_cmd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.cmd_ = value;
}

// int64 args = 7;
inline void Fcntl::clear_args() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.args_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int64_t Fcntl::args() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fcntl.args)
  return _internal_args();
}
inline void Fcntl::set_args(::int64_t value) {
  _internal_set_args(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fcntl.args)
}
inline ::int64_t Fcntl::_internal_args() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.args_;
}
inline void Fcntl::_internal_set_args(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.args_ = value;
}

// -------------------------------------------------------------------

// Dup

// .gvisor.common.ContextData context_data = 1;
inline bool Dup::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Dup::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Dup::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.context_data)
  return _internal_context_data();
}
inline void Dup::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Dup.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Dup::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Dup::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Dup.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Dup::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Dup::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Dup.context_data)
  return _msg;
}
inline void Dup::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Dup.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Dup::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Dup::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Dup::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Dup::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.exit)
  return _internal_exit();
}
inline void Dup::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Dup.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Dup::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Dup::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Dup.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Dup::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Dup::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Dup.exit)
  return _msg;
}
inline void Dup::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Dup.exit)
}

// uint64 sysno = 3;
inline void Dup::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Dup::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.sysno)
  return _internal_sysno();
}
inline void Dup::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Dup.sysno)
}
inline ::uint64_t Dup::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Dup::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 old_fd = 4;
inline void Dup::clear_old_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.old_fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Dup::old_fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.old_fd)
  return _internal_old_fd();
}
inline void Dup::set_old_fd(::int32_t value) {
  _internal_set_old_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Dup.old_fd)
}
inline ::int32_t Dup::_internal_old_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.old_fd_;
}
inline void Dup::_internal_set_old_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.old_fd_ = value;
}

// int32 new_fd = 5;
inline void Dup::clear_new_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.new_fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Dup::new_fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.new_fd)
  return _internal_new_fd();
}
inline void Dup::set_new_fd(::int32_t value) {
  _internal_set_new_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Dup.new_fd)
}
inline ::int32_t Dup::_internal_new_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.new_fd_;
}
inline void Dup::_internal_set_new_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.new_fd_ = value;
}

// string fd_path = 6;
inline void Dup::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Dup::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Dup::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Dup.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Dup::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Dup.fd_path)
  return _s;
}
inline const ::std::string& Dup::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Dup::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Dup::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Dup::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Dup.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Dup::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Dup.fd_path)
}

// uint32 flags = 7;
inline void Dup::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint32_t Dup::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Dup.flags)
  return _internal_flags();
}
inline void Dup::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Dup.flags)
}
inline ::uint32_t Dup::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Dup::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Signalfd

// .gvisor.common.ContextData context_data = 1;
inline bool Signalfd::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Signalfd::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Signalfd::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.context_data)
  return _internal_context_data();
}
inline void Signalfd::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Signalfd.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Signalfd::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Signalfd::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Signalfd.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Signalfd::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Signalfd::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Signalfd.context_data)
  return _msg;
}
inline void Signalfd::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Signalfd.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Signalfd::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Signalfd::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Signalfd::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Signalfd::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.exit)
  return _internal_exit();
}
inline void Signalfd::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Signalfd.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Signalfd::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Signalfd::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Signalfd.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Signalfd::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Signalfd::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Signalfd.exit)
  return _msg;
}
inline void Signalfd::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Signalfd.exit)
}

// uint64 sysno = 3;
inline void Signalfd::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Signalfd::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.sysno)
  return _internal_sysno();
}
inline void Signalfd::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Signalfd.sysno)
}
inline ::uint64_t Signalfd::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Signalfd::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void Signalfd::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t Signalfd::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.fd)
  return _internal_fd();
}
inline void Signalfd::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Signalfd.fd)
}
inline ::int32_t Signalfd::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Signalfd::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Signalfd::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Signalfd::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Signalfd::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Signalfd.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Signalfd::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Signalfd.fd_path)
  return _s;
}
inline const ::std::string& Signalfd::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Signalfd::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Signalfd::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Signalfd::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Signalfd.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Signalfd::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Signalfd.fd_path)
}

// uint64 sigset = 6;
inline void Signalfd::clear_sigset() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sigset_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t Signalfd::sigset() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.sigset)
  return _internal_sigset();
}
inline void Signalfd::set_sigset(::uint64_t value) {
  _internal_set_sigset(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Signalfd.sigset)
}
inline ::uint64_t Signalfd::_internal_sigset() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sigset_;
}
inline void Signalfd::_internal_set_sigset(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sigset_ = value;
}

// int32 flags = 7;
inline void Signalfd::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Signalfd::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Signalfd.flags)
  return _internal_flags();
}
inline void Signalfd::set_flags(::int32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Signalfd.flags)
}
inline ::int32_t Signalfd::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Signalfd::_internal_set_flags(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Chroot

// .gvisor.common.ContextData context_data = 1;
inline bool Chroot::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Chroot::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Chroot::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chroot.context_data)
  return _internal_context_data();
}
inline void Chroot::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Chroot.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Chroot::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Chroot::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chroot.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Chroot::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Chroot::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chroot.context_data)
  return _msg;
}
inline void Chroot::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chroot.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Chroot::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Chroot::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& Chroot::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Chroot::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chroot.exit)
  return _internal_exit();
}
inline void Chroot::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Chroot.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Chroot::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Chroot::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chroot.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Chroot::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Chroot::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chroot.exit)
  return _msg;
}
inline void Chroot::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chroot.exit)
}

// uint64 sysno = 3;
inline void Chroot::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Chroot::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chroot.sysno)
  return _internal_sysno();
}
inline void Chroot::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chroot.sysno)
}
inline ::uint64_t Chroot::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Chroot::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// string pathname = 4;
inline void Chroot::clear_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Chroot::pathname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Chroot.pathname)
  return _internal_pathname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Chroot::set_pathname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.pathname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Chroot.pathname)
}
inline ::std::string* PROTOBUF_NONNULL Chroot::mutable_pathname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_pathname();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Chroot.pathname)
  return _s;
}
inline const ::std::string& Chroot::_internal_pathname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pathname_.Get();
}
inline void Chroot::_internal_set_pathname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Chroot::_internal_mutable_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pathname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Chroot::release_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Chroot.pathname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.pathname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pathname_.Set("", GetArena());
  }
  return released;
}
inline void Chroot::set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.pathname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pathname_.IsDefault()) {
    _impl_.pathname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Chroot.pathname)
}

// -------------------------------------------------------------------

// Eventfd

// .gvisor.common.ContextData context_data = 1;
inline bool Eventfd::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Eventfd::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Eventfd::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Eventfd.context_data)
  return _internal_context_data();
}
inline void Eventfd::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Eventfd.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Eventfd::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Eventfd::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Eventfd.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Eventfd::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Eventfd::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Eventfd.context_data)
  return _msg;
}
inline void Eventfd::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Eventfd.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Eventfd::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Eventfd::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Eventfd::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Eventfd::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Eventfd.exit)
  return _internal_exit();
}
inline void Eventfd::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Eventfd.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Eventfd::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Eventfd::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Eventfd.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Eventfd::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Eventfd::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Eventfd.exit)
  return _msg;
}
inline void Eventfd::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Eventfd.exit)
}

// uint64 sysno = 3;
inline void Eventfd::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Eventfd::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Eventfd.sysno)
  return _internal_sysno();
}
inline void Eventfd::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Eventfd.sysno)
}
inline ::uint64_t Eventfd::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Eventfd::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 val = 4;
inline void Eventfd::clear_val() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.val_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t Eventfd::val() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Eventfd.val)
  return _internal_val();
}
inline void Eventfd::set_val(::int32_t value) {
  _internal_set_val(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Eventfd.val)
}
inline ::int32_t Eventfd::_internal_val() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.val_;
}
inline void Eventfd::_internal_set_val(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.val_ = value;
}

// uint32 flags = 5;
inline void Eventfd::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint32_t Eventfd::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Eventfd.flags)
  return _internal_flags();
}
inline void Eventfd::set_flags(::uint32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Eventfd.flags)
}
inline ::uint32_t Eventfd::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Eventfd::_internal_set_flags(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Clone

// .gvisor.common.ContextData context_data = 1;
inline bool Clone::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Clone::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Clone::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.context_data)
  return _internal_context_data();
}
inline void Clone::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Clone.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Clone::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Clone::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Clone.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Clone::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Clone::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Clone.context_data)
  return _msg;
}
inline void Clone::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Clone.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Clone::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Clone::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Clone::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Clone::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.exit)
  return _internal_exit();
}
inline void Clone::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Clone.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Clone::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Clone::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Clone.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Clone::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Clone::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Clone.exit)
  return _msg;
}
inline void Clone::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Clone.exit)
}

// uint64 sysno = 3;
inline void Clone::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Clone::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.sysno)
  return _internal_sysno();
}
inline void Clone::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Clone.sysno)
}
inline ::uint64_t Clone::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Clone::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// uint64 flags = 4;
inline void Clone::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t Clone::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.flags)
  return _internal_flags();
}
inline void Clone::set_flags(::uint64_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Clone.flags)
}
inline ::uint64_t Clone::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Clone::_internal_set_flags(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// uint64 stack = 5;
inline void Clone::clear_stack() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stack_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Clone::stack() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.stack)
  return _internal_stack();
}
inline void Clone::set_stack(::uint64_t value) {
  _internal_set_stack(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Clone.stack)
}
inline ::uint64_t Clone::_internal_stack() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.stack_;
}
inline void Clone::_internal_set_stack(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.stack_ = value;
}

// uint64 new_tid = 6;
inline void Clone::clear_new_tid() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.new_tid_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t Clone::new_tid() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.new_tid)
  return _internal_new_tid();
}
inline void Clone::set_new_tid(::uint64_t value) {
  _internal_set_new_tid(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Clone.new_tid)
}
inline ::uint64_t Clone::_internal_new_tid() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.new_tid_;
}
inline void Clone::_internal_set_new_tid(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.new_tid_ = value;
}

// uint64 tls = 7;
inline void Clone::clear_tls() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tls_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint64_t Clone::tls() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Clone.tls)
  return _internal_tls();
}
inline void Clone::set_tls(::uint64_t value) {
  _internal_set_tls(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Clone.tls)
}
inline ::uint64_t Clone::_internal_tls() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.tls_;
}
inline void Clone::_internal_set_tls(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.tls_ = value;
}

// -------------------------------------------------------------------

// Bind

// .gvisor.common.ContextData context_data = 1;
inline bool Bind::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Bind::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Bind::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.context_data)
  return _internal_context_data();
}
inline void Bind::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Bind.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Bind::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Bind::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Bind.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Bind::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Bind::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Bind.context_data)
  return _msg;
}
inline void Bind::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Bind.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Bind::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Bind::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& Bind::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Bind::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.exit)
  return _internal_exit();
}
inline void Bind::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Bind.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Bind::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Bind::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Bind.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Bind::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Bind::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Bind.exit)
  return _msg;
}
inline void Bind::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Bind.exit)
}

// uint64 sysno = 3;
inline void Bind::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Bind::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.sysno)
  return _internal_sysno();
}
inline void Bind::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Bind.sysno)
}
inline ::uint64_t Bind::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Bind::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void Bind::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Bind::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.fd)
  return _internal_fd();
}
inline void Bind::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Bind.fd)
}
inline ::int32_t Bind::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Bind::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Bind::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Bind::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Bind::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Bind.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Bind::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Bind.fd_path)
  return _s;
}
inline const ::std::string& Bind::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Bind::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Bind::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Bind::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Bind.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Bind::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Bind.fd_path)
}

// bytes address = 6;
inline void Bind::clear_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Bind::address() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Bind.address)
  return _internal_address();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Bind::set_address(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.address_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Bind.address)
}
inline ::std::string* PROTOBUF_NONNULL Bind::mutable_address()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_address();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Bind.address)
  return _s;
}
inline const ::std::string& Bind::_internal_address() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.address_.Get();
}
inline void Bind::_internal_set_address(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Bind::_internal_mutable_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.address_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Bind::release_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Bind.address)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.address_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.address_.Set("", GetArena());
  }
  return released;
}
inline void Bind::set_allocated_address(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.address_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.address_.IsDefault()) {
    _impl_.address_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Bind.address)
}

// -------------------------------------------------------------------

// Accept

// .gvisor.common.ContextData context_data = 1;
inline bool Accept::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Accept::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Accept::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.context_data)
  return _internal_context_data();
}
inline void Accept::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Accept.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Accept::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Accept::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Accept.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Accept::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Accept::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Accept.context_data)
  return _msg;
}
inline void Accept::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Accept.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Accept::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Accept::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& Accept::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Accept::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.exit)
  return _internal_exit();
}
inline void Accept::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Accept.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Accept::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Accept::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Accept.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Accept::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Accept::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Accept.exit)
  return _msg;
}
inline void Accept::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Accept.exit)
}

// uint64 sysno = 3;
inline void Accept::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t Accept::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.sysno)
  return _internal_sysno();
}
inline void Accept::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Accept.sysno)
}
inline ::uint64_t Accept::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Accept::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void Accept::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t Accept::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.fd)
  return _internal_fd();
}
inline void Accept::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Accept.fd)
}
inline ::int32_t Accept::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void Accept::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void Accept::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& Accept::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Accept::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Accept.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL Accept::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Accept.fd_path)
  return _s;
}
inline const ::std::string& Accept::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void Accept::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Accept::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Accept::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Accept.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void Accept::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Accept.fd_path)
}

// bytes address = 6;
inline void Accept::clear_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& Accept::address() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.address)
  return _internal_address();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void Accept::set_address(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.address_.SetBytes(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.Accept.address)
}
inline ::std::string* PROTOBUF_NONNULL Accept::mutable_address()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_address();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Accept.address)
  return _s;
}
inline const ::std::string& Accept::_internal_address() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.address_.Get();
}
inline void Accept::_internal_set_address(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.address_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL Accept::_internal_mutable_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.address_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE Accept::release_address() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Accept.address)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.address_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.address_.Set("", GetArena());
  }
  return released;
}
inline void Accept::set_allocated_address(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.address_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.address_.IsDefault()) {
    _impl_.address_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Accept.address)
}

// int32 flags = 7;
inline void Accept::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int32_t Accept::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Accept.flags)
  return _internal_flags();
}
inline void Accept::set_flags(::int32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Accept.flags)
}
inline ::int32_t Accept::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void Accept::_internal_set_flags(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// TimerfdCreate

// .gvisor.common.ContextData context_data = 1;
inline bool TimerfdCreate::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& TimerfdCreate::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& TimerfdCreate::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdCreate.context_data)
  return _internal_context_data();
}
inline void TimerfdCreate::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdCreate.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdCreate::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdCreate::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdCreate.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdCreate::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdCreate::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdCreate.context_data)
  return _msg;
}
inline void TimerfdCreate::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdCreate.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool TimerfdCreate::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void TimerfdCreate::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& TimerfdCreate::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& TimerfdCreate::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdCreate.exit)
  return _internal_exit();
}
inline void TimerfdCreate::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdCreate.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdCreate::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdCreate::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdCreate.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdCreate::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdCreate::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdCreate.exit)
  return _msg;
}
inline void TimerfdCreate::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdCreate.exit)
}

// uint64 sysno = 3;
inline void TimerfdCreate::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t TimerfdCreate::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdCreate.sysno)
  return _internal_sysno();
}
inline void TimerfdCreate::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdCreate.sysno)
}
inline ::uint64_t TimerfdCreate::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void TimerfdCreate::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 clock_id = 4;
inline void TimerfdCreate::clear_clock_id() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.clock_id_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t TimerfdCreate::clock_id() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdCreate.clock_id)
  return _internal_clock_id();
}
inline void TimerfdCreate::set_clock_id(::int32_t value) {
  _internal_set_clock_id(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdCreate.clock_id)
}
inline ::int32_t TimerfdCreate::_internal_clock_id() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.clock_id_;
}
inline void TimerfdCreate::_internal_set_clock_id(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.clock_id_ = value;
}

// int32 flags = 5;
inline void TimerfdCreate::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t TimerfdCreate::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdCreate.flags)
  return _internal_flags();
}
inline void TimerfdCreate::set_flags(::int32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdCreate.flags)
}
inline ::int32_t TimerfdCreate::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void TimerfdCreate::_internal_set_flags(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// Timespec

// int64 sec = 1;
inline void Timespec::clear_sec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sec_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline ::int64_t Timespec::sec() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Timespec.sec)
  return _internal_sec();
}
inline void Timespec::set_sec(::int64_t value) {
  _internal_set_sec(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Timespec.sec)
}
inline ::int64_t Timespec::_internal_sec() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sec_;
}
inline void Timespec::_internal_set_sec(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sec_ = value;
}

// int64 nsec = 2;
inline void Timespec::clear_nsec() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nsec_ = ::int64_t{0};
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline ::int64_t Timespec::nsec() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Timespec.nsec)
  return _internal_nsec();
}
inline void Timespec::set_nsec(::int64_t value) {
  _internal_set_nsec(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Timespec.nsec)
}
inline ::int64_t Timespec::_internal_nsec() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.nsec_;
}
inline void Timespec::_internal_set_nsec(::int64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.nsec_ = value;
}

// -------------------------------------------------------------------

// ItimerSpec

// .gvisor.syscall.Timespec interval = 1;
inline bool ItimerSpec::has_interval() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.interval_ != nullptr);
  return value;
}
inline void ItimerSpec::clear_interval() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.interval_ != nullptr) _impl_.interval_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::gvisor::syscall::Timespec& ItimerSpec::_internal_interval() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Timespec* p = _impl_.interval_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Timespec>(&::gvisor::syscall::Timespec_globals_);
}
inline const ::gvisor::syscall::Timespec& ItimerSpec::interval() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.ItimerSpec.interval)
  return _internal_interval();
}
inline void ItimerSpec::unsafe_arena_set_allocated_interval(
    ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.interval_);
  }
  _impl_.interval_ = reinterpret_cast<::gvisor::syscall::Timespec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.ItimerSpec.interval)
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE ItimerSpec::release_interval() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::syscall::Timespec* released = _impl_.interval_;
  _impl_.interval_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE ItimerSpec::unsafe_arena_release_interval() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.ItimerSpec.interval)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::syscall::Timespec* temp = _impl_.interval_;
  _impl_.interval_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NONNULL ItimerSpec::_internal_mutable_interval() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.interval_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Timespec>(GetArena());
    _impl_.interval_ = reinterpret_cast<::gvisor::syscall::Timespec*>(p);
  }
  return _impl_.interval_;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NONNULL ItimerSpec::mutable_interval()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::syscall::Timespec* _msg = _internal_mutable_interval();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.ItimerSpec.interval)
  return _msg;
}
inline void ItimerSpec::set_allocated_interval(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.interval_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.interval_ = reinterpret_cast<::gvisor::syscall::Timespec*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.ItimerSpec.interval)
}

// .gvisor.syscall.Timespec value = 2;
inline bool ItimerSpec::has_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.value_ != nullptr);
  return value;
}
inline void ItimerSpec::clear_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.value_ != nullptr) _impl_.value_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Timespec& ItimerSpec::_internal_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Timespec* p = _impl_.value_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Timespec>(&::gvisor::syscall::Timespec_globals_);
}
inline const ::gvisor::syscall::Timespec& ItimerSpec::value() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.ItimerSpec.value)
  return _internal_value();
}
inline void ItimerSpec::unsafe_arena_set_allocated_value(
    ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.value_);
  }
  _impl_.value_ = reinterpret_cast<::gvisor::syscall::Timespec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.ItimerSpec.value)
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE ItimerSpec::release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Timespec* released = _impl_.value_;
  _impl_.value_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NULLABLE ItimerSpec::unsafe_arena_release_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.ItimerSpec.value)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Timespec* temp = _impl_.value_;
  _impl_.value_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NONNULL ItimerSpec::_internal_mutable_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.value_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Timespec>(GetArena());
    _impl_.value_ = reinterpret_cast<::gvisor::syscall::Timespec*>(p);
  }
  return _impl_.value_;
}
inline ::gvisor::syscall::Timespec* PROTOBUF_NONNULL ItimerSpec::mutable_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Timespec* _msg = _internal_mutable_value();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.ItimerSpec.value)
  return _msg;
}
inline void ItimerSpec::set_allocated_value(::gvisor::syscall::Timespec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.value_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.value_ = reinterpret_cast<::gvisor::syscall::Timespec*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.ItimerSpec.value)
}

// -------------------------------------------------------------------

// TimerfdSetTime

// .gvisor.common.ContextData context_data = 1;
inline bool TimerfdSetTime::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& TimerfdSetTime::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& TimerfdSetTime::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.context_data)
  return _internal_context_data();
}
inline void TimerfdSetTime::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdSetTime.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdSetTime::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdSetTime::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdSetTime.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdSetTime::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdSetTime::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdSetTime.context_data)
  return _msg;
}
inline void TimerfdSetTime::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdSetTime.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool TimerfdSetTime::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void TimerfdSetTime::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& TimerfdSetTime::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& TimerfdSetTime::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.exit)
  return _internal_exit();
}
inline void TimerfdSetTime::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdSetTime.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdSetTime::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdSetTime::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdSetTime.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdSetTime::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdSetTime::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdSetTime.exit)
  return _msg;
}
inline void TimerfdSetTime::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdSetTime.exit)
}

// uint64 sysno = 3;
inline void TimerfdSetTime::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::uint64_t TimerfdSetTime::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.sysno)
  return _internal_sysno();
}
inline void TimerfdSetTime::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdSetTime.sysno)
}
inline ::uint64_t TimerfdSetTime::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void TimerfdSetTime::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void TimerfdSetTime::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int32_t TimerfdSetTime::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.fd)
  return _internal_fd();
}
inline void TimerfdSetTime::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdSetTime.fd)
}
inline ::int32_t TimerfdSetTime::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void TimerfdSetTime::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void TimerfdSetTime::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& TimerfdSetTime::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TimerfdSetTime::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdSetTime.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL TimerfdSetTime::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdSetTime.fd_path)
  return _s;
}
inline const ::std::string& TimerfdSetTime::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void TimerfdSetTime::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TimerfdSetTime::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TimerfdSetTime::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdSetTime.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void TimerfdSetTime::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdSetTime.fd_path)
}

// int32 flags = 6;
inline void TimerfdSetTime::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int32_t TimerfdSetTime::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.flags)
  return _internal_flags();
}
inline void TimerfdSetTime::set_flags(::int32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdSetTime.flags)
}
inline ::int32_t TimerfdSetTime::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void TimerfdSetTime::_internal_set_flags(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// .gvisor.syscall.ItimerSpec new_value = 7;
inline bool TimerfdSetTime::has_new_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.new_value_ != nullptr);
  return value;
}
inline void TimerfdSetTime::clear_new_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.new_value_ != nullptr) _impl_.new_value_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdSetTime::_internal_new_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::ItimerSpec* p = _impl_.new_value_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::ItimerSpec>(&::gvisor::syscall::ItimerSpec_globals_);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdSetTime::new_value() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.new_value)
  return _internal_new_value();
}
inline void TimerfdSetTime::unsafe_arena_set_allocated_new_value(
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.new_value_);
  }
  _impl_.new_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdSetTime.new_value)
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdSetTime::release_new_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* released = _impl_.new_value_;
  _impl_.new_value_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdSetTime::unsafe_arena_release_new_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdSetTime.new_value)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* temp = _impl_.new_value_;
  _impl_.new_value_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdSetTime::_internal_mutable_new_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.new_value_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::ItimerSpec>(GetArena());
    _impl_.new_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(p);
  }
  return _impl_.new_value_;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdSetTime::mutable_new_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* _msg = _internal_mutable_new_value();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdSetTime.new_value)
  return _msg;
}
inline void TimerfdSetTime::set_allocated_new_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.new_value_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.new_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdSetTime.new_value)
}

// .gvisor.syscall.ItimerSpec old_value = 8;
inline bool TimerfdSetTime::has_old_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000010U);
  PROTOBUF_ASSUME(!value || _impl_.old_value_ != nullptr);
  return value;
}
inline void TimerfdSetTime::clear_old_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.old_value_ != nullptr) _impl_.old_value_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdSetTime::_internal_old_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::ItimerSpec* p = _impl_.old_value_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::ItimerSpec>(&::gvisor::syscall::ItimerSpec_globals_);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdSetTime::old_value() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdSetTime.old_value)
  return _internal_old_value();
}
inline void TimerfdSetTime::unsafe_arena_set_allocated_old_value(
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.old_value_);
  }
  _impl_.old_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdSetTime.old_value)
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdSetTime::release_old_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::syscall::ItimerSpec* released = _impl_.old_value_;
  _impl_.old_value_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdSetTime::unsafe_arena_release_old_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdSetTime.old_value)

  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::syscall::ItimerSpec* temp = _impl_.old_value_;
  _impl_.old_value_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdSetTime::_internal_mutable_old_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.old_value_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::ItimerSpec>(GetArena());
    _impl_.old_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(p);
  }
  return _impl_.old_value_;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdSetTime::mutable_old_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  ::gvisor::syscall::ItimerSpec* _msg = _internal_mutable_old_value();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdSetTime.old_value)
  return _msg;
}
inline void TimerfdSetTime::set_allocated_old_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.old_value_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
  }

  _impl_.old_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdSetTime.old_value)
}

// -------------------------------------------------------------------

// TimerfdGetTime

// .gvisor.common.ContextData context_data = 1;
inline bool TimerfdGetTime::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& TimerfdGetTime::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& TimerfdGetTime::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.context_data)
  return _internal_context_data();
}
inline void TimerfdGetTime::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdGetTime.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdGetTime::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE TimerfdGetTime::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdGetTime.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdGetTime::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL TimerfdGetTime::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdGetTime.context_data)
  return _msg;
}
inline void TimerfdGetTime::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdGetTime.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool TimerfdGetTime::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void TimerfdGetTime::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& TimerfdGetTime::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& TimerfdGetTime::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.exit)
  return _internal_exit();
}
inline void TimerfdGetTime::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdGetTime.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdGetTime::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE TimerfdGetTime::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdGetTime.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdGetTime::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL TimerfdGetTime::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdGetTime.exit)
  return _msg;
}
inline void TimerfdGetTime::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdGetTime.exit)
}

// uint64 sysno = 3;
inline void TimerfdGetTime::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t TimerfdGetTime::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.sysno)
  return _internal_sysno();
}
inline void TimerfdGetTime::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdGetTime.sysno)
}
inline ::uint64_t TimerfdGetTime::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void TimerfdGetTime::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void TimerfdGetTime::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t TimerfdGetTime::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.fd)
  return _internal_fd();
}
inline void TimerfdGetTime::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdGetTime.fd)
}
inline ::int32_t TimerfdGetTime::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void TimerfdGetTime::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void TimerfdGetTime::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& TimerfdGetTime::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void TimerfdGetTime::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.TimerfdGetTime.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL TimerfdGetTime::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdGetTime.fd_path)
  return _s;
}
inline const ::std::string& TimerfdGetTime::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void TimerfdGetTime::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL TimerfdGetTime::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE TimerfdGetTime::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdGetTime.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void TimerfdGetTime::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdGetTime.fd_path)
}

// .gvisor.syscall.ItimerSpec cur_value = 6;
inline bool TimerfdGetTime::has_cur_value() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.cur_value_ != nullptr);
  return value;
}
inline void TimerfdGetTime::clear_cur_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.cur_value_ != nullptr) _impl_.cur_value_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdGetTime::_internal_cur_value() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::ItimerSpec* p = _impl_.cur_value_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::ItimerSpec>(&::gvisor::syscall::ItimerSpec_globals_);
}
inline const ::gvisor::syscall::ItimerSpec& TimerfdGetTime::cur_value() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.TimerfdGetTime.cur_value)
  return _internal_cur_value();
}
inline void TimerfdGetTime::unsafe_arena_set_allocated_cur_value(
    ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.cur_value_);
  }
  _impl_.cur_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.TimerfdGetTime.cur_value)
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdGetTime::release_cur_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* released = _impl_.cur_value_;
  _impl_.cur_value_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE TimerfdGetTime::unsafe_arena_release_cur_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.TimerfdGetTime.cur_value)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* temp = _impl_.cur_value_;
  _impl_.cur_value_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdGetTime::_internal_mutable_cur_value() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.cur_value_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::ItimerSpec>(GetArena());
    _impl_.cur_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(p);
  }
  return _impl_.cur_value_;
}
inline ::gvisor::syscall::ItimerSpec* PROTOBUF_NONNULL TimerfdGetTime::mutable_cur_value()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::ItimerSpec* _msg = _internal_mutable_cur_value();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.TimerfdGetTime.cur_value)
  return _msg;
}
inline void TimerfdGetTime::set_allocated_cur_value(::gvisor::syscall::ItimerSpec* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.cur_value_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.cur_value_ = reinterpret_cast<::gvisor::syscall::ItimerSpec*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.TimerfdGetTime.cur_value)
}

// -------------------------------------------------------------------

// Fork

// .gvisor.common.ContextData context_data = 1;
inline bool Fork::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& Fork::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& Fork::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fork.context_data)
  return _internal_context_data();
}
inline void Fork::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Fork.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Fork::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE Fork::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Fork.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Fork::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL Fork::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Fork.context_data)
  return _msg;
}
inline void Fork::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Fork.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool Fork::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void Fork::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& Fork::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& Fork::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fork.exit)
  return _internal_exit();
}
inline void Fork::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.Fork.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Fork::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE Fork::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.Fork.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Fork::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL Fork::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.Fork.exit)
  return _msg;
}
inline void Fork::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.Fork.exit)
}

// uint64 sysno = 3;
inline void Fork::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t Fork::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.Fork.sysno)
  return _internal_sysno();
}
inline void Fork::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.Fork.sysno)
}
inline ::uint64_t Fork::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void Fork::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// -------------------------------------------------------------------

// InotifyInit

// .gvisor.common.ContextData context_data = 1;
inline bool InotifyInit::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& InotifyInit::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& InotifyInit::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyInit.context_data)
  return _internal_context_data();
}
inline void InotifyInit::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyInit.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyInit::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyInit::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyInit.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyInit::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyInit::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyInit.context_data)
  return _msg;
}
inline void InotifyInit::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyInit.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool InotifyInit::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void InotifyInit::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& InotifyInit::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& InotifyInit::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyInit.exit)
  return _internal_exit();
}
inline void InotifyInit::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyInit.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyInit::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyInit::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyInit.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyInit::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyInit::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyInit.exit)
  return _msg;
}
inline void InotifyInit::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyInit.exit)
}

// uint64 sysno = 3;
inline void InotifyInit::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t InotifyInit::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyInit.sysno)
  return _internal_sysno();
}
inline void InotifyInit::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyInit.sysno)
}
inline ::uint64_t InotifyInit::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void InotifyInit::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 flags = 4;
inline void InotifyInit::clear_flags() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t InotifyInit::flags() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyInit.flags)
  return _internal_flags();
}
inline void InotifyInit::set_flags(::int32_t value) {
  _internal_set_flags(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyInit.flags)
}
inline ::int32_t InotifyInit::_internal_flags() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.flags_;
}
inline void InotifyInit::_internal_set_flags(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.flags_ = value;
}

// -------------------------------------------------------------------

// InotifyAddWatch

// .gvisor.common.ContextData context_data = 1;
inline bool InotifyAddWatch::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& InotifyAddWatch::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& InotifyAddWatch::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.context_data)
  return _internal_context_data();
}
inline void InotifyAddWatch::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyAddWatch.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyAddWatch::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyAddWatch::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyAddWatch.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyAddWatch::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyAddWatch::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyAddWatch.context_data)
  return _msg;
}
inline void InotifyAddWatch::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyAddWatch.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool InotifyAddWatch::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000008U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void InotifyAddWatch::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline const ::gvisor::syscall::Exit& InotifyAddWatch::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& InotifyAddWatch::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.exit)
  return _internal_exit();
}
inline void InotifyAddWatch::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyAddWatch.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyAddWatch::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyAddWatch::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyAddWatch.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyAddWatch::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyAddWatch::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyAddWatch.exit)
  return _msg;
}
inline void InotifyAddWatch::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyAddWatch.exit)
}

// uint64 sysno = 3;
inline void InotifyAddWatch::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::uint64_t InotifyAddWatch::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.sysno)
  return _internal_sysno();
}
inline void InotifyAddWatch::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyAddWatch.sysno)
}
inline ::uint64_t InotifyAddWatch::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void InotifyAddWatch::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void InotifyAddWatch::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t InotifyAddWatch::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.fd)
  return _internal_fd();
}
inline void InotifyAddWatch::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyAddWatch.fd)
}
inline ::int32_t InotifyAddWatch::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void InotifyAddWatch::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void InotifyAddWatch::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& InotifyAddWatch::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void InotifyAddWatch::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyAddWatch.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL InotifyAddWatch::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyAddWatch.fd_path)
  return _s;
}
inline const ::std::string& InotifyAddWatch::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void InotifyAddWatch::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL InotifyAddWatch::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE InotifyAddWatch::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyAddWatch.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void InotifyAddWatch::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyAddWatch.fd_path)
}

// string pathname = 6;
inline void InotifyAddWatch::clear_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::std::string& InotifyAddWatch::pathname() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.pathname)
  return _internal_pathname();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void InotifyAddWatch::set_pathname(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  _impl_.pathname_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyAddWatch.pathname)
}
inline ::std::string* PROTOBUF_NONNULL InotifyAddWatch::mutable_pathname()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::std::string* _s = _internal_mutable_pathname();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyAddWatch.pathname)
  return _s;
}
inline const ::std::string& InotifyAddWatch::_internal_pathname() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.pathname_.Get();
}
inline void InotifyAddWatch::_internal_set_pathname(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.pathname_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL InotifyAddWatch::_internal_mutable_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.pathname_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE InotifyAddWatch::release_pathname() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyAddWatch.pathname)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000002U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  auto* released = _impl_.pathname_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.pathname_.Set("", GetArena());
  }
  return released;
}
inline void InotifyAddWatch::set_allocated_pathname(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  _impl_.pathname_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.pathname_.IsDefault()) {
    _impl_.pathname_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyAddWatch.pathname)
}

// uint32 mask = 7;
inline void InotifyAddWatch::clear_mask() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mask_ = 0u;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::uint32_t InotifyAddWatch::mask() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyAddWatch.mask)
  return _internal_mask();
}
inline void InotifyAddWatch::set_mask(::uint32_t value) {
  _internal_set_mask(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyAddWatch.mask)
}
inline ::uint32_t InotifyAddWatch::_internal_mask() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.mask_;
}
inline void InotifyAddWatch::_internal_set_mask(::uint32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.mask_ = value;
}

// -------------------------------------------------------------------

// InotifyRmWatch

// .gvisor.common.ContextData context_data = 1;
inline bool InotifyRmWatch::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& InotifyRmWatch::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& InotifyRmWatch::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.context_data)
  return _internal_context_data();
}
inline void InotifyRmWatch::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyRmWatch.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyRmWatch::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE InotifyRmWatch::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyRmWatch.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyRmWatch::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL InotifyRmWatch::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyRmWatch.context_data)
  return _msg;
}
inline void InotifyRmWatch::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyRmWatch.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool InotifyRmWatch::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000004U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void InotifyRmWatch::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline const ::gvisor::syscall::Exit& InotifyRmWatch::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& InotifyRmWatch::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.exit)
  return _internal_exit();
}
inline void InotifyRmWatch::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.InotifyRmWatch.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyRmWatch::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE InotifyRmWatch::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyRmWatch.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyRmWatch::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL InotifyRmWatch::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyRmWatch.exit)
  return _msg;
}
inline void InotifyRmWatch::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyRmWatch.exit)
}

// uint64 sysno = 3;
inline void InotifyRmWatch::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::uint64_t InotifyRmWatch::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.sysno)
  return _internal_sysno();
}
inline void InotifyRmWatch::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyRmWatch.sysno)
}
inline ::uint64_t InotifyRmWatch::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void InotifyRmWatch::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 fd = 4;
inline void InotifyRmWatch::clear_fd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t InotifyRmWatch::fd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.fd)
  return _internal_fd();
}
inline void InotifyRmWatch::set_fd(::int32_t value) {
  _internal_set_fd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyRmWatch.fd)
}
inline ::int32_t InotifyRmWatch::_internal_fd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_;
}
inline void InotifyRmWatch::_internal_set_fd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_ = value;
}

// string fd_path = 5;
inline void InotifyRmWatch::clear_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.ClearToEmpty();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
inline const ::std::string& InotifyRmWatch::fd_path() const
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.fd_path)
  return _internal_fd_path();
}
template <typename Arg_, typename... Args_>
PROTOBUF_ALWAYS_INLINE void InotifyRmWatch::set_fd_path(Arg_&& arg, Args_... args) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  _impl_.fd_path_.Set(static_cast<Arg_&&>(arg), args..., GetArena());
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyRmWatch.fd_path)
}
inline ::std::string* PROTOBUF_NONNULL InotifyRmWatch::mutable_fd_path()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::std::string* _s = _internal_mutable_fd_path();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.InotifyRmWatch.fd_path)
  return _s;
}
inline const ::std::string& InotifyRmWatch::_internal_fd_path() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.fd_path_.Get();
}
inline void InotifyRmWatch::_internal_set_fd_path(const ::std::string& value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.fd_path_.Set(value, GetArena());
}
inline ::std::string* PROTOBUF_NONNULL InotifyRmWatch::_internal_mutable_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  return _impl_.fd_path_.Mutable( GetArena());
}
inline ::std::string* PROTOBUF_NULLABLE InotifyRmWatch::release_fd_path() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.InotifyRmWatch.fd_path)
  if (!CheckHasBit(_impl_._has_bits_[0], 0x00000001U)) {
    return nullptr;
  }
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  auto* released = _impl_.fd_path_.Release();
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  return released;
}
inline void InotifyRmWatch::set_allocated_fd_path(::std::string* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  _impl_.fd_path_.SetAllocated(value, GetArena());
  if (::google::protobuf::internal::DebugHardenForceCopyDefaultString() && _impl_.fd_path_.IsDefault()) {
    _impl_.fd_path_.Set("", GetArena());
  }
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.InotifyRmWatch.fd_path)
}

// int32 wd = 6;
inline void InotifyRmWatch::clear_wd() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.wd_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t InotifyRmWatch::wd() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.InotifyRmWatch.wd)
  return _internal_wd();
}
inline void InotifyRmWatch::set_wd(::int32_t value) {
  _internal_set_wd(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.InotifyRmWatch.wd)
}
inline ::int32_t InotifyRmWatch::_internal_wd() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.wd_;
}
inline void InotifyRmWatch::_internal_set_wd(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.wd_ = value;
}

// -------------------------------------------------------------------

// SocketPair

// .gvisor.common.ContextData context_data = 1;
inline bool SocketPair::has_context_data() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000001U);
  PROTOBUF_ASSUME(!value || _impl_.context_data_ != nullptr);
  return value;
}
inline const ::gvisor::common::ContextData& SocketPair::_internal_context_data() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::common::ContextData* p = _impl_.context_data_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::common::ContextData>(&::gvisor::common::ContextData_globals_);
}
inline const ::gvisor::common::ContextData& SocketPair::context_data() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.context_data)
  return _internal_context_data();
}
inline void SocketPair::unsafe_arena_set_allocated_context_data(
    ::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }
  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.SocketPair.context_data)
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE SocketPair::release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* released = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::common::ContextData* PROTOBUF_NULLABLE SocketPair::unsafe_arena_release_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.SocketPair.context_data)

  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* temp = _impl_.context_data_;
  _impl_.context_data_ = nullptr;
  return temp;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL SocketPair::_internal_mutable_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::common::ContextData>(GetArena());
    _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(p);
  }
  return _impl_.context_data_;
}
inline ::gvisor::common::ContextData* PROTOBUF_NONNULL SocketPair::mutable_context_data()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  ::gvisor::common::ContextData* _msg = _internal_mutable_context_data();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.SocketPair.context_data)
  return _msg;
}
inline void SocketPair::set_allocated_context_data(::gvisor::common::ContextData* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.context_data_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = reinterpret_cast<::google::protobuf::Message*>(value)->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000001U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
  }

  _impl_.context_data_ = reinterpret_cast<::gvisor::common::ContextData*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.SocketPair.context_data)
}

// .gvisor.syscall.Exit exit = 2;
inline bool SocketPair::has_exit() const {
  bool value = CheckHasBit(_impl_._has_bits_[0], 0x00000002U);
  PROTOBUF_ASSUME(!value || _impl_.exit_ != nullptr);
  return value;
}
inline void SocketPair::clear_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ != nullptr) _impl_.exit_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
inline const ::gvisor::syscall::Exit& SocketPair::_internal_exit() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  const ::gvisor::syscall::Exit* p = _impl_.exit_;
  return p != nullptr ? *p : *::google::protobuf::internal::MessageGlobalsBase::ToDefaultInstance<::gvisor::syscall::Exit>(&::gvisor::syscall::Exit_globals_);
}
inline const ::gvisor::syscall::Exit& SocketPair::exit() const ABSL_ATTRIBUTE_LIFETIME_BOUND {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.exit)
  return _internal_exit();
}
inline void SocketPair::unsafe_arena_set_allocated_exit(
    ::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (GetArena() == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }
  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  if (value != nullptr) {
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }
  // @@protoc_insertion_point(field_unsafe_arena_set_allocated:gvisor.syscall.SocketPair.exit)
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE SocketPair::release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* released = _impl_.exit_;
  _impl_.exit_ = nullptr;
  if (::google::protobuf::internal::DebugHardenForceCopyInRelease()) {
    auto* old = reinterpret_cast<::google::protobuf::MessageLite*>(released);
    released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    if (GetArena() == nullptr) {
      delete old;
    }
  } else {
    if (GetArena() != nullptr) {
      released = ::google::protobuf::internal::DuplicateIfNonNull(released);
    }
  }
  return released;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NULLABLE SocketPair::unsafe_arena_release_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  // @@protoc_insertion_point(field_release:gvisor.syscall.SocketPair.exit)

  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* temp = _impl_.exit_;
  _impl_.exit_ = nullptr;
  return temp;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL SocketPair::_internal_mutable_exit() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.exit_ == nullptr) {
    auto* p = Super_::DefaultConstruct<::gvisor::syscall::Exit>(GetArena());
    _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(p);
  }
  return _impl_.exit_;
}
inline ::gvisor::syscall::Exit* PROTOBUF_NONNULL SocketPair::mutable_exit()
    ABSL_ATTRIBUTE_LIFETIME_BOUND {
  SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  ::gvisor::syscall::Exit* _msg = _internal_mutable_exit();
  // @@protoc_insertion_point(field_mutable:gvisor.syscall.SocketPair.exit)
  return _msg;
}
inline void SocketPair::set_allocated_exit(::gvisor::syscall::Exit* PROTOBUF_NULLABLE value) {
  ::google::protobuf::Arena* message_arena = GetArena();
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (message_arena == nullptr) {
    delete reinterpret_cast<::google::protobuf::MessageLite*>(_impl_.exit_);
  }

  if (value != nullptr) {
    ::google::protobuf::Arena* submessage_arena = value->GetArena();
    if (message_arena != submessage_arena) {
      value = ::google::protobuf::internal::GetOwnedMessage(message_arena, value, submessage_arena);
    }
    SetHasBit(_impl_._has_bits_[0], 0x00000002U);
  } else {
    ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
  }

  _impl_.exit_ = reinterpret_cast<::gvisor::syscall::Exit*>(value);
  // @@protoc_insertion_point(field_set_allocated:gvisor.syscall.SocketPair.exit)
}

// uint64 sysno = 3;
inline void SocketPair::clear_sysno() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = ::uint64_t{0u};
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
inline ::uint64_t SocketPair::sysno() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.sysno)
  return _internal_sysno();
}
inline void SocketPair::set_sysno(::uint64_t value) {
  _internal_set_sysno(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000004U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.sysno)
}
inline ::uint64_t SocketPair::_internal_sysno() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.sysno_;
}
inline void SocketPair::_internal_set_sysno(::uint64_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.sysno_ = value;
}

// int32 domain = 4;
inline void SocketPair::clear_domain() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.domain_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000008U);
}
inline ::int32_t SocketPair::domain() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.domain)
  return _internal_domain();
}
inline void SocketPair::set_domain(::int32_t value) {
  _internal_set_domain(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000008U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.domain)
}
inline ::int32_t SocketPair::_internal_domain() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.domain_;
}
inline void SocketPair::_internal_set_domain(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.domain_ = value;
}

// int32 type = 5;
inline void SocketPair::clear_type() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
inline ::int32_t SocketPair::type() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.type)
  return _internal_type();
}
inline void SocketPair::set_type(::int32_t value) {
  _internal_set_type(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000010U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.type)
}
inline ::int32_t SocketPair::_internal_type() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.type_;
}
inline void SocketPair::_internal_set_type(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.type_ = value;
}

// int32 protocol = 6;
inline void SocketPair::clear_protocol() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000020U);
}
inline ::int32_t SocketPair::protocol() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.protocol)
  return _internal_protocol();
}
inline void SocketPair::set_protocol(::int32_t value) {
  _internal_set_protocol(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000020U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.protocol)
}
inline ::int32_t SocketPair::_internal_protocol() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.protocol_;
}
inline void SocketPair::_internal_set_protocol(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.protocol_ = value;
}

// int32 socket1 = 7;
inline void SocketPair::clear_socket1() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.socket1_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000040U);
}
inline ::int32_t SocketPair::socket1() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.socket1)
  return _internal_socket1();
}
inline void SocketPair::set_socket1(::int32_t value) {
  _internal_set_socket1(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000040U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.socket1)
}
inline ::int32_t SocketPair::_internal_socket1() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.socket1_;
}
inline void SocketPair::_internal_set_socket1(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.socket1_ = value;
}

// int32 socket2 = 8;
inline void SocketPair::clear_socket2() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.socket2_ = 0;
  ClearHasBit(_impl_._has_bits_[0], 0x00000080U);
}
inline ::int32_t SocketPair::socket2() const {
  // @@protoc_insertion_point(field_get:gvisor.syscall.SocketPair.socket2)
  return _internal_socket2();
}
inline void SocketPair::set_socket2(::int32_t value) {
  _internal_set_socket2(value);
  SetHasBit(_impl_._has_bits_[0], 0x00000080U);
  // @@protoc_insertion_point(field_set:gvisor.syscall.SocketPair.socket2)
}
inline ::int32_t SocketPair::_internal_socket2() const {
  ::google::protobuf::internal::TSanRead(&_impl_);
  return _impl_.socket2_;
}
inline void SocketPair::_internal_set_socket2(::int32_t value) {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  _impl_.socket2_ = value;
}

#ifdef __GNUC__
#pragma GCC diagnostic pop
#endif  // __GNUC__

// @@protoc_insertion_point(namespace_scope)
}  // namespace syscall
}  // namespace gvisor


// @@protoc_insertion_point(global_scope)

#include "google/protobuf/port_undef.inc"
// clang-format on

#endif  // pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_2epb_2eh
