// Generated by the protocol buffer compiler.  DO NOT EDIT!
// NO CHECKED-IN PROTOBUF GENCODE
// source: pkg/sentry/seccheck/points/syscall.proto
// Protobuf C++ Version: 7.36.1

#include "pkg/sentry/seccheck/points/syscall.pb.h"

#include <algorithm>
#include <type_traits>
#include "google/protobuf/io/coded_stream.h"
#include "google/protobuf/generated_message_tctable_impl.h"
#include "google/protobuf/internal_visibility.h"
#include "google/protobuf/extension_set.h"
#include "google/protobuf/generated_message_util.h"
#include "google/protobuf/wire_format_lite.h"
#include "google/protobuf/descriptor.h"
#include "google/protobuf/generated_message_reflection.h"
#include "google/protobuf/reflection_ops.h"
#include "google/protobuf/wire_format.h"
// @@protoc_insertion_point(includes)

// Must be included last.
#include "google/protobuf/port_def.inc"
PROTOBUF_PRAGMA_INIT_SEG
namespace _pb = ::google::protobuf;
namespace _pbi = ::google::protobuf::internal;
namespace _fl = ::google::protobuf::internal::field_layout;
#ifdef PROTOBUF_MESSAGE_GLOBALS
namespace {
PROTOBUF_CONSTINIT ::google::protobuf::internal::ReflectionData
    file_reflection_data[] = {
        // ::gvisor::syscall::Exit
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Syscall
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Open
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Close
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Read
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Write
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Connect
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Execve
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Socket
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Chdir
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Setresid
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Setid
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::StructRlimit
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Prlimit
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Pipe
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Fcntl
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Dup
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Signalfd
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Chroot
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Eventfd
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Clone
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Bind
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Accept
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::TimerfdCreate
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Timespec
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::ItimerSpec
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::TimerfdSetTime
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::TimerfdGetTime
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::Fork
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::InotifyInit
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::InotifyAddWatch
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::InotifyRmWatch
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
        // ::gvisor::syscall::SocketPair
        {&::_pbi::kDescriptorMethods, &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto, /* tracker*/ nullptr,},
};
}  // namespace
#endif
namespace gvisor {
namespace syscall {
class Timespec::_Internal {
 public:
  using HasBits = decltype(::std::declval<Timespec>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Timespec, _impl_._has_bits_);
};

constexpr Timespec::ParseTableT_ Timespec::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Timespec, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Timespec>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 nsec = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Timespec, _impl_.nsec_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Timespec, _impl_.nsec_)}},
      // int64 sec = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Timespec, _impl_.sec_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Timespec, _impl_.sec_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 sec = 1;
      {PROTOBUF_FIELD_OFFSET(Timespec, _impl_.sec_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 nsec = 2;
      {PROTOBUF_FIELD_OFFSET(Timespec, _impl_.nsec_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Timespec::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : sec_{::int64_t{0}},
        nsec_{::int64_t{0}} {}

template <typename>
constexpr Timespec::Timespec(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Timespec::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Timespec(arena);
}
constexpr auto Timespec::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Timespec), alignof(Timespec));
}
constexpr auto Timespec::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Timespec::MergeImpl,
          Super_::GetNewImpl<Timespec>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Timespec::SharedDtor,
          Super_::GetClearImpl<Timespec>(), &Timespec::ByteSizeLong,
              &Timespec::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Timespec, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[24],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TimespecGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TimespecGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Timespec_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Timespec::InternalGenerateClassData_(
            _default, &Timespec_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Timespec>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TimespecGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Timespec _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Timespec>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TimespecGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TimespecGlobalsTypeInternal Timespec_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Timespec_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Timespec_globals_.GetClassData();
#else
  return Timespec_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class StructRlimit::_Internal {
 public:
  using HasBits = decltype(::std::declval<StructRlimit>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_._has_bits_);
};

constexpr StructRlimit::ParseTableT_ StructRlimit::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::StructRlimit>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint64 max = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(StructRlimit, _impl_.max_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.max_)}},
      // uint64 cur = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(StructRlimit, _impl_.cur_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.cur_)}},
    }}, {{
      65535, 65535
    }}, {{
      // uint64 cur = 1;
      {PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.cur_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 max = 2;
      {PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.max_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr StructRlimit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : cur_{::uint64_t{0u}},
        max_{::uint64_t{0u}} {}

template <typename>
constexpr StructRlimit::StructRlimit(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL StructRlimit::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) StructRlimit(arena);
}
constexpr auto StructRlimit::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(StructRlimit), alignof(StructRlimit));
}
constexpr auto StructRlimit::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &StructRlimit::MergeImpl,
          Super_::GetNewImpl<StructRlimit>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &StructRlimit::SharedDtor,
          Super_::GetClearImpl<StructRlimit>(), &StructRlimit::ByteSizeLong,
              &StructRlimit::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[12],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct StructRlimitGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr StructRlimitGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 StructRlimit_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(StructRlimit::InternalGenerateClassData_(
            _default, &StructRlimit_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<StructRlimit>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~StructRlimitGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) StructRlimit _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<StructRlimit>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(StructRlimitGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST StructRlimitGlobalsTypeInternal StructRlimit_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* StructRlimit_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return StructRlimit_globals_.GetClassData();
#else
  return StructRlimit_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Exit::_Internal {
 public:
  using HasBits = decltype(::std::declval<Exit>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Exit, _impl_._has_bits_);
};

constexpr Exit::ParseTableT_ Exit::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Exit, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      0,  // num_aux_entries
      offsetof(ParseTableT_, field_names),  // no aux_entries
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int64 errorno = 2;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Exit, _impl_.errorno_), 1>(),
       {16, 1, 0,
        PROTOBUF_FIELD_OFFSET(Exit, _impl_.errorno_)}},
      // int64 result = 1;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Exit, _impl_.result_), 0>(),
       {8, 0, 0,
        PROTOBUF_FIELD_OFFSET(Exit, _impl_.result_)}},
    }}, {{
      65535, 65535
    }}, {{
      // int64 result = 1;
      {PROTOBUF_FIELD_OFFSET(Exit, _impl_.result_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // int64 errorno = 2;
      {PROTOBUF_FIELD_OFFSET(Exit, _impl_.errorno_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    // no aux_entries
    {{
    }},
  };
}


inline constexpr Exit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : result_{::int64_t{0}},
        errorno_{::int64_t{0}} {}

template <typename>
constexpr Exit::Exit(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Exit::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Exit(arena);
}
constexpr auto Exit::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Exit), alignof(Exit));
}
constexpr auto Exit::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Exit::MergeImpl,
          Super_::GetNewImpl<Exit>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Exit::SharedDtor,
          Super_::GetClearImpl<Exit>(), &Exit::ByteSizeLong,
              &Exit::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Exit, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[0],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExitGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExitGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Exit_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Exit::InternalGenerateClassData_(
            _default, &Exit_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Exit>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExitGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Exit _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Exit>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExitGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExitGlobalsTypeInternal Exit_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Exit_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Exit_globals_.GetClassData();
#else
  return Exit_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class ItimerSpec::_Internal {
 public:
  using HasBits = decltype(::std::declval<ItimerSpec>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_._has_bits_);
};

constexpr ItimerSpec::ParseTableT_ ItimerSpec::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_._has_bits_),
      0, // no _extensions_
      2, 8,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967292,  // skipmap
      offsetof(ParseTableT_, field_entries),
      2,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::ItimerSpec>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .gvisor.syscall.Timespec value = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.value_)}},
      // .gvisor.syscall.Timespec interval = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.interval_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.syscall.Timespec interval = 1;
      {PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.interval_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Timespec value = 2;
      {PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.value_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Timespec>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Timespec_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Timespec>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Timespec_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr ItimerSpec::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : interval_{nullptr},
        value_{nullptr} {}

template <typename>
constexpr ItimerSpec::ItimerSpec(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL ItimerSpec::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) ItimerSpec(arena);
}
constexpr auto ItimerSpec::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(ItimerSpec), alignof(ItimerSpec));
}
constexpr auto ItimerSpec::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &ItimerSpec::MergeImpl,
          Super_::GetNewImpl<ItimerSpec>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &ItimerSpec::SharedDtor,
          Super_::GetClearImpl<ItimerSpec>(), &ItimerSpec::ByteSizeLong,
              &ItimerSpec::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[25],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ItimerSpecGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ItimerSpecGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 ItimerSpec_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(ItimerSpec::InternalGenerateClassData_(
            _default, &ItimerSpec_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<ItimerSpec>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ItimerSpecGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) ItimerSpec _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<ItimerSpec>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ItimerSpecGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ItimerSpecGlobalsTypeInternal ItimerSpec_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* ItimerSpec_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return ItimerSpec_globals_.GetClassData();
#else
  return ItimerSpec_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Write::_Internal {
 public:
  using HasBits = decltype(::std::declval<Write>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Write, _impl_._has_bits_);
};

constexpr Write::ParseTableT_ Write::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Write, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Write>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Write, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Write, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.fd_path_)}},
      // uint64 count = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Write, _impl_.count_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.count_)}},
      // bool has_offset = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Write, _impl_.has_offset_), 7>(),
       {56, 7, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.has_offset_)}},
      // int64 offset = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Write, _impl_.offset_), 6>(),
       {64, 6, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.offset_)}},
      // uint32 flags = 9;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Write, _impl_.flags_), 8>(),
       {72, 8, 0,
        PROTOBUF_FIELD_OFFSET(Write, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint64 count = 6;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.count_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // bool has_offset = 7;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.has_offset_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // int64 offset = 8;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.offset_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // uint32 flags = 9;
      {PROTOBUF_FIELD_OFFSET(Write, _impl_.flags_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\24\0\0\0\0\7\0\0\0\0\0\0\0\0\0\0"
      "gvisor.syscall.Write"
      "fd_path"
    }},
  };
}


inline constexpr Write::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}},
        count_{::uint64_t{0u}},
        offset_{::int64_t{0}},
        has_offset_{false},
        flags_{0u} {}

template <typename>
constexpr Write::Write(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Write::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Write(arena);
}
constexpr auto Write::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Write), alignof(Write));
}
constexpr auto Write::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Write::MergeImpl,
          Super_::GetNewImpl<Write>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Write::SharedDtor,
          Super_::GetClearImpl<Write>(), &Write::ByteSizeLong,
              &Write::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Write, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[5],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct WriteGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr WriteGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Write_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Write::InternalGenerateClassData_(
            _default, &Write_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Write>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~WriteGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Write _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Write>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(WriteGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST WriteGlobalsTypeInternal Write_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Write_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Write_globals_.GetClassData();
#else
  return Write_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class TimerfdSetTime::_Internal {
 public:
  using HasBits = decltype(::std::declval<TimerfdSetTime>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_._has_bits_);
};

constexpr TimerfdSetTime::ParseTableT_ TimerfdSetTime::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::TimerfdSetTime>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // .gvisor.syscall.ItimerSpec old_value = 8;
      {::_pbi::TcParser::FastMtS1,
       {66, 4, 3,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.old_value_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TimerfdSetTime, _impl_.sysno_), 5>(),
       {24, 5, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TimerfdSetTime, _impl_.fd_), 6>(),
       {32, 6, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.fd_path_)}},
      // int32 flags = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TimerfdSetTime, _impl_.flags_), 7>(),
       {48, 7, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.flags_)}},
      // .gvisor.syscall.ItimerSpec new_value = 7;
      {::_pbi::TcParser::FastMtS1,
       {58, 3, 2,
        PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.new_value_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.sysno_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.fd_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 flags = 6;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.flags_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // .gvisor.syscall.ItimerSpec new_value = 7;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.new_value_), _Internal::kHasBitsOffset + 3, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.ItimerSpec old_value = 8;
      {PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.old_value_), _Internal::kHasBitsOffset + 4, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::ItimerSpec>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::ItimerSpec_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::ItimerSpec>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::ItimerSpec_globals_},
        #endif
    }},
    {{
      "\35\0\0\0\0\7\0\0\0\0\0\0\0\0\0\0"
      "gvisor.syscall.TimerfdSetTime"
      "fd_path"
    }},
  };
}


inline constexpr TimerfdSetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        new_value_{nullptr},
        old_value_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        flags_{0} {}

template <typename>
constexpr TimerfdSetTime::TimerfdSetTime(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TimerfdSetTime::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TimerfdSetTime(arena);
}
constexpr auto TimerfdSetTime::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TimerfdSetTime), alignof(TimerfdSetTime));
}
constexpr auto TimerfdSetTime::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TimerfdSetTime::MergeImpl,
          Super_::GetNewImpl<TimerfdSetTime>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TimerfdSetTime::SharedDtor,
          Super_::GetClearImpl<TimerfdSetTime>(), &TimerfdSetTime::ByteSizeLong,
              &TimerfdSetTime::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[26],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TimerfdSetTimeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TimerfdSetTimeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TimerfdSetTime_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TimerfdSetTime::InternalGenerateClassData_(
            _default, &TimerfdSetTime_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TimerfdSetTime>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TimerfdSetTimeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TimerfdSetTime _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TimerfdSetTime>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TimerfdSetTimeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TimerfdSetTimeGlobalsTypeInternal TimerfdSetTime_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TimerfdSetTime_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TimerfdSetTime_globals_.GetClassData();
#else
  return TimerfdSetTime_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class TimerfdGetTime::_Internal {
 public:
  using HasBits = decltype(::std::declval<TimerfdGetTime>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_._has_bits_);
};

constexpr TimerfdGetTime::ParseTableT_ TimerfdGetTime::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      3,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::TimerfdGetTime>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TimerfdGetTime, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TimerfdGetTime, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.fd_path_)}},
      // .gvisor.syscall.ItimerSpec cur_value = 6;
      {::_pbi::TcParser::FastMtS1,
       {50, 3, 2,
        PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.cur_value_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // .gvisor.syscall.ItimerSpec cur_value = 6;
      {PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.cur_value_), _Internal::kHasBitsOffset + 3, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::ItimerSpec>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::ItimerSpec_globals_},
        #endif
    }},
    {{
      "\35\0\0\0\0\7\0\0"
      "gvisor.syscall.TimerfdGetTime"
      "fd_path"
    }},
  };
}


inline constexpr TimerfdGetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        cur_value_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0} {}

template <typename>
constexpr TimerfdGetTime::TimerfdGetTime(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TimerfdGetTime::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TimerfdGetTime(arena);
}
constexpr auto TimerfdGetTime::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(TimerfdGetTime), alignof(TimerfdGetTime));
}
constexpr auto TimerfdGetTime::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TimerfdGetTime::MergeImpl,
          Super_::GetNewImpl<TimerfdGetTime>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TimerfdGetTime::SharedDtor,
          Super_::GetClearImpl<TimerfdGetTime>(), &TimerfdGetTime::ByteSizeLong,
              &TimerfdGetTime::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[27],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TimerfdGetTimeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TimerfdGetTimeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TimerfdGetTime_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TimerfdGetTime::InternalGenerateClassData_(
            _default, &TimerfdGetTime_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TimerfdGetTime>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TimerfdGetTimeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TimerfdGetTime _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TimerfdGetTime>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TimerfdGetTimeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TimerfdGetTimeGlobalsTypeInternal TimerfdGetTime_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TimerfdGetTime_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TimerfdGetTime_globals_.GetClassData();
#else
  return TimerfdGetTime_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class TimerfdCreate::_Internal {
 public:
  using HasBits = decltype(::std::declval<TimerfdCreate>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_._has_bits_);
};

constexpr TimerfdCreate::ParseTableT_ TimerfdCreate::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::TimerfdCreate>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(TimerfdCreate, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.sysno_)}},
      // int32 clock_id = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TimerfdCreate, _impl_.clock_id_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.clock_id_)}},
      // int32 flags = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(TimerfdCreate, _impl_.flags_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 clock_id = 4;
      {PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.clock_id_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 flags = 5;
      {PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.flags_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr TimerfdCreate::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        clock_id_{0},
        flags_{0} {}

template <typename>
constexpr TimerfdCreate::TimerfdCreate(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL TimerfdCreate::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) TimerfdCreate(arena);
}
constexpr auto TimerfdCreate::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(TimerfdCreate), alignof(TimerfdCreate));
}
constexpr auto TimerfdCreate::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &TimerfdCreate::MergeImpl,
          Super_::GetNewImpl<TimerfdCreate>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &TimerfdCreate::SharedDtor,
          Super_::GetClearImpl<TimerfdCreate>(), &TimerfdCreate::ByteSizeLong,
              &TimerfdCreate::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[23],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct TimerfdCreateGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr TimerfdCreateGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 TimerfdCreate_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(TimerfdCreate::InternalGenerateClassData_(
            _default, &TimerfdCreate_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<TimerfdCreate>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~TimerfdCreateGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) TimerfdCreate _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<TimerfdCreate>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(TimerfdCreateGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST TimerfdCreateGlobalsTypeInternal TimerfdCreate_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* TimerfdCreate_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return TimerfdCreate_globals_.GetClassData();
#else
  return TimerfdCreate_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Syscall::_Internal {
 public:
  using HasBits = decltype(::std::declval<Syscall>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Syscall, _impl_._has_bits_);
};

constexpr Syscall::ParseTableT_ Syscall::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Syscall, _impl_._has_bits_),
      0, // no _extensions_
      10, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966276,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Syscall>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.exit_)}},
      {::_pbi::TcParser::MiniParse, {}},
      // uint64 sysno = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.sysno_), 2>(),
       {32, 2, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.sysno_)}},
      // uint64 arg1 = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg1_), 3>(),
       {40, 3, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg1_)}},
      // uint64 arg2 = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg2_), 4>(),
       {48, 4, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg2_)}},
      // uint64 arg3 = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg3_), 5>(),
       {56, 5, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg3_)}},
      // uint64 arg4 = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg4_), 6>(),
       {64, 6, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg4_)}},
      // uint64 arg5 = 9;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg5_), 7>(),
       {72, 7, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg5_)}},
      // uint64 arg6 = 10;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Syscall, _impl_.arg6_), 8>(),
       {80, 8, 0,
        PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg6_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 4;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg1 = 5;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg1_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg2 = 6;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg2_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg3 = 7;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg3_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg4 = 8;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg4_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg5 = 9;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg5_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 arg6 = 10;
      {PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg6_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Syscall::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        arg1_{::uint64_t{0u}},
        arg2_{::uint64_t{0u}},
        arg3_{::uint64_t{0u}},
        arg4_{::uint64_t{0u}},
        arg5_{::uint64_t{0u}},
        arg6_{::uint64_t{0u}} {}

template <typename>
constexpr Syscall::Syscall(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Syscall::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Syscall(arena);
}
constexpr auto Syscall::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Syscall), alignof(Syscall));
}
constexpr auto Syscall::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Syscall::MergeImpl,
          Super_::GetNewImpl<Syscall>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Syscall::SharedDtor,
          Super_::GetClearImpl<Syscall>(), &Syscall::ByteSizeLong,
              &Syscall::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Syscall, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[1],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SyscallGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SyscallGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Syscall_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Syscall::InternalGenerateClassData_(
            _default, &Syscall_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Syscall>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SyscallGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Syscall _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Syscall>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SyscallGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SyscallGlobalsTypeInternal Syscall_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Syscall_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Syscall_globals_.GetClassData();
#else
  return Syscall_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class SocketPair::_Internal {
 public:
  using HasBits = decltype(::std::declval<SocketPair>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(SocketPair, _impl_._has_bits_);
};

constexpr SocketPair::ParseTableT_ SocketPair::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(SocketPair, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::SocketPair>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 socket2 = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SocketPair, _impl_.socket2_), 7>(),
       {64, 7, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.socket2_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(SocketPair, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.sysno_)}},
      // int32 domain = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SocketPair, _impl_.domain_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.domain_)}},
      // int32 type = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SocketPair, _impl_.type_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.type_)}},
      // int32 protocol = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SocketPair, _impl_.protocol_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.protocol_)}},
      // int32 socket1 = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(SocketPair, _impl_.socket1_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.socket1_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 domain = 4;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.domain_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 type = 5;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 protocol = 6;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.protocol_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 socket1 = 7;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.socket1_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 socket2 = 8;
      {PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.socket2_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr SocketPair::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        domain_{0},
        type_{0},
        protocol_{0},
        socket1_{0},
        socket2_{0} {}

template <typename>
constexpr SocketPair::SocketPair(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL SocketPair::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) SocketPair(arena);
}
constexpr auto SocketPair::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(SocketPair), alignof(SocketPair));
}
constexpr auto SocketPair::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &SocketPair::MergeImpl,
          Super_::GetNewImpl<SocketPair>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &SocketPair::SharedDtor,
          Super_::GetClearImpl<SocketPair>(), &SocketPair::ByteSizeLong,
              &SocketPair::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(SocketPair, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[32],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SocketPairGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SocketPairGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 SocketPair_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(SocketPair::InternalGenerateClassData_(
            _default, &SocketPair_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<SocketPair>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SocketPairGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) SocketPair _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<SocketPair>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SocketPairGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SocketPairGlobalsTypeInternal SocketPair_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* SocketPair_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return SocketPair_globals_.GetClassData();
#else
  return SocketPair_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Socket::_Internal {
 public:
  using HasBits = decltype(::std::declval<Socket>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Socket, _impl_._has_bits_);
};

constexpr Socket::ParseTableT_ Socket::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Socket, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Socket>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Socket, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.sysno_)}},
      // int32 domain = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Socket, _impl_.domain_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.domain_)}},
      // int32 type = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Socket, _impl_.type_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.type_)}},
      // int32 protocol = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Socket, _impl_.protocol_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Socket, _impl_.protocol_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 domain = 4;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.domain_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 type = 5;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.type_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 protocol = 6;
      {PROTOBUF_FIELD_OFFSET(Socket, _impl_.protocol_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Socket::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        domain_{0},
        type_{0},
        protocol_{0} {}

template <typename>
constexpr Socket::Socket(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Socket::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Socket(arena);
}
constexpr auto Socket::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Socket), alignof(Socket));
}
constexpr auto Socket::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Socket::MergeImpl,
          Super_::GetNewImpl<Socket>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Socket::SharedDtor,
          Super_::GetClearImpl<Socket>(), &Socket::ByteSizeLong,
              &Socket::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Socket, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[8],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SocketGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SocketGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Socket_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Socket::InternalGenerateClassData_(
            _default, &Socket_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Socket>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SocketGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Socket _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Socket>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SocketGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SocketGlobalsTypeInternal Socket_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Socket_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Socket_globals_.GetClassData();
#else
  return Socket_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Signalfd::_Internal {
 public:
  using HasBits = decltype(::std::declval<Signalfd>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Signalfd, _impl_._has_bits_);
};

constexpr Signalfd::ParseTableT_ Signalfd::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Signalfd, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Signalfd>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Signalfd, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Signalfd, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.fd_path_)}},
      // uint64 sigset = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Signalfd, _impl_.sigset_), 6>(),
       {48, 6, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.sigset_)}},
      // int32 flags = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Signalfd, _impl_.flags_), 5>(),
       {56, 5, 0,
        PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.flags_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint64 sigset = 6;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.sigset_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 flags = 7;
      {PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.flags_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\27\0\0\0\0\7\0\0"
      "gvisor.syscall.Signalfd"
      "fd_path"
    }},
  };
}


inline constexpr Signalfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        flags_{0},
        sigset_{::uint64_t{0u}} {}

template <typename>
constexpr Signalfd::Signalfd(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Signalfd::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Signalfd(arena);
}
constexpr auto Signalfd::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Signalfd), alignof(Signalfd));
}
constexpr auto Signalfd::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Signalfd::MergeImpl,
          Super_::GetNewImpl<Signalfd>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Signalfd::SharedDtor,
          Super_::GetClearImpl<Signalfd>(), &Signalfd::ByteSizeLong,
              &Signalfd::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Signalfd, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[17],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SignalfdGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SignalfdGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Signalfd_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Signalfd::InternalGenerateClassData_(
            _default, &Signalfd_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Signalfd>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SignalfdGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Signalfd _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Signalfd>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SignalfdGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SignalfdGlobalsTypeInternal Signalfd_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Signalfd_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Signalfd_globals_.GetClassData();
#else
  return Signalfd_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Setresid::_Internal {
 public:
  using HasBits = decltype(::std::declval<Setresid>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Setresid, _impl_._has_bits_);
};

constexpr Setresid::ParseTableT_ Setresid::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Setresid, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Setresid>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Setresid, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.sysno_)}},
      // uint32 rid = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Setresid, _impl_.rid_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.rid_)}},
      // uint32 eid = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Setresid, _impl_.eid_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.eid_)}},
      // uint32 sid = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Setresid, _impl_.sid_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Setresid, _impl_.sid_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint32 rid = 4;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.rid_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 eid = 5;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.eid_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 sid = 6;
      {PROTOBUF_FIELD_OFFSET(Setresid, _impl_.sid_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Setresid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        rid_{0u},
        eid_{0u},
        sid_{0u} {}

template <typename>
constexpr Setresid::Setresid(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Setresid::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Setresid(arena);
}
constexpr auto Setresid::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Setresid), alignof(Setresid));
}
constexpr auto Setresid::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Setresid::MergeImpl,
          Super_::GetNewImpl<Setresid>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Setresid::SharedDtor,
          Super_::GetClearImpl<Setresid>(), &Setresid::ByteSizeLong,
              &Setresid::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Setresid, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[10],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SetresidGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SetresidGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Setresid_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Setresid::InternalGenerateClassData_(
            _default, &Setresid_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Setresid>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SetresidGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Setresid _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Setresid>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SetresidGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SetresidGlobalsTypeInternal Setresid_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Setresid_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Setresid_globals_.GetClassData();
#else
  return Setresid_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Setid::_Internal {
 public:
  using HasBits = decltype(::std::declval<Setid>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Setid, _impl_._has_bits_);
};

constexpr Setid::ParseTableT_ Setid::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Setid, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Setid>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint32 id = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Setid, _impl_.id_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Setid, _impl_.id_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Setid, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Setid, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Setid, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Setid, _impl_.sysno_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Setid, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Setid, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Setid, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint32 id = 4;
      {PROTOBUF_FIELD_OFFSET(Setid, _impl_.id_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Setid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        id_{0u} {}

template <typename>
constexpr Setid::Setid(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Setid::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Setid(arena);
}
constexpr auto Setid::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Setid), alignof(Setid));
}
constexpr auto Setid::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Setid::MergeImpl,
          Super_::GetNewImpl<Setid>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Setid::SharedDtor,
          Super_::GetClearImpl<Setid>(), &Setid::ByteSizeLong,
              &Setid::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Setid, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[11],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct SetidGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr SetidGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Setid_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Setid::InternalGenerateClassData_(
            _default, &Setid_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Setid>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~SetidGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Setid _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Setid>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(SetidGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST SetidGlobalsTypeInternal Setid_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Setid_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Setid_globals_.GetClassData();
#else
  return Setid_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Read::_Internal {
 public:
  using HasBits = decltype(::std::declval<Read>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Read, _impl_._has_bits_);
};

constexpr Read::ParseTableT_ Read::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Read, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Read>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Read, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Read, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.fd_path_)}},
      // uint64 count = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Read, _impl_.count_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.count_)}},
      // bool has_offset = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<bool, offsetof(Read, _impl_.has_offset_), 7>(),
       {56, 7, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.has_offset_)}},
      // int64 offset = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Read, _impl_.offset_), 6>(),
       {64, 6, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.offset_)}},
      // uint32 flags = 9;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Read, _impl_.flags_), 8>(),
       {72, 8, 0,
        PROTOBUF_FIELD_OFFSET(Read, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint64 count = 6;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.count_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // bool has_offset = 7;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.has_offset_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kBool)},
      // int64 offset = 8;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.offset_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // uint32 flags = 9;
      {PROTOBUF_FIELD_OFFSET(Read, _impl_.flags_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\23\0\0\0\0\7\0\0\0\0\0\0\0\0\0\0"
      "gvisor.syscall.Read"
      "fd_path"
    }},
  };
}


inline constexpr Read::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}},
        count_{::uint64_t{0u}},
        offset_{::int64_t{0}},
        has_offset_{false},
        flags_{0u} {}

template <typename>
constexpr Read::Read(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Read::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Read(arena);
}
constexpr auto Read::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Read), alignof(Read));
}
constexpr auto Read::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Read::MergeImpl,
          Super_::GetNewImpl<Read>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Read::SharedDtor,
          Super_::GetClearImpl<Read>(), &Read::ByteSizeLong,
              &Read::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Read, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[4],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ReadGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ReadGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Read_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Read::InternalGenerateClassData_(
            _default, &Read_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Read>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ReadGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Read _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Read>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ReadGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ReadGlobalsTypeInternal Read_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Read_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Read_globals_.GetClassData();
#else
  return Read_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Prlimit::_Internal {
 public:
  using HasBits = decltype(::std::declval<Prlimit>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Prlimit, _impl_._has_bits_);
};

constexpr Prlimit::ParseTableT_ Prlimit::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Prlimit, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      4,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Prlimit>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Prlimit, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.sysno_)}},
      // int32 pid = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Prlimit, _impl_.pid_), 6>(),
       {32, 6, 0,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.pid_)}},
      // int64 resource = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Prlimit, _impl_.resource_), 5>(),
       {40, 5, 0,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.resource_)}},
      // .gvisor.syscall.StructRlimit new_limit = 6;
      {::_pbi::TcParser::FastMtS1,
       {50, 2, 2,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.new_limit_)}},
      // .gvisor.syscall.StructRlimit old_limit = 7;
      {::_pbi::TcParser::FastMtS1,
       {58, 3, 3,
        PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.old_limit_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 pid = 4;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.pid_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 resource = 5;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.resource_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // .gvisor.syscall.StructRlimit new_limit = 6;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.new_limit_), _Internal::kHasBitsOffset + 2, 2, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.StructRlimit old_limit = 7;
      {PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.old_limit_), _Internal::kHasBitsOffset + 3, 3, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::StructRlimit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::StructRlimit_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::StructRlimit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::StructRlimit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Prlimit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        new_limit_{nullptr},
        old_limit_{nullptr},
        sysno_{::uint64_t{0u}},
        resource_{::int64_t{0}},
        pid_{0} {}

template <typename>
constexpr Prlimit::Prlimit(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Prlimit::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Prlimit(arena);
}
constexpr auto Prlimit::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Prlimit), alignof(Prlimit));
}
constexpr auto Prlimit::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Prlimit::MergeImpl,
          Super_::GetNewImpl<Prlimit>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Prlimit::SharedDtor,
          Super_::GetClearImpl<Prlimit>(), &Prlimit::ByteSizeLong,
              &Prlimit::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Prlimit, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[13],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PrlimitGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PrlimitGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Prlimit_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Prlimit::InternalGenerateClassData_(
            _default, &Prlimit_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Prlimit>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PrlimitGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Prlimit _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Prlimit>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PrlimitGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PrlimitGlobalsTypeInternal Prlimit_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Prlimit_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Prlimit_globals_.GetClassData();
#else
  return Prlimit_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Pipe::_Internal {
 public:
  using HasBits = decltype(::std::declval<Pipe>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Pipe, _impl_._has_bits_);
};

constexpr Pipe::ParseTableT_ Pipe::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Pipe, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Pipe>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Pipe, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.sysno_)}},
      // uint32 flags = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Pipe, _impl_.flags_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.flags_)}},
      // int32 reader = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Pipe, _impl_.reader_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.reader_)}},
      // int32 writer = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Pipe, _impl_.writer_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Pipe, _impl_.writer_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint32 flags = 4;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.flags_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // int32 reader = 5;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.reader_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 writer = 6;
      {PROTOBUF_FIELD_OFFSET(Pipe, _impl_.writer_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Pipe::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        flags_{0u},
        reader_{0},
        writer_{0} {}

template <typename>
constexpr Pipe::Pipe(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Pipe::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Pipe(arena);
}
constexpr auto Pipe::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Pipe), alignof(Pipe));
}
constexpr auto Pipe::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Pipe::MergeImpl,
          Super_::GetNewImpl<Pipe>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Pipe::SharedDtor,
          Super_::GetClearImpl<Pipe>(), &Pipe::ByteSizeLong,
              &Pipe::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Pipe, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[14],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct PipeGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr PipeGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Pipe_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Pipe::InternalGenerateClassData_(
            _default, &Pipe_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Pipe>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~PipeGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Pipe _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Pipe>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(PipeGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST PipeGlobalsTypeInternal Pipe_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Pipe_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Pipe_globals_.GetClassData();
#else
  return Pipe_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Open::_Internal {
 public:
  using HasBits = decltype(::std::declval<Open>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Open, _impl_._has_bits_);
};

constexpr Open::ParseTableT_ Open::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Open, _impl_._has_bits_),
      0, // no _extensions_
      8, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967040,  // skipmap
      offsetof(ParseTableT_, field_entries),
      8,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Open>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // uint32 mode = 8;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Open, _impl_.mode_), 7>(),
       {64, 7, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.mode_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Open, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Open, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.fd_path_)}},
      // string pathname = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.pathname_)}},
      // uint32 flags = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Open, _impl_.flags_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Open, _impl_.flags_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string pathname = 6;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.pathname_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint32 flags = 7;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.flags_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
      // uint32 mode = 8;
      {PROTOBUF_FIELD_OFFSET(Open, _impl_.mode_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\23\0\0\0\0\7\10\0\0\0\0\0\0\0\0\0"
      "gvisor.syscall.Open"
      "fd_path"
      "pathname"
    }},
  };
}


inline constexpr Open::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        pathname_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}},
        flags_{0u},
        mode_{0u} {}

template <typename>
constexpr Open::Open(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Open::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Open(arena);
}
constexpr auto Open::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Open), alignof(Open));
}
constexpr auto Open::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Open::MergeImpl,
          Super_::GetNewImpl<Open>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Open::SharedDtor,
          Super_::GetClearImpl<Open>(), &Open::ByteSizeLong,
              &Open::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Open, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[2],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct OpenGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr OpenGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Open_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Open::InternalGenerateClassData_(
            _default, &Open_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Open>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~OpenGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Open _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Open>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(OpenGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST OpenGlobalsTypeInternal Open_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Open_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Open_globals_.GetClassData();
#else
  return Open_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class InotifyRmWatch::_Internal {
 public:
  using HasBits = decltype(::std::declval<InotifyRmWatch>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_._has_bits_);
};

constexpr InotifyRmWatch::ParseTableT_ InotifyRmWatch::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::InotifyRmWatch>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(InotifyRmWatch, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(InotifyRmWatch, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.fd_path_)}},
      // int32 wd = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(InotifyRmWatch, _impl_.wd_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.wd_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 wd = 6;
      {PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.wd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\35\0\0\0\0\7\0\0"
      "gvisor.syscall.InotifyRmWatch"
      "fd_path"
    }},
  };
}


inline constexpr InotifyRmWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        wd_{0} {}

template <typename>
constexpr InotifyRmWatch::InotifyRmWatch(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL InotifyRmWatch::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) InotifyRmWatch(arena);
}
constexpr auto InotifyRmWatch::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(InotifyRmWatch), alignof(InotifyRmWatch));
}
constexpr auto InotifyRmWatch::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &InotifyRmWatch::MergeImpl,
          Super_::GetNewImpl<InotifyRmWatch>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &InotifyRmWatch::SharedDtor,
          Super_::GetClearImpl<InotifyRmWatch>(), &InotifyRmWatch::ByteSizeLong,
              &InotifyRmWatch::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[31],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct InotifyRmWatchGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr InotifyRmWatchGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 InotifyRmWatch_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(InotifyRmWatch::InternalGenerateClassData_(
            _default, &InotifyRmWatch_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<InotifyRmWatch>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~InotifyRmWatchGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) InotifyRmWatch _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<InotifyRmWatch>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(InotifyRmWatchGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST InotifyRmWatchGlobalsTypeInternal InotifyRmWatch_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* InotifyRmWatch_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return InotifyRmWatch_globals_.GetClassData();
#else
  return InotifyRmWatch_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class InotifyInit::_Internal {
 public:
  using HasBits = decltype(::std::declval<InotifyInit>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_._has_bits_);
};

constexpr InotifyInit::ParseTableT_ InotifyInit::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::InotifyInit>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // int32 flags = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(InotifyInit, _impl_.flags_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.flags_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(InotifyInit, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.sysno_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 flags = 4;
      {PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.flags_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr InotifyInit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        flags_{0} {}

template <typename>
constexpr InotifyInit::InotifyInit(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL InotifyInit::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) InotifyInit(arena);
}
constexpr auto InotifyInit::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(InotifyInit), alignof(InotifyInit));
}
constexpr auto InotifyInit::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &InotifyInit::MergeImpl,
          Super_::GetNewImpl<InotifyInit>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &InotifyInit::SharedDtor,
          Super_::GetClearImpl<InotifyInit>(), &InotifyInit::ByteSizeLong,
              &InotifyInit::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[29],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct InotifyInitGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr InotifyInitGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 InotifyInit_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(InotifyInit::InternalGenerateClassData_(
            _default, &InotifyInit_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<InotifyInit>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~InotifyInitGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) InotifyInit _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<InotifyInit>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(InotifyInitGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST InotifyInitGlobalsTypeInternal InotifyInit_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* InotifyInit_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return InotifyInit_globals_.GetClassData();
#else
  return InotifyInit_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class InotifyAddWatch::_Internal {
 public:
  using HasBits = decltype(::std::declval<InotifyAddWatch>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_._has_bits_);
};

constexpr InotifyAddWatch::ParseTableT_ InotifyAddWatch::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::InotifyAddWatch>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(InotifyAddWatch, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(InotifyAddWatch, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.fd_path_)}},
      // string pathname = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.pathname_)}},
      // uint32 mask = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(InotifyAddWatch, _impl_.mask_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.mask_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string pathname = 6;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.pathname_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint32 mask = 7;
      {PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.mask_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\36\0\0\0\0\7\10\0"
      "gvisor.syscall.InotifyAddWatch"
      "fd_path"
      "pathname"
    }},
  };
}


inline constexpr InotifyAddWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        pathname_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        mask_{0u} {}

template <typename>
constexpr InotifyAddWatch::InotifyAddWatch(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL InotifyAddWatch::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) InotifyAddWatch(arena);
}
constexpr auto InotifyAddWatch::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(InotifyAddWatch), alignof(InotifyAddWatch));
}
constexpr auto InotifyAddWatch::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &InotifyAddWatch::MergeImpl,
          Super_::GetNewImpl<InotifyAddWatch>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &InotifyAddWatch::SharedDtor,
          Super_::GetClearImpl<InotifyAddWatch>(), &InotifyAddWatch::ByteSizeLong,
              &InotifyAddWatch::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[30],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct InotifyAddWatchGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr InotifyAddWatchGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 InotifyAddWatch_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(InotifyAddWatch::InternalGenerateClassData_(
            _default, &InotifyAddWatch_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<InotifyAddWatch>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~InotifyAddWatchGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) InotifyAddWatch _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<InotifyAddWatch>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(InotifyAddWatchGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST InotifyAddWatchGlobalsTypeInternal InotifyAddWatch_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* InotifyAddWatch_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return InotifyAddWatch_globals_.GetClassData();
#else
  return InotifyAddWatch_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Fork::_Internal {
 public:
  using HasBits = decltype(::std::declval<Fork>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Fork, _impl_._has_bits_);
};

constexpr Fork::ParseTableT_ Fork::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Fork, _impl_._has_bits_),
      0, // no _extensions_
      3, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967288,  // skipmap
      offsetof(ParseTableT_, field_entries),
      3,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Fork>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Fork, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Fork, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Fork, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Fork, _impl_.sysno_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Fork, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Fork, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Fork, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Fork::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}} {}

template <typename>
constexpr Fork::Fork(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Fork::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Fork(arena);
}
constexpr auto Fork::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Fork), alignof(Fork));
}
constexpr auto Fork::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Fork::MergeImpl,
          Super_::GetNewImpl<Fork>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Fork::SharedDtor,
          Super_::GetClearImpl<Fork>(), &Fork::ByteSizeLong,
              &Fork::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Fork, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[28],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ForkGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ForkGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Fork_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Fork::InternalGenerateClassData_(
            _default, &Fork_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Fork>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ForkGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Fork _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Fork>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ForkGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ForkGlobalsTypeInternal Fork_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Fork_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Fork_globals_.GetClassData();
#else
  return Fork_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Fcntl::_Internal {
 public:
  using HasBits = decltype(::std::declval<Fcntl>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Fcntl, _impl_._has_bits_);
};

constexpr Fcntl::ParseTableT_ Fcntl::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Fcntl, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Fcntl>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Fcntl, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Fcntl, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.fd_path_)}},
      // int32 cmd = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Fcntl, _impl_.cmd_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.cmd_)}},
      // int64 args = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Fcntl, _impl_.args_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.args_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // int32 cmd = 6;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.cmd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int64 args = 7;
      {PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.args_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\24\0\0\0\0\7\0\0"
      "gvisor.syscall.Fcntl"
      "fd_path"
    }},
  };
}


inline constexpr Fcntl::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        cmd_{0},
        args_{::int64_t{0}} {}

template <typename>
constexpr Fcntl::Fcntl(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Fcntl::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Fcntl(arena);
}
constexpr auto Fcntl::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Fcntl), alignof(Fcntl));
}
constexpr auto Fcntl::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Fcntl::MergeImpl,
          Super_::GetNewImpl<Fcntl>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Fcntl::SharedDtor,
          Super_::GetClearImpl<Fcntl>(), &Fcntl::ByteSizeLong,
              &Fcntl::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Fcntl, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[15],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct FcntlGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr FcntlGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Fcntl_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Fcntl::InternalGenerateClassData_(
            _default, &Fcntl_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Fcntl>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~FcntlGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Fcntl _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Fcntl>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(FcntlGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST FcntlGlobalsTypeInternal Fcntl_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Fcntl_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Fcntl_globals_.GetClassData();
#else
  return Fcntl_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Execve::_Internal {
 public:
  using HasBits = decltype(::std::declval<Execve>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Execve, _impl_._has_bits_);
};

constexpr Execve::ParseTableT_ Execve::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Execve, _impl_._has_bits_),
      0, // no _extensions_
      9, 120,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294966784,  // skipmap
      offsetof(ParseTableT_, field_entries),
      9,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Execve>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 4, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 5, 1,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Execve, _impl_.sysno_), 6>(),
       {24, 6, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Execve, _impl_.fd_), 7>(),
       {32, 7, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 2, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.fd_path_)}},
      // string pathname = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 3, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.pathname_)}},
      // repeated string argv = 7;
      {::_pbi::TcParser::FastUR1,
       {58, 0, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.argv_)}},
      // repeated string envv = 8;
      {::_pbi::TcParser::FastUR1,
       {66, 1, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.envv_)}},
      // uint32 flags = 9;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Execve, _impl_.flags_), 8>(),
       {72, 8, 0,
        PROTOBUF_FIELD_OFFSET(Execve, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.context_data_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.exit_), _Internal::kHasBitsOffset + 5, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.sysno_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.fd_), _Internal::kHasBitsOffset + 7, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.fd_path_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string pathname = 6;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.pathname_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // repeated string argv = 7;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.argv_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // repeated string envv = 8;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.envv_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcRepeated | ::_fl::kUtf8String | ::_fl::kRepSString)},
      // uint32 flags = 9;
      {PROTOBUF_FIELD_OFFSET(Execve, _impl_.flags_), _Internal::kHasBitsOffset + 8, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\25\0\0\0\0\7\10\4\4\0\0\0\0\0\0\0"
      "gvisor.syscall.Execve"
      "fd_path"
      "pathname"
      "argv"
      "envv"
    }},
  };
}


inline constexpr Execve::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : argv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::syscall::Execve,
            PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.argv_)>()
         }
        ,
        envv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::syscall::Execve,
            PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.envv_)>()
         }
        ,
        fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        pathname_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}},
        flags_{0u} {}

template <typename>
constexpr Execve::Execve(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Execve::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Execve(arena);
}
constexpr auto Execve::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Execve), alignof(Execve));
}
constexpr auto Execve::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Execve::MergeImpl,
          Super_::GetNewImpl<Execve>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Execve::SharedDtor,
          Super_::GetClearImpl<Execve>(), &Execve::ByteSizeLong,
              &Execve::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Execve, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[7],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ExecveGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ExecveGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Execve_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Execve::InternalGenerateClassData_(
            _default, &Execve_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Execve>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ExecveGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Execve _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Execve>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ExecveGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ExecveGlobalsTypeInternal Execve_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Execve_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Execve_globals_.GetClassData();
#else
  return Execve_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Eventfd::_Internal {
 public:
  using HasBits = decltype(::std::declval<Eventfd>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Eventfd, _impl_._has_bits_);
};

constexpr Eventfd::ParseTableT_ Eventfd::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Eventfd, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Eventfd>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Eventfd, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.sysno_)}},
      // int32 val = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Eventfd, _impl_.val_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.val_)}},
      // uint32 flags = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Eventfd, _impl_.flags_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.flags_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 val = 4;
      {PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.val_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // uint32 flags = 5;
      {PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.flags_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Eventfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        val_{0},
        flags_{0u} {}

template <typename>
constexpr Eventfd::Eventfd(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Eventfd::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Eventfd(arena);
}
constexpr auto Eventfd::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Eventfd), alignof(Eventfd));
}
constexpr auto Eventfd::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Eventfd::MergeImpl,
          Super_::GetNewImpl<Eventfd>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Eventfd::SharedDtor,
          Super_::GetClearImpl<Eventfd>(), &Eventfd::ByteSizeLong,
              &Eventfd::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Eventfd, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[19],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct EventfdGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr EventfdGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Eventfd_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Eventfd::InternalGenerateClassData_(
            _default, &Eventfd_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Eventfd>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~EventfdGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Eventfd _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Eventfd>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(EventfdGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST EventfdGlobalsTypeInternal Eventfd_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Eventfd_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Eventfd_globals_.GetClassData();
#else
  return Eventfd_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Dup::_Internal {
 public:
  using HasBits = decltype(::std::declval<Dup>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Dup, _impl_._has_bits_);
};

constexpr Dup::ParseTableT_ Dup::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Dup, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Dup>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Dup, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.sysno_)}},
      // int32 old_fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Dup, _impl_.old_fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.old_fd_)}},
      // int32 new_fd = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Dup, _impl_.new_fd_), 5>(),
       {40, 5, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.new_fd_)}},
      // string fd_path = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 0, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.fd_path_)}},
      // uint32 flags = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Dup, _impl_.flags_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Dup, _impl_.flags_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 old_fd = 4;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.old_fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // int32 new_fd = 5;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.new_fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 6;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // uint32 flags = 7;
      {PROTOBUF_FIELD_OFFSET(Dup, _impl_.flags_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\22\0\0\0\0\0\7\0"
      "gvisor.syscall.Dup"
      "fd_path"
    }},
  };
}


inline constexpr Dup::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        old_fd_{0},
        new_fd_{0},
        flags_{0u} {}

template <typename>
constexpr Dup::Dup(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Dup::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Dup(arena);
}
constexpr auto Dup::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Dup), alignof(Dup));
}
constexpr auto Dup::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Dup::MergeImpl,
          Super_::GetNewImpl<Dup>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Dup::SharedDtor,
          Super_::GetClearImpl<Dup>(), &Dup::ByteSizeLong,
              &Dup::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Dup, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[16],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct DupGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr DupGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Dup_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Dup::InternalGenerateClassData_(
            _default, &Dup_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Dup>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~DupGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Dup _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Dup>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(DupGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST DupGlobalsTypeInternal Dup_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Dup_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Dup_globals_.GetClassData();
#else
  return Dup_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Connect::_Internal {
 public:
  using HasBits = decltype(::std::declval<Connect>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Connect, _impl_._has_bits_);
};

constexpr Connect::ParseTableT_ Connect::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Connect, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Connect>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Connect, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Connect, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.fd_path_)}},
      // bytes address = 6;
      {::_pbi::TcParser::FastBS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(Connect, _impl_.address_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bytes address = 6;
      {PROTOBUF_FIELD_OFFSET(Connect, _impl_.address_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\26\0\0\0\0\7\0\0"
      "gvisor.syscall.Connect"
      "fd_path"
    }},
  };
}


inline constexpr Connect::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}} {}

template <typename>
constexpr Connect::Connect(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Connect::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Connect(arena);
}
constexpr auto Connect::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Connect), alignof(Connect));
}
constexpr auto Connect::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Connect::MergeImpl,
          Super_::GetNewImpl<Connect>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Connect::SharedDtor,
          Super_::GetClearImpl<Connect>(), &Connect::ByteSizeLong,
              &Connect::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Connect, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[6],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ConnectGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ConnectGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Connect_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Connect::InternalGenerateClassData_(
            _default, &Connect_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Connect>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ConnectGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Connect _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Connect>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ConnectGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ConnectGlobalsTypeInternal Connect_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Connect_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Connect_globals_.GetClassData();
#else
  return Connect_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Close::_Internal {
 public:
  using HasBits = decltype(::std::declval<Close>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Close, _impl_._has_bits_);
};

constexpr Close::ParseTableT_ Close::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Close, _impl_._has_bits_),
      0, // no _extensions_
      5, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967264,  // skipmap
      offsetof(ParseTableT_, field_entries),
      5,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Close>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Close, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Close, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Close, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Close, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Close, _impl_.fd_), 4>(),
       {32, 4, 0,
        PROTOBUF_FIELD_OFFSET(Close, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Close, _impl_.fd_path_)}},
      {::_pbi::TcParser::MiniParse, {}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Close, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Close, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Close, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Close, _impl_.fd_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Close, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\24\0\0\0\0\7\0\0"
      "gvisor.syscall.Close"
      "fd_path"
    }},
  };
}


inline constexpr Close::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}} {}

template <typename>
constexpr Close::Close(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Close::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Close(arena);
}
constexpr auto Close::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Close), alignof(Close));
}
constexpr auto Close::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Close::MergeImpl,
          Super_::GetNewImpl<Close>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Close::SharedDtor,
          Super_::GetClearImpl<Close>(), &Close::ByteSizeLong,
              &Close::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Close, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[3],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloseGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloseGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Close_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Close::InternalGenerateClassData_(
            _default, &Close_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Close>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloseGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Close _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Close>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloseGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloseGlobalsTypeInternal Close_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Close_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Close_globals_.GetClassData();
#else
  return Close_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Clone::_Internal {
 public:
  using HasBits = decltype(::std::declval<Clone>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Clone, _impl_._has_bits_);
};

constexpr Clone::ParseTableT_ Clone::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Clone, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Clone>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 0, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 1, 1,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Clone, _impl_.sysno_), 2>(),
       {24, 2, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.sysno_)}},
      // uint64 flags = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Clone, _impl_.flags_), 3>(),
       {32, 3, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.flags_)}},
      // uint64 stack = 5;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Clone, _impl_.stack_), 4>(),
       {40, 4, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.stack_)}},
      // uint64 new_tid = 6;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Clone, _impl_.new_tid_), 5>(),
       {48, 5, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.new_tid_)}},
      // uint64 tls = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Clone, _impl_.tls_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Clone, _impl_.tls_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.context_data_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.exit_), _Internal::kHasBitsOffset + 1, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.sysno_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 flags = 4;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.flags_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 stack = 5;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.stack_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 new_tid = 6;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.new_tid_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // uint64 tls = 7;
      {PROTOBUF_FIELD_OFFSET(Clone, _impl_.tls_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
    }},
  };
}


inline constexpr Clone::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        flags_{::uint64_t{0u}},
        stack_{::uint64_t{0u}},
        new_tid_{::uint64_t{0u}},
        tls_{::uint64_t{0u}} {}

template <typename>
constexpr Clone::Clone(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Clone::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Clone(arena);
}
constexpr auto Clone::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::ZeroInit(sizeof(Clone), alignof(Clone));
}
constexpr auto Clone::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Clone::MergeImpl,
          Super_::GetNewImpl<Clone>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Clone::SharedDtor,
          Super_::GetClearImpl<Clone>(), &Clone::ByteSizeLong,
              &Clone::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Clone, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[20],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct CloneGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr CloneGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Clone_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Clone::InternalGenerateClassData_(
            _default, &Clone_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Clone>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~CloneGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Clone _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Clone>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(CloneGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST CloneGlobalsTypeInternal Clone_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Clone_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Clone_globals_.GetClassData();
#else
  return Clone_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Chroot::_Internal {
 public:
  using HasBits = decltype(::std::declval<Chroot>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Chroot, _impl_._has_bits_);
};

constexpr Chroot::ParseTableT_ Chroot::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Chroot, _impl_._has_bits_),
      0, // no _extensions_
      4, 24,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967280,  // skipmap
      offsetof(ParseTableT_, field_entries),
      4,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Chroot>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      // string pathname = 4;
      {::_pbi::TcParser::FastUS1,
       {34, 0, 0,
        PROTOBUF_FIELD_OFFSET(Chroot, _impl_.pathname_)}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 1, 0,
        PROTOBUF_FIELD_OFFSET(Chroot, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 2, 1,
        PROTOBUF_FIELD_OFFSET(Chroot, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Chroot, _impl_.sysno_), 3>(),
       {24, 3, 0,
        PROTOBUF_FIELD_OFFSET(Chroot, _impl_.sysno_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Chroot, _impl_.context_data_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Chroot, _impl_.exit_), _Internal::kHasBitsOffset + 2, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Chroot, _impl_.sysno_), _Internal::kHasBitsOffset + 3, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // string pathname = 4;
      {PROTOBUF_FIELD_OFFSET(Chroot, _impl_.pathname_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\25\0\0\0\10\0\0\0"
      "gvisor.syscall.Chroot"
      "pathname"
    }},
  };
}


inline constexpr Chroot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : pathname_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}} {}

template <typename>
constexpr Chroot::Chroot(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Chroot::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Chroot(arena);
}
constexpr auto Chroot::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Chroot), alignof(Chroot));
}
constexpr auto Chroot::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Chroot::MergeImpl,
          Super_::GetNewImpl<Chroot>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Chroot::SharedDtor,
          Super_::GetClearImpl<Chroot>(), &Chroot::ByteSizeLong,
              &Chroot::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Chroot, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[18],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ChrootGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ChrootGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Chroot_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Chroot::InternalGenerateClassData_(
            _default, &Chroot_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Chroot>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ChrootGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Chroot _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Chroot>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ChrootGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ChrootGlobalsTypeInternal Chroot_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Chroot_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Chroot_globals_.GetClassData();
#else
  return Chroot_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Chdir::_Internal {
 public:
  using HasBits = decltype(::std::declval<Chdir>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Chdir, _impl_._has_bits_);
};

constexpr Chdir::ParseTableT_ Chdir::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Chdir, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Chdir>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Chdir, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.sysno_)}},
      // int64 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Chdir, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.fd_path_)}},
      // string pathname = 6;
      {::_pbi::TcParser::FastUS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(Chdir, _impl_.pathname_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int64 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt64)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // string pathname = 6;
      {PROTOBUF_FIELD_OFFSET(Chdir, _impl_.pathname_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\24\0\0\0\0\7\10\0"
      "gvisor.syscall.Chdir"
      "fd_path"
      "pathname"
    }},
  };
}


inline constexpr Chdir::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        pathname_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{::int64_t{0}} {}

template <typename>
constexpr Chdir::Chdir(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Chdir::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Chdir(arena);
}
constexpr auto Chdir::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Chdir), alignof(Chdir));
}
constexpr auto Chdir::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Chdir::MergeImpl,
          Super_::GetNewImpl<Chdir>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Chdir::SharedDtor,
          Super_::GetClearImpl<Chdir>(), &Chdir::ByteSizeLong,
              &Chdir::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Chdir, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[9],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct ChdirGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr ChdirGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Chdir_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Chdir::InternalGenerateClassData_(
            _default, &Chdir_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Chdir>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~ChdirGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Chdir _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Chdir>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(ChdirGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST ChdirGlobalsTypeInternal Chdir_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Chdir_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Chdir_globals_.GetClassData();
#else
  return Chdir_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Bind::_Internal {
 public:
  using HasBits = decltype(::std::declval<Bind>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Bind, _impl_._has_bits_);
};

constexpr Bind::ParseTableT_ Bind::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Bind, _impl_._has_bits_),
      0, // no _extensions_
      6, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967232,  // skipmap
      offsetof(ParseTableT_, field_entries),
      6,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Bind>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Bind, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Bind, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.fd_path_)}},
      // bytes address = 6;
      {::_pbi::TcParser::FastBS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(Bind, _impl_.address_)}},
      {::_pbi::TcParser::MiniParse, {}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bytes address = 6;
      {PROTOBUF_FIELD_OFFSET(Bind, _impl_.address_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\23\0\0\0\0\7\0\0"
      "gvisor.syscall.Bind"
      "fd_path"
    }},
  };
}


inline constexpr Bind::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0} {}

template <typename>
constexpr Bind::Bind(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Bind::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Bind(arena);
}
constexpr auto Bind::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Bind), alignof(Bind));
}
constexpr auto Bind::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Bind::MergeImpl,
          Super_::GetNewImpl<Bind>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Bind::SharedDtor,
          Super_::GetClearImpl<Bind>(), &Bind::ByteSizeLong,
              &Bind::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Bind, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[21],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct BindGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr BindGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Bind_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Bind::InternalGenerateClassData_(
            _default, &Bind_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Bind>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~BindGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Bind _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Bind>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(BindGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST BindGlobalsTypeInternal Bind_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Bind_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Bind_globals_.GetClassData();
#else
  return Bind_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
class Accept::_Internal {
 public:
  using HasBits = decltype(::std::declval<Accept>()._impl_._has_bits_);
  static constexpr ::int32_t kHasBitsOffset =
      8 * PROTOBUF_FIELD_OFFSET(Accept, _impl_._has_bits_);
};

constexpr Accept::ParseTableT_ Accept::InternalGenerateParseTable_(const ::_pbi::ClassData* class_data) {
  return ParseTableT_{
    {
      PROTOBUF_FIELD_OFFSET(Accept, _impl_._has_bits_),
      0, // no _extensions_
      7, 56,  // max_field_number, fast_idx_mask
      offsetof(ParseTableT_, field_lookup_table),
      4294967168,  // skipmap
      offsetof(ParseTableT_, field_entries),
      7,  // num_field_entries
      2,  // num_aux_entries
      offsetof(ParseTableT_, aux_entries),
      class_data,
      nullptr,  // post_loop_handler
      ::_pbi::TcParser::MpUnknownFields,  // fallback
      #ifdef PROTOBUF_PREFETCH_PARSE_TABLE
      ::_pbi::TcParser::GetTable<::gvisor::syscall::Accept>(),  // to_prefetch
      #endif  // PROTOBUF_PREFETCH_PARSE_TABLE
    }, {{
      {::_pbi::TcParser::MiniParse, {}},
      // .gvisor.common.ContextData context_data = 1;
      {::_pbi::TcParser::FastMtS1,
       {10, 2, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.context_data_)}},
      // .gvisor.syscall.Exit exit = 2;
      {::_pbi::TcParser::FastMtS1,
       {18, 3, 1,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.exit_)}},
      // uint64 sysno = 3;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint64_t, offsetof(Accept, _impl_.sysno_), 4>(),
       {24, 4, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.sysno_)}},
      // int32 fd = 4;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Accept, _impl_.fd_), 5>(),
       {32, 5, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.fd_)}},
      // string fd_path = 5;
      {::_pbi::TcParser::FastUS1,
       {42, 0, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.fd_path_)}},
      // bytes address = 6;
      {::_pbi::TcParser::FastBS1,
       {50, 1, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.address_)}},
      // int32 flags = 7;
      {::_pbi::TcParser::SingularVarintNoZag1<::uint32_t, offsetof(Accept, _impl_.flags_), 6>(),
       {56, 6, 0,
        PROTOBUF_FIELD_OFFSET(Accept, _impl_.flags_)}},
    }}, {{
      65535, 65535
    }}, {{
      // .gvisor.common.ContextData context_data = 1;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.context_data_), _Internal::kHasBitsOffset + 2, 0, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // .gvisor.syscall.Exit exit = 2;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.exit_), _Internal::kHasBitsOffset + 3, 1, (0 | ::_fl::kFcOptional | ::_fl::kMessage | ::_fl::kTvTable)},
      // uint64 sysno = 3;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.sysno_), _Internal::kHasBitsOffset + 4, 0, (0 | ::_fl::kFcOptional | ::_fl::kUInt64)},
      // int32 fd = 4;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.fd_), _Internal::kHasBitsOffset + 5, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
      // string fd_path = 5;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.fd_path_), _Internal::kHasBitsOffset + 0, 0, (0 | ::_fl::kFcOptional | ::_fl::kUtf8String | ::_fl::kRepAString)},
      // bytes address = 6;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.address_), _Internal::kHasBitsOffset + 1, 0, (0 | ::_fl::kFcOptional | ::_fl::kBytes | ::_fl::kRepAString)},
      // int32 flags = 7;
      {PROTOBUF_FIELD_OFFSET(Accept, _impl_.flags_), _Internal::kHasBitsOffset + 6, 0, (0 | ::_fl::kFcOptional | ::_fl::kInt32)},
    }},
    {{
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::common::ContextData>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::common::ContextData_globals_},
        #endif
        #ifndef PROTOBUF_MESSAGE_GLOBALS
        {::_pbi::TcParser::GetTable<::gvisor::syscall::Exit>()},
        #else
        {::_pbi::FieldAuxMessageGlobals(), &::gvisor::syscall::Exit_globals_},
        #endif
    }},
    {{
      "\25\0\0\0\0\7\0\0"
      "gvisor.syscall.Accept"
      "fd_path"
    }},
  };
}


inline constexpr Accept::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    ::_pbi::ConstantInitialized) noexcept
      : fd_path_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        address_(
            &::google::protobuf::internal::fixed_address_empty_string,
            ::_pbi::ConstantInitialized()),
        context_data_{nullptr},
        exit_{nullptr},
        sysno_{::uint64_t{0u}},
        fd_{0},
        flags_{0} {}

template <typename>
constexpr Accept::Accept(::_pbi::ConstantInitialized,
                       const ::_pbi::ClassData* PROTOBUF_NONNULL class_data)
    : Super_(
#if defined(PROTOBUF_CUSTOM_VTABLE)
          class_data
#endif  // PROTOBUF_CUSTOM_VTABLE
          ),
      _impl_(internal_visibility(), ::_pbi::ConstantInitialized()) {
}
inline void* PROTOBUF_NONNULL Accept::PlacementNew_(
    const void* PROTOBUF_NONNULL, void* PROTOBUF_NONNULL mem,
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena) {
  return ::new (mem) Accept(arena);
}
constexpr auto Accept::InternalNewImpl_() {
  return ::google::protobuf::internal::MessageCreator::CopyInit(sizeof(Accept), alignof(Accept));
}
constexpr auto Accept::InternalGenerateClassData_(
    const MessageLite& prototype,
    const ::google::protobuf::internal::TcParseTableBase* tc_table) {
  return ::google::protobuf::internal::ClassDataFull{
      ::google::protobuf::internal::ClassData{
          &prototype,
#ifndef PROTOBUF_MESSAGE_GLOBALS
          &_table_.header,
#else
          tc_table,
#endif
          nullptr,  // IsInitialized
          &Accept::MergeImpl,
          Super_::GetNewImpl<Accept>(),
#if defined(PROTOBUF_CUSTOM_VTABLE)
          &Accept::SharedDtor,
          Super_::GetClearImpl<Accept>(), &Accept::ByteSizeLong,
              &Accept::_InternalSerialize,
#endif  // PROTOBUF_CUSTOM_VTABLE
          PROTOBUF_FIELD_OFFSET(Accept, _impl_._cached_size_),
          false,
      },
#ifdef PROTOBUF_MESSAGE_GLOBALS
      &file_reflection_data[22],
#else   // !PROTOBUF_MESSAGE_GLOBALS
      &::_pbi::kDescriptorMethods,
      &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
      nullptr,  // tracker
#endif  // PROTOBUF_MESSAGE_GLOBALS
  };
}
struct AcceptGlobalsTypeInternal : ::_pbi::MessageGlobalsBase {
  constexpr AcceptGlobalsTypeInternal()
      :
#ifndef PROTOBUF_MESSAGE_GLOBALS
        _default(::_pbi::ConstantInitialized{},
                 Accept_class_data_.base())
#else   // !PROTOBUF_MESSAGE_GLOBALS
        MessageGlobalsBase(Accept::InternalGenerateClassData_(
            _default, &Accept_globals_._table.header)),
        _default(::_pbi::ConstantInitialized{}, GetClassData()),
        _table(::_pbi::PrivateAccess::GenerateParseTable<Accept>(
            GetClassData()))
#endif  // PROTOBUF_MESSAGE_GLOBALS
  {
  }
  ~AcceptGlobalsTypeInternal() {}
  union {
    alignas(::_pbi::kMaxMessageAlignment) Accept _default;
  };
#ifdef PROTOBUF_MESSAGE_GLOBALS
  decltype(::_pbi::PrivateAccess::GenerateParseTable<Accept>(
      ::std::declval<const ::_pbi::ClassData*>())) _table;
#endif
};
#ifdef PROTOBUF_MESSAGE_GLOBALS
static_assert(PROTOBUF_FIELD_OFFSET(AcceptGlobalsTypeInternal, _default) ==
              ::_pbi::MessageGlobalsBase::OffsetToDefault());
#endif  // PROTOBUF_MESSAGE_GLOBALS

PROTOBUF_ATTRIBUTE_NO_DESTROY PROTOBUF_CONSTINIT
    PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 PROTOBUF_MESSAGE_GLOBALS_CONST AcceptGlobalsTypeInternal Accept_globals_
        PROTOBUF_MESSAGE_GLOBALS_SECTION(.data.rel.ro);
#if defined(PROTOBUF_CUSTOM_VTABLE)
namespace {
const ::_pbi::ClassData* Accept_get_class_data() {
#ifdef PROTOBUF_MESSAGE_GLOBALS
  return Accept_globals_.GetClassData();
#else
  return Accept_class_data_.base();
#endif  // PROTOBUF_MESSAGE_GLOBALS
}
}  // namespace
#endif  // PROTOBUF_CUSTOM_VTABLE
}  // namespace syscall
}  // namespace gvisor
static constexpr const ::_pb::EnumDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto = nullptr;
static constexpr const ::_pb::ServiceDescriptor* PROTOBUF_NONNULL* PROTOBUF_NULLABLE
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto = nullptr;
const ::uint32_t
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto::offsets[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
        protodesc_cold) = {
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Exit, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Exit, _impl_.result_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Exit, _impl_.errorno_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg1_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg2_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg3_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg4_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg5_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Syscall, _impl_.arg6_),
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_._has_bits_),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.pathname_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.flags_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Open, _impl_.mode_),
        2,
        3,
        4,
        5,
        0,
        1,
        6,
        7,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Close, _impl_.fd_path_),
        1,
        2,
        3,
        4,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.count_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.has_offset_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.offset_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Read, _impl_.flags_),
        1,
        2,
        3,
        4,
        0,
        5,
        7,
        6,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.count_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.has_offset_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.offset_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Write, _impl_.flags_),
        1,
        2,
        3,
        4,
        0,
        5,
        7,
        6,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Connect, _impl_.address_),
        2,
        3,
        4,
        5,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_._has_bits_),
        12, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.pathname_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.argv_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.envv_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.flags_),
        4,
        5,
        6,
        7,
        2,
        3,
        0,
        1,
        8,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.domain_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Socket, _impl_.protocol_),
        0,
        1,
        2,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chdir, _impl_.pathname_),
        2,
        3,
        4,
        5,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.rid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.eid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setresid, _impl_.sid_),
        0,
        1,
        2,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setid, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setid, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setid, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setid, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Setid, _impl_.id_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::StructRlimit, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::StructRlimit, _impl_.cur_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::StructRlimit, _impl_.max_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.pid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.resource_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.new_limit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Prlimit, _impl_.old_limit_),
        0,
        1,
        4,
        6,
        5,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.flags_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.reader_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Pipe, _impl_.writer_),
        0,
        1,
        2,
        3,
        4,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.cmd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fcntl, _impl_.args_),
        1,
        2,
        3,
        4,
        0,
        5,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.old_fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.new_fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Dup, _impl_.flags_),
        1,
        2,
        3,
        4,
        5,
        0,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.sigset_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Signalfd, _impl_.flags_),
        1,
        2,
        3,
        4,
        0,
        6,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chroot, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chroot, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chroot, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chroot, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Chroot, _impl_.pathname_),
        1,
        2,
        3,
        0,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_.val_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Eventfd, _impl_.flags_),
        0,
        1,
        2,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.flags_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.stack_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.new_tid_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Clone, _impl_.tls_),
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Bind, _impl_.address_),
        2,
        3,
        4,
        5,
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.address_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Accept, _impl_.flags_),
        2,
        3,
        4,
        5,
        0,
        1,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_._has_bits_),
        8, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_.clock_id_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdCreate, _impl_.flags_),
        0,
        1,
        2,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Timespec, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Timespec, _impl_.sec_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Timespec, _impl_.nsec_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::ItimerSpec, _impl_._has_bits_),
        5, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::ItimerSpec, _impl_.interval_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::ItimerSpec, _impl_.value_),
        0,
        1,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_._has_bits_),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.flags_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.new_value_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdSetTime, _impl_.old_value_),
        1,
        2,
        5,
        6,
        0,
        7,
        3,
        4,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::TimerfdGetTime, _impl_.cur_value_),
        1,
        2,
        4,
        5,
        0,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fork, _impl_._has_bits_),
        6, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fork, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fork, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Fork, _impl_.sysno_),
        0,
        1,
        2,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyInit, _impl_._has_bits_),
        7, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyInit, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyInit, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyInit, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyInit, _impl_.flags_),
        0,
        1,
        2,
        3,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_._has_bits_),
        10, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.pathname_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyAddWatch, _impl_.mask_),
        2,
        3,
        4,
        5,
        0,
        1,
        6,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_._has_bits_),
        9, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.fd_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.fd_path_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::InotifyRmWatch, _impl_.wd_),
        1,
        2,
        3,
        4,
        0,
        5,
        0x081, // bitmap
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_._has_bits_),
        11, // hasbit index offset
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.context_data_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.exit_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.sysno_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.domain_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.type_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.protocol_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.socket1_),
        PROTOBUF_FIELD_OFFSET(::gvisor::syscall::SocketPair, _impl_.socket2_),
        0,
        1,
        2,
        3,
        4,
        5,
        6,
        7,
};

static const ::_pbi::MigrationSchema
    schemas[] ABSL_ATTRIBUTE_SECTION_VARIABLE(protodesc_cold) = {
        {0, sizeof(::gvisor::syscall::Exit)},
        {7, sizeof(::gvisor::syscall::Syscall)},
        {28, sizeof(::gvisor::syscall::Open)},
        {47, sizeof(::gvisor::syscall::Close)},
        {60, sizeof(::gvisor::syscall::Read)},
        {81, sizeof(::gvisor::syscall::Write)},
        {102, sizeof(::gvisor::syscall::Connect)},
        {117, sizeof(::gvisor::syscall::Execve)},
        {138, sizeof(::gvisor::syscall::Socket)},
        {153, sizeof(::gvisor::syscall::Chdir)},
        {168, sizeof(::gvisor::syscall::Setresid)},
        {183, sizeof(::gvisor::syscall::Setid)},
        {194, sizeof(::gvisor::syscall::StructRlimit)},
        {201, sizeof(::gvisor::syscall::Prlimit)},
        {218, sizeof(::gvisor::syscall::Pipe)},
        {233, sizeof(::gvisor::syscall::Fcntl)},
        {250, sizeof(::gvisor::syscall::Dup)},
        {267, sizeof(::gvisor::syscall::Signalfd)},
        {284, sizeof(::gvisor::syscall::Chroot)},
        {295, sizeof(::gvisor::syscall::Eventfd)},
        {308, sizeof(::gvisor::syscall::Clone)},
        {325, sizeof(::gvisor::syscall::Bind)},
        {340, sizeof(::gvisor::syscall::Accept)},
        {357, sizeof(::gvisor::syscall::TimerfdCreate)},
        {370, sizeof(::gvisor::syscall::Timespec)},
        {377, sizeof(::gvisor::syscall::ItimerSpec)},
        {384, sizeof(::gvisor::syscall::TimerfdSetTime)},
        {403, sizeof(::gvisor::syscall::TimerfdGetTime)},
        {418, sizeof(::gvisor::syscall::Fork)},
        {427, sizeof(::gvisor::syscall::InotifyInit)},
        {438, sizeof(::gvisor::syscall::InotifyAddWatch)},
        {455, sizeof(::gvisor::syscall::InotifyRmWatch)},
        {470, sizeof(::gvisor::syscall::SocketPair)},
};
static const ::_pbi::MessageGlobalsBase* PROTOBUF_NONNULL const
    file_message_globals[] = {
        &::gvisor::syscall::Exit_globals_,
        &::gvisor::syscall::Syscall_globals_,
        &::gvisor::syscall::Open_globals_,
        &::gvisor::syscall::Close_globals_,
        &::gvisor::syscall::Read_globals_,
        &::gvisor::syscall::Write_globals_,
        &::gvisor::syscall::Connect_globals_,
        &::gvisor::syscall::Execve_globals_,
        &::gvisor::syscall::Socket_globals_,
        &::gvisor::syscall::Chdir_globals_,
        &::gvisor::syscall::Setresid_globals_,
        &::gvisor::syscall::Setid_globals_,
        &::gvisor::syscall::StructRlimit_globals_,
        &::gvisor::syscall::Prlimit_globals_,
        &::gvisor::syscall::Pipe_globals_,
        &::gvisor::syscall::Fcntl_globals_,
        &::gvisor::syscall::Dup_globals_,
        &::gvisor::syscall::Signalfd_globals_,
        &::gvisor::syscall::Chroot_globals_,
        &::gvisor::syscall::Eventfd_globals_,
        &::gvisor::syscall::Clone_globals_,
        &::gvisor::syscall::Bind_globals_,
        &::gvisor::syscall::Accept_globals_,
        &::gvisor::syscall::TimerfdCreate_globals_,
        &::gvisor::syscall::Timespec_globals_,
        &::gvisor::syscall::ItimerSpec_globals_,
        &::gvisor::syscall::TimerfdSetTime_globals_,
        &::gvisor::syscall::TimerfdGetTime_globals_,
        &::gvisor::syscall::Fork_globals_,
        &::gvisor::syscall::InotifyInit_globals_,
        &::gvisor::syscall::InotifyAddWatch_globals_,
        &::gvisor::syscall::InotifyRmWatch_globals_,
        &::gvisor::syscall::SocketPair_globals_,
};
const char descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto[] ABSL_ATTRIBUTE_SECTION_VARIABLE(
    protodesc_cold) = {
    "\n(pkg/sentry/seccheck/points/syscall.pro"
    "to\022\016gvisor.syscall\032\'pkg/sentry/seccheck/"
    "points/common.proto\"\'\n\004Exit\022\016\n\006result\030\001 "
    "\001(\003\022\017\n\007errorno\030\002 \001(\003\"\302\001\n\007Syscall\0220\n\014cont"
    "ext_data\030\001 \001(\0132\032.gvisor.common.ContextDa"
    "ta\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r"
    "\n\005sysno\030\004 \001(\004\022\014\n\004arg1\030\005 \001(\004\022\014\n\004arg2\030\006 \001("
    "\004\022\014\n\004arg3\030\007 \001(\004\022\014\n\004arg4\030\010 \001(\004\022\014\n\004arg5\030\t "
    "\001(\004\022\014\n\004arg6\030\n \001(\004\"\267\001\n\004Open\0220\n\014context_da"
    "ta\030\001 \001(\0132\032.gvisor.common.ContextData\022\"\n\004"
    "exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysn"
    "o\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022\017\n\007fd_path\030\005 \001(\t\022\020\n\010"
    "pathname\030\006 \001(\t\022\r\n\005flags\030\007 \001(\r\022\014\n\004mode\030\010 "
    "\001(\r\"\211\001\n\005Close\0220\n\014context_data\030\001 \001(\0132\032.gv"
    "isor.common.ContextData\022\"\n\004exit\030\002 \001(\0132\024."
    "gvisor.syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd"
    "\030\004 \001(\003\022\017\n\007fd_path\030\005 \001(\t\"\312\001\n\004Read\0220\n\014cont"
    "ext_data\030\001 \001(\0132\032.gvisor.common.ContextDa"
    "ta\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r"
    "\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022\017\n\007fd_path\030\005 \001"
    "(\t\022\r\n\005count\030\006 \001(\004\022\022\n\nhas_offset\030\007 \001(\010\022\016\n"
    "\006offset\030\010 \001(\003\022\r\n\005flags\030\t \001(\r\"\313\001\n\005Write\0220"
    "\n\014context_data\030\001 \001(\0132\032.gvisor.common.Con"
    "textData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall."
    "Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022\017\n\007fd_pa"
    "th\030\005 \001(\t\022\r\n\005count\030\006 \001(\004\022\022\n\nhas_offset\030\007 "
    "\001(\010\022\016\n\006offset\030\010 \001(\003\022\r\n\005flags\030\t \001(\r\"\234\001\n\007C"
    "onnect\0220\n\014context_data\030\001 \001(\0132\032.gvisor.co"
    "mmon.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor."
    "syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022"
    "\017\n\007fd_path\030\005 \001(\t\022\017\n\007address\030\006 \001(\014\"\307\001\n\006Ex"
    "ecve\0220\n\014context_data\030\001 \001(\0132\032.gvisor.comm"
    "on.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.sy"
    "scall.Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022\017\n"
    "\007fd_path\030\005 \001(\t\022\020\n\010pathname\030\006 \001(\t\022\014\n\004argv"
    "\030\007 \003(\t\022\014\n\004envv\030\010 \003(\t\022\r\n\005flags\030\t \001(\r\"\235\001\n\006"
    "Socket\0220\n\014context_data\030\001 \001(\0132\032.gvisor.co"
    "mmon.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor."
    "syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\016\n\006domain\030\004 "
    "\001(\005\022\014\n\004type\030\005 \001(\005\022\020\n\010protocol\030\006 \001(\005\"\233\001\n\005"
    "Chdir\0220\n\014context_data\030\001 \001(\0132\032.gvisor.com"
    "mon.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.s"
    "yscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\003\022\017"
    "\n\007fd_path\030\005 \001(\t\022\020\n\010pathname\030\006 \001(\t\"\226\001\n\010Se"
    "tresid\0220\n\014context_data\030\001 \001(\0132\032.gvisor.co"
    "mmon.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor."
    "syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\013\n\003rid\030\004 \001(\r"
    "\022\013\n\003eid\030\005 \001(\r\022\013\n\003sid\030\006 \001(\r\"x\n\005Setid\0220\n\014c"
    "ontext_data\030\001 \001(\0132\032.gvisor.common.Contex"
    "tData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exi"
    "t\022\r\n\005sysno\030\003 \001(\004\022\n\n\002id\030\004 \001(\r\"(\n\014StructRl"
    "imit\022\013\n\003cur\030\001 \001(\004\022\013\n\003max\030\002 \001(\004\"\357\001\n\007Prlim"
    "it\0220\n\014context_data\030\001 \001(\0132\032.gvisor.common"
    ".ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.sysc"
    "all.Exit\022\r\n\005sysno\030\003 \001(\004\022\013\n\003pid\030\004 \001(\005\022\020\n\010"
    "resource\030\005 \001(\003\022/\n\tnew_limit\030\006 \001(\0132\034.gvis"
    "or.syscall.StructRlimit\022/\n\told_limit\030\007 \001"
    "(\0132\034.gvisor.syscall.StructRlimit\"\232\001\n\004Pip"
    "e\0220\n\014context_data\030\001 \001(\0132\032.gvisor.common."
    "ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.sysca"
    "ll.Exit\022\r\n\005sysno\030\003 \001(\004\022\r\n\005flags\030\004 \001(\r\022\016\n"
    "\006reader\030\005 \001(\005\022\016\n\006writer\030\006 \001(\005\"\244\001\n\005Fcntl\022"
    "0\n\014context_data\030\001 \001(\0132\032.gvisor.common.Co"
    "ntextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall"
    ".Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_p"
    "ath\030\005 \001(\t\022\013\n\003cmd\030\006 \001(\005\022\014\n\004args\030\007 \001(\003\"\252\001\n"
    "\003Dup\0220\n\014context_data\030\001 \001(\0132\032.gvisor.comm"
    "on.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.sy"
    "scall.Exit\022\r\n\005sysno\030\003 \001(\004\022\016\n\006old_fd\030\004 \001("
    "\005\022\016\n\006new_fd\030\005 \001(\005\022\017\n\007fd_path\030\006 \001(\t\022\r\n\005fl"
    "ags\030\007 \001(\r\"\253\001\n\010Signalfd\0220\n\014context_data\030\001"
    " \001(\0132\032.gvisor.common.ContextData\022\"\n\004exit"
    "\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003 "
    "\001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_path\030\005 \001(\t\022\016\n\006sigs"
    "et\030\006 \001(\004\022\r\n\005flags\030\007 \001(\005\"\177\n\006Chroot\0220\n\014con"
    "text_data\030\001 \001(\0132\032.gvisor.common.ContextD"
    "ata\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022"
    "\r\n\005sysno\030\003 \001(\004\022\020\n\010pathname\030\004 \001(\t\"\212\001\n\007Eve"
    "ntfd\0220\n\014context_data\030\001 \001(\0132\032.gvisor.comm"
    "on.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.sy"
    "scall.Exit\022\r\n\005sysno\030\003 \001(\004\022\013\n\003val\030\004 \001(\005\022\r"
    "\n\005flags\030\005 \001(\r\"\250\001\n\005Clone\0220\n\014context_data\030"
    "\001 \001(\0132\032.gvisor.common.ContextData\022\"\n\004exi"
    "t\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003"
    " \001(\004\022\r\n\005flags\030\004 \001(\004\022\r\n\005stack\030\005 \001(\004\022\017\n\007ne"
    "w_tid\030\006 \001(\004\022\013\n\003tls\030\007 \001(\004\"\231\001\n\004Bind\0220\n\014con"
    "text_data\030\001 \001(\0132\032.gvisor.common.ContextD"
    "ata\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022"
    "\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_path\030\005 "
    "\001(\t\022\017\n\007address\030\006 \001(\014\"\252\001\n\006Accept\0220\n\014conte"
    "xt_data\030\001 \001(\0132\032.gvisor.common.ContextDat"
    "a\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n"
    "\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_path\030\005 \001("
    "\t\022\017\n\007address\030\006 \001(\014\022\r\n\005flags\030\007 \001(\005\"\225\001\n\rTi"
    "merfdCreate\0220\n\014context_data\030\001 \001(\0132\032.gvis"
    "or.common.ContextData\022\"\n\004exit\030\002 \001(\0132\024.gv"
    "isor.syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\020\n\010cloc"
    "k_id\030\004 \001(\005\022\r\n\005flags\030\005 \001(\005\"%\n\010Timespec\022\013\n"
    "\003sec\030\001 \001(\003\022\014\n\004nsec\030\002 \001(\003\"a\n\nItimerSpec\022*"
    "\n\010interval\030\001 \001(\0132\030.gvisor.syscall.Timesp"
    "ec\022\'\n\005value\030\002 \001(\0132\030.gvisor.syscall.Times"
    "pec\"\377\001\n\016TimerfdSetTime\0220\n\014context_data\030\001"
    " \001(\0132\032.gvisor.common.ContextData\022\"\n\004exit"
    "\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003 "
    "\001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_path\030\005 \001(\t\022\r\n\005flag"
    "s\030\006 \001(\005\022-\n\tnew_value\030\007 \001(\0132\032.gvisor.sysc"
    "all.ItimerSpec\022-\n\told_value\030\010 \001(\0132\032.gvis"
    "or.syscall.ItimerSpec\"\301\001\n\016TimerfdGetTime"
    "\0220\n\014context_data\030\001 \001(\0132\032.gvisor.common.C"
    "ontextData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscal"
    "l.Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_"
    "path\030\005 \001(\t\022-\n\tcur_value\030\006 \001(\0132\032.gvisor.s"
    "yscall.ItimerSpec\"k\n\004Fork\0220\n\014context_dat"
    "a\030\001 \001(\0132\032.gvisor.common.ContextData\022\"\n\004e"
    "xit\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno"
    "\030\003 \001(\004\"\201\001\n\013InotifyInit\0220\n\014context_data\030\001"
    " \001(\0132\032.gvisor.common.ContextData\022\"\n\004exit"
    "\030\002 \001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003 "
    "\001(\004\022\r\n\005flags\030\004 \001(\005\"\263\001\n\017InotifyAddWatch\0220"
    "\n\014context_data\030\001 \001(\0132\032.gvisor.common.Con"
    "textData\022\"\n\004exit\030\002 \001(\0132\024.gvisor.syscall."
    "Exit\022\r\n\005sysno\030\003 \001(\004\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_pa"
    "th\030\005 \001(\t\022\020\n\010pathname\030\006 \001(\t\022\014\n\004mask\030\007 \001(\r"
    "\"\236\001\n\016InotifyRmWatch\0220\n\014context_data\030\001 \001("
    "\0132\032.gvisor.common.ContextData\022\"\n\004exit\030\002 "
    "\001(\0132\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003 \001(\004"
    "\022\n\n\002fd\030\004 \001(\005\022\017\n\007fd_path\030\005 \001(\t\022\n\n\002wd\030\006 \001("
    "\005\"\303\001\n\nSocketPair\0220\n\014context_data\030\001 \001(\0132\032"
    ".gvisor.common.ContextData\022\"\n\004exit\030\002 \001(\013"
    "2\024.gvisor.syscall.Exit\022\r\n\005sysno\030\003 \001(\004\022\016\n"
    "\006domain\030\004 \001(\005\022\014\n\004type\030\005 \001(\005\022\020\n\010protocol\030"
    "\006 \001(\005\022\017\n\007socket1\030\007 \001(\005\022\017\n\007socket2\030\010 \001(\005b"
    "\006proto3"
};
static const ::_pbi::DescriptorTable* PROTOBUF_NONNULL const
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_deps[1] = {
        &::descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fcommon_2eproto,
};
static ::absl::once_flag descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_once;
PROTOBUF_CONSTINIT const ::_pbi::DescriptorTable descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto = {
    false,
    false,
    5287,
    descriptor_table_protodef_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
    "pkg/sentry/seccheck/points/syscall.proto",
    &descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_once,
    descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto_deps,
    1,
    33,
    schemas,
    file_message_globals,
    TableStruct_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto::offsets,
    file_level_enum_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
    file_level_service_descriptors_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto,
};
namespace gvisor {
namespace syscall {
// ===================================================================

Exit::Exit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Exit_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Exit)
}
Exit::Exit(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Exit& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Exit_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Exit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Exit::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, result_),
           0,
           offsetof(Impl_, errorno_) -
               offsetof(Impl_, result_) +
               sizeof(Impl_::errorno_));
}
Exit::~Exit() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Exit)
  SharedDtor(*this);
}
inline void Exit::SharedDtor(MessageLite& self) {
  Exit& this_ = static_cast<Exit&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Exit_class_data_ =
        Exit::InternalGenerateClassData_(Exit_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Exit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Exit_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Exit_class_data_.tc_table);
  return Exit_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Exit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Exit_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Exit_globals_));
  return Exit_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Exit::ParseTableT_
    Exit::_table_ =
        Exit::InternalGenerateParseTable_(Exit_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Exit::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Exit)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&this_._impl_.result_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.errorno_) -
                 reinterpret_cast<char*>(&this_._impl_.result_)) +
                 sizeof(_impl_.errorno_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Exit::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Exit& this_ = static_cast<const Exit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Exit::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Exit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Exit)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 result = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_result() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_result(), target);
    }
  }

  // int64 errorno = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_errorno() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_errorno(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Exit)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Exit::ByteSizeLong(const MessageLite& base) {
  const Exit& this_ = static_cast<const Exit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Exit::ByteSizeLong() const {
  const Exit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Exit)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // int64 result = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_result() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_result());
      }
    }
    // int64 errorno = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_errorno() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_errorno());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Exit::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Exit*>(&to_msg);
  auto& from = static_cast<const Exit&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Exit)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_result() != 0) {
        _this->_impl_.result_ = from._impl_.result_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_errorno() != 0) {
        _this->_impl_.errorno_ = from._impl_.errorno_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Exit::CopyFrom(const Exit& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Exit)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Exit::InternalSwap(Exit* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Exit, _impl_.errorno_)
      + sizeof(Exit::_impl_.errorno_)
      - PROTOBUF_FIELD_OFFSET(Exit, _impl_.result_)>(
          reinterpret_cast<char*>(&_impl_.result_),
          reinterpret_cast<char*>(&other->_impl_.result_));
}

::google::protobuf::Metadata Exit::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Syscall::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Syscall::Syscall(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Syscall_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Syscall)
}
PROTOBUF_NDEBUG_INLINE Syscall::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Syscall& from_msg)
      : _has_bits_{from._has_bits_} {}

Syscall::Syscall(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Syscall& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Syscall_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Syscall* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, arg6_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::arg6_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Syscall)
}
PROTOBUF_NDEBUG_INLINE Syscall::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Syscall::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, arg6_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::arg6_));
}
Syscall::~Syscall() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Syscall)
  SharedDtor(*this);
}
inline void Syscall::SharedDtor(MessageLite& self) {
  Syscall& this_ = static_cast<Syscall&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Syscall_class_data_ =
        Syscall::InternalGenerateClassData_(Syscall_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Syscall::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Syscall_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Syscall_class_data_.tc_table);
  return Syscall_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Syscall::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Syscall_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Syscall_globals_));
  return Syscall_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Syscall::ParseTableT_
    Syscall::_table_ =
        Syscall::InternalGenerateParseTable_(Syscall_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Syscall::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Syscall)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000fcU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.arg5_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.arg5_));
  }
  this_._impl_.arg6_ = ::uint64_t{0u};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Syscall::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Syscall& this_ = static_cast<const Syscall&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Syscall::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Syscall& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Syscall)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 4;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          4, this_._internal_sysno(), target);
    }
  }

  // uint64 arg1 = 5;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_arg1() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          5, this_._internal_arg1(), target);
    }
  }

  // uint64 arg2 = 6;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_arg2() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_arg2(), target);
    }
  }

  // uint64 arg3 = 7;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_arg3() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          7, this_._internal_arg3(), target);
    }
  }

  // uint64 arg4 = 8;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_arg4() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          8, this_._internal_arg4(), target);
    }
  }

  // uint64 arg5 = 9;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_arg5() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          9, this_._internal_arg5(), target);
    }
  }

  // uint64 arg6 = 10;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_arg6() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          10, this_._internal_arg6(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Syscall)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Syscall::ByteSizeLong(const MessageLite& base) {
  const Syscall& this_ = static_cast<const Syscall&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Syscall::ByteSizeLong() const {
  const Syscall& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Syscall)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 4;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // uint64 arg1 = 5;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_arg1() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg1());
      }
    }
    // uint64 arg2 = 6;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_arg2() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg2());
      }
    }
    // uint64 arg3 = 7;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_arg3() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg3());
      }
    }
    // uint64 arg4 = 8;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_arg4() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg4());
      }
    }
    // uint64 arg5 = 9;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_arg5() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg5());
      }
    }
  }
   {
    // uint64 arg6 = 10;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_arg6() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_arg6());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Syscall::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Syscall*>(&to_msg);
  auto& from = static_cast<const Syscall&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Syscall)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_arg1() != 0) {
        _this->_impl_.arg1_ = from._impl_.arg1_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_arg2() != 0) {
        _this->_impl_.arg2_ = from._impl_.arg2_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_arg3() != 0) {
        _this->_impl_.arg3_ = from._impl_.arg3_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_arg4() != 0) {
        _this->_impl_.arg4_ = from._impl_.arg4_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_arg5() != 0) {
        _this->_impl_.arg5_ = from._impl_.arg5_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_arg6() != 0) {
      _this->_impl_.arg6_ = from._impl_.arg6_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Syscall::CopyFrom(const Syscall& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Syscall)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Syscall::InternalSwap(Syscall* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Syscall, _impl_.arg6_)
      + sizeof(Syscall::_impl_.arg6_)
      - PROTOBUF_FIELD_OFFSET(Syscall, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Syscall::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Open::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Open::Open(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Open_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Open)
}
PROTOBUF_NDEBUG_INLINE Open::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Open& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        pathname_(arena, from.pathname_) {}

Open::Open(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Open& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Open_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Open* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, mode_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::mode_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Open)
}
PROTOBUF_NDEBUG_INLINE Open::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        pathname_(arena) {}

inline void Open::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, mode_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::mode_));
}
Open::~Open() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Open)
  SharedDtor(*this);
}
inline void Open::SharedDtor(MessageLite& self) {
  Open& this_ = static_cast<Open&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.pathname_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Open_class_data_ =
        Open::InternalGenerateClassData_(Open_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Open::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Open_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Open_class_data_.tc_table);
  return Open_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Open::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Open_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Open_globals_));
  return Open_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Open::ParseTableT_
    Open::_table_ =
        Open::InternalGenerateParseTable_(Open_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Open::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Open)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.pathname_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f0U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.mode_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.mode_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Open::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Open& this_ = static_cast<const Open&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Open::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Open& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Open)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Open.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string pathname = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_pathname().empty()) {
      const ::std::string& _s = this_._internal_pathname();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Open.pathname");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // uint32 flags = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          7, this_._internal_flags(), target);
    }
  }

  // uint32 mode = 8;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_mode() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          8, this_._internal_mode(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Open)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Open::ByteSizeLong(const MessageLite& base) {
  const Open& this_ = static_cast<const Open&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Open::ByteSizeLong() const {
  const Open& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Open)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // string pathname = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_pathname().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_pathname());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
    // uint32 flags = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
    // uint32 mode = 8;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_mode() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_mode());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Open::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Open*>(&to_msg);
  auto& from = static_cast<const Open&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Open)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_pathname().empty()) {
        _this->_internal_set_pathname(from._internal_pathname());
      } else {
        if (_this->_impl_.pathname_.IsDefault()) {
          _this->_internal_set_pathname("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_mode() != 0) {
        _this->_impl_.mode_ = from._impl_.mode_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Open::CopyFrom(const Open& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Open)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Open::InternalSwap(Open* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.pathname_, &other->_impl_.pathname_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Open, _impl_.mode_)
      + sizeof(Open::_impl_.mode_)
      - PROTOBUF_FIELD_OFFSET(Open, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Open::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Close::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Close::Close(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Close_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Close)
}
PROTOBUF_NDEBUG_INLINE Close::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Close& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Close::Close(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Close& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Close_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Close* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, fd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::fd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Close)
}
PROTOBUF_NDEBUG_INLINE Close::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Close::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, fd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::fd_));
}
Close::~Close() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Close)
  SharedDtor(*this);
}
inline void Close::SharedDtor(MessageLite& self) {
  Close& this_ = static_cast<Close&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Close_class_data_ =
        Close::InternalGenerateClassData_(Close_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Close::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Close_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Close_class_data_.tc_table);
  return Close_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Close::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Close_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Close_globals_));
  return Close_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Close::ParseTableT_
    Close::_table_ =
        Close::InternalGenerateParseTable_(Close_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Close::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Close)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000018U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Close::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Close& this_ = static_cast<const Close&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Close::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Close& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Close)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Close.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Close)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Close::ByteSizeLong(const MessageLite& base) {
  const Close& this_ = static_cast<const Close&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Close::ByteSizeLong() const {
  const Close& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Close)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Close::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Close*>(&to_msg);
  auto& from = static_cast<const Close&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Close)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Close::CopyFrom(const Close& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Close)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Close::InternalSwap(Close* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Close, _impl_.fd_)
      + sizeof(Close::_impl_.fd_)
      - PROTOBUF_FIELD_OFFSET(Close, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Close::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Read::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Read::Read(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Read_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Read)
}
PROTOBUF_NDEBUG_INLINE Read::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Read& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Read::Read(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Read& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Read_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Read* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Read)
}
PROTOBUF_NDEBUG_INLINE Read::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Read::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Read::~Read() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Read)
  SharedDtor(*this);
}
inline void Read::SharedDtor(MessageLite& self) {
  Read& this_ = static_cast<Read&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Read_class_data_ =
        Read::InternalGenerateClassData_(Read_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Read::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Read_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Read_class_data_.tc_table);
  return Read_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Read::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Read_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Read_globals_));
  return Read_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Read::ParseTableT_
    Read::_table_ =
        Read::InternalGenerateParseTable_(Read_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Read::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Read)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f8U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.has_offset_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.has_offset_));
  }
  this_._impl_.flags_ = 0u;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Read::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Read& this_ = static_cast<const Read&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Read::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Read& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Read)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Read.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // uint64 count = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_count() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_count(), target);
    }
  }

  // bool has_offset = 7;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_has_offset() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          7, this_._internal_has_offset(), target);
    }
  }

  // int64 offset = 8;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_offset() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<8>(
              stream, this_._internal_offset(), target);
    }
  }

  // uint32 flags = 9;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          9, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Read)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Read::ByteSizeLong(const MessageLite& base) {
  const Read& this_ = static_cast<const Read&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Read::ByteSizeLong() const {
  const Read& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Read)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
    // uint64 count = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_count() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_count());
      }
    }
    // int64 offset = 8;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_offset() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_offset());
      }
    }
    // bool has_offset = 7;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_has_offset() != 0) {
        total_size += 2;
      }
    }
  }
   {
    // uint32 flags = 9;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Read::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Read*>(&to_msg);
  auto& from = static_cast<const Read&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Read)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_count() != 0) {
        _this->_impl_.count_ = from._impl_.count_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_offset() != 0) {
        _this->_impl_.offset_ = from._impl_.offset_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_has_offset() != 0) {
        _this->_impl_.has_offset_ = from._impl_.has_offset_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_flags() != 0) {
      _this->_impl_.flags_ = from._impl_.flags_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Read::CopyFrom(const Read& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Read)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Read::InternalSwap(Read* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Read, _impl_.flags_)
      + sizeof(Read::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Read, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Read::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Write::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Write::Write(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Write_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Write)
}
PROTOBUF_NDEBUG_INLINE Write::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Write& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Write::Write(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Write& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Write_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Write* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Write)
}
PROTOBUF_NDEBUG_INLINE Write::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Write::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Write::~Write() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Write)
  SharedDtor(*this);
}
inline void Write::SharedDtor(MessageLite& self) {
  Write& this_ = static_cast<Write&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Write_class_data_ =
        Write::InternalGenerateClassData_(Write_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Write::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Write_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Write_class_data_.tc_table);
  return Write_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Write::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Write_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Write_globals_));
  return Write_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Write::ParseTableT_
    Write::_table_ =
        Write::InternalGenerateParseTable_(Write_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Write::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Write)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000f8U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.has_offset_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.has_offset_));
  }
  this_._impl_.flags_ = 0u;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Write::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Write& this_ = static_cast<const Write&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Write::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Write& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Write)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Write.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // uint64 count = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_count() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_count(), target);
    }
  }

  // bool has_offset = 7;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_has_offset() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteBoolToArray(
          7, this_._internal_has_offset(), target);
    }
  }

  // int64 offset = 8;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_offset() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<8>(
              stream, this_._internal_offset(), target);
    }
  }

  // uint32 flags = 9;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          9, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Write)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Write::ByteSizeLong(const MessageLite& base) {
  const Write& this_ = static_cast<const Write&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Write::ByteSizeLong() const {
  const Write& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Write)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
    // uint64 count = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_count() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_count());
      }
    }
    // int64 offset = 8;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_offset() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_offset());
      }
    }
    // bool has_offset = 7;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_has_offset() != 0) {
        total_size += 2;
      }
    }
  }
   {
    // uint32 flags = 9;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Write::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Write*>(&to_msg);
  auto& from = static_cast<const Write&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Write)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_count() != 0) {
        _this->_impl_.count_ = from._impl_.count_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_offset() != 0) {
        _this->_impl_.offset_ = from._impl_.offset_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_has_offset() != 0) {
        _this->_impl_.has_offset_ = from._impl_.has_offset_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_flags() != 0) {
      _this->_impl_.flags_ = from._impl_.flags_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Write::CopyFrom(const Write& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Write)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Write::InternalSwap(Write* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Write, _impl_.flags_)
      + sizeof(Write::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Write, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Write::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Connect::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Connect::Connect(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Connect_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Connect)
}
PROTOBUF_NDEBUG_INLINE Connect::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Connect& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        address_(arena, from.address_) {}

Connect::Connect(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Connect& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Connect_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Connect* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, fd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::fd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Connect)
}
PROTOBUF_NDEBUG_INLINE Connect::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        address_(arena) {}

inline void Connect::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, fd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::fd_));
}
Connect::~Connect() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Connect)
  SharedDtor(*this);
}
inline void Connect::SharedDtor(MessageLite& self) {
  Connect& this_ = static_cast<Connect&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.address_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Connect_class_data_ =
        Connect::InternalGenerateClassData_(Connect_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Connect::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Connect_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Connect_class_data_.tc_table);
  return Connect_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Connect::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Connect_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Connect_globals_));
  return Connect_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Connect::ParseTableT_
    Connect::_table_ =
        Connect::InternalGenerateParseTable_(Connect_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Connect::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Connect)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.address_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000030U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Connect::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Connect& this_ = static_cast<const Connect&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Connect::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Connect& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Connect)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Connect.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // bytes address = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      target = stream->WriteBytesMaybeAliased(6, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Connect)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Connect::ByteSizeLong(const MessageLite& base) {
  const Connect& this_ = static_cast<const Connect&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Connect::ByteSizeLong() const {
  const Connect& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Connect)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // bytes address = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_address());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Connect::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Connect*>(&to_msg);
  auto& from = static_cast<const Connect&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Connect)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_address().empty()) {
        _this->_internal_set_address(from._internal_address());
      } else {
        if (_this->_impl_.address_.IsDefault()) {
          _this->_internal_set_address("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Connect::CopyFrom(const Connect& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Connect)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Connect::InternalSwap(Connect* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Connect, _impl_.fd_)
      + sizeof(Connect::_impl_.fd_)
      - PROTOBUF_FIELD_OFFSET(Connect, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Connect::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Execve::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000010U);
}
Execve::Execve(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Execve_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Execve)
}
PROTOBUF_NDEBUG_INLINE Execve::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Execve& from_msg)
      : _has_bits_{from._has_bits_},
        argv_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::syscall::Execve,
              PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.argv_)>()
          , arena, from.argv_
        }
        ,
        envv_ {
          visibility, ::_pbi::InternalMetadataOffset::Build<
              ::gvisor::syscall::Execve,
              PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.envv_)>()
          , arena, from.envv_
        }
        ,
        fd_path_(arena, from.fd_path_),
        pathname_(arena, from.pathname_) {}

Execve::Execve(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Execve& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Execve_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Execve* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000020U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Execve)
}
PROTOBUF_NDEBUG_INLINE Execve::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : argv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::syscall::Execve,
            PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.argv_)>()
         }
        ,
        envv_ { visibility, ::_pbi::InternalMetadataOffset::Build<
            ::gvisor::syscall::Execve,
            PROTOBUF_FIELD_OFFSET(::gvisor::syscall::Execve, _impl_.envv_)>()
         }
        ,
        fd_path_(arena),
        pathname_(arena) {}

inline void Execve::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Execve::~Execve() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Execve)
  SharedDtor(*this);
}
inline void Execve::SharedDtor(MessageLite& self) {
  Execve& this_ = static_cast<Execve&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.pathname_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Execve_class_data_ =
        Execve::InternalGenerateClassData_(Execve_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Execve::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Execve_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Execve_class_data_.tc_table);
  return Execve_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Execve::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Execve_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Execve_globals_));
  return Execve_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Execve::ParseTableT_
    Execve::_table_ =
        Execve::InternalGenerateParseTable_(Execve_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Execve::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Execve)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _impl_.argv_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _impl_.envv_.Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      this_._impl_.pathname_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000c0U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  this_._impl_.flags_ = 0u;
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Execve::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Execve& this_ = static_cast<const Execve&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Execve::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Execve& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Execve)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Execve.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string pathname = 6;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (!this_._internal_pathname().empty()) {
      const ::std::string& _s = this_._internal_pathname();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Execve.pathname");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // repeated string argv = 7;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    for (int i = 0, n = this_._internal_argv_size(); i < n; ++i) {
      const auto& s = this_._internal_argv().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Execve.argv");
      target = stream->WriteString(7, s, target);
    }
  }

  // repeated string envv = 8;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    for (int i = 0, n = this_._internal_envv_size(); i < n; ++i) {
      const auto& s = this_._internal_envv().Get(i);
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          s.data(), static_cast<int>(s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Execve.envv");
      target = stream->WriteString(8, s, target);
    }
  }

  // uint32 flags = 9;
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          9, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Execve)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Execve::ByteSizeLong(const MessageLite& base) {
  const Execve& this_ = static_cast<const Execve&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Execve::ByteSizeLong() const {
  const Execve& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Execve)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // repeated string argv = 7;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_argv().size());
      for (int i = 0, n = this_._internal_argv().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_argv().Get(i));
      }
    }
    // repeated string envv = 8;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size +=
          1 * ::google::protobuf::internal::FromIntSize(this_._internal_envv().size());
      for (int i = 0, n = this_._internal_envv().size(); i < n; ++i) {
        total_size += ::google::protobuf::internal::WireFormatLite::StringSize(
            this_._internal_envv().Get(i));
      }
    }
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // string pathname = 6;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!this_._internal_pathname().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_pathname());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
  }
   {
    // uint32 flags = 9;
    if (CheckHasBit(cached_has_bits, 0x00000100U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Execve::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Execve*>(&to_msg);
  auto& from = static_cast<const Execve&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Execve)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      _this->_internal_mutable_argv()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_argv());
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      _this->_internal_mutable_envv()->InternalMergeFromWithArena(
          ::google::protobuf::MessageLite::internal_visibility(), arena,
          from._internal_envv());
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (!from._internal_pathname().empty()) {
        _this->_internal_set_pathname(from._internal_pathname());
      } else {
        if (_this->_impl_.pathname_.IsDefault()) {
          _this->_internal_set_pathname("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  if (CheckHasBit(cached_has_bits, 0x00000100U)) {
    if (from._internal_flags() != 0) {
      _this->_impl_.flags_ = from._impl_.flags_;
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Execve::CopyFrom(const Execve& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Execve)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Execve::InternalSwap(Execve* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  _impl_.argv_.InternalSwap(&other->_impl_.argv_);
  _impl_.envv_.InternalSwap(&other->_impl_.envv_);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.pathname_, &other->_impl_.pathname_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Execve, _impl_.flags_)
      + sizeof(Execve::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Execve, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Execve::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Socket::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Socket::Socket(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Socket_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Socket)
}
PROTOBUF_NDEBUG_INLINE Socket::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Socket& from_msg)
      : _has_bits_{from._has_bits_} {}

Socket::Socket(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Socket& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Socket_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Socket* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, protocol_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::protocol_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Socket)
}
PROTOBUF_NDEBUG_INLINE Socket::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Socket::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, protocol_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::protocol_));
}
Socket::~Socket() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Socket)
  SharedDtor(*this);
}
inline void Socket::SharedDtor(MessageLite& self) {
  Socket& this_ = static_cast<Socket&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Socket_class_data_ =
        Socket::InternalGenerateClassData_(Socket_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Socket::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Socket_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Socket_class_data_.tc_table);
  return Socket_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Socket::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Socket_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Socket_globals_));
  return Socket_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Socket::ParseTableT_
    Socket::_table_ =
        Socket::InternalGenerateParseTable_(Socket_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Socket::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Socket)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000003cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.protocol_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.protocol_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Socket::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Socket& this_ = static_cast<const Socket&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Socket::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Socket& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Socket)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 domain = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_domain() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_domain(), target);
    }
  }

  // int32 type = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_type() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_type(), target);
    }
  }

  // int32 protocol = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_protocol() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_protocol(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Socket)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Socket::ByteSizeLong(const MessageLite& base) {
  const Socket& this_ = static_cast<const Socket&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Socket::ByteSizeLong() const {
  const Socket& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Socket)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 domain = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_domain() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_domain());
      }
    }
    // int32 type = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_type() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_type());
      }
    }
    // int32 protocol = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_protocol() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_protocol());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Socket::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Socket*>(&to_msg);
  auto& from = static_cast<const Socket&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Socket)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_domain() != 0) {
        _this->_impl_.domain_ = from._impl_.domain_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_protocol() != 0) {
        _this->_impl_.protocol_ = from._impl_.protocol_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Socket::CopyFrom(const Socket& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Socket)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Socket::InternalSwap(Socket* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Socket, _impl_.protocol_)
      + sizeof(Socket::_impl_.protocol_)
      - PROTOBUF_FIELD_OFFSET(Socket, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Socket::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Chdir::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Chdir::Chdir(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Chdir_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Chdir)
}
PROTOBUF_NDEBUG_INLINE Chdir::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Chdir& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        pathname_(arena, from.pathname_) {}

Chdir::Chdir(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Chdir& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Chdir_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Chdir* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, fd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::fd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Chdir)
}
PROTOBUF_NDEBUG_INLINE Chdir::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        pathname_(arena) {}

inline void Chdir::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, fd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::fd_));
}
Chdir::~Chdir() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Chdir)
  SharedDtor(*this);
}
inline void Chdir::SharedDtor(MessageLite& self) {
  Chdir& this_ = static_cast<Chdir&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.pathname_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Chdir_class_data_ =
        Chdir::InternalGenerateClassData_(Chdir_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Chdir::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Chdir_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Chdir_class_data_.tc_table);
  return Chdir_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Chdir::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Chdir_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Chdir_globals_));
  return Chdir_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Chdir::ParseTableT_
    Chdir::_table_ =
        Chdir::InternalGenerateParseTable_(Chdir_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Chdir::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Chdir)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.pathname_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000030U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Chdir::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Chdir& this_ = static_cast<const Chdir&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Chdir::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Chdir& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Chdir)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int64 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Chdir.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string pathname = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_pathname().empty()) {
      const ::std::string& _s = this_._internal_pathname();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Chdir.pathname");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Chdir)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Chdir::ByteSizeLong(const MessageLite& base) {
  const Chdir& this_ = static_cast<const Chdir&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Chdir::ByteSizeLong() const {
  const Chdir& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Chdir)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // string pathname = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_pathname().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_pathname());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_fd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Chdir::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Chdir*>(&to_msg);
  auto& from = static_cast<const Chdir&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Chdir)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_pathname().empty()) {
        _this->_internal_set_pathname(from._internal_pathname());
      } else {
        if (_this->_impl_.pathname_.IsDefault()) {
          _this->_internal_set_pathname("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Chdir::CopyFrom(const Chdir& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Chdir)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Chdir::InternalSwap(Chdir* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.pathname_, &other->_impl_.pathname_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Chdir, _impl_.fd_)
      + sizeof(Chdir::_impl_.fd_)
      - PROTOBUF_FIELD_OFFSET(Chdir, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Chdir::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Setresid::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Setresid::Setresid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Setresid_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Setresid)
}
PROTOBUF_NDEBUG_INLINE Setresid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Setresid& from_msg)
      : _has_bits_{from._has_bits_} {}

Setresid::Setresid(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Setresid& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Setresid_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Setresid* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, sid_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::sid_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Setresid)
}
PROTOBUF_NDEBUG_INLINE Setresid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Setresid::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, sid_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::sid_));
}
Setresid::~Setresid() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Setresid)
  SharedDtor(*this);
}
inline void Setresid::SharedDtor(MessageLite& self) {
  Setresid& this_ = static_cast<Setresid&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Setresid_class_data_ =
        Setresid::InternalGenerateClassData_(Setresid_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Setresid::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Setresid_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Setresid_class_data_.tc_table);
  return Setresid_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Setresid::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Setresid_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Setresid_globals_));
  return Setresid_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Setresid::ParseTableT_
    Setresid::_table_ =
        Setresid::InternalGenerateParseTable_(Setresid_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Setresid::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Setresid)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000003cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.sid_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.sid_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Setresid::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Setresid& this_ = static_cast<const Setresid&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Setresid::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Setresid& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Setresid)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // uint32 rid = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_rid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          4, this_._internal_rid(), target);
    }
  }

  // uint32 eid = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_eid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          5, this_._internal_eid(), target);
    }
  }

  // uint32 sid = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_sid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          6, this_._internal_sid(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Setresid)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Setresid::ByteSizeLong(const MessageLite& base) {
  const Setresid& this_ = static_cast<const Setresid&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Setresid::ByteSizeLong() const {
  const Setresid& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Setresid)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // uint32 rid = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_rid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_rid());
      }
    }
    // uint32 eid = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_eid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_eid());
      }
    }
    // uint32 sid = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_sid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_sid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Setresid::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Setresid*>(&to_msg);
  auto& from = static_cast<const Setresid&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Setresid)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_rid() != 0) {
        _this->_impl_.rid_ = from._impl_.rid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_eid() != 0) {
        _this->_impl_.eid_ = from._impl_.eid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_sid() != 0) {
        _this->_impl_.sid_ = from._impl_.sid_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Setresid::CopyFrom(const Setresid& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Setresid)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Setresid::InternalSwap(Setresid* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Setresid, _impl_.sid_)
      + sizeof(Setresid::_impl_.sid_)
      - PROTOBUF_FIELD_OFFSET(Setresid, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Setresid::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Setid::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Setid::Setid(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Setid_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Setid)
}
PROTOBUF_NDEBUG_INLINE Setid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Setid& from_msg)
      : _has_bits_{from._has_bits_} {}

Setid::Setid(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Setid& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Setid_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Setid* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, id_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::id_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Setid)
}
PROTOBUF_NDEBUG_INLINE Setid::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Setid::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, id_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::id_));
}
Setid::~Setid() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Setid)
  SharedDtor(*this);
}
inline void Setid::SharedDtor(MessageLite& self) {
  Setid& this_ = static_cast<Setid&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Setid_class_data_ =
        Setid::InternalGenerateClassData_(Setid_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Setid::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Setid_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Setid_class_data_.tc_table);
  return Setid_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Setid::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Setid_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Setid_globals_));
  return Setid_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Setid::ParseTableT_
    Setid::_table_ =
        Setid::InternalGenerateParseTable_(Setid_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Setid::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Setid)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.id_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.id_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Setid::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Setid& this_ = static_cast<const Setid&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Setid::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Setid& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Setid)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // uint32 id = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_id() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          4, this_._internal_id(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Setid)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Setid::ByteSizeLong(const MessageLite& base) {
  const Setid& this_ = static_cast<const Setid&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Setid::ByteSizeLong() const {
  const Setid& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Setid)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // uint32 id = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_id() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_id());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Setid::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Setid*>(&to_msg);
  auto& from = static_cast<const Setid&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Setid)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_id() != 0) {
        _this->_impl_.id_ = from._impl_.id_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Setid::CopyFrom(const Setid& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Setid)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Setid::InternalSwap(Setid* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Setid, _impl_.id_)
      + sizeof(Setid::_impl_.id_)
      - PROTOBUF_FIELD_OFFSET(Setid, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Setid::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

StructRlimit::StructRlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StructRlimit_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.StructRlimit)
}
StructRlimit::StructRlimit(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const StructRlimit& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, StructRlimit_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE StructRlimit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void StructRlimit::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, cur_),
           0,
           offsetof(Impl_, max_) -
               offsetof(Impl_, cur_) +
               sizeof(Impl_::max_));
}
StructRlimit::~StructRlimit() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.StructRlimit)
  SharedDtor(*this);
}
inline void StructRlimit::SharedDtor(MessageLite& self) {
  StructRlimit& this_ = static_cast<StructRlimit&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull StructRlimit_class_data_ =
        StructRlimit::InternalGenerateClassData_(StructRlimit_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StructRlimit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StructRlimit_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(StructRlimit_class_data_.tc_table);
  return StructRlimit_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
StructRlimit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&StructRlimit_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&StructRlimit_globals_));
  return StructRlimit_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const StructRlimit::ParseTableT_
    StructRlimit::_table_ =
        StructRlimit::InternalGenerateParseTable_(StructRlimit_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void StructRlimit::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.StructRlimit)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&this_._impl_.cur_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.max_) -
                 reinterpret_cast<char*>(&this_._impl_.cur_)) +
                 sizeof(_impl_.max_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL StructRlimit::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const StructRlimit& this_ = static_cast<const StructRlimit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL StructRlimit::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const StructRlimit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.StructRlimit)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // uint64 cur = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_cur() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          1, this_._internal_cur(), target);
    }
  }

  // uint64 max = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_max() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          2, this_._internal_max(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.StructRlimit)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t StructRlimit::ByteSizeLong(const MessageLite& base) {
  const StructRlimit& this_ = static_cast<const StructRlimit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t StructRlimit::ByteSizeLong() const {
  const StructRlimit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.StructRlimit)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // uint64 cur = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_cur() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_cur());
      }
    }
    // uint64 max = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_max() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_max());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void StructRlimit::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<StructRlimit*>(&to_msg);
  auto& from = static_cast<const StructRlimit&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.StructRlimit)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_cur() != 0) {
        _this->_impl_.cur_ = from._impl_.cur_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_max() != 0) {
        _this->_impl_.max_ = from._impl_.max_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void StructRlimit::CopyFrom(const StructRlimit& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.StructRlimit)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void StructRlimit::InternalSwap(StructRlimit* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.max_)
      + sizeof(StructRlimit::_impl_.max_)
      - PROTOBUF_FIELD_OFFSET(StructRlimit, _impl_.cur_)>(
          reinterpret_cast<char*>(&_impl_.cur_),
          reinterpret_cast<char*>(&other->_impl_.cur_));
}

::google::protobuf::Metadata StructRlimit::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Prlimit::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Prlimit::Prlimit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Prlimit_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Prlimit)
}
PROTOBUF_NDEBUG_INLINE Prlimit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Prlimit& from_msg)
      : _has_bits_{from._has_bits_} {}

Prlimit::Prlimit(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Prlimit& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Prlimit_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Prlimit* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  _impl_.new_limit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.new_limit_)
                 : nullptr;
  _impl_.old_limit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.old_limit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, pid_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::pid_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Prlimit)
}
PROTOBUF_NDEBUG_INLINE Prlimit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Prlimit::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, pid_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::pid_));
}
Prlimit::~Prlimit() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Prlimit)
  SharedDtor(*this);
}
inline void Prlimit::SharedDtor(MessageLite& self) {
  Prlimit& this_ = static_cast<Prlimit&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  delete this_._impl_.new_limit_;
  delete this_._impl_.old_limit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Prlimit_class_data_ =
        Prlimit::InternalGenerateClassData_(Prlimit_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Prlimit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Prlimit_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Prlimit_class_data_.tc_table);
  return Prlimit_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Prlimit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Prlimit_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Prlimit_globals_));
  return Prlimit_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Prlimit::ParseTableT_
    Prlimit::_table_ =
        Prlimit::InternalGenerateParseTable_(Prlimit_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Prlimit::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Prlimit)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.new_limit_ != nullptr);
      this_._impl_.new_limit_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.old_limit_ != nullptr);
      this_._impl_.old_limit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000070U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.pid_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.pid_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Prlimit::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Prlimit& this_ = static_cast<const Prlimit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Prlimit::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Prlimit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Prlimit)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 pid = 4;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_pid() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_pid(), target);
    }
  }

  // int64 resource = 5;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_resource() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<5>(
              stream, this_._internal_resource(), target);
    }
  }

  // .gvisor.syscall.StructRlimit new_limit = 6;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.new_limit_, this_._impl_.new_limit_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.StructRlimit old_limit = 7;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        7, *this_._impl_.old_limit_, this_._impl_.old_limit_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Prlimit)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Prlimit::ByteSizeLong(const MessageLite& base) {
  const Prlimit& this_ = static_cast<const Prlimit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Prlimit::ByteSizeLong() const {
  const Prlimit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Prlimit)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // .gvisor.syscall.StructRlimit new_limit = 6;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.new_limit_);
    }
    // .gvisor.syscall.StructRlimit old_limit = 7;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.old_limit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int64 resource = 5;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_resource() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_resource());
      }
    }
    // int32 pid = 4;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_pid() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_pid());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Prlimit::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Prlimit*>(&to_msg);
  auto& from = static_cast<const Prlimit&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Prlimit)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.new_limit_ != nullptr);
      if (_this->_impl_.new_limit_ == nullptr) {
        _this->_impl_.new_limit_ = Super_::CopyConstruct(arena, *from._impl_.new_limit_);
      } else {
        _this->_impl_.new_limit_->MergeFrom(*from._impl_.new_limit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.old_limit_ != nullptr);
      if (_this->_impl_.old_limit_ == nullptr) {
        _this->_impl_.old_limit_ = Super_::CopyConstruct(arena, *from._impl_.old_limit_);
      } else {
        _this->_impl_.old_limit_->MergeFrom(*from._impl_.old_limit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_resource() != 0) {
        _this->_impl_.resource_ = from._impl_.resource_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_pid() != 0) {
        _this->_impl_.pid_ = from._impl_.pid_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Prlimit::CopyFrom(const Prlimit& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Prlimit)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Prlimit::InternalSwap(Prlimit* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.pid_)
      + sizeof(Prlimit::_impl_.pid_)
      - PROTOBUF_FIELD_OFFSET(Prlimit, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Prlimit::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Pipe::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Pipe::Pipe(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Pipe_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Pipe)
}
PROTOBUF_NDEBUG_INLINE Pipe::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Pipe& from_msg)
      : _has_bits_{from._has_bits_} {}

Pipe::Pipe(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Pipe& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Pipe_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Pipe* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, writer_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::writer_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Pipe)
}
PROTOBUF_NDEBUG_INLINE Pipe::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Pipe::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, writer_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::writer_));
}
Pipe::~Pipe() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Pipe)
  SharedDtor(*this);
}
inline void Pipe::SharedDtor(MessageLite& self) {
  Pipe& this_ = static_cast<Pipe&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Pipe_class_data_ =
        Pipe::InternalGenerateClassData_(Pipe_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Pipe::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Pipe_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Pipe_class_data_.tc_table);
  return Pipe_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Pipe::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Pipe_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Pipe_globals_));
  return Pipe_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Pipe::ParseTableT_
    Pipe::_table_ =
        Pipe::InternalGenerateParseTable_(Pipe_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Pipe::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Pipe)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000003cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.writer_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.writer_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Pipe::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Pipe& this_ = static_cast<const Pipe&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Pipe::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Pipe& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Pipe)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // uint32 flags = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          4, this_._internal_flags(), target);
    }
  }

  // int32 reader = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_reader() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_reader(), target);
    }
  }

  // int32 writer = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_writer() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_writer(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Pipe)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Pipe::ByteSizeLong(const MessageLite& base) {
  const Pipe& this_ = static_cast<const Pipe&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Pipe::ByteSizeLong() const {
  const Pipe& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Pipe)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // uint32 flags = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
    // int32 reader = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_reader() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_reader());
      }
    }
    // int32 writer = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_writer() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_writer());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Pipe::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Pipe*>(&to_msg);
  auto& from = static_cast<const Pipe&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Pipe)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_reader() != 0) {
        _this->_impl_.reader_ = from._impl_.reader_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_writer() != 0) {
        _this->_impl_.writer_ = from._impl_.writer_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Pipe::CopyFrom(const Pipe& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Pipe)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Pipe::InternalSwap(Pipe* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Pipe, _impl_.writer_)
      + sizeof(Pipe::_impl_.writer_)
      - PROTOBUF_FIELD_OFFSET(Pipe, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Pipe::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Fcntl::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Fcntl::Fcntl(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Fcntl_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Fcntl)
}
PROTOBUF_NDEBUG_INLINE Fcntl::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Fcntl& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Fcntl::Fcntl(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Fcntl& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Fcntl_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Fcntl* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, args_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::args_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Fcntl)
}
PROTOBUF_NDEBUG_INLINE Fcntl::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Fcntl::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, args_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::args_));
}
Fcntl::~Fcntl() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Fcntl)
  SharedDtor(*this);
}
inline void Fcntl::SharedDtor(MessageLite& self) {
  Fcntl& this_ = static_cast<Fcntl&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Fcntl_class_data_ =
        Fcntl::InternalGenerateClassData_(Fcntl_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Fcntl::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Fcntl_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Fcntl_class_data_.tc_table);
  return Fcntl_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Fcntl::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Fcntl_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Fcntl_globals_));
  return Fcntl_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Fcntl::ParseTableT_
    Fcntl::_table_ =
        Fcntl::InternalGenerateParseTable_(Fcntl_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Fcntl::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Fcntl)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000078U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.args_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.args_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Fcntl::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Fcntl& this_ = static_cast<const Fcntl&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Fcntl::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Fcntl& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Fcntl)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Fcntl.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // int32 cmd = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_cmd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_cmd(), target);
    }
  }

  // int64 args = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_args() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<7>(
              stream, this_._internal_args(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Fcntl)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Fcntl::ByteSizeLong(const MessageLite& base) {
  const Fcntl& this_ = static_cast<const Fcntl&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Fcntl::ByteSizeLong() const {
  const Fcntl& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Fcntl)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // int32 cmd = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_cmd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_cmd());
      }
    }
    // int64 args = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_args() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_args());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Fcntl::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Fcntl*>(&to_msg);
  auto& from = static_cast<const Fcntl&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Fcntl)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_cmd() != 0) {
        _this->_impl_.cmd_ = from._impl_.cmd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_args() != 0) {
        _this->_impl_.args_ = from._impl_.args_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Fcntl::CopyFrom(const Fcntl& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Fcntl)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Fcntl::InternalSwap(Fcntl* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.args_)
      + sizeof(Fcntl::_impl_.args_)
      - PROTOBUF_FIELD_OFFSET(Fcntl, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Fcntl::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Dup::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Dup::Dup(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Dup_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Dup)
}
PROTOBUF_NDEBUG_INLINE Dup::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Dup& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Dup::Dup(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Dup& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Dup_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Dup* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Dup)
}
PROTOBUF_NDEBUG_INLINE Dup::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Dup::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Dup::~Dup() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Dup)
  SharedDtor(*this);
}
inline void Dup::SharedDtor(MessageLite& self) {
  Dup& this_ = static_cast<Dup&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Dup_class_data_ =
        Dup::InternalGenerateClassData_(Dup_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Dup::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Dup_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Dup_class_data_.tc_table);
  return Dup_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Dup::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Dup_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Dup_globals_));
  return Dup_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Dup::ParseTableT_
    Dup::_table_ =
        Dup::InternalGenerateParseTable_(Dup_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Dup::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Dup)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000078U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Dup::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Dup& this_ = static_cast<const Dup&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Dup::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Dup& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Dup)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 old_fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_old_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_old_fd(), target);
    }
  }

  // int32 new_fd = 5;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_new_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_new_fd(), target);
    }
  }

  // string fd_path = 6;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Dup.fd_path");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // uint32 flags = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          7, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Dup)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Dup::ByteSizeLong(const MessageLite& base) {
  const Dup& this_ = static_cast<const Dup&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Dup::ByteSizeLong() const {
  const Dup& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Dup)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string fd_path = 6;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 old_fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_old_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_old_fd());
      }
    }
    // int32 new_fd = 5;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_new_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_new_fd());
      }
    }
    // uint32 flags = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Dup::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Dup*>(&to_msg);
  auto& from = static_cast<const Dup&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Dup)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_old_fd() != 0) {
        _this->_impl_.old_fd_ = from._impl_.old_fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_new_fd() != 0) {
        _this->_impl_.new_fd_ = from._impl_.new_fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Dup::CopyFrom(const Dup& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Dup)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Dup::InternalSwap(Dup* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Dup, _impl_.flags_)
      + sizeof(Dup::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Dup, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Dup::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Signalfd::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Signalfd::Signalfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Signalfd_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Signalfd)
}
PROTOBUF_NDEBUG_INLINE Signalfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Signalfd& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

Signalfd::Signalfd(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Signalfd& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Signalfd_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Signalfd* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, sigset_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::sigset_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Signalfd)
}
PROTOBUF_NDEBUG_INLINE Signalfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void Signalfd::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, sigset_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::sigset_));
}
Signalfd::~Signalfd() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Signalfd)
  SharedDtor(*this);
}
inline void Signalfd::SharedDtor(MessageLite& self) {
  Signalfd& this_ = static_cast<Signalfd&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Signalfd_class_data_ =
        Signalfd::InternalGenerateClassData_(Signalfd_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Signalfd::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Signalfd_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Signalfd_class_data_.tc_table);
  return Signalfd_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Signalfd::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Signalfd_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Signalfd_globals_));
  return Signalfd_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Signalfd::ParseTableT_
    Signalfd::_table_ =
        Signalfd::InternalGenerateParseTable_(Signalfd_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Signalfd::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Signalfd)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000078U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.sigset_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.sigset_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Signalfd::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Signalfd& this_ = static_cast<const Signalfd&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Signalfd::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Signalfd& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Signalfd)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Signalfd.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // uint64 sigset = 6;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_sigset() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_sigset(), target);
    }
  }

  // int32 flags = 7;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_flags() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<7>(
              stream, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Signalfd)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Signalfd::ByteSizeLong(const MessageLite& base) {
  const Signalfd& this_ = static_cast<const Signalfd&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Signalfd::ByteSizeLong() const {
  const Signalfd& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Signalfd)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // int32 flags = 7;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_flags());
      }
    }
    // uint64 sigset = 6;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_sigset() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sigset());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Signalfd::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Signalfd*>(&to_msg);
  auto& from = static_cast<const Signalfd&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Signalfd)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_sigset() != 0) {
        _this->_impl_.sigset_ = from._impl_.sigset_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Signalfd::CopyFrom(const Signalfd& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Signalfd)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Signalfd::InternalSwap(Signalfd* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.sigset_)
      + sizeof(Signalfd::_impl_.sigset_)
      - PROTOBUF_FIELD_OFFSET(Signalfd, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Signalfd::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Chroot::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
Chroot::Chroot(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Chroot_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Chroot)
}
PROTOBUF_NDEBUG_INLINE Chroot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Chroot& from_msg)
      : _has_bits_{from._has_bits_},
        pathname_(arena, from.pathname_) {}

Chroot::Chroot(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Chroot& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Chroot_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Chroot* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  _impl_.sysno_ = from._impl_.sysno_;

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Chroot)
}
PROTOBUF_NDEBUG_INLINE Chroot::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : pathname_(arena) {}

inline void Chroot::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, sysno_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::sysno_));
}
Chroot::~Chroot() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Chroot)
  SharedDtor(*this);
}
inline void Chroot::SharedDtor(MessageLite& self) {
  Chroot& this_ = static_cast<Chroot&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.pathname_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Chroot_class_data_ =
        Chroot::InternalGenerateClassData_(Chroot_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Chroot::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Chroot_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Chroot_class_data_.tc_table);
  return Chroot_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Chroot::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Chroot_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Chroot_globals_));
  return Chroot_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Chroot::ParseTableT_
    Chroot::_table_ =
        Chroot::InternalGenerateParseTable_(Chroot_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Chroot::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Chroot)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.pathname_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  this_._impl_.sysno_ = ::uint64_t{0u};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Chroot::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Chroot& this_ = static_cast<const Chroot&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Chroot::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Chroot& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Chroot)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // string pathname = 4;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_pathname().empty()) {
      const ::std::string& _s = this_._internal_pathname();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Chroot.pathname");
      target = stream->WriteStringMaybeAliased(4, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Chroot)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Chroot::ByteSizeLong(const MessageLite& base) {
  const Chroot& this_ = static_cast<const Chroot&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Chroot::ByteSizeLong() const {
  const Chroot& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Chroot)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // string pathname = 4;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_pathname().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_pathname());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Chroot::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Chroot*>(&to_msg);
  auto& from = static_cast<const Chroot&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Chroot)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_pathname().empty()) {
        _this->_internal_set_pathname(from._internal_pathname());
      } else {
        if (_this->_impl_.pathname_.IsDefault()) {
          _this->_internal_set_pathname("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Chroot::CopyFrom(const Chroot& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Chroot)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Chroot::InternalSwap(Chroot* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.pathname_, &other->_impl_.pathname_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Chroot, _impl_.sysno_)
      + sizeof(Chroot::_impl_.sysno_)
      - PROTOBUF_FIELD_OFFSET(Chroot, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Chroot::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Eventfd::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Eventfd::Eventfd(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Eventfd_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Eventfd)
}
PROTOBUF_NDEBUG_INLINE Eventfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Eventfd& from_msg)
      : _has_bits_{from._has_bits_} {}

Eventfd::Eventfd(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Eventfd& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Eventfd_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Eventfd* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Eventfd)
}
PROTOBUF_NDEBUG_INLINE Eventfd::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Eventfd::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Eventfd::~Eventfd() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Eventfd)
  SharedDtor(*this);
}
inline void Eventfd::SharedDtor(MessageLite& self) {
  Eventfd& this_ = static_cast<Eventfd&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Eventfd_class_data_ =
        Eventfd::InternalGenerateClassData_(Eventfd_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Eventfd::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Eventfd_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Eventfd_class_data_.tc_table);
  return Eventfd_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Eventfd::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Eventfd_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Eventfd_globals_));
  return Eventfd_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Eventfd::ParseTableT_
    Eventfd::_table_ =
        Eventfd::InternalGenerateParseTable_(Eventfd_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Eventfd::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Eventfd)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000001cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Eventfd::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Eventfd& this_ = static_cast<const Eventfd&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Eventfd::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Eventfd& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Eventfd)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 val = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_val() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_val(), target);
    }
  }

  // uint32 flags = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          5, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Eventfd)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Eventfd::ByteSizeLong(const MessageLite& base) {
  const Eventfd& this_ = static_cast<const Eventfd&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Eventfd::ByteSizeLong() const {
  const Eventfd& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Eventfd)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 val = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_val() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_val());
      }
    }
    // uint32 flags = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Eventfd::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Eventfd*>(&to_msg);
  auto& from = static_cast<const Eventfd&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Eventfd)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_val() != 0) {
        _this->_impl_.val_ = from._impl_.val_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Eventfd::CopyFrom(const Eventfd& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Eventfd)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Eventfd::InternalSwap(Eventfd* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.flags_)
      + sizeof(Eventfd::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Eventfd, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Eventfd::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Clone::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Clone::Clone(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Clone_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Clone)
}
PROTOBUF_NDEBUG_INLINE Clone::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Clone& from_msg)
      : _has_bits_{from._has_bits_} {}

Clone::Clone(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Clone& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Clone_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Clone* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, tls_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::tls_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Clone)
}
PROTOBUF_NDEBUG_INLINE Clone::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Clone::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, tls_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::tls_));
}
Clone::~Clone() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Clone)
  SharedDtor(*this);
}
inline void Clone::SharedDtor(MessageLite& self) {
  Clone& this_ = static_cast<Clone&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Clone_class_data_ =
        Clone::InternalGenerateClassData_(Clone_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Clone::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Clone_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Clone_class_data_.tc_table);
  return Clone_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Clone::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Clone_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Clone_globals_));
  return Clone_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Clone::ParseTableT_
    Clone::_table_ =
        Clone::InternalGenerateParseTable_(Clone_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Clone::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Clone)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000007cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.tls_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.tls_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Clone::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Clone& this_ = static_cast<const Clone&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Clone::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Clone& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Clone)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // uint64 flags = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_flags() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          4, this_._internal_flags(), target);
    }
  }

  // uint64 stack = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_stack() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          5, this_._internal_stack(), target);
    }
  }

  // uint64 new_tid = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_new_tid() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          6, this_._internal_new_tid(), target);
    }
  }

  // uint64 tls = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_tls() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          7, this_._internal_tls(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Clone)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Clone::ByteSizeLong(const MessageLite& base) {
  const Clone& this_ = static_cast<const Clone&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Clone::ByteSizeLong() const {
  const Clone& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Clone)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // uint64 flags = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_flags());
      }
    }
    // uint64 stack = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_stack() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_stack());
      }
    }
    // uint64 new_tid = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_new_tid() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_new_tid());
      }
    }
    // uint64 tls = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_tls() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_tls());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Clone::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Clone*>(&to_msg);
  auto& from = static_cast<const Clone&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Clone)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_stack() != 0) {
        _this->_impl_.stack_ = from._impl_.stack_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_new_tid() != 0) {
        _this->_impl_.new_tid_ = from._impl_.new_tid_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_tls() != 0) {
        _this->_impl_.tls_ = from._impl_.tls_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Clone::CopyFrom(const Clone& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Clone)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Clone::InternalSwap(Clone* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Clone, _impl_.tls_)
      + sizeof(Clone::_impl_.tls_)
      - PROTOBUF_FIELD_OFFSET(Clone, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Clone::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Bind::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Bind::Bind(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Bind_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Bind)
}
PROTOBUF_NDEBUG_INLINE Bind::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Bind& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        address_(arena, from.address_) {}

Bind::Bind(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Bind& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Bind_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Bind* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, fd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::fd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Bind)
}
PROTOBUF_NDEBUG_INLINE Bind::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        address_(arena) {}

inline void Bind::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, fd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::fd_));
}
Bind::~Bind() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Bind)
  SharedDtor(*this);
}
inline void Bind::SharedDtor(MessageLite& self) {
  Bind& this_ = static_cast<Bind&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.address_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Bind_class_data_ =
        Bind::InternalGenerateClassData_(Bind_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Bind::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Bind_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Bind_class_data_.tc_table);
  return Bind_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Bind::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Bind_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Bind_globals_));
  return Bind_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Bind::ParseTableT_
    Bind::_table_ =
        Bind::InternalGenerateParseTable_(Bind_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Bind::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Bind)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.address_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000030U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Bind::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Bind& this_ = static_cast<const Bind&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Bind::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Bind& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Bind)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Bind.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // bytes address = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      target = stream->WriteBytesMaybeAliased(6, _s, target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Bind)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Bind::ByteSizeLong(const MessageLite& base) {
  const Bind& this_ = static_cast<const Bind&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Bind::ByteSizeLong() const {
  const Bind& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Bind)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // bytes address = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_address());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Bind::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Bind*>(&to_msg);
  auto& from = static_cast<const Bind&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Bind)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_address().empty()) {
        _this->_internal_set_address(from._internal_address());
      } else {
        if (_this->_impl_.address_.IsDefault()) {
          _this->_internal_set_address("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Bind::CopyFrom(const Bind& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Bind)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Bind::InternalSwap(Bind* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Bind, _impl_.fd_)
      + sizeof(Bind::_impl_.fd_)
      - PROTOBUF_FIELD_OFFSET(Bind, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Bind::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Accept::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
Accept::Accept(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Accept_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Accept)
}
PROTOBUF_NDEBUG_INLINE Accept::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Accept& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        address_(arena, from.address_) {}

Accept::Accept(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Accept& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Accept_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Accept* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Accept)
}
PROTOBUF_NDEBUG_INLINE Accept::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        address_(arena) {}

inline void Accept::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
Accept::~Accept() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Accept)
  SharedDtor(*this);
}
inline void Accept::SharedDtor(MessageLite& self) {
  Accept& this_ = static_cast<Accept&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.address_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Accept_class_data_ =
        Accept::InternalGenerateClassData_(Accept_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Accept::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Accept_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Accept_class_data_.tc_table);
  return Accept_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Accept::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Accept_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Accept_globals_));
  return Accept_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Accept::ParseTableT_
    Accept::_table_ =
        Accept::InternalGenerateParseTable_(Accept_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Accept::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Accept)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.address_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000070U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Accept::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Accept& this_ = static_cast<const Accept&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Accept::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Accept& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Accept)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.Accept.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // bytes address = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_address().empty()) {
      const ::std::string& _s = this_._internal_address();
      target = stream->WriteBytesMaybeAliased(6, _s, target);
    }
  }

  // int32 flags = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_flags() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<7>(
              stream, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Accept)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Accept::ByteSizeLong(const MessageLite& base) {
  const Accept& this_ = static_cast<const Accept&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Accept::ByteSizeLong() const {
  const Accept& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Accept)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // bytes address = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_address().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::BytesSize(
                                        this_._internal_address());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // int32 flags = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Accept::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Accept*>(&to_msg);
  auto& from = static_cast<const Accept&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Accept)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_address().empty()) {
        _this->_internal_set_address(from._internal_address());
      } else {
        if (_this->_impl_.address_.IsDefault()) {
          _this->_internal_set_address("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Accept::CopyFrom(const Accept& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Accept)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Accept::InternalSwap(Accept* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.address_, &other->_impl_.address_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Accept, _impl_.flags_)
      + sizeof(Accept::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(Accept, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Accept::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void TimerfdCreate::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
TimerfdCreate::TimerfdCreate(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdCreate_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.TimerfdCreate)
}
PROTOBUF_NDEBUG_INLINE TimerfdCreate::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::TimerfdCreate& from_msg)
      : _has_bits_{from._has_bits_} {}

TimerfdCreate::TimerfdCreate(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TimerfdCreate& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdCreate_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TimerfdCreate* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.TimerfdCreate)
}
PROTOBUF_NDEBUG_INLINE TimerfdCreate::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void TimerfdCreate::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
TimerfdCreate::~TimerfdCreate() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.TimerfdCreate)
  SharedDtor(*this);
}
inline void TimerfdCreate::SharedDtor(MessageLite& self) {
  TimerfdCreate& this_ = static_cast<TimerfdCreate&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TimerfdCreate_class_data_ =
        TimerfdCreate::InternalGenerateClassData_(TimerfdCreate_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdCreate::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdCreate_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TimerfdCreate_class_data_.tc_table);
  return TimerfdCreate_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdCreate::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdCreate_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TimerfdCreate_globals_));
  return TimerfdCreate_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TimerfdCreate::ParseTableT_
    TimerfdCreate::_table_ =
        TimerfdCreate::InternalGenerateParseTable_(TimerfdCreate_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TimerfdCreate::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.TimerfdCreate)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000001cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TimerfdCreate::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TimerfdCreate& this_ = static_cast<const TimerfdCreate&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TimerfdCreate::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TimerfdCreate& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.TimerfdCreate)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 clock_id = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_clock_id() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_clock_id(), target);
    }
  }

  // int32 flags = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_flags() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.TimerfdCreate)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TimerfdCreate::ByteSizeLong(const MessageLite& base) {
  const TimerfdCreate& this_ = static_cast<const TimerfdCreate&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TimerfdCreate::ByteSizeLong() const {
  const TimerfdCreate& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.TimerfdCreate)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 clock_id = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_clock_id() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_clock_id());
      }
    }
    // int32 flags = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TimerfdCreate::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TimerfdCreate*>(&to_msg);
  auto& from = static_cast<const TimerfdCreate&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.TimerfdCreate)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_clock_id() != 0) {
        _this->_impl_.clock_id_ = from._impl_.clock_id_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TimerfdCreate::CopyFrom(const TimerfdCreate& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.TimerfdCreate)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TimerfdCreate::InternalSwap(TimerfdCreate* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.flags_)
      + sizeof(TimerfdCreate::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(TimerfdCreate, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata TimerfdCreate::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

Timespec::Timespec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Timespec_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Timespec)
}
Timespec::Timespec(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Timespec& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Timespec_get_class_data()),
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena),
#endif  // PROTOBUF_CUSTOM_VTABLE
      _impl_(from._impl_) {
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}
PROTOBUF_NDEBUG_INLINE Timespec::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Timespec::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sec_),
           0,
           offsetof(Impl_, nsec_) -
               offsetof(Impl_, sec_) +
               sizeof(Impl_::nsec_));
}
Timespec::~Timespec() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Timespec)
  SharedDtor(*this);
}
inline void Timespec::SharedDtor(MessageLite& self) {
  Timespec& this_ = static_cast<Timespec&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Timespec_class_data_ =
        Timespec::InternalGenerateClassData_(Timespec_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Timespec::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Timespec_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Timespec_class_data_.tc_table);
  return Timespec_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Timespec::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Timespec_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Timespec_globals_));
  return Timespec_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Timespec::ParseTableT_
    Timespec::_table_ =
        Timespec::InternalGenerateParseTable_(Timespec_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Timespec::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Timespec)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    ::memset(&this_._impl_.sec_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.nsec_) -
                 reinterpret_cast<char*>(&this_._impl_.sec_)) +
                 sizeof(_impl_.nsec_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Timespec::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Timespec& this_ = static_cast<const Timespec&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Timespec::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Timespec& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Timespec)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // int64 sec = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (this_._internal_sec() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<1>(
              stream, this_._internal_sec(), target);
    }
  }

  // int64 nsec = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (this_._internal_nsec() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt64ToArrayWithField<2>(
              stream, this_._internal_nsec(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Timespec)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Timespec::ByteSizeLong(const MessageLite& base) {
  const Timespec& this_ = static_cast<const Timespec&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Timespec::ByteSizeLong() const {
  const Timespec& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Timespec)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // int64 sec = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (this_._internal_sec() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_sec());
      }
    }
    // int64 nsec = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (this_._internal_nsec() != 0) {
        total_size += ::_pbi::WireFormatLite::Int64SizePlusOne(
            this_._internal_nsec());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Timespec::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Timespec*>(&to_msg);
  auto& from = static_cast<const Timespec&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Timespec)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (from._internal_sec() != 0) {
        _this->_impl_.sec_ = from._impl_.sec_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (from._internal_nsec() != 0) {
        _this->_impl_.nsec_ = from._impl_.nsec_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Timespec::CopyFrom(const Timespec& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Timespec)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Timespec::InternalSwap(Timespec* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Timespec, _impl_.nsec_)
      + sizeof(Timespec::_impl_.nsec_)
      - PROTOBUF_FIELD_OFFSET(Timespec, _impl_.sec_)>(
          reinterpret_cast<char*>(&_impl_.sec_),
          reinterpret_cast<char*>(&other->_impl_.sec_));
}

::google::protobuf::Metadata Timespec::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

ItimerSpec::ItimerSpec(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ItimerSpec_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.ItimerSpec)
}
PROTOBUF_NDEBUG_INLINE ItimerSpec::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::ItimerSpec& from_msg)
      : _has_bits_{from._has_bits_} {}

ItimerSpec::ItimerSpec(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const ItimerSpec& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, ItimerSpec_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  ItimerSpec* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.interval_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.interval_)
                 : nullptr;
  _impl_.value_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.value_)
                 : nullptr;

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.ItimerSpec)
}
PROTOBUF_NDEBUG_INLINE ItimerSpec::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void ItimerSpec::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, interval_),
           0,
           offsetof(Impl_, value_) -
               offsetof(Impl_, interval_) +
               sizeof(Impl_::value_));
}
ItimerSpec::~ItimerSpec() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.ItimerSpec)
  SharedDtor(*this);
}
inline void ItimerSpec::SharedDtor(MessageLite& self) {
  ItimerSpec& this_ = static_cast<ItimerSpec&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.interval_;
  delete this_._impl_.value_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull ItimerSpec_class_data_ =
        ItimerSpec::InternalGenerateClassData_(ItimerSpec_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ItimerSpec::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ItimerSpec_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(ItimerSpec_class_data_.tc_table);
  return ItimerSpec_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
ItimerSpec::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&ItimerSpec_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&ItimerSpec_globals_));
  return ItimerSpec_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const ItimerSpec::ParseTableT_
    ItimerSpec::_table_ =
        ItimerSpec::InternalGenerateParseTable_(ItimerSpec_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void ItimerSpec::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.ItimerSpec)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.interval_ != nullptr);
      this_._impl_.interval_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.value_ != nullptr);
      this_._impl_.value_->Clear();
    }
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL ItimerSpec::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const ItimerSpec& this_ = static_cast<const ItimerSpec&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL ItimerSpec::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const ItimerSpec& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.ItimerSpec)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.syscall.Timespec interval = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.interval_, this_._impl_.interval_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Timespec value = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.value_, this_._impl_.value_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.ItimerSpec)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t ItimerSpec::ByteSizeLong(const MessageLite& base) {
  const ItimerSpec& this_ = static_cast<const ItimerSpec&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t ItimerSpec::ByteSizeLong() const {
  const ItimerSpec& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.ItimerSpec)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    // .gvisor.syscall.Timespec interval = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.interval_);
    }
    // .gvisor.syscall.Timespec value = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.value_);
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void ItimerSpec::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<ItimerSpec*>(&to_msg);
  auto& from = static_cast<const ItimerSpec&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.ItimerSpec)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.interval_ != nullptr);
      if (_this->_impl_.interval_ == nullptr) {
        _this->_impl_.interval_ = Super_::CopyConstruct(arena, *from._impl_.interval_);
      } else {
        _this->_impl_.interval_->MergeFrom(*from._impl_.interval_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.value_ != nullptr);
      if (_this->_impl_.value_ == nullptr) {
        _this->_impl_.value_ = Super_::CopyConstruct(arena, *from._impl_.value_);
      } else {
        _this->_impl_.value_->MergeFrom(*from._impl_.value_);
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void ItimerSpec::CopyFrom(const ItimerSpec& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.ItimerSpec)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void ItimerSpec::InternalSwap(ItimerSpec* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.value_)
      + sizeof(ItimerSpec::_impl_.value_)
      - PROTOBUF_FIELD_OFFSET(ItimerSpec, _impl_.interval_)>(
          reinterpret_cast<char*>(&_impl_.interval_),
          reinterpret_cast<char*>(&other->_impl_.interval_));
}

::google::protobuf::Metadata ItimerSpec::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void TimerfdSetTime::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
TimerfdSetTime::TimerfdSetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdSetTime_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.TimerfdSetTime)
}
PROTOBUF_NDEBUG_INLINE TimerfdSetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::TimerfdSetTime& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

TimerfdSetTime::TimerfdSetTime(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TimerfdSetTime& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdSetTime_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TimerfdSetTime* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  _impl_.new_value_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.new_value_)
                 : nullptr;
  _impl_.old_value_ = (CheckHasBit(cached_has_bits, 0x00000010U))
                 ? Super_::CopyConstruct(arena, *from._impl_.old_value_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.TimerfdSetTime)
}
PROTOBUF_NDEBUG_INLINE TimerfdSetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void TimerfdSetTime::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
TimerfdSetTime::~TimerfdSetTime() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.TimerfdSetTime)
  SharedDtor(*this);
}
inline void TimerfdSetTime::SharedDtor(MessageLite& self) {
  TimerfdSetTime& this_ = static_cast<TimerfdSetTime&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  delete this_._impl_.new_value_;
  delete this_._impl_.old_value_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TimerfdSetTime_class_data_ =
        TimerfdSetTime::InternalGenerateClassData_(TimerfdSetTime_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdSetTime::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdSetTime_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TimerfdSetTime_class_data_.tc_table);
  return TimerfdSetTime_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdSetTime::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdSetTime_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TimerfdSetTime_globals_));
  return TimerfdSetTime_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TimerfdSetTime::ParseTableT_
    TimerfdSetTime::_table_ =
        TimerfdSetTime::InternalGenerateParseTable_(TimerfdSetTime_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TimerfdSetTime::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.TimerfdSetTime)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000001fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.new_value_ != nullptr);
      this_._impl_.new_value_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(this_._impl_.old_value_ != nullptr);
      this_._impl_.old_value_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000e0U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TimerfdSetTime::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TimerfdSetTime& this_ = static_cast<const TimerfdSetTime&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TimerfdSetTime::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TimerfdSetTime& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.TimerfdSetTime)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.TimerfdSetTime.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // int32 flags = 6;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_flags() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_flags(), target);
    }
  }

  // .gvisor.syscall.ItimerSpec new_value = 7;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        7, *this_._impl_.new_value_, this_._impl_.new_value_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.ItimerSpec old_value = 8;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        8, *this_._impl_.old_value_, this_._impl_.old_value_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.TimerfdSetTime)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TimerfdSetTime::ByteSizeLong(const MessageLite& base) {
  const TimerfdSetTime& this_ = static_cast<const TimerfdSetTime&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TimerfdSetTime::ByteSizeLong() const {
  const TimerfdSetTime& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.TimerfdSetTime)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // .gvisor.syscall.ItimerSpec new_value = 7;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.new_value_);
    }
    // .gvisor.syscall.ItimerSpec old_value = 8;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.old_value_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // int32 flags = 6;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TimerfdSetTime::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TimerfdSetTime*>(&to_msg);
  auto& from = static_cast<const TimerfdSetTime&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.TimerfdSetTime)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.new_value_ != nullptr);
      if (_this->_impl_.new_value_ == nullptr) {
        _this->_impl_.new_value_ = Super_::CopyConstruct(arena, *from._impl_.new_value_);
      } else {
        _this->_impl_.new_value_->MergeFrom(*from._impl_.new_value_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      ABSL_DCHECK(from._impl_.old_value_ != nullptr);
      if (_this->_impl_.old_value_ == nullptr) {
        _this->_impl_.old_value_ = Super_::CopyConstruct(arena, *from._impl_.old_value_);
      } else {
        _this->_impl_.old_value_->MergeFrom(*from._impl_.old_value_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TimerfdSetTime::CopyFrom(const TimerfdSetTime& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.TimerfdSetTime)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TimerfdSetTime::InternalSwap(TimerfdSetTime* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.flags_)
      + sizeof(TimerfdSetTime::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(TimerfdSetTime, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata TimerfdSetTime::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void TimerfdGetTime::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
TimerfdGetTime::TimerfdGetTime(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdGetTime_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.TimerfdGetTime)
}
PROTOBUF_NDEBUG_INLINE TimerfdGetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::TimerfdGetTime& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

TimerfdGetTime::TimerfdGetTime(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const TimerfdGetTime& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, TimerfdGetTime_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  TimerfdGetTime* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  _impl_.cur_value_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.cur_value_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, fd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::fd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.TimerfdGetTime)
}
PROTOBUF_NDEBUG_INLINE TimerfdGetTime::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void TimerfdGetTime::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, fd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::fd_));
}
TimerfdGetTime::~TimerfdGetTime() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.TimerfdGetTime)
  SharedDtor(*this);
}
inline void TimerfdGetTime::SharedDtor(MessageLite& self) {
  TimerfdGetTime& this_ = static_cast<TimerfdGetTime&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  delete this_._impl_.cur_value_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull TimerfdGetTime_class_data_ =
        TimerfdGetTime::InternalGenerateClassData_(TimerfdGetTime_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdGetTime::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdGetTime_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(TimerfdGetTime_class_data_.tc_table);
  return TimerfdGetTime_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
TimerfdGetTime::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&TimerfdGetTime_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&TimerfdGetTime_globals_));
  return TimerfdGetTime_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const TimerfdGetTime::ParseTableT_
    TimerfdGetTime::_table_ =
        TimerfdGetTime::InternalGenerateParseTable_(TimerfdGetTime_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void TimerfdGetTime::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.TimerfdGetTime)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.cur_value_ != nullptr);
      this_._impl_.cur_value_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000030U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.fd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.fd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL TimerfdGetTime::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const TimerfdGetTime& this_ = static_cast<const TimerfdGetTime&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL TimerfdGetTime::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const TimerfdGetTime& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.TimerfdGetTime)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.TimerfdGetTime.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // .gvisor.syscall.ItimerSpec cur_value = 6;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        6, *this_._impl_.cur_value_, this_._impl_.cur_value_->GetCachedSize(), target,
        stream);
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.TimerfdGetTime)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t TimerfdGetTime::ByteSizeLong(const MessageLite& base) {
  const TimerfdGetTime& this_ = static_cast<const TimerfdGetTime&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t TimerfdGetTime::ByteSizeLong() const {
  const TimerfdGetTime& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.TimerfdGetTime)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // .gvisor.syscall.ItimerSpec cur_value = 6;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.cur_value_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void TimerfdGetTime::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<TimerfdGetTime*>(&to_msg);
  auto& from = static_cast<const TimerfdGetTime&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.TimerfdGetTime)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.cur_value_ != nullptr);
      if (_this->_impl_.cur_value_ == nullptr) {
        _this->_impl_.cur_value_ = Super_::CopyConstruct(arena, *from._impl_.cur_value_);
      } else {
        _this->_impl_.cur_value_->MergeFrom(*from._impl_.cur_value_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void TimerfdGetTime::CopyFrom(const TimerfdGetTime& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.TimerfdGetTime)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void TimerfdGetTime::InternalSwap(TimerfdGetTime* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.fd_)
      + sizeof(TimerfdGetTime::_impl_.fd_)
      - PROTOBUF_FIELD_OFFSET(TimerfdGetTime, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata TimerfdGetTime::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void Fork::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
Fork::Fork(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Fork_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.Fork)
}
PROTOBUF_NDEBUG_INLINE Fork::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::Fork& from_msg)
      : _has_bits_{from._has_bits_} {}

Fork::Fork(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const Fork& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, Fork_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  Fork* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  _impl_.sysno_ = from._impl_.sysno_;

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.Fork)
}
PROTOBUF_NDEBUG_INLINE Fork::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void Fork::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, sysno_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::sysno_));
}
Fork::~Fork() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.Fork)
  SharedDtor(*this);
}
inline void Fork::SharedDtor(MessageLite& self) {
  Fork& this_ = static_cast<Fork&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull Fork_class_data_ =
        Fork::InternalGenerateClassData_(Fork_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Fork::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Fork_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(Fork_class_data_.tc_table);
  return Fork_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
Fork::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&Fork_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&Fork_globals_));
  return Fork_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const Fork::ParseTableT_
    Fork::_table_ =
        Fork::InternalGenerateParseTable_(Fork_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void Fork::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.Fork)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  this_._impl_.sysno_ = ::uint64_t{0u};
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL Fork::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const Fork& this_ = static_cast<const Fork&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL Fork::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const Fork& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.Fork)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.Fork)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t Fork::ByteSizeLong(const MessageLite& base) {
  const Fork& this_ = static_cast<const Fork&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t Fork::ByteSizeLong() const {
  const Fork& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.Fork)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void Fork::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<Fork*>(&to_msg);
  auto& from = static_cast<const Fork&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.Fork)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void Fork::CopyFrom(const Fork& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.Fork)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void Fork::InternalSwap(Fork* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(Fork, _impl_.sysno_)
      + sizeof(Fork::_impl_.sysno_)
      - PROTOBUF_FIELD_OFFSET(Fork, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata Fork::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void InotifyInit::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
InotifyInit::InotifyInit(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyInit_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.InotifyInit)
}
PROTOBUF_NDEBUG_INLINE InotifyInit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::InotifyInit& from_msg)
      : _has_bits_{from._has_bits_} {}

InotifyInit::InotifyInit(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const InotifyInit& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyInit_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  InotifyInit* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, flags_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::flags_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.InotifyInit)
}
PROTOBUF_NDEBUG_INLINE InotifyInit::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void InotifyInit::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, flags_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::flags_));
}
InotifyInit::~InotifyInit() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.InotifyInit)
  SharedDtor(*this);
}
inline void InotifyInit::SharedDtor(MessageLite& self) {
  InotifyInit& this_ = static_cast<InotifyInit&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull InotifyInit_class_data_ =
        InotifyInit::InternalGenerateClassData_(InotifyInit_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyInit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyInit_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(InotifyInit_class_data_.tc_table);
  return InotifyInit_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyInit::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyInit_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&InotifyInit_globals_));
  return InotifyInit_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const InotifyInit::ParseTableT_
    InotifyInit::_table_ =
        InotifyInit::InternalGenerateParseTable_(InotifyInit_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void InotifyInit::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.InotifyInit)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x0000000cU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.flags_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.flags_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL InotifyInit::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const InotifyInit& this_ = static_cast<const InotifyInit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL InotifyInit::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const InotifyInit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.InotifyInit)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 flags = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_flags() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_flags(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.InotifyInit)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t InotifyInit::ByteSizeLong(const MessageLite& base) {
  const InotifyInit& this_ = static_cast<const InotifyInit&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t InotifyInit::ByteSizeLong() const {
  const InotifyInit& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.InotifyInit)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 flags = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_flags() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_flags());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void InotifyInit::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<InotifyInit*>(&to_msg);
  auto& from = static_cast<const InotifyInit&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.InotifyInit)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_flags() != 0) {
        _this->_impl_.flags_ = from._impl_.flags_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void InotifyInit::CopyFrom(const InotifyInit& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.InotifyInit)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void InotifyInit::InternalSwap(InotifyInit* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.flags_)
      + sizeof(InotifyInit::_impl_.flags_)
      - PROTOBUF_FIELD_OFFSET(InotifyInit, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata InotifyInit::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void InotifyAddWatch::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000004U);
}
InotifyAddWatch::InotifyAddWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyAddWatch_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.InotifyAddWatch)
}
PROTOBUF_NDEBUG_INLINE InotifyAddWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::InotifyAddWatch& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_),
        pathname_(arena, from.pathname_) {}

InotifyAddWatch::InotifyAddWatch(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const InotifyAddWatch& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyAddWatch_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  InotifyAddWatch* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000008U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, mask_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::mask_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.InotifyAddWatch)
}
PROTOBUF_NDEBUG_INLINE InotifyAddWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena),
        pathname_(arena) {}

inline void InotifyAddWatch::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, mask_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::mask_));
}
InotifyAddWatch::~InotifyAddWatch() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.InotifyAddWatch)
  SharedDtor(*this);
}
inline void InotifyAddWatch::SharedDtor(MessageLite& self) {
  InotifyAddWatch& this_ = static_cast<InotifyAddWatch&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  this_._impl_.pathname_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull InotifyAddWatch_class_data_ =
        InotifyAddWatch::InternalGenerateClassData_(InotifyAddWatch_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyAddWatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyAddWatch_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(InotifyAddWatch_class_data_.tc_table);
  return InotifyAddWatch_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyAddWatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyAddWatch_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&InotifyAddWatch_globals_));
  return InotifyAddWatch_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const InotifyAddWatch::ParseTableT_
    InotifyAddWatch::_table_ =
        InotifyAddWatch::InternalGenerateParseTable_(InotifyAddWatch_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void InotifyAddWatch::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.InotifyAddWatch)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000000fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      this_._impl_.pathname_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000070U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.mask_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.mask_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL InotifyAddWatch::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const InotifyAddWatch& this_ = static_cast<const InotifyAddWatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL InotifyAddWatch::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const InotifyAddWatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.InotifyAddWatch)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.InotifyAddWatch.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // string pathname = 6;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    if (!this_._internal_pathname().empty()) {
      const ::std::string& _s = this_._internal_pathname();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.InotifyAddWatch.pathname");
      target = stream->WriteStringMaybeAliased(6, _s, target);
    }
  }

  // uint32 mask = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_mask() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt32ToArray(
          7, this_._internal_mask(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.InotifyAddWatch)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t InotifyAddWatch::ByteSizeLong(const MessageLite& base) {
  const InotifyAddWatch& this_ = static_cast<const InotifyAddWatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t InotifyAddWatch::ByteSizeLong() const {
  const InotifyAddWatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.InotifyAddWatch)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // string pathname = 6;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!this_._internal_pathname().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_pathname());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // uint32 mask = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_mask() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt32SizePlusOne(
            this_._internal_mask());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void InotifyAddWatch::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<InotifyAddWatch*>(&to_msg);
  auto& from = static_cast<const InotifyAddWatch&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.InotifyAddWatch)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000007fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      if (!from._internal_pathname().empty()) {
        _this->_internal_set_pathname(from._internal_pathname());
      } else {
        if (_this->_impl_.pathname_.IsDefault()) {
          _this->_internal_set_pathname("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_mask() != 0) {
        _this->_impl_.mask_ = from._impl_.mask_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void InotifyAddWatch::CopyFrom(const InotifyAddWatch& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.InotifyAddWatch)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void InotifyAddWatch::InternalSwap(InotifyAddWatch* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.pathname_, &other->_impl_.pathname_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.mask_)
      + sizeof(InotifyAddWatch::_impl_.mask_)
      - PROTOBUF_FIELD_OFFSET(InotifyAddWatch, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata InotifyAddWatch::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void InotifyRmWatch::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000002U);
}
InotifyRmWatch::InotifyRmWatch(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyRmWatch_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.InotifyRmWatch)
}
PROTOBUF_NDEBUG_INLINE InotifyRmWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::InotifyRmWatch& from_msg)
      : _has_bits_{from._has_bits_},
        fd_path_(arena, from.fd_path_) {}

InotifyRmWatch::InotifyRmWatch(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const InotifyRmWatch& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, InotifyRmWatch_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  InotifyRmWatch* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000004U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, wd_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::wd_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.InotifyRmWatch)
}
PROTOBUF_NDEBUG_INLINE InotifyRmWatch::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
      : fd_path_(arena) {}

inline void InotifyRmWatch::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, wd_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::wd_));
}
InotifyRmWatch::~InotifyRmWatch() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.InotifyRmWatch)
  SharedDtor(*this);
}
inline void InotifyRmWatch::SharedDtor(MessageLite& self) {
  InotifyRmWatch& this_ = static_cast<InotifyRmWatch&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  this_._impl_.fd_path_.Destroy();
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull InotifyRmWatch_class_data_ =
        InotifyRmWatch::InternalGenerateClassData_(InotifyRmWatch_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyRmWatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyRmWatch_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(InotifyRmWatch_class_data_.tc_table);
  return InotifyRmWatch_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
InotifyRmWatch::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&InotifyRmWatch_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&InotifyRmWatch_globals_));
  return InotifyRmWatch_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const InotifyRmWatch::ParseTableT_
    InotifyRmWatch::_table_ =
        InotifyRmWatch::InternalGenerateParseTable_(InotifyRmWatch_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void InotifyRmWatch::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.InotifyRmWatch)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000007U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      this_._impl_.fd_path_.ClearNonDefaultToEmpty();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x00000038U)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.wd_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.wd_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL InotifyRmWatch::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const InotifyRmWatch& this_ = static_cast<const InotifyRmWatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL InotifyRmWatch::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const InotifyRmWatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.InotifyRmWatch)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 fd = 4;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_fd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_fd(), target);
    }
  }

  // string fd_path = 5;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    if (!this_._internal_fd_path().empty()) {
      const ::std::string& _s = this_._internal_fd_path();
      ::google::protobuf::internal::WireFormatLite::VerifyUtf8String(
          _s.data(), static_cast<int>(_s.length()), ::google::protobuf::internal::WireFormatLite::SERIALIZE, "gvisor.syscall.InotifyRmWatch.fd_path");
      target = stream->WriteStringMaybeAliased(5, _s, target);
    }
  }

  // int32 wd = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_wd() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_wd(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.InotifyRmWatch)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t InotifyRmWatch::ByteSizeLong(const MessageLite& base) {
  const InotifyRmWatch& this_ = static_cast<const InotifyRmWatch&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t InotifyRmWatch::ByteSizeLong() const {
  const InotifyRmWatch& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.InotifyRmWatch)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    // string fd_path = 5;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!this_._internal_fd_path().empty()) {
        total_size += 1 + ::google::protobuf::internal::WireFormatLite::StringSize(
                                        this_._internal_fd_path());
      }
    }
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 fd = 4;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_fd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_fd());
      }
    }
    // int32 wd = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_wd() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_wd());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void InotifyRmWatch::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<InotifyRmWatch*>(&to_msg);
  auto& from = static_cast<const InotifyRmWatch&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.InotifyRmWatch)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x0000003fU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      if (!from._internal_fd_path().empty()) {
        _this->_internal_set_fd_path(from._internal_fd_path());
      } else {
        if (_this->_impl_.fd_path_.IsDefault()) {
          _this->_internal_set_fd_path("");
        }
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_fd() != 0) {
        _this->_impl_.fd_ = from._impl_.fd_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_wd() != 0) {
        _this->_impl_.wd_ = from._impl_.wd_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void InotifyRmWatch::CopyFrom(const InotifyRmWatch& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.InotifyRmWatch)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void InotifyRmWatch::InternalSwap(InotifyRmWatch* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  auto* arena = GetArena();
  ABSL_DCHECK_EQ(arena, other->GetArena());
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::_pbi::ArenaStringPtr::InternalSwap(&_impl_.fd_path_, &other->_impl_.fd_path_, arena);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.wd_)
      + sizeof(InotifyRmWatch::_impl_.wd_)
      - PROTOBUF_FIELD_OFFSET(InotifyRmWatch, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata InotifyRmWatch::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// ===================================================================

void SocketPair::clear_context_data() {
  ::google::protobuf::internal::TSanWrite(&_impl_);
  if (_impl_.context_data_ != nullptr) _impl_.context_data_->Clear();
  ClearHasBit(_impl_._has_bits_[0], 0x00000001U);
}
SocketPair::SocketPair(::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, SocketPair_get_class_data()) {
#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SharedCtor(arena);
  // @@protoc_insertion_point(arena_constructor:gvisor.syscall.SocketPair)
}
PROTOBUF_NDEBUG_INLINE SocketPair::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena, const Impl_& from,
    [[maybe_unused]] const ::gvisor::syscall::SocketPair& from_msg)
      : _has_bits_{from._has_bits_} {}

SocketPair::SocketPair(
    ::google::protobuf::Arena* PROTOBUF_NULLABLE arena,
    const SocketPair& from)
#if defined(PROTOBUF_CUSTOM_VTABLE)
    : Super_(arena, SocketPair_get_class_data()) {

#else   // PROTOBUF_CUSTOM_VTABLE
    : Super_(arena) {
#endif  // PROTOBUF_CUSTOM_VTABLE
  SocketPair* const _this = this;
  (void)_this;
  _internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
  new (&_impl_) Impl_(internal_visibility(), arena, from._impl_, from);
  ::uint32_t cached_has_bits = _impl_._has_bits_[0];
  _impl_.context_data_ = (CheckHasBit(cached_has_bits, 0x00000001U))
                 ? Super_::CopyConstruct(arena, *from._impl_.context_data_)
                 : nullptr;
  _impl_.exit_ = (CheckHasBit(cached_has_bits, 0x00000002U))
                 ? Super_::CopyConstruct(arena, *from._impl_.exit_)
                 : nullptr;
  ::memcpy(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, sysno_),
           reinterpret_cast<const char*>(&from._impl_) +
               offsetof(Impl_, sysno_),
           offsetof(Impl_, socket2_) -
               offsetof(Impl_, sysno_) +
               sizeof(Impl_::socket2_));

  // @@protoc_insertion_point(copy_constructor:gvisor.syscall.SocketPair)
}
PROTOBUF_NDEBUG_INLINE SocketPair::Impl_::Impl_(
    [[maybe_unused]] ::google::protobuf::internal::InternalVisibility visibility,
    [[maybe_unused]] ::google::protobuf::Arena* PROTOBUF_NULLABLE arena)
     {}

inline void SocketPair::SharedCtor(::_pb::Arena* PROTOBUF_NULLABLE arena) {
  new (&_impl_) Impl_(internal_visibility(), arena);
  ::memset(reinterpret_cast<char*>(&_impl_) +
               offsetof(Impl_, context_data_),
           0,
           offsetof(Impl_, socket2_) -
               offsetof(Impl_, context_data_) +
               sizeof(Impl_::socket2_));
}
SocketPair::~SocketPair() {
  // @@protoc_insertion_point(destructor:gvisor.syscall.SocketPair)
  SharedDtor(*this);
}
inline void SocketPair::SharedDtor(MessageLite& self) {
  SocketPair& this_ = static_cast<SocketPair&>(self);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  this_._internal_metadata_.Delete<::google::protobuf::UnknownFieldSet>();
  ABSL_DCHECK(this_.GetArena() == nullptr);
  delete this_._impl_.context_data_;
  delete this_._impl_.exit_;
  this_._impl_.~Impl_();
}

#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const
    ::google::protobuf::internal::ClassDataFull SocketPair_class_data_ =
        SocketPair::InternalGenerateClassData_(SocketPair_globals_._default);

PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SocketPair::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SocketPair_class_data_);
  ::google::protobuf::internal::PrefetchToLocalCache(SocketPair_class_data_.tc_table);
  return SocketPair_class_data_.base();
}
#else
PROTOBUF_ATTRIBUTE_WEAK const ::google::protobuf::internal::ClassData* PROTOBUF_NONNULL
SocketPair::GetClassData() const {
  ::google::protobuf::internal::PrefetchToLocalCache(&SocketPair_globals_);
  ::google::protobuf::internal::PrefetchToLocalCache(
      ::google::protobuf::internal::MessageGlobalsBase::ToParseTableBase(&SocketPair_globals_));
  return SocketPair_globals_.GetClassData();
}
#endif  // !PROTOBUF_MESSAGE_GLOBALS
#ifndef PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_CONSTINIT
PROTOBUF_ATTRIBUTE_INIT_PRIORITY1 const SocketPair::ParseTableT_
    SocketPair::_table_ =
        SocketPair::InternalGenerateParseTable_(SocketPair_class_data_.base());
#endif  // !PROTOBUF_MESSAGE_GLOBALS
PROTOBUF_NOINLINE void SocketPair::Clear() {
  auto& this_ [[maybe_unused]] = *this;
  // @@protoc_insertion_point(message_clear_start:gvisor.syscall.SocketPair)
  ::google::protobuf::internal::TSanWrite(&_impl_);
  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x00000003U)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(this_._impl_.context_data_ != nullptr);
      this_._impl_.context_data_->Clear();
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(this_._impl_.exit_ != nullptr);
      this_._impl_.exit_->Clear();
    }
  }
  if (BatchCheckHasBit(cached_has_bits, 0x000000fcU)) {
    ::memset(&this_._impl_.sysno_, 0,
             static_cast<::size_t>(
                 reinterpret_cast<char*>(&this_._impl_.socket2_) -
                 reinterpret_cast<char*>(&this_._impl_.sysno_)) +
                 sizeof(_impl_.socket2_));
  }
  _impl_._has_bits_.Clear();
  _internal_metadata_.Clear<::google::protobuf::UnknownFieldSet>();
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::uint8_t* PROTOBUF_NONNULL SocketPair::_InternalSerialize(
    const ::google::protobuf::MessageLite& base, ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) {
  const SocketPair& this_ = static_cast<const SocketPair&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::uint8_t* PROTOBUF_NONNULL SocketPair::_InternalSerialize(
    ::uint8_t* PROTOBUF_NONNULL target,
    ::google::protobuf::io::EpsCopyOutputStream* PROTOBUF_NONNULL stream) const {
  const SocketPair& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    this_.CheckHasBitConsistency();
  }
  // @@protoc_insertion_point(serialize_to_array_start:gvisor.syscall.SocketPair)
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = this_._impl_._has_bits_[0];
  // .gvisor.common.ContextData context_data = 1;
  if (CheckHasBit(cached_has_bits, 0x00000001U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        1, *this_._impl_.context_data_, this_._impl_.context_data_->GetCachedSize(), target,
        stream);
  }

  // .gvisor.syscall.Exit exit = 2;
  if (CheckHasBit(cached_has_bits, 0x00000002U)) {
    target = ::google::protobuf::internal::WireFormatLite::InternalWriteMessage(
        2, *this_._impl_.exit_, this_._impl_.exit_->GetCachedSize(), target,
        stream);
  }

  // uint64 sysno = 3;
  if (CheckHasBit(cached_has_bits, 0x00000004U)) {
    if (this_._internal_sysno() != 0) {
      target = stream->EnsureSpace(target);
      target = ::_pbi::WireFormatLite::WriteUInt64ToArray(
          3, this_._internal_sysno(), target);
    }
  }

  // int32 domain = 4;
  if (CheckHasBit(cached_has_bits, 0x00000008U)) {
    if (this_._internal_domain() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<4>(
              stream, this_._internal_domain(), target);
    }
  }

  // int32 type = 5;
  if (CheckHasBit(cached_has_bits, 0x00000010U)) {
    if (this_._internal_type() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<5>(
              stream, this_._internal_type(), target);
    }
  }

  // int32 protocol = 6;
  if (CheckHasBit(cached_has_bits, 0x00000020U)) {
    if (this_._internal_protocol() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<6>(
              stream, this_._internal_protocol(), target);
    }
  }

  // int32 socket1 = 7;
  if (CheckHasBit(cached_has_bits, 0x00000040U)) {
    if (this_._internal_socket1() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<7>(
              stream, this_._internal_socket1(), target);
    }
  }

  // int32 socket2 = 8;
  if (CheckHasBit(cached_has_bits, 0x00000080U)) {
    if (this_._internal_socket2() != 0) {
      target =
          ::google::protobuf::internal::WireFormatLite::WriteInt32ToArrayWithField<8>(
              stream, this_._internal_socket2(), target);
    }
  }

  if (ABSL_PREDICT_FALSE(this_._internal_metadata_.have_unknown_fields())) {
    target =
        ::_pbi::WireFormat::InternalSerializeUnknownFieldsToArray(
            this_._internal_metadata_.unknown_fields<::google::protobuf::UnknownFieldSet>(::google::protobuf::UnknownFieldSet::default_instance), target, stream);
  }
  // @@protoc_insertion_point(serialize_to_array_end:gvisor.syscall.SocketPair)
  return target;
}

#if defined(PROTOBUF_CUSTOM_VTABLE)
::size_t SocketPair::ByteSizeLong(const MessageLite& base) {
  const SocketPair& this_ = static_cast<const SocketPair&>(base);
#else   // PROTOBUF_CUSTOM_VTABLE
::size_t SocketPair::ByteSizeLong() const {
  const SocketPair& this_ = *this;
#endif  // PROTOBUF_CUSTOM_VTABLE
  // @@protoc_insertion_point(message_byte_size_start:gvisor.syscall.SocketPair)
  ::size_t total_size = 0;

  ::uint32_t cached_has_bits [[maybe_unused]] = 0;

  ::_pbi::Prefetch5LinesFrom7Lines(&this_);
  cached_has_bits = this_._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    // .gvisor.common.ContextData context_data = 1;
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.context_data_);
    }
    // .gvisor.syscall.Exit exit = 2;
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      total_size += 1 +
                    ::google::protobuf::internal::WireFormatLite::MessageSize(*this_._impl_.exit_);
    }
    // uint64 sysno = 3;
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (this_._internal_sysno() != 0) {
        total_size += ::_pbi::WireFormatLite::UInt64SizePlusOne(
            this_._internal_sysno());
      }
    }
    // int32 domain = 4;
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (this_._internal_domain() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_domain());
      }
    }
    // int32 type = 5;
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (this_._internal_type() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_type());
      }
    }
    // int32 protocol = 6;
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (this_._internal_protocol() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_protocol());
      }
    }
    // int32 socket1 = 7;
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (this_._internal_socket1() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_socket1());
      }
    }
    // int32 socket2 = 8;
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (this_._internal_socket2() != 0) {
        total_size += ::_pbi::WireFormatLite::Int32SizePlusOne(
            this_._internal_socket2());
      }
    }
  }
  return this_.MaybeComputeUnknownFieldsSize(total_size,
                                             &this_._impl_._cached_size_);
}

void SocketPair::MergeImpl(::google::protobuf::MessageLite& to_msg,
                      const ::google::protobuf::MessageLite& from_msg) {
   auto* const _this = static_cast<SocketPair*>(&to_msg);
  auto& from = static_cast<const SocketPair&>(from_msg);
  if constexpr (::_pbi::DebugHardenCheckHasBitConsistency()) {
    from.CheckHasBitConsistency();
  }
  ::google::protobuf::Arena* arena = _this->GetArena();
  // @@protoc_insertion_point(class_specific_merge_from_start:gvisor.syscall.SocketPair)
  ABSL_DCHECK_NE(&from, _this);
  ::uint32_t cached_has_bits = 0;
  (void)cached_has_bits;

  cached_has_bits = from._impl_._has_bits_[0];
  if (BatchCheckHasBit(cached_has_bits, 0x000000ffU)) {
    if (CheckHasBit(cached_has_bits, 0x00000001U)) {
      ABSL_DCHECK(from._impl_.context_data_ != nullptr);
      if (_this->_impl_.context_data_ == nullptr) {
        _this->_impl_.context_data_ = Super_::CopyConstruct(arena, *from._impl_.context_data_);
      } else {
        _this->_impl_.context_data_->MergeFrom(*from._impl_.context_data_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000002U)) {
      ABSL_DCHECK(from._impl_.exit_ != nullptr);
      if (_this->_impl_.exit_ == nullptr) {
        _this->_impl_.exit_ = Super_::CopyConstruct(arena, *from._impl_.exit_);
      } else {
        _this->_impl_.exit_->MergeFrom(*from._impl_.exit_);
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000004U)) {
      if (from._internal_sysno() != 0) {
        _this->_impl_.sysno_ = from._impl_.sysno_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000008U)) {
      if (from._internal_domain() != 0) {
        _this->_impl_.domain_ = from._impl_.domain_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000010U)) {
      if (from._internal_type() != 0) {
        _this->_impl_.type_ = from._impl_.type_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000020U)) {
      if (from._internal_protocol() != 0) {
        _this->_impl_.protocol_ = from._impl_.protocol_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000040U)) {
      if (from._internal_socket1() != 0) {
        _this->_impl_.socket1_ = from._impl_.socket1_;
      }
    }
    if (CheckHasBit(cached_has_bits, 0x00000080U)) {
      if (from._internal_socket2() != 0) {
        _this->_impl_.socket2_ = from._impl_.socket2_;
      }
    }
  }
  _this->_impl_._has_bits_[0] |= cached_has_bits;
  _this->_internal_metadata_.MergeFrom<::google::protobuf::UnknownFieldSet>(
      from._internal_metadata_);
}

void SocketPair::CopyFrom(const SocketPair& from) {
  // @@protoc_insertion_point(class_specific_copy_from_start:gvisor.syscall.SocketPair)
  if (&from == this) return;
  Clear();
  MergeFrom(from);
}


void SocketPair::InternalSwap(SocketPair* PROTOBUF_RESTRICT PROTOBUF_NONNULL other) {
  using ::std::swap;
  _internal_metadata_.InternalSwap(&other->_internal_metadata_);
  swap(_impl_._has_bits_[0], other->_impl_._has_bits_[0]);
  ::google::protobuf::internal::memswap<
      PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.socket2_)
      + sizeof(SocketPair::_impl_.socket2_)
      - PROTOBUF_FIELD_OFFSET(SocketPair, _impl_.context_data_)>(
          reinterpret_cast<char*>(&_impl_.context_data_),
          reinterpret_cast<char*>(&other->_impl_.context_data_));
}

::google::protobuf::Metadata SocketPair::GetMetadata() const {
  return Super_::GetMetadataImpl(GetClassData()->full());
}
// @@protoc_insertion_point(namespace_scope)
}  // namespace syscall
}  // namespace gvisor
namespace google {
namespace protobuf {
}  // namespace protobuf
}  // namespace google
// @@protoc_insertion_point(global_scope)
PROTOBUF_ATTRIBUTE_INIT_PRIORITY2 static ::std::false_type
    _static_init2_ [[maybe_unused]] =
        (::_pbi::AddDescriptors(&descriptor_table_pkg_2fsentry_2fseccheck_2fpoints_2fsyscall_2eproto),
         ::std::false_type{});
#include "google/protobuf/port_undef.inc"
